unit Axes;
{$I DFS.inc}{ Copyright (C) 2007 C M Western; see GraphC.pas for license}
interface

uses
  {$IFNDEF CLX_} Messages, Graphics, Controls, Forms, Dialogs, StdCtrls, Buttons, ExtCtrls,
    {$IFDEF DFS_DELPHI_4_UP} ComCtrls, {$ENDIF}
  {$ELSE} QGraphics, QControls, QForms, QDialogs, QStdCtrls, QButtons, QExtCtrls, QComCtrls, {$ENDIF}
  SysUtils, Classes, AlEdit, Expr, PenChse, MarkChse, FrameGrp, MathSup,
  TextStrm, PropStrm, Gstyle, EditExtr, NewSpin;

type

  { TAxesDlg }

  TAxesDlg = class(TForm)
    GroupBoxX: TGroupBox;
    Lbl0: TLabel;
    xlEdit: TAlignedEdit;
    xlAutoCheck: TCheckBox;
    Lbl1: TLabel;
    xuEdit: TAlignedEdit;
    xuAutoCheck: TCheckBox;
    GroupBoxY: TGroupBox;
    Lbl2: TLabel;
    Lbl3: TLabel;
    yuEdit: TAlignedEdit;
    yuAutoCheck: TCheckBox;
    ylEdit: TAlignedEdit;
    ylAutoCheck: TCheckBox;
    ScaleOnCurveCheck: TCheckBox;
    Label1: TLabel;
    XLabelEdit: TEdit;
    Label2: TLabel;
    YLabelEdit: TEdit;
    Label3: TLabel;
    TitleEdit: TEdit;
    GroupBoxFont: TGroupBox;
    FontSample: TLabel;
    SetFont: TButton;
    AxisStyleChooser: TPenChoose;
    OKBtn: TButton;
    CancelBtn: TButton;
    ApplyButton: TButton;
    FontDialog: TFontDialog;
    GroupBoxOC: TGroupBox;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    ocuEdit: TAlignedEdit;
    ocuAutoCheck: TCheckBox;
    oclEdit: TAlignedEdit;
    oclAutoCheck: TCheckBox;
    OCLabelEdit: TEdit;
    GroupBoxExtra: TGroupBox;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    exuEdit: TAlignedEdit;
    exuAutoCheck: TCheckBox;
    exlEdit: TAlignedEdit;
    exlAutoCheck: TCheckBox;
    exLabelExit: TEdit;
    ShowFitDetailsCheck: TCheckBox;
    CurvePointsEdit: TSpinBox;
    CurvePointsLabel: TLabel;
    MiscGroupBox: TGroupBox;
    procedure OKBtnClick(Sender: TObject);
    procedure AutoKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure AutoMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure LimitState(Sender: TObject);
    procedure SetFontClick(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure ApplyButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FExLabel: string;
    FTitle: string;
    FOCLabel: string;
    FYLabel: string;
    function GetString(const Index: Integer): string;
    procedure SetString(const Index: Integer; const Value: string);
    function GetAuto(const Index: Integer): Boolean;
    function GetDouble(const Index: Integer): Double;
    procedure SetAuto(const Index: Integer; const Value: Boolean);
    procedure SetDouble(const Index: Integer; const Value: Double);
    procedure SetLimits(const Index: Integer; var Point: TDPoint);
    function GetPlotFont: TFont;
    procedure SetPlotFont(const Value: TFont);
    function GetAxisStyle: TLineStyle;
    procedure SetAxisStyle(const Value: TLineStyle);
    function GetnCurvePoints: Integer;
    procedure SetnCurvePoints(const Value: Integer);
  public
    UpdatePlot: procedure of object;
    procedure CheckEntries;
    procedure SetXLimits(var Point: TDPoint);
    procedure SetYLimits(var Point: TDPoint);
    procedure SetOCLimits(var Point: TDPoint);
    procedure SetExLimits(var Point: TDPoint);
    procedure SaveOptionsToStream(ts: TTextStream; Section: string);
    procedure LoadOptionsFromPropStream(ps: TPropReader; Section: string);
    procedure SetDefaults;
  published
    property XLabel: string  index 101 read GetString write SetString;
    property YLabel: string  index 102 read GetString write SetString;
    property OCLabel: string index 103 read GetString write SetString;
    property ExLabel: string index 104 read GetString write SetString;
    property Title: string   index 105 read GetString write SetString;
    property xl: Double  index 1 read GetDouble write SetDouble;
    property xu: Double  index 2 read GetDouble write SetDouble;
    property yl: Double  index 3 read GetDouble write SetDouble;
    property yu: Double  index 4 read GetDouble write SetDouble;
    property ocl: Double index 5 read GetDouble write SetDouble;
    property ocu: Double index 6 read GetDouble write SetDouble;
    property exl: Double index 7 read GetDouble write SetDouble;
    property exu: Double index 8 read GetDouble write SetDouble;
    property xlAuto: Boolean  index 1 read GetAuto write SetAuto;
    property xuAuto: Boolean  index 2 read GetAuto write SetAuto;
    property ylAuto: Boolean  index 3 read GetAuto write SetAuto;
    property yuAuto: Boolean  index 4 read GetAuto write SetAuto;
    property oclAuto: Boolean index 5 read GetAuto write SetAuto;
    property ocuAuto: Boolean index 6 read GetAuto write SetAuto;
    property exlAuto: Boolean index 7 read GetAuto write SetAuto;
    property exuAuto: Boolean index 8 read GetAuto write SetAuto;
    property ScaleOnCurve: Boolean index 150 read GetAuto write SetAuto;
    property PrintFitDetails: Boolean index 151 read GetAuto write SetAuto;
    property PlotFont: TFont read GetPlotFont write SetPlotFont;
    property AxisStyle: TLineStyle read GetAxisStyle write SetAxisStyle;
    property nCurvePoints: Integer read GetnCurvePoints write SetnCurvePoints;
  end;

function AxesDlg: TAxesDlg;

implementation

{$IFNDEF LCL_}
{$R *.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

var
  GlobalAxesDlg: TAxesDlg;

function AxesDlg: TAxesDlg;
begin
  if not Assigned(GlobalAxesDlg) then
    GlobalAxesDlg := TAxesDlg.Create(Application);
  Result := GlobalAxesDlg;
end;

procedure TAxesDlg.CheckEntries;
  procedure CheckLimit(l: TCustomEdit);
  begin
    if l.Enabled then
      try
        if l.Text <> '' then
          StrToFloat(l.Text);
      except
        l.SetFocus;
        raise;
      end;
  end;
begin
  CheckLimit(xlEdit);
  CheckLimit(xuEdit);
  CheckLimit(ylEdit);
  CheckLimit(yuEdit);
  CheckLimit(oclEdit);
  CheckLimit(ocuEdit);
  CheckLimit(exlEdit);
  CheckLimit(exuEdit);
end;

procedure TAxesDlg.FormCreate(Sender: TObject);
begin
  SetDefaults;
end;

procedure TAxesDlg.OKBtnClick(Sender: TObject);
begin
  ApplyButtonClick(Sender);
  Close;
end;

{ Disable axes limit text windows if auto set }
procedure TAxesDlg.LimitState(Sender: TObject);
var
  i: Integer;
begin
  if Sender is TCheckBox then
    for i := 0 to ComponentCount -1 do
       if (Components[i] is TCustomEdit)
          and (TCustomEdit(Components[i]).Tag = TCheckBox(Sender).Tag) then begin
            if Components[i] is TAlignedEdit then
              with TAlignedEdit(Components[i]) do begin
                Enabled := not TCheckBox(Sender).Checked;
                if Enabled then
                  Font.Color := clWindowText
                else
                  Font.Color := clGrayText;
              end
            else
              TCustomEdit(Components[i]).Enabled := TCheckBox(Sender).Checked;
            Exit;
       end;
end;

procedure TAxesDlg.AutoKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  LimitState(Sender);
end;

procedure TAxesDlg.AutoMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  LimitState(Sender);
end;

procedure TAxesDlg.SetFontClick(Sender: TObject);
begin
  FontDialog.Font := FontSample.Font;
  if FontDialog.Execute then
    FontSample.Font := FontDialog.Font;
end;

procedure TAxesDlg.SetLimits(const Index: Integer; var Point: TDPoint);
  procedure SetLimit(const Index: Integer; var Value: Double);
  begin
    if GetAuto(Index) then
      SetDouble(Index,Value)
    else
      Value := GetDouble(Index);
  end;
begin
  SetLimit(Index,Point.X);
  SetLimit(Index+1,Point.Y);
end;

procedure TAxesDlg.SetXLimits(var Point: TDPoint);
begin
  SetLimits(1,Point);
end;

procedure TAxesDlg.SetYLimits(var Point: TDPoint);
begin
  SetLimits(3,Point);
end;

procedure TAxesDlg.SetOCLimits(var Point: TDPoint);
begin
  SetLimits(5,Point);
end;

procedure TAxesDlg.SetExLimits(var Point: TDPoint);
begin
  SetLimits(7,Point);
end;

procedure TAxesDlg.LoadOptionsFromPropStream(ps: TPropReader; Section: string);
begin
  ps.ReadProperties(self, Section,
'nCurvePoints,ScaleOnCurve,xl,xlAuto,xu,xuAuto,yl,ylAuto,yu,yuAuto,ocl,oclAuto,ocu,ocuAuto,PlotFont,AxisStyle,'
+'exl,exlAuto,exu,exuAuto,XLabel,YLabel,OCLabel,ExLabel,Title,PrintFitDetails'
  );
end;

procedure TAxesDlg.SaveOptionsToStream(ts: TTextStream; Section: string);
begin
  SavePropertiesToStream(self, ts, Section,
'nCurvePoints,ScaleOnCurve,xl,xlAuto,xu,xuAuto,yl,ylAuto,yu,yuAuto,ocl,oclAuto,ocu,ocuAuto,PlotFont,AxisStyle,'
+'exl,exlAuto,exu,exuAuto,XLabel,YLabel,OCLabel,ExLabel,Title,PrintFitDetails'
  );
end;

procedure TAxesDlg.SetDefaults;
begin
  xlAuto := True;
  xuAuto := True;
  ylAuto := True;
  yuAuto := True;
  oclAuto := True;
  ocuAuto := True;
  exlAuto := True;
  exuAuto := True;
  xl := 0;
  xu := 1;
  yl := 0;
  yu := 1;
  ocl := -1;
  ocu := 1;
  XLabel := 'x';
  YLabel := 'y';
  OCLabel := 'Obs-Calc';
  ExLabel := 'x';
  Title := 'Title';
  ScaleOnCurve := True;
  nCurvePoints := 200;
  PrintFitDetails := True;
end;

function TAxesDlg.GetDouble(const Index: Integer): Double;
var
  s: String;
begin
  s := GetString(Index);
  if s <> '' then
    Result := StrToFloat(s)
  else
    Result := 0;
end;

procedure TAxesDlg.SetDouble(const Index: Integer; const Value: Double);
begin
  SetString(Index,FloatToStr(Value));
end;

procedure TAxesDlg.SetAuto(const Index: Integer; const Value: Boolean);
var
  i: Integer;
begin
  for i := 0 to ComponentCount - 1 do
    if (Components[i].Tag = Index) and (Components[i] is TCheckBox) then begin
      TCheckBox(Components[i]).Checked := Value;
      LimitState(Components[i]);
      Break;
    end;
end;

function TAxesDlg.GetAuto(const Index: Integer): Boolean;
var
  i: Integer;
begin
  for i := 0 to ComponentCount - 1 do
    if (Components[i].Tag = Index) and (Components[i] is TCheckBox) then begin
      Result := TCheckBox(Components[i]).Checked;
      Exit;
    end;
  raise Exception.Create('GetAuto');
end;

function TAxesDlg.GetAxisStyle: TLineStyle;
begin
  Result := AxisStyleChooser.LineStyle;
end;

procedure TAxesDlg.SetAxisStyle(const Value: TLineStyle);
begin
  AxisStyleChooser.LineStyle := Value;
end;

function TAxesDlg.GetnCurvePoints: Integer;
begin
  Result := CurvePointsEdit.Value;
end;

procedure TAxesDlg.SetnCurvePoints(const Value: Integer);
begin
  CurvePointsEdit.Value := Value;
end;

function TAxesDlg.GetPlotFont: TFont;
begin
  Result := FontSample.Font;
end;

procedure TAxesDlg.SetPlotFont(const Value: TFont);
begin
  FontSample.Font := Value;
end;

function TAxesDlg.GetString(const Index: Integer): string;
var
  i: Integer;
begin
  for i := 0 to ComponentCount - 1 do
    if (Components[i].Tag = Index) and (Components[i] is TCustomEdit) then begin
      Result := TCustomEdit(Components[i]).Text;
      Exit;
    end;
  raise Exception.Create('GetString');
end;

procedure TAxesDlg.SetString(const Index: Integer; const Value: string);
var
  i: Integer;
begin
  for i := 0 to ComponentCount - 1 do
    if (Components[i].Tag = Index) and (Components[i] is TCustomEdit) then begin
      TCustomEdit(Components[i]).Text := Value;
      Break;
    end;
end;

procedure TAxesDlg.CancelBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TAxesDlg.ApplyButtonClick(Sender: TObject);
begin
  CheckEntries;
  if Assigned(UpdatePlot) then
    UpdatePlot;
end;

end.
