unit BaseDataC;
{$I DFS.inc}{ Copyright (C) 2008 C M Western; see GraphC.pas for license}
interface

uses
  {$IFNDEF CLX_} Messages, Graphics, Controls, Forms, Dialogs,
                 Grids, StdCtrls, ExtCtrls, Buttons,
  {$ELSE} QGraphics, QControls, QForms, QDialogs, QGrids, QStdCtrls, QExtCtrls, QButtons, Types, {$ENDIF}
  SysUtils, Classes, NGrid, FormGrid, Expr, Graphs, GStyle, AlEdit, Style,
  TextStrm, PropStrm, EditExtr, NewSpin, Menus, Gendlg, DataSource,
  MDIChildForm, Types, MathSup, IPlotLoader, Units, ExpPlotLoader, Axes, GraphM;

type

  TJoinStyle = (JoinNone, JoinCurrent, JoinAll);

  TBaseDataChild = class(TMDIChildForm)
    Data: TFormulaGrid;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    DataPopupMenu: TPopupMenu;
    InsertRowItem: TMenuItem;
    DeleteRowItem: TMenuItem;
    N1: TMenuItem;
    CutItem: TMenuItem;
    CopyItem: TMenuItem;
    PasteItem: TMenuItem;
    ClearItem: TMenuItem;
    MainMenuItem: TMainMenu;
    DataItem: TMenuItem;
    FitItem: TMenuItem;
    StatusPanel: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    CloseFitInfo: TSpeedButton;
    ShowDataPointsBtn: TSpeedButton;
    ShowFittedButton: TSpeedButton;
    JoinNoneBtn: TSpeedButton;
    JoinCurrentBtn: TSpeedButton;
    JoinAllBtn: TSpeedButton;
    AverageErrorT: TEdit;
    nObsT: TAlignedEdit;
    SwapColumnsItem: TMenuItem;
    SortAllItem: TMenuItem;
    SortItem: TMenuItem;
    SetEntireColumnItem: TMenuItem;
    FindNextDuplicateItem: TMenuItem;
    N2: TMenuItem;
    TransformItem: TMenuItem;
    StyleItem: TMenuItem;
    procedure DataChangeCellQuery(Sender: TObject; Col, Row: Longint;
      var CanChange: Boolean);
    procedure DataChanged(Sender: TObject; ACol, ARow: Longint);
    procedure DataGetCell(Sender: TObject; Col, Row: Longint;
      var v: Double; var Valid: Boolean);
    procedure FormDestroy(Sender: TObject);
    procedure CloseFitInfoClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DataDblClick(Sender: TObject);
    procedure InsertRowItemClick(Sender: TObject);
    procedure DeleteRowItemClick(Sender: TObject);
    procedure ShowDataPointsBtnClick(Sender: TObject);
    procedure ShowFittedButtonClick(Sender: TObject);
    procedure JoinNoneBtnClick(Sender: TObject);
    procedure NonLinearItemClick(Sender: TObject);
    procedure SwapColumnsItemClick(Sender: TObject);
    procedure SortAllItemClick(Sender: TObject);
    procedure SortItemClick(Sender: TObject);
    procedure SetEntireColumnItemClick(Sender: TObject);
    procedure FindNextDuplicateItemClick(Sender: TObject);
    procedure TransformItemClick(Sender: TObject);
    procedure StyleItemClick(Sender: TObject);
  private
    FWeighted: Boolean;
    FTransformX, FTransformf: Boolean;
    FxFunctionText, FyFunctionText: string;
    FLineStyle: TLineStyle;
    FMarkStyle: TMarkStyle;
    ColInfo: array [1..6] of TColInfo;
    FHideData: Boolean;
    FHideFitted: Boolean;
    FGradient, FIntercept, FSigma, FGradientSigma, FInterceptSigma :Double;
    FLastnPar: Integer;
    LastSortCol: Integer;
    WasSelectedValue: Boolean; { Set if result of last GetData(..) was selected by toolbar }
    FID: Integer;
    FJoinStyle: TJoinStyle;
    FDataAsLine: Boolean;
    FFittedAsLine: Boolean;
    FFittedMarkStyle: TMarkStyle;
    FDataLineStyle: TLineStyle;
    function GetDataSource: TInteractiveDataSource;
    procedure SetxFunctionText(s: string);
    procedure SetyFunctionText(s: string);
    procedure EnsureDifferentStyle;
    procedure SetLineStyle(const Value: TLineStyle);
    procedure SetMarkStyle(const Value: TMarkStyle);
    procedure SetHideData(const Value: Boolean);
    procedure SetHideFitted(const Value: Boolean);
    procedure SetModified(const Value: Boolean);
    function GetModified: Boolean;
    procedure SetJoinStyle(const Value: TJoinStyle);
    procedure SetDataLineStyle(const Value: TLineStyle);
    procedure SetFittedMarkStyle(const Value: TMarkStyle);
    procedure SetXLabel(const Value: string);
    procedure SetYLabel(const Value: string);
    function GetXLabel: string;
    function GetYLabel: string;
  protected
    FDataSource: TInteractiveDataSource;
    FFitExpr: TExpr;
    FnDim: Integer;
    FVariables: array of TVariable;
    CurVals: array of Double;
    {$IFDEF MSWINDOWS}
    procedure WMGetMinMaxInfo(var Message: TWMGetMinMaxInfo); message WM_GETMINMAXINFO;
    {$ENDIF}
    {$IFDEF CLX_}
    procedure ConstrainedResize(var MinWidth, MinHeight, MaxWidth, MaxHeight: Integer); override;
    {$ENDIF}
    procedure StuffColTitles(ComboBox: TCustomComboBox; nCols: Integer);
    function GetVariable(Index: Integer): TVariable; virtual;
    procedure SetWeighted(w: Boolean); virtual;
    procedure SetnDim(const Value: Integer); virtual;
    procedure FinishGridChange(DoReSize: Boolean);
    function AxesDlg: TAxesDlg;
    procedure SetAllValues; virtual;
    function PlotAgainst: Integer; virtual;
    function IsSelected(Col:Integer; Value: Double; out Selecting: Boolean):Boolean; virtual;
    function CurValChanged(First: Boolean): Boolean; virtual;
    procedure AddIndependentVariables(Lex: TLex); virtual;
  public
    FitOK:Boolean;
    FNPoints:Longint;
    StyleDlg: TStyleDlg;
    VirginForm: Boolean;
    ExtraLimits: TDPoint;
    nObs: Integer;
    function xTransCol: Integer;
    function WeightCol: Integer;
    function GetOmCcol: Integer;
    function nEditableCols: Integer;
    procedure UpdateVariableNames;
    procedure SetFitExpr(Expr: TExpr; nPar: Longint);
    property OmCcol:Integer read GetOmCcol;
    property TransformX: Boolean read FTransformX;
    property Transformf: Boolean read FTransformf;
    property FitExpr: TExpr read FFitExpr;
    procedure MergeItem(Sender: TObject);
    procedure ShowCols(G:TGraphM; var x, y:Integer; First, Last: Longint; FindWidth: Boolean);
    procedure ShowFitInfo(G:TGraphM; var x, y: Integer; FindWidth: Boolean);
    procedure ShowTitle(G:TGraphM; var x, y: Integer; FindWidth: Boolean);
    procedure PlotData(G:TGraphM);
    procedure PlotOC(G:TGraphM);
    procedure UpdateStatus(AnObs: Longint); virtual;
    procedure DetermineLimits(var Limits: array of TDPoint; var nPoints, nyPoints: Longint; SetXMin, SetXMax, OnCurve, OCLimits: Boolean;
                              ExtraAxis: Integer = -1);
    procedure SetDefaults;
    function GetData(r: Longint; var x, y, w: Double; SetVariableControlsState: Boolean = False; ExtraAxis: Integer = -1): Boolean;
    function GetOC(r: Longint; var x, y, w, yc: Double; ExtraAxis: Integer = -1): Boolean;
    function GetInfoSize(G:TGraphM): TPoint;
    procedure ShowInfo(var Line: Integer; ShowData: Boolean; G:TGraphM);
    procedure LoadStyleOptions;
    procedure UnLoadStyleOptions;
    procedure SaveOptionsToStream(s: TTextStream);
    procedure SaveFitDataToStream(s: TTextStream);
    procedure ReadFromPropStream(p: TPropReader);
    procedure ReadFitDataFromPropStream(p: TPropReader);
    procedure ShellSort(N: Longint; SortOn: Integer; Ascending: Integer);
    procedure SwapColumns(i, j: Integer);
    function AllWeightsOne: Boolean;

    function xVar: TVariable; virtual;
    function yVar: TVariable; virtual;
    property Variable[Index:Integer]:TVariable read GetVariable;
    property XLabel: string read GetXLabel write SetXLabel;
    property YLabel: string read GetYLabel write SetYLabel;

    property Modified: Boolean read GetModified write SetModified;
    property DataSource: TInteractiveDataSource read GetDataSource;
  published
    property xFunctionText: string read FxFunctionText write SetxFunctionText;
    property yFunctionText: string read FyFunctionText write SetyFunctionText;
    property Weighted: Boolean read FWeighted write SetWeighted;
    property LineStyle:TLineStyle read FLineStyle write SetLineStyle;
    property FittedMarkStyle:TMarkStyle read FFittedMarkStyle write SetFittedMarkStyle;
    property MarkStyle:TMarkStyle read FMarkStyle write SetMarkStyle;
    property DataLineStyle:TLineStyle read FDataLineStyle write SetDataLineStyle;
    property HideData: Boolean read FHideData write SetHideData;
    property HideFitted: Boolean read FHideFitted write SetHideFitted;
    property DataAsLine: Boolean read FDataAsLine write FDataAsLine;
    property FittedAsLine: Boolean read FFittedAsLine write FFittedAsLine default True;
    property Gradient: Double read FGradient write FGradient;
    property Intercept: Double read FIntercept write FIntercept;
    property Sigma: Double read FSigma write FSigma;
    property GradientSigma: Double read FGradientSigma write FGradientSigma;
    property InterceptSigma: Double read FInterceptSigma write FInterceptSigma;
    property NPoints:Longint read FNPoints write FNPoints;
    property LastnPar: Integer read FLastnPar write FLastnPar;
    property nDim: Integer read FnDim write SetnDim;
    property ID: Integer read FID write FID;
    property JoinStyle: TJoinStyle read FJoinStyle write SetJoinStyle;
  end;

var
  BaseDataChild: TBaseDataChild;

type

  TDCDataSource = class (TInteractiveDataSource)
  public
    DataChild: TBaseDataChild;
    procedure AddIndependentVariables(Lex:TLex); override;
    function nRows: Integer; override;
    function GetData(r: Longint; var x, y, w: Double; SetVariableControlsState: Boolean = False; ExtraAxis: Integer = -1): Boolean; override;
    function Weighted: Boolean; override;
    function Caption: string; override;
    procedure SetFitExpr(Expr: TExpr; nPar: Integer); override;
  end;

  TLeastPlotLoaderConnector = class(TInteractivePlotLoaderConnector)
  private
    FDataChild: TBaseDataChild;
    FatalError: Boolean;
  protected
    function GetComment: string; override;
    procedure SetComment(const AValue: string); override;
  public
    property ExperimentalPlot: TBaseDataChild read FDataChild;
    constructor Create(AnObject: TObject; AStreamName: string); override;
    destructor Destroy; override;
    procedure ReportError(Msg: string; Fatal: Boolean); override;
    procedure SetIntensityUnits (IntensityUnits: TIntensityUnits); override;
    procedure SetUnits (Plot: Integer; AUnits: TResonanceUnits); override;
    function Another(Index: Integer): TPlotLoaderConnector; override;
  end;

implementation

uses {$IFDEF MSWINDOWS} Windows, {$ENDIF}
     BaseGraphC, IntVec, Gmarks, Setcol, Transfrm, Huge,
     SwapCols, Vector, Math,
     TextAssign, MatrixVec, Sort;

const
  PrintColSize = 15;

{$IFNDEF LCL_}
{$R *.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

procedure TBaseDataChild.DataChangeCellQuery(Sender: TObject; Col,
  Row: Longint; var CanChange: Boolean);
begin
  if Col = OmCcol then CanChange := FitExpr <> nil;
end;

function TBaseDataChild.GetOmCcol:Integer;
begin
  Result := nDim+2;
  if Weighted then Inc(Result);
  if TransformX then Inc(Result);
  if Transformf then Inc(Result);
end;

function TBaseDataChild.GetVariable(Index: Integer): TVariable;
begin
  Result := FVariables[Index];
end;

function TBaseDataChild.GetXLabel: string;
begin
  Result := AxesDlg.XLabel
end;

function TBaseDataChild.GetYLabel: string;
begin
  Result := AxesDlg.YLabel
end;

function TBaseDataChild.xTransCol: Integer;
begin
  Result := WeightCol;
  if Weighted then Inc(Result);
end;

function TBaseDataChild.xVar: TVariable;
begin
  Result := FVariables[0];
end;

function TBaseDataChild.yVar: TVariable;
begin
  Result := FVariables[1];
end;

{$IFDEF MSWINDOWS}
procedure TBaseDataChild.WMGetMinMaxInfo(var Message: TWMGetMinMaxInfo);
begin
  inherited;
  {$IFNDEF VER80}
    Message.MinMaxInfo.ptMaxTrackSize.x := Data.NaturalWidth + 2*GetSystemMetrics(SM_CXFRAME);
  {$ELSE}
    Message.MinMaxInfo^.ptMaxTrackSize.x := Data.NaturalWidth + 2*GetSystemMetrics(SM_CXFRAME);
  {$ENDIF}
  Message.Result := 0;
end;
{$ENDIF}

{$IFDEF CLX_}
procedure TDataChild.ConstrainedResize(var MinWidth, MinHeight, MaxWidth, MaxHeight: Integer);
begin
  inherited;
  if Assigned(Data) then begin
    MaxWidth := Data.NaturalWidth;
    if FitInfo.Visible and ( FitInfo.NaturalWidth > MaxWidth ) then
      MaxWidth := FitInfo.NaturalWidth;
    MaxWidth := MaxWidth + 2*Application.Style.DefaultFrameWidth;
    Constraints.MaxWidth := MaxWidth;
  end;
end;
{$ENDIF}

procedure TBaseDataChild.SetFitExpr(Expr: TExpr; nPar: Longint);
var
  s: Extended;
  v, Calc, x, y, w: Double;
  r: Longint;
  WasModified: Boolean;
begin
  LastnPar := nPar;
  if (Expr = nil) and (FitExpr = nil) then Exit;
  FitOK := True; { Force Full Invalidate }
  DataChanged(Self, -1, -1);
  if Expr = nil then Exit;
  FitOK := True;
  FFitExpr := Expr;
  s := 0;
  nObs := 0;
  with Data do
    try
      BeginUpdate;
      for r := 1 to RowCount-1 do
        if GetData(r, x, y, w, true) then try
          Calc := Expr.Value;
          v := y - Calc;
          WasModified := Modified;
          Value[r,OmCcol] := v;
          Modified := WasModified;
          if Weighted and (w > 0) then begin
            s := s + Sqr(v/w);
            Inc(nObs);
          end else begin
            s := s + Sqr(v);
            Inc(nObs);
          end;
        except
      end;
    finally
      EndUpdate;
    end;
  if (nObs <> nPar) and (nObs > 0) then begin
    Sigma := Sqrt(s/(nObs-nPar));
    AverageErrorT.Text := DtosM(Sigma, 7, -1);
    AverageErrorT.Visible := True;
  end else
    AverageErrorT.Visible := False;
  UpdateStatus(nObs);
end;

procedure TBaseDataChild.SetFittedMarkStyle(const Value: TMarkStyle);
begin
  FFittedMarkStyle.Assign(Value);
end;

procedure TBaseDataChild.MergeItem(Sender: TObject);
var
  s:TFileStream;
begin
  s := nil;
  if OpenDialog.Execute then
  try
    s := TFileStream.Create(OpenDialog.Filename, fmOpenRead or fmShareDenyWrite);
    Data.OffsetLoadFromStream(s, Data.Row, Data.Col);
  finally
    s.Free;
  end;
end;

procedure TBaseDataChild.DataChanged(Sender: TObject; ACol, ARow: Longint);
begin
  if csDestroying in ComponentState then
    Exit;
  if ACol = OmCcol then Exit;
  if FitOK then begin
    FitOK := False;
    Data.InvalidateCol(OmCcol);
    AverageErrorT.Text := '';
    nObsT.Text := '';
    FFitExpr := nil;
  end;
  UpdateStatus(-1);
end;

procedure TBaseDataChild.ShowCols(G:TGraphM; var x, y:Integer; First, Last: Longint; FindWidth: Boolean);
var
  Col, iy, ix, dx, dy: Integer;
  procedure Vline(x: Integer);
  begin
    if not FindWidth then begin
      G.Move(x, iy+dy);
      G.Vector(x, y);
    end;
  end;
  procedure Show1Col;
  var
    ix: Integer;
  begin
    y := iy;
    ix := x;
    Inc(x, dx);
    ShowCol(G, Col, x, y, First, Last, FindWidth, False, ColInfo[Col], Data, 7);
    Inc(Col);
    VLine(ix + dx div 2);
  end;
begin
  dx := Round(G.AvCharWidth);
  dy := Round(G.CellHeight);
  Dec(y, dy div 2);
  Col := 1;
  iy := y;
  ix := x;
  if not FindWidth then
    G.LineStyleP(clBlack, 0, lpSolid);
  Show1Col;
  Show1Col;
  if Weighted then Show1Col;
  if TransformX then Show1Col;
  if Transformf then Show1Col;
  if FitOK then Show1Col;
  if not FindWidth then begin
    G.Move(ix + dx div 2, y);
    G.Vector(x + dx div 2, y);
    G.Vector(x + dx div 2, iy+dy);
    G.Vector(ix + dx div 2, iy+dy);
    G.Move(ix + dx div 2, iy - dy div 4);
    G.Vector(x + dx div 2, iy - dy div 4);
  end;
  Inc(x, dx);
  Dec(y, dy);
end;

function TBaseDataChild.GetData(r: Longint; var x, y, w: Double; SetVariableControlsState: Boolean; ExtraAxis: Integer): Boolean;
var
  Col: Integer;
  v: Double;
  Selected, Selecting: Boolean;
begin
  Result := False;
  if PlotAgainst > nDim then Exit;
  WasSelectedValue := True; { Should be passed as a parameter, but use global variable instead }
  if Data.GetCell(r, PlotAgainst, x) then begin
    if TransformX then
        if not Data.GetCell(r, xTransCol, x) then Exit;
    if Data.GetCell(r, nDim+1, y) then begin
      for Col := 1 to nDim do
        if Col <> PlotAgainst then begin
          if not Data.GetCell(r, Col, v) then Exit;
            Selected := (nDim > 1) and IsSelected(Col-1,v,Selecting);
          WasSelectedValue := WasSelectedValue and Selected;
          { Discard points that are not selected if selection wanted }
          if not Selected and Selecting and (Col <> ExtraAxis) { Disable for second axis for 3d plots } then
            Exit;
          if SetVariableControlsState then
            Variable[Col-1].Value := v;
        end else if SetVariableControlsState then
          Variable[Col-1].Value := x;
      if Weighted then begin
        if not Data.GetCell(r, WeightCol, w) then Exit;
        if w <= 0 then Exit;
      end else
        w := 1;
      if Transformf then
        if not Data.GetCell(r, OmCcol-1, y) then Exit;
      Result := True;
    end;
  end;
  DataSource.DataSourceFitter.ObservedVariable.Value := y;
  DataSource.DataSourceFitter.WeightVariable.Value := w;
end;

function TBaseDataChild.GetDataSource: TInteractiveDataSource;
begin
  if not Assigned(FDataSource) then begin
    FDataSource := TDCDataSource.Create(Self);
    TDCDataSource(FDataSource).DataChild := Self;
//    FDataSource.NLFitDlg.StatusWindow := MainForm.StatusBar;
  end;
  Result := FDataSource;
end;

function TBaseDataChild.GetOC(r: Longint; var x, y, w, yc: Double;  ExtraAxis: Integer): Boolean;
begin
  Result := FitOK;
  if Result then
    Result := GetData(r, x, y, w, True, ExtraAxis);
  if Result then
    try
      Result := False;
      if FitExpr = nil then
        yc := y - (Gradient*x + Intercept)
      else begin
        yc := FitExpr.Value;
        // if Transformf then
          // raise Exception.Create('Help');
        yc := y - yc;
      end;
      Result := True;
    except
    end;
end;

function TBaseDataChild.GetInfoSize(G:TGraphM):TPoint;
var
  dy, Last, FitX, DataX, y: Integer;
begin
  FitX := 0;
  y := G.YPixels;
  dy := Round(G.CellHeight);
  if not HideData or (FitOK and not HideFitted) then
    ShowTitle(G, FitX, y, True);
  if FitOK and not HideFitted then
    ShowFitInfo(G, FitX, y, True);
  Last := y div dy - 2;
  if Last > Data.RowCount - 1 then Last := Data.RowCount - 1;
  DataX := 0;
  if not HideData then
    ShowCols(G, DataX, y, 1, Last, True);
  if DataX > FitX then
    Result.X := DataX
  else
    Result.X := FitX;
  Result.Y := G.YPixels - y;
  if Result.Y = 0 then Result.X := 0;
end;

procedure TBaseDataChild.ShowTitle(G:TGraphM; var x, y: Integer; FindWidth: Boolean);
var
  lx, dx, dy: Integer;
begin
  dy := Round(G.CellHeight);
  dx := Round(G.AvCharWidth);
  if (Copy(Caption, 1, 9) <> '[Untitled') or (MainForm.MDIChildCount > 2) then begin
    lx := G.TextExtent(Caption).cx;
    if not FindWidth then
      G.TextI(0, y, Caption);
    if MainForm.MDIChildCount > 2 then begin
      if FindWidth then
        Inc(lx, 13*dx)
      else begin
        G.LineStyle := LineStyle;
        G.Move(lx+dx, y);
        G.Vector(lx+13*dx, y + dy);
        G.MarkT(lx+7*dx, y + dy div 2, MarkStyle);
      end;
    end;
    if x < lx then x := lx;
    Dec(y, dy);
  end;
  if (CompareText(Caption, OpenDialog.FileName) <> 0) and (Length(OpenDialog.FileName) <> 0) then begin
    lx := G.TextExtent(Caption).cx;
    if not FindWidth then
      G.TextI(0, y, OpenDialog.FileName);
    if x < lx then x := lx;
    Dec(y, dy);
  end;
end;

const
  {$IFDEF LCL_}
  PlusOrMinus = #$C2#$B1;  { UTF8 sequence }
  {$ELSE}
  PlusOrMinus = '';
  {$ENDIF}
procedure TBaseDataChild.ShowFitInfo(G:TGraphM; var x, y: Integer; FindWidth: Boolean);
var
  dy: Integer;
  procedure Txt(s: string);
  begin
    if FindWidth then begin
      if Length(s)*G.AvCharWidth > x then x := Round(Length(s)*G.AvCharWidth);
    end else
      G.TextI(0, y, s);
    Dec(y, dy);
  end;
begin
  dy := Round(G.CellHeight);
  if FitExpr = nil then begin
    Txt('Linear Least Squares Fit');
    Txt(Format('Intercept: %s  '+PlusOrMinus+' %s', [dtosm(Intercept, 12, -1), dtosm(InterceptSigma, 7, -1)]));
    Txt(Format('Gradient: %s '+PlusOrMinus+' %s', [dtosm(Gradient, 12, -1), dtosm(GradientSigma, 7, -1)]));
  end else
    DataSource.NLFitDlg.ShowFitInfo(G, x, y, FindWidth);
  Txt(Format('Average Error:  %s', [dtosm(Sigma, 7, -1)]));
  Txt(Concat(IntToStr(nObs), ' Observations'));
  Txt('Errors are one standard deviation');
end;

procedure TBaseDataChild.ShowInfo(var Line: Integer; ShowData: Boolean; G:TGraphM);
var
  Last: Longint;
  dy, x: Integer;
begin
  dy := Round(G.CellHeight);
  G.Justify := taLeftJustify;
  x := 0;
  if (ShowData and not HideData) or (FitOK and not HideFitted) then
    ShowTitle(G, x, Line, False);
  x := 0;
  if FitOK and not HideFitted then
    ShowFitInfo(G, x, Line, False);
  x := 0;
  Last := Line div dy;
  if Last > Data.RowCount - 1 then Last := Data.RowCount - 1;
  if ShowData and not HideData then
    ShowCols(G, x, Line, 1, Last, False);
end;

function TBaseDataChild.PlotAgainst: Integer;
begin
  Result := 1;
end;

procedure TBaseDataChild.PlotData(G:TGraphM);
var
  xl, xu, yl, yu, x, y, w: Double;
  i: Integer;
  First, SetExtra: Boolean;
  r: Longint;
  { Save selected points, so they can be joined with a line }
  xs, ys: array of Double;
  nxys: Integer;
  IncludeWeights: Boolean;
  procedure PurgeCurrentLine;
  var
    i: Integer;
  begin
    if nxys > 1 then begin
      G.ClipPoint(xs[0],ys[0]);
      for i := 1 to nxys-1 do
        G.ClipJoin(xs[i],ys[i]);
    end;
    nxys := 0;
  end;
begin
  G.StyleT(LineStyle);
  if not HideData then begin
    nxys := 0;
    SetExtra := (JoinStyle = JoinAll) and (nDim > 1);
    if SetExtra then
      SetLength(curvals, nDim);
    IncludeWeights := not AllWeightsOne;
    First := False;
    for r := 1 to Data.RowCount-1 do
      if GetData(r, x, y, w, SetExtra) then
        if IncludeWeights then
          G.ErrorBarY(x, y-w, y+w)
        else begin
          if SetExtra and CurValChanged(First) then
            PurgeCurrentLine;
          First := False;
          if SetExtra or (WasSelectedValue and (JoinStyle = JoinCurrent) and (nDim > 1)) then begin
            Inc(nxys);
            if nxys > Length(xs) then begin
              SetLength(xs, nxys+100); SetLength(ys, nxys+100);
            end;
            xs[nxys-1] := x; ys[nxys-1] := y;
          end;
          G.SymbolT(x, y, MarkStyle);
        end;
    PurgeCurrentLine;
  end;
  if FitOK and not HideFitted then begin
    G.MapArg(xl, xu, yl, yu);
    if FitExpr = nil then begin
      G.ClipPoint(xl, Gradient*xl+Intercept);
      G.ClipJoin(xu, Gradient*xu+Intercept);
    end else begin
      First := True;
      SetAllValues;
      for i := 0 to AxesDlg.nCurvePoints do begin
        xVar.Value := xl + i*(xu-xl)/AxesDlg.nCurvePoints;
        try
          y := FitExpr.Value;
          if First then begin
            G.ClipPoint(xVar.Value, y);
            First := False;
          end else
            G.ClipJoin(xVar.Value, y);
        except
        end;
      end;
    end;
  end;
end;

procedure TBaseDataChild.PlotOC(G:TGraphM);
var
  r: Longint;
  x, y, w, yc: Double;
  IncludeWeights: Boolean;
begin
  if not FitOK or HideData then Exit;
  IncludeWeights := not AllWeightsOne;
  G.StyleT(LineStyle);
  for r := 1 to Data.RowCount-1 do
    if GetOC(r, x, yc, w, y) then
      if IncludeWeights then
        G.ErrorBarY(x, y-w, y+w)
      else
        G.SymbolT(x, y, MarkStyle);
end;

procedure TBaseDataChild.DataGetCell(Sender: TObject; Col, Row: Longint;
  var v: Double; var Valid: Boolean);
var
  x, y, w: Double;
begin
  if Col = OmCcol then begin
    Valid := FitOK and GetData(Row, x, y, w);
    if Valid and (FitExpr = nil) then
      v := y - Intercept - Gradient*x;
  end;
end;

procedure TBaseDataChild.UpdateVariableNames;
var
  i: Integer;
begin
  for i := 1 to nDim do begin
    Data.Formula[i] := '';
    Data.Title[i] := Variable[i-1].Name;
    Data.ColumnVariableName[i] := Variable[i-1].Name;
  end;
end;

procedure TBaseDataChild.FinishGridChange(DoReSize: Boolean);
var
  i: Integer;
begin
  Data.ColCount := OmCcol+1;
  if Weighted then begin
    Data.Formula[WeightCol] := '';
    Data.Title[WeightCol] := 'Sigma';
  end;
  UpdateVariableNames;
  Data.Formula[nDim+1] := '';
  if nDim = 1 then
    Data.Title[2] := 'y'
  else
    Data.Title[nDim+1] := 'f';
  Data.ColumnVariableName[nDim+1] := Data.Title[nDim+1];
  if TransformX then begin
    try
      Data.Formula[xTransCol] := xFunctionText;
    except
      FTransformX := False;
      raise;
    end;
    Data.Title[xTransCol] := xFunctionText;
  end;
  if Transformf then begin
    try
      Data.Formula[OmCcol-1] := yFunctionText;
    except
      FTransformf := False;
      raise;
    end;
    Data.Title[OmCcol-1] := yFunctionText;
  end;
  Data.Formula[OmCcol] := '';
  Data.Title[OmCcol] := 'Obs-Calc';
  if DoReSize then begin
    for i := 1 to Data.ColCount-1 do
      Data.ColWidths[i] := Data.DefaultColWidth;
  end;
  { FormResize(Self); }
  DataChanged(Self, -1, -1);
  {$IFDEF LCL_}
    Constraints.MaxWidth := Data.NaturalWidth;
  {$ENDIF}
  if DoReSize then
    ClientWidth := Data.NaturalWidth;
end;

procedure TBaseDataChild.SetWeighted(w: Boolean);
begin
  if w <> Weighted then begin
    FWeighted := w;
    FinishGridChange(True);
  end;
end;

procedure TBaseDataChild.SetxFunctionText(s: string);
begin
  s := Trim(s);
  if s = xVar.Name then
    s := '';
  if CompareText(s, xFunctionText) <> 0 then begin
    FxFunctionText := s;
    FTransformX := s <> '';
    FinishGridChange(False);
  end;
end;

procedure TBaseDataChild.SetXLabel(const Value: string);
begin
  AxesDlg.XLabel := Value;
end;

procedure TBaseDataChild.SetyFunctionText(s: string);
begin
  s := Trim(s);
  if s = Data.Title[nDim+1] then
    s := '';
  if CompareText(s, yFunctionText) <> 0 then begin
    FyFunctionText := s;
    FTransformf := s <> '';
    FinishGridChange(False);
  end;
end;

procedure TBaseDataChild.SetYLabel(const Value: string);
begin
  AxesDlg.YLabel := Value;
end;

procedure TBaseDataChild.SwapColumns(i, j: Integer);
var
  r: Longint;
  t: Double;
begin
  if i = j then Exit;
  for r := 1 to Data.RowCount-1 do
    if Data.Valid[r,i] then
      if Data.Valid[r,j] then begin
        t := Data[r,i];
        Data[r,i] := Data[r,j];
        Data[r,j] := t;
      end else begin
        Data[r,j] := Data[r,i];
        Data.Valid[r,i] := False;
      end
    else if Data.Valid[r,j] then begin
      Data[r,i] := Data[r,j];
      Data.Valid[r,j] := False;
    end;
end;

procedure TBaseDataChild.SwapColumnsItemClick(Sender: TObject);
begin
  with SwapCols.SwapColsDlg do begin
    StuffColTitles(Column1, nEditableCols);
    StuffColTitles(Column2, nEditableCols);
    if Column2.ItemIndex = Column1.ItemIndex then
      Column2.ItemIndex := (Column1.ItemIndex+1) mod nEditableCols;
    if ShowModal = mrOK then
      SwapColumns(Column1.ItemIndex+1, Column2.ItemIndex+1);
  end;
end;

procedure TBaseDataChild.ShellSort(N: Longint; SortOn: Integer; Ascending: Integer);
var
  Offset, I, J, T: Longint;
  IsInOrder: Boolean;
  OrgOrder: TIntColumnVector;
  A: TColumnVector;
  c: Integer;
  v: Double;
  function Compare(i, j: Longint):Integer;
  { < 0 if i wanted before j }
  var
    v1, v2: Double;
  begin
    Result := 0;
    if A.GetCell(i, v1) then begin
      if A.GetCell(j, v2) then begin
        if v1 < v2 then
          Result := -1*Ascending
        else if v1 > v2 then
          Result := 1*Ascending;
      end else
        Result := -1;
    end else if A.Valid[j] then
      Result := 1;
  end;
begin
  OrgOrder := nil;
  A := nil;
  try
    OrgOrder := TIntColumnVector.Create(N);
    for I := 1 to N do
      OrgOrder[I] := I;
    { copy column to allow sorting on, for example, the obs-calc column }
    A := TColumnVector.Create(N);
    for I := 1 to N do
      if Data.GetCell(I, SortOn, v) then
        A[I] := v
      else
        A.Valid[I] := False;
    Offset := N;
    while Offset > 1 do begin
      Offset := Offset shr 1;
      repeat
        IsInOrder := True;
        for J := 1 to N - Offset do begin
          I := J + Offset;
          c := Compare(I, J);
          if (c < 0) or ((c = 0) and (OrgOrder[I] < OrgOrder[J])) then begin
            IsInOrder := False;
            T := OrgOrder[I];
            OrgOrder[I] := OrgOrder[J];
            OrgOrder[J] := T;
            { Swap values by binary copy so nan's are copied properly }
            System.Move(A.Item[i]^, v, SizeOf(Double));
            System.Move(A.Item[j]^, A.Item[i]^, SizeOf(Double));
            System.Move(v, A.Item[j]^, SizeOf(Double));
            Data.ExchangeRows(I,J);
          end;
        end;
      until IsInOrder;
    end;
  finally
    OrgOrder.Free;
    A.Free;
  end;
end;

procedure TBaseDataChild.AddIndependentVariables(Lex: TLex);
begin
  Lex.AddVariable(xVar);
end;

function TBaseDataChild.AllWeightsOne: Boolean;
var
  r: Integer;
  w: Double;
begin
  Result := True;
  if not Weighted then
    Exit;
  for r := 1 to Data.RowCount-1 do
    if Data.GetCell(r, WeightCol, w) then begin
      Result := Result and ((w <= 0) or (w = 1));
      if not Result then
        Exit;
    end;
end;

procedure TBaseDataChild.SortAllItemClick(Sender: TObject);
var
  c: Integer;
begin
  ShellSort(Data.RowCount-1, nDim+1, 1);
  for c := 1 to nDim do
    ShellSort(Data.RowCount-1, c, 1);
end;

procedure TBaseDataChild.SortItemClick(Sender: TObject);
var
  SortOn: Integer;
  Ascending: Integer;
begin
  StuffColTitles(SortDlg.SortOn, Data.ColCount-1);
  if SortDlg.ShowModal = mrOK then begin
    SortOn := SortDlg.SortOn.ItemIndex+1;
    Ascending := 1 - SortDlg.Order.ItemIndex*2;
    ShellSort(Data.RowCount-1, SortOn, Ascending);
  end;
end;

procedure TBaseDataChild.DataDblClick(Sender: TObject);
var
  Ascending: Integer;
begin
  with Data.Selection do
    if (Right > 0) and (Right = Left) and (Bottom = Data.DataExtent.Bottom) and (Top = 1) then begin
      if LastSortCol = Left then
        Ascending := -1
      else
        Ascending := 1;
      ShellSort(Data.RowCount-1, Left, Ascending);
      if LastSortCol = Left then
        LastSortCol := -1 { Ascending next time }
      else
        LastSortCol := Left;
    end;
end;

procedure TBaseDataChild.UpdateStatus(AnObs: Longint);
begin
  Data.InvalidateCol(OmCcol);
  if AnObs >= 0 then begin
    Data.EditorMode := False;
    nObs := AnObs;
    nObsT.Text := IntToStr(nObs);
  end;
  if Assigned(BaseGraphChild) then begin
    BaseGraphChild.G.PlotChanged;
    BaseGraphChild.OC.PlotChanged;
  end;
end;

procedure TBaseDataChild.DetermineLimits(var Limits: array of TDPoint; var nPoints, nyPoints: Longint; SetXMin, SetXMax, OnCurve, OCLimits: Boolean; ExtraAxis: Integer);
var
  r, np: Longint;
  w, x, y, yc: Double;
  i: Integer;
  Valid: Boolean;
  ExVar: TVariable; { Extra axis variable }
  IncludeWeights: Boolean;
  procedure CheckY(const y: Double);
  begin
    if IsNanOrInf(y) then
      Exit;
    if nyPoints = 0 then begin
      Limits[1].X := y;
      Limits[1].Y := y;
    end else begin
      if Limits[1].X > y then Limits[1].X := y;
      if Limits[1].Y < y then Limits[1].Y := y;
    end;
    Inc(nyPoints);
  end;
begin
  SetAllValues;
  if ExtraAxis > 0 then
    ExVar := Variable[ExtraAxis-1]
  else
    ExVar := nil;
  if OnCurve then begin
    if FitOK and not HideFitted then
      if FitExpr = nil then begin
        CheckY(Gradient*Limits[0].X+Intercept);
        CheckY(Gradient*Limits[0].Y+Intercept);
      end else begin
        for i := 0 to AxesDlg.nCurvePoints do begin
          xVar.Value := Limits[0].X + i*(Limits[0].Y-Limits[0].X)/AxesDlg.nCurvePoints;
          try
            CheckY(FitExpr.Value);
          except
          end;
        end;
      end;
  end else if not (HideData and HideFitted) then begin
    np := 0;
    IncludeWeights := not AllWeightsOne;
    for r := 1 to Data.RowCount-1 do begin
      if OCLimits then
        Valid := GetOC(r, x, yc, w, y, ExtraAxis)
      else
        Valid := GetData(r, x, y, w, ExtraAxis > 0, ExtraAxis);
      if Valid and (SetXmin or (x >= Limits[0].X)) and (SetXmax or (x <= Limits[0].Y)) then begin
        if not IncludeWeights then w := 0;
        if nPoints = 0 then begin
          if SetXMin then
            Limits[0].X := x;
          if SetXMax then
            Limits[0].Y := x;
          if Assigned(ExVar) then begin
            Limits[2].X := ExVar.Value;
            Limits[2].Y := ExVar.Value;
          end;
        end else begin
          if SetXMin and (Limits[0].X > x) then Limits[0].X := x;
          if SetXMax and (Limits[0].Y < x) then Limits[0].Y := x;
          if Assigned(ExVar) then begin
            if Limits[2].X > ExVar.Value then Limits[2].X := ExVar.Value;
            if Limits[2].Y < ExVar.Value then Limits[2].Y := ExVar.Value;
          end;
        end;
        Inc(nPoints);
        if not HideData then begin
          CheckY(y-w);
          if Limits[1].Y < y+w then Limits[1].Y := y+w;
        end;
        Inc(np);
      end;
    end;
    nObs := np;
    nObsT.Text := IntToStr(nObs);
  end;
end;

procedure TBaseDataChild.NonLinearItemClick(Sender: TObject);
begin
  DataSource.NLFitDlg.Show;
end;

procedure TBaseDataChild.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FLineStyle);
  FreeAndNil(FDataLineStyle);
  FreeAndNil(FMarkStyle);
  FreeAndNil(FFittedMarkStyle);
  FreeAndNil(StyleDlg);
end;

function TBaseDataChild.AxesDlg: TAxesDlg;
begin
  Result := Axes.AxesDlg; { Could overrride if needed }
end;

procedure TBaseDataChild.CloseFitInfoClick(Sender: TObject);
begin
  DataChanged(Self, -1, -1);
end;

function TBaseDataChild.CurValChanged(First: Boolean): Boolean;
begin
  Result := First;
end;

procedure TBaseDataChild.FormCreate(Sender: TObject);
begin
  FnDim := -1;
  FittedAsLine := True;
  FLineStyle := TLineStyle.Create;
  FFittedMarkStyle := TMarkStyle.Create;
  FMarkStyle := TMarkStyle.Create;
  FDataLineStyle := TLineStyle.Create;
  EnsureDifferentStyle;
  SetDefaults;
  Modified := False;
  VirginForm := True;
  FID := 0;
  FJoinStyle := JoinCurrent;
  {$IFDEF LCLcarbon}
    MainMenuItem.Free; { Confuses Carbon Menu system }
  {$ENDIF}
end;

procedure TBaseDataChild.SetAllValues;
begin
 {}
end;

procedure TBaseDataChild.SetDataLineStyle(const Value: TLineStyle);
begin
  FDataLineStyle.Assign(Value);
end;

procedure TBaseDataChild.SetDefaults;
begin
  nDim := 1;
  Weighted := False;
  xFunctionText := '';
  yFunctionText := '';
  Data.RowCount := 100;
//  Data.RowCount := MainForm.MaximumDataPoints+1;
end;

procedure TBaseDataChild.EnsureDifferentStyle;
var
  Patterns: set of TLinePattern;
  Marks: set of TMarkType;
  i: Integer;
begin
  Patterns := [];
  Marks := [];
  with MainForm do
    for i := 0 to MDIChildCount - 1 do
      if (MDIChildren[i] is TBaseDataChild) and (MDIChildren[i] <> Self) then
        with TBaseDataChild(MDIChildren[i]) do begin
          Patterns := Patterns + [LineStyle.Pattern];
          Marks := Marks + [MarkStyle.MarkType];
        end;
  while (LineStyle.Pattern in Patterns) and (LineStyle.Pattern < lpEndPoints) do
    LineStyle.Pattern := Succ(LineStyle.Pattern);
  while (MarkStyle.MarkType in Marks) and (MarkStyle.MarkType < mtVbar) do
    MarkStyle.MarkType := Succ(MarkStyle.MarkType);
end;

procedure TBaseDataChild.SetEntireColumnItemClick(Sender: TObject);
var
  Expr: Texpr;
  Lex: TLex;
  i: TVariable;
  ConstantPi: TConstant;
  Cols: array of TVariable;
  r: Longint;
  c, Col: Integer;
  Rect: TGridRect;
begin
  Lex := nil;
  Expr := nil;
  i := nil;
  ConstantPi := nil;
  with SetColumnDlg do begin
    First.MaxValue := Data.RowCount - 1;
    Last.MaxValue := Data.RowCount - 1;
    Rect := Data.Selection;
    if Abs(Rect.Top - Rect.Bottom) <= 1 then
      Rect := Data.DataExtent;
    if Rect.Top < Rect.Bottom then begin
      First.Value := Rect.Top;
      Last.Value := Rect.Bottom;
    end else begin
      First.Value := Rect.Bottom;
      Last.Value := Rect.Top;
    end;
    StuffColTitles(ColumnToSet, nEditableCols);
    if ShowModal = mrOK then
      try
        Col := ColumnToSet.ItemIndex + 1;
        if Col = 0 then Exit;
        Lex := TLex.Create;
        Lex.Options := Lex.Options - [xFundamentalConstants];
        if LoadedFunctions.Count > 0 then
          Lex.Options := Lex.Options + [xLoadedFunctions];
        i := TVariable.Create('i');
        Lex.AddVariable(i);
        ConstantPi := TConstant.Create(Pi, 'pi');
        Lex.AddFunction(ConstantPi);
        SetLength(Cols, nEditableCols+1); { Element 0 unused }
        for c := 1 to nEditableCols do begin
          Cols[c] := TVariable.Create(Data.Title[c]);
          Lex.AddVariable(Cols[c]);
        end;
        Expr := TExpr.Create(Lex);
        Expr.Text := Formula.Text;
        for r := First.Value to Last.Value do begin
          i.Value := r;
          for c := 1 to nEditableCols do
            Data.GetCell(r, c, Cols[c].Value);
          try
            Data[r, Col] := Expr.Value;
          except
            Data.Valid[r, Col] := False;
          end;
        end;
      finally
        Expr.Free;
        Lex.Free;
        i.Free;
        for c := 1 to nEditableCols do
          Cols[c].Free;
        ConstantPi.Free;
      end;
  end;
end;

procedure TBaseDataChild.TransformItemClick(Sender: TObject);
var
  DefaultXLabel, DefaultYLabel: Boolean;
  function IsDefault(const FunctionText, MLabel, VarName: string): Boolean;
  begin
    if FunctionText = '' then
      Result := CompareText(MLabel,VarName) = 0
    else
      Result := CompareText(FunctionText,MLabel) = 0;
  end;
begin
  TransformationsDlg.xFunction.Text := xFunctionText;
  DefaultXLabel := IsDefault(xFunctionText, XLabel, xVar.Name);
  TransformationsDlg.yFunction.Text := yFunctionText;
  DefaultYLabel := IsDefault(yFunctionText, YLabel, yVar.Name);
  TransformationsDlg.xVarName := xVar.Name;
  TransformationsDlg.yVarName := yVar.Name;
  TransformationsDlg.BaseDataChild := Self;
  if TransformationsDlg.ShowModal = mrOK then begin
    xFunctionText := TransformationsDlg.xFunction.Text;
    if DefaultXLabel then
      XLabel := TransformationsDlg.xFunction.Text;
    yFunctionText := TransformationsDlg.yFunction.Text;
    if DefaultYLabel then
      YLabel := TransformationsDlg.yFunction.Text;
  end
end;

procedure TBaseDataChild.StyleItemClick(Sender: TObject);
var
  w: TStyleDlg;
begin
  if StyleDlg = nil then begin
    w := TStyleDlg.Create(Self);
    StyleDlg := w;
    w.DataChild := Self;
    LoadStyleOptions;
  end;
  StyleDlg.Show;
end;

procedure TBaseDataChild.LoadStyleOptions;
begin
  StyleDlg.FittedAsLine.Checked := FittedAsLine;
  StyleDlg.DataAsLine.Checked := DataAsLine;
  StyleDlg.FittedPenChoose.LineStyle := LineStyle;
  StyleDlg.FittedMarkChoose.MarkStyle := FittedMarkStyle;
  StyleDlg.DataMarkChoose.MarkStyle := MarkStyle;
  StyleDlg.DataPenChoose.LineStyle := DataLineStyle;
  StyleDlg.HideData.Checked := HideData;
  StyleDlg.HideFitted.Checked := HideFitted;
  StyleDlg.DataName.Text := Caption;
end;

procedure TBaseDataChild.UnLoadStyleOptions;
begin
  FLineStyle.Assign(StyleDlg.FittedPenChoose.LineStyle);
  FMarkStyle.Assign(StyleDlg.DataMarkChoose.MarkStyle);
  MarkStyle := StyleDlg.DataMarkChoose.MarkStyle;
  DataLineStyle := StyleDlg.DataPenChoose.LineStyle;
  HideData := StyleDlg.HideData.Checked;
  HideFitted := StyleDlg.HideFitted.Checked;
  FittedAsLine := StyleDlg.FittedAsLine.Checked;
  DataAsLine := StyleDlg.DataAsLine.Checked;
//  if Length(StyleDlg.DataName.Text) = 0 then
//    SetFileName(OpenDialog.FileName)
//  else
    Caption := StyleDlg.DataName.Text;
  Modified := True;
  UpdateStatus(nObs);
end;

procedure TBaseDataChild.SetLineStyle(const Value: TLineStyle);
begin
  FLineStyle.Assign(Value);
end;

procedure TBaseDataChild.SetMarkStyle(const Value: TMarkStyle);
begin
  FMarkStyle.Assign(Value);
end;

procedure TBaseDataChild.SetHideData(const Value: Boolean);
begin
  if FHideData <> Value then begin
    FHideData := Value;
    ShowDataPointsBtn.Down := not Value;
    UpdateStatus(nObs);
  end;
end;

procedure TBaseDataChild.SetHideFitted(const Value: Boolean);
begin
  if FHideFitted <> Value then begin
    FHideFitted := Value;
    ShowFittedButton.Down := not Value;
    UpdateStatus(nObs);
  end;
end;

procedure TBaseDataChild.SaveFitDataToStream(s: TTextStream);
begin
  if FitOK or Assigned(FDataSource) then begin
    s.PutString('[Fit]');
    s.NewLine;
  end;
  if FitOK then begin
    s.PutString('FitOK=True');
    s.NewLine;
    SavePropertiesToStream(self, s, '', 'Sigma,NPoints');
    if FitExpr = nil then
      SavePropertiesToStream(self, s, '', 'Gradient,Intercept,GradientSigma,InterceptSigma')
    else
      SavePropertiesToStream(self, s, '', 'LastnPar');
  end;
  if Assigned(FDataSource) then
    DataSource.NLFitDlg.SaveOptionsToStream(s);
end;

procedure TBaseDataChild.ReadFitDataFromPropStream(p: TPropReader);
var
  s: string;
begin
  GetDataSource;
  if p.Sections['Fit'] <> nil then begin
    FitOK := False;
    if p.ReadString('Fit', 'FunctionText', s) or p.ReadString('Fit', 'Function', s)
       or p.ReadString('Fit', 'FunctionTextMemo.Lines.count', s) then begin
      DataSource.NLFitDlg.ReadFromPropStream(p);
    end;
    if p.ReadBool('Fit', 'FitOK', FitOK) then begin
      p.ReadProperties(self, 'Fit', 'Sigma,NPoints,LastnPar,Gradient,Intercept,GradientSigma,InterceptSigma');
      if p.ReadString('Fit', 'LastnPar', s) then begin
        if Assigned(FDataSource) and (DataSource.DataSourceFitter.Expr <> nil) then
          SetFitExpr(DataSource.DataSourceFitter.Expr, LastnPar);
      end;
    end;
  end;
end;

procedure TBaseDataChild.SaveOptionsToStream(s: TTextStream);
begin
  SavePropertiesToStream(self, s, 'Local',
'Caption,Weighted,xFunctionText,yFunctionText,LineStyle,HideFitted,MarkStyle,HideData,nDim,Data.ColumnVariableNames,ID,JoinStyle,'
+'DataAsLine,FittedAsLine,FittedMarkStyle,DataLineStyle'
  );
  SaveFitDataToStream(s);
  s.PutString('[Data]');
  s.NewLine;
end;

procedure TBaseDataChild.ReadFromPropStream(p: TPropReader);
var
  s: THugeMemoryStream;
  DataStrings: TStrings;
  i: Integer;
begin
  p.ReadProperties(self, 'Local',
'Caption,Weighted,LineStyle,HideFitted,MarkStyle,HideData,nDim,Data.ColumnVariableNames,ID,JoinStyle,'
+'DataAsLine,FittedAsLine,FittedMarkStyle,DataLineStyle'
  );
  if p.ReadInteger('Local', 'Data.ColumnVariableNames.Count', i) then begin
    for i := 0 to nDim-1 do
      Variable[i].Name := Data.ColumnVariableNames[i];
    FinishGridChange(False);
//    MainForm.SetupVarsToolbar;
  end;
  p.ReadProperties(self, 'Local', 'xFunctionText,yFunctionText');
  DataStrings := p.Sections['Data'];
  s := nil;
  if DataStrings <> nil then
    try
      s := THugeMemoryStream.Create;
      DataStrings.SaveToStream(s);
      s.Position := 0;
      Data.LoadFromStream(s);
    finally
      s.Free;
    end;
//  MainForm.UpdateToolBar;
  ReadFitDataFromPropStream(p);
//  MainForm.LoadOptionsFromPropStream(p);
end;

procedure TBaseDataChild.SetnDim(const Value: Integer);
begin
  if (FnDim <> Value) and (Value > 0) then begin
    FnDim := Value;
    FinishGridChange(True);
    YLabel := Data.Title[nDim+1];
  end;
end;

function TBaseDataChild.WeightCol: Integer;
begin
  Result := FnDim + 2;
end;

function TBaseDataChild.nEditableCols: Integer;
begin
  Result := nDim+1;
  if Weighted then Inc(Result);
end;

procedure TBaseDataChild.StuffColTitles(ComboBox: TCustomComboBox; nCols: Integer);
var
  i, LastIndex: Integer;
begin
  with ComboBox do begin
    LastIndex := ItemIndex;
    Clear;
    for i := 1 to nCols do
      Items.Add(Data.Title[i]);
    if (LastIndex >= 0) and (LastIndex < nCols) then
      ItemIndex := LastIndex
    else
      ItemIndex := 0;
  end;
end;

function TBaseDataChild.GetModified: Boolean;
begin
  Result := Data.Modified or (Assigned(FDataSource) and DataSource.NLFitDlg.Modified);
end;

procedure TBaseDataChild.SetModified(const Value: Boolean);
begin
  Data.Modified := Value;
  if not Value and Assigned(FDataSource) then
    DataSource.NLFitDlg.Modified := Value;
end;

procedure TBaseDataChild.InsertRowItemClick(Sender: TObject);
var
  v: Double;
  r, c: Longint;
begin
  with Data do begin
    RowCount := RowCount + 1;
    for c := 1 to nEditableCols do begin
      for r := RowCount-1 downto Row+1 do
        if GetCell(r-1, c, v) then
          Value[r,c] := v
        else
          Valid[r,c] := False;
      Valid[Row, c] := False;
    end;
  end;
end;

function TBaseDataChild.IsSelected(Col: Integer; Value: Double; out Selecting: Boolean): Boolean;
begin
  Result := False;
  Selecting := False;
end;

procedure TBaseDataChild.DeleteRowItemClick(Sender: TObject);
var
  v: Double;
  r, c, n: Longint;
begin
  with Data do begin
    n := Selection.Bottom - Selection.Top + 1;
    for c := 1 to nEditableCols do begin
      for r := Selection.Bottom to RowCount-1 do
        if GetCell(r, c, v) then
          Value[r-n,c] := v
        else
          Valid[r-n,c] := False;
    end;
    RowCount := RowCount - n;
  end;
end;

procedure TBaseDataChild.ShowDataPointsBtnClick(Sender: TObject);
begin
  HideData := not ShowDataPointsBtn.Down;
end;

procedure TBaseDataChild.ShowFittedButtonClick(Sender: TObject);
begin
  HideFitted := not ShowFittedButton.Down;
end;

procedure TBaseDataChild.SetJoinStyle(const Value: TJoinStyle);
begin
  if FJoinStyle <> Value then begin
    FJoinStyle := Value;
    case Value of
      JoinNone:    JoinNoneBtn.Down := true;
      JoinCurrent: JoinCurrentBtn.Down := True;
      JoinAll:     JoinAllBtn.Down := true;
    end;
    UpdateStatus(nObs);
  end;
end;

procedure TBaseDataChild.JoinNoneBtnClick(Sender: TObject);
begin
  JoinStyle := TJoinStyle((Sender as TComponent).Tag);
end;

procedure TBaseDataChild.FindNextDuplicateItemClick(Sender: TObject);
var
  Vals: array of Double;
  CurValid, Rising: array of Boolean;
  v1, v2: Double;
  c, c1, Scale, csmall, Difference: Integer;
  r, r1: Longint;
  Distance: array of Longint;
  NullRow, Wrapped: Boolean;
begin
  SetLength(Vals, nDim);
  SetLength(CurValid, nDim);
  SetLength(Rising, nDim);
  SetLength(Distance, nDim);
  { Scan to find first difference in each column }
  for c := 0 to nDim-1 do begin
    r := 1;
    while (r <= Data.RowCount-1) and not Data.GetCell(r, c+1, v1) do
      Inc(r);
    r1 := r+1;
    while (r1 <= Data.RowCount-1)
          and (not Data.GetCell(r1, c+1, v2) or SameValue(v2, v1)) do
      Inc(r1);
    Rising[c] := v2 >= v1;
    Distance[c] := r1-r;
  end;
  { Figure out probable sort order from order of distances }
  Scale := -1;
  for c := 0 to nDim-1 do begin
    { find smallest remaining difference }
    csmall := -1;
    for c1 := 0 to nDim-1 do
      if Distance[c1] > 0 then
        if (csmall < 0) or (Distance[c1] < Distance[csmall]) then
          csmall := c1;
    Distance[csmall] := Scale;
    Scale := Scale*10;
  end;
  r := Data.Row;
  { Get initial values }
  for c := 0 to nDim-1 do
    CurValid[c] := Data.GetCell(r, c+1, Vals[c]);
  r1 := -1; { First unsorted row }
  Wrapped := False;
  repeat
    Inc(r);
    if r >= Data.RowCount then begin
      r := 1;
      Wrapped := True;
    end;
    NullRow := True;
    for c := 0 to nDim-1 do begin
      NullRow := NullRow and not Data.Valid[r,c+1];
      if not NullRow then
        Break;
    end;
    if NullRow then
      Continue;
    Difference := 0;
    for c := 0 to nDim-1 do begin
      if Data.GetCell(r, c+1, v1) then begin
        if CurValid[c] then begin
          if not SameValue(v1, Vals[c]) then
            { A difference }
            if Rising[c] xor (v1 < Vals[c]) then
              Inc(Difference,-Distance[c])
            else
              Inc(Difference,Distance[c]);
        end else begin
          Inc(Difference,Distance[c]);
        end;
        Vals[c] := v1;
        CurValid[c] := True;
      end else if CurValid[c] then begin
        Inc(Difference,-Distance[c]);
        CurValid[c] := False;
      end;
      { else no difference }
    end;
    if Difference = 0 then begin
      Data.Row := r; { Found a duplicate }
//      MainForm.StatusBar.SimpleText := 'Duplicate found';
      Exit;
    end;
    if not (Wrapped xor (Difference > 0)) then begin
      if r1 < 0 then
        r1 := r;
    end;
    Wrapped := False;
  until r = Data.Row;
  if r1 < 0 then
    MessageDlg('No duplicates found', mtInformation, [mbOK], 0)
  else begin
    Data.Row := r1;
    MessageDlg('No duplicates found, but data may not be sorted', mtInformation, [mbOK], 0);
  end;
end;

{ TDCDataSource }

procedure TDCDataSource.AddIndependentVariables(Lex: TLex);
begin
  DataChild.AddIndependentVariables(Lex);
end;

function TDCDataSource.Caption: string;
begin
  Result := DataChild.Caption;
end;

function TDCDataSource.GetData(r: Integer; var x, y, w: Double;
  SetVariableControlsState: Boolean; ExtraAxis: Integer): Boolean;
begin
  Result := DataChild.GetData(r,x,y,w,SetVariableControlsState,ExtraAxis);
end;

function TDCDataSource.nRows: Integer;
begin
  Result :=  DataChild.Data.RowCount-1
end;

procedure TDCDataSource.SetFitExpr(Expr: TExpr; nPar: Integer);
begin
  DataChild.SetFitExpr(Expr, nPar);
end;

function TDCDataSource.Weighted: Boolean;
begin
  Result := DataChild.Weighted;
end;

{ TLeastPlotLoaderConnector }

function TLeastPlotLoaderConnector.Another(Index: Integer): TPlotLoaderConnector;
var
  NewDataChild: TBaseDataChild;
begin
  { Open another data child }
  NewDataChild := TBaseDataChild.Create(MainForm);
//  NewDataChild.FileNew(nil);
  Result := TLeastPlotLoaderConnector.Create(NewDataChild, StreamName);
end;

constructor TLeastPlotLoaderConnector.Create(AnObject: TObject; AStreamName: string);
begin
  inherited;
  FDataChild := AnObject as TBaseDataChild;
  FatalError := False;
end;

destructor TLeastPlotLoaderConnector.Destroy;
var
  i: Integer;
begin
  if not FatalError then begin
    { Copy back data }
    FDataChild.Weighted := High(Weights) > 0;
    with FDataChild.Data do begin
      BeginUpdate;
      if RowCount <= Length(Frequency) then
        RowCount := Length(Frequency)*2;
      for i := 0 to High(Frequency) do begin
        Value[i+1,1] := Frequency[i];
        Value[i+1,2] := Intensity[i];
        if FDataChild.Weighted then
          Value[i+1,3] := Weights[i];
      end;
      EndUpdate;
    end;
  end;
  inherited;
end;

function TLeastPlotLoaderConnector.GetComment: string;
begin
  Result := FDataChild.AxesDlg.Title;
end;

procedure TLeastPlotLoaderConnector.ReportError(Msg: string; Fatal: Boolean);
begin
  FatalError := FatalError or Fatal;
  inherited;
end;

procedure TLeastPlotLoaderConnector.SetComment(const AValue: string);
begin
  FDataChild.AxesDlg.Title := AValue;
end;

procedure TLeastPlotLoaderConnector.SetIntensityUnits(IntensityUnits: TIntensityUnits);
begin
  { TODO }
end;

procedure TLeastPlotLoaderConnector.SetUnits(Plot: Integer; AUnits: TResonanceUnits);
begin
  FDataChild.XLabel := TypeOfUnit(AUnits) + '/' + NameOfUnit(AUnits);
end;

end.


