unit DatLoader;
{$I DFS.inc}{ Copyright (C) 2005 C M Western; see NLFit.pas for license}
interface

uses ExpPlotLoader, TextStrm;

type

  TDatFileLoader = class(TExperimentalPlotLoader)
  private
    S: TTextStream;
    procedure ReadBody;
    function DetectChannels: Integer;
  public
    function Load(var Log: TextFile; AskAboutWeights: Boolean = true): Boolean; override;
    destructor Destroy; override;
  end;

implementation

uses Math, SysUtils, Classes;

{ TDatFileLoader }

destructor TDatFileLoader.Destroy;
begin
  FreeAndNil(S);
  inherited;
end;

function TDatFileLoader.DetectChannels: Integer;
var
  nCol: Integer;
  Value: Double;
  MinCols: Integer; { Number of complete rows of data }
  i: Integer;
begin
  MinCols := 0;
  Result := 0;
  while not S.AtEndOfFile do begin
    nCol := 0;
    while not S.AtEndOfLine do
      if S.GetFreeItem then begin
        if not S.IsRealToken(Value) then
          Break;
        if nCol >= Length(ChannelDest) then
          SetnChannels(nCol+1);
        AddChange(nCol,Value);
        Inc(nCol);
      end;
    S.NextLine;
    if nCol > 0 then begin
      Inc(Result);
      if MinCols = 0 then
        MinCols := nCol
      else if nCol < MinCols then
        MinCols := nCol;
    end;
  end;
  { Treat channel as valid if > 50% of the lines have data }
  if (MinCols > 0) and (MinCols < Length(ChannelDest)) then
    for i := MinCols to High(ChannelDest) do
      if (nValues[i] >= Result div 2) and (nValues[i] > 0) then
        MinCols := i+1;
  SetLength(ChannelDest,MinCols);
  SetnChannels(MinCols);
  if MinCols < 1 then
    Exit;
  AutoDetectFrequency;
end;

function TDatFileLoader.Load(var Log: TextFile; AskAboutWeights: Boolean): Boolean;
var
  N: Integer;
begin
  S := TTextStream.Create(Stream);
  S.ItemSeparators := [isTab, isComma];
  { Determine number of Columns and Rows of data }
  N := DetectChannels;
  Result := False;
  if (N = 0) or (Length(ChannelDest) = 0) then
    Exit;
  CheckAvailableChannels(Length(ChannelDest), N, AskAboutWeights);
  ReadBody;
  Result := True;
end;

procedure TDatFileLoader.ReadBody;
var
  i, nCol: Integer;
  Value: Double;
begin
  { Repeat read logic, this time setting arrays }
  S.Position := 0;
  i := 0;
  while not S.AtEndOfFile do begin
    nCol := 0;
    while not S.AtEndOfLine do
      if S.GetFreeItem then begin
        if not S.IsRealToken(Value) or (nCol >= Length(ChannelDest)) then
          Break;
        SetValue(-1, nCol, i, Value);
        Inc(nCol);
      end;
    S.NextLine;
    if nCol > 0 then
      Inc(i);
  end;
end;

end.
