unit PersistentVariables;
{$I DFS.inc}{Copyright (C) 2008 C M Western; see NLFit.pas for license}
interface
{ TPersistent descendant that allows published properties to be used as Expr variables }

uses TypInfo, Classes, Expr;

type

  TVariabledPersistent = class;

  TPersistentVariable = class (TArray)
  private
    Owner: TVariabledPersistent;
    PropIndex: Integer;
  public
    constructor Create(AOwner: TVariabledPersistent; APropIndex: Integer);
    function Value(var a:array of Double):Double; override;
    procedure Store(var a:array of Double); override;
  end;

  TVariabledPersistent = class(TPersistent)
  private
    PropList: PPropList;
    FName: string;
    FPrefixVariables: Boolean;
    function GetVariable(Index: Integer): TArray;
    function GetnVariables: Integer;
  protected
    PersistentVariables: array of TArray;
    procedure SetName(const NewName: string); virtual;
  public
    constructor Create;
    destructor Destroy; override;
    procedure AddVariables(Lex: TLex; Prefix:string); virtual;
    property nVariables: Integer read GetnVariables;
    property Variable[Index:Integer]:TArray read GetVariable;
    property Name: string read FName write SetName;
    property PrefixVariables: Boolean read FPrefixVariables write FPrefixVariables;
  end;


implementation

{ TVariabledPersistent }

procedure TVariabledPersistent.AddVariables(Lex: TLex; Prefix: string);
var
  i: Integer;
begin
  if PrefixVariables and (Prefix = '') then
    Prefix := Prefix + Name + '.';
  for i := 0 to nVariables - 1 do
    Lex.AddFunction(Prefix+Variable[i].Name,Variable[i]);
end;

constructor TVariabledPersistent.Create;
begin
  inherited;
  SetLength(PersistentVariables,GetPropList(Self, PropList));
end;

destructor TVariabledPersistent.Destroy;
var
  I: Integer;
begin
  FreeMem(PropList);
  for I := 0 to nVariables - 1 do
    PersistentVariables[i].Free;
  inherited;
end;

function TVariabledPersistent.GetnVariables: Integer;
begin
  Result := Length(PersistentVariables)
end;

function TVariabledPersistent.GetVariable(Index: Integer): TArray;
begin
  if not Assigned(PersistentVariables[Index]) then
    PersistentVariables[Index] := TPersistentVariable.Create(Self,Index);
  Result := PersistentVariables[Index];
end;

procedure TVariabledPersistent.SetName(const NewName: string);
begin
  FName := NewName;
end;

{ TPersistentVariable }

constructor TPersistentVariable.Create(AOwner: TVariabledPersistent; APropIndex: Integer);
begin
  PropIndex := APropIndex;
  Owner := AOwner;
  inherited Create(Owner.PropList^[PropIndex]^.Name,0);
end;

procedure TPersistentVariable.Store(var a: array of Double);
begin
  with Owner do
    case PropList^[PropIndex]^.PropType^.Kind of
      tkInteger: SetOrdProp(Owner,PropList^[PropIndex],Round(a[0]));
      tkFloat:   SetFloatProp(Owner,PropList^[PropIndex],a[0]);
    end;
end;

function TPersistentVariable.Value(var a: array of Double): Double;
begin
  with Owner do
    case PropList^[PropIndex]^.PropType^.Kind of
      tkInteger: Result := GetOrdProp(Owner,PropList^[PropIndex]);
      tkFloat:   Result := GetFloatProp(Owner,PropList^[PropIndex]);
    end;
end;

end.
