unit SIFLoader;
{$I DFS.inc}{ Copyright (C) 2008 C M Western; see NLFit.pas for license}
{ Import Andor .SIF files }
interface

uses ExpPlotLoader, Classes, TextStrm;

type

  TSIFFileLoader = class(TExperimentalPlotLoader)
  private
    S: TTextStream;
  public
    destructor Destroy; override;
    function Load(var Log: TextFile; AskAboutWeights: Boolean): Boolean; override;
  end;

implementation

uses SysUtils, Types, Math, Units;

{ TSIFFileLoader }

const
  Marker:string = 'Andor Technology Multi-Channel File'#$0A;

destructor TSIFFileLoader.Destroy;
begin
  FreeAndNil(S);
  inherited;
end;

function TSIFFileLoader.Load(var Log: TextFile; AskAboutWeights: Boolean): Boolean;
var
  Token: string[255];
  DetectorType: string;
  WavPoly: array of Double;
  nChannels, nPoints: Integer;
  function NextC: Char;
  begin
    Read(Result,1);
  end;
  function NextToken: Char;
  var
    I: Integer;
  begin
    I := 0;
    repeat
      Inc(I);
      if I > 255 then
        I := 255; { Simply truncate long tokens }
      Result := NextC;
      Token[I] := Result;
    until Result in [' ',#$0A];
    SetLength(Token,I-1);
  end;
  procedure SkipTokens(n: Integer);
  var
    I: Integer;
  begin
    for I := 0 to n - 1 do
      NextToken;
  end;
  procedure SkipLines(n:Integer);
  var
    I: Integer;
  begin
    for I := 0 to n - 1 do
      while NextToken <> #$0A do
       ;
  end;
  function NextInteger: Integer;
  begin
    NextToken;
    Result := StrToInt(Token);
  end;
  procedure NextString;
  begin
    SetLength(Token,NextInteger);
    if Length(Token) > 0 then
      Read(Token[1],Length(Token));
  end;
  function Poly(Bin: Integer):Double;
  var
    I: Integer;
  begin
    Result := WavPoly[High(WavPoly)];
    for I := High(WavPoly) - 1 downto 0 do
      Result := Result*Bin + WavPoly[I];
  end;
  function ReadSection(FirstPass: Boolean): Boolean;
  var
    ImageArea, FrameArea: TRect;
    FirstImage, LastImage, TotalSize, TotalArea, nImages: Integer;
    FrameBins, s: TSize;
    I: Integer;
    TimeStamp: Smallint;
    c: Char;
    J: Integer;
  begin
    SkipLines(1);
    NextToken;
    DetectorType := Token;
    SkipLines(9);
    SetLength(WavPoly,0);
    repeat
      c := NextToken;
      SetLength(WavPoly, Length(WavPoly)+1);
      WavPoly[High(WavPoly)] := StrToFloat(Token);
    until c = #$0A;
    SkipLines(3);
    if (Copy(DetectorType,1,3) = 'Luc') or (Copy(DetectorType,1,2) = 'DH') then
      SkipLines(2);
    NextString; { Frame Axis }
    NextString; { Data Type }
    NextString; { Image Axis }
    NextToken;
    with ImageArea do begin { Note that this info is not used }
      Left := NextInteger;
      Bottom := NextInteger;
      Right := NextInteger;
      Top := NextInteger;
    end;
    LastImage := NextInteger;
    FirstImage := NextInteger;
    TotalSize := NextInteger;
    TotalArea := NextInteger;
    NextToken;
    with FrameArea do begin
      Left := NextInteger;
      Bottom := NextInteger;
      Right := NextInteger;
      Top := NextInteger;
    end;
    with FrameBins do begin
      cy := NextInteger;
      cx := NextInteger;
    end;
    s.cx := (FrameArea.Right-FrameArea.Left+1) div FrameBins.cx;
    s.cy := (FrameArea.Bottom-FrameArea.Top+1) div FrameBins.cy;
    nImages := LastImage - FirstImage + 1;
    if (s.cx * s.cy <> TotalArea) or (TotalArea*nImages <> TotalSize) then
      raise Exception.CreateFmt('Image size inconsistent: %d %d %d %d %d',[s.cx, s.cy, TotalArea, nImages, TotalSize]);
    for I := 0 to nImages - 1 do
      NextString;
    Read(TimeStamp,SizeOf(TimeStamp));
    nPoints := Max(nPoints,s.cx);
    Inc(nChannels,nImages*s.cy);
    if FirstPass then
      Stream.Seek(SizeOf(Single)*TotalSize,soFromCurrent)
    else { TODO: Should read images if s.cy > 1? }
      for I := 0 to nImages*s.cy-1 do begin
        ReadAndCopyArray(I,1,tySingle,s.cx);
        Connector.SetUnits(I,nmAir);
        for J := 0 to s.cx - 1 do
          Connector.SetValue(I,0,J,Poly(J*FrameBins.cx + FrameBins.cx div 2 + 1));
      end;
    NextString;
    Result := NextInteger = 1;
  end;
var
  Buffer: string;
  Position: Int64;
begin
  Result := False;
  if (Ext = '') or (Ext = '.sif') then begin
    Position := Stream.Position;
    SetLength(Buffer, Length(Marker));
    if not ( (Stream.Read(Buffer[1],Length(Marker)) = Length(Marker))
             and (Buffer = Marker) ) then begin
      { Unrecognized }
      Stream.Position := Position;
      Exit;
    end;
  end else
    Exit; { Wrong extension }
  Result := True;
  SkipLines(1);
  nChannels := 0;
  nPoints := 0;
  Position := Stream.Position;
  while ReadSection(True) do
    ;
  Stream.Position := Position;
  SetupChannels(nChannels,nPoints,False);
  while ReadSection(False) do
    ;
end;

initialization
  RegisterPlotLoader(TSIFFileLoader, 'ANDOR SIF Files', '*.sif');
end.
