unit CheckImport;
{ Check an imported file yields the same .pgo file }

{$I DFS.inc}{Copyright (C) 2009 C M Western; see commandline.pas for license}

interface

uses TextStrm, Mixture;

function CheckLImport(Mixture: TMixture; S: TTextStream;
                          var nTried: Integer; Tolerance: Double; var Failures: Integer):Double;

implementation

uses SysUtils, MathSup, Worker, MNode, ImportLMatrix;

function CheckLImport(Mixture: TMixture; S: TTextStream;
                          var nTried: Integer; Tolerance: Double; var Failures: Integer):Double;
  procedure CompareNode(A, B: TMNode);
    procedure Fail(Name: string);
    begin
      Inc(Failures);
      if Failures > 0 then begin
        WriteLn(Log, Name, ' in ', A.FullHeirarchicName);
      end;
    end;
    function CheckStr(Name, sA, sB: string):Boolean;
    begin
      Result := sA = sB;
      if not Result then
        Fail(Format('%s difference: %s %s', [Name, sA, sB]));
    end;
    function CheckInt(Name: string; iA, iB: Integer):Boolean;
    begin
      Result := iA = iB;
      if not Result then
        Fail(Format('%s difference: %d %d', [Name, iA, iB]));
    end;
    procedure CheckNo(Name: string; n1, n2, tol: Double);
    var
      Msg: string;
      Bad: Boolean;
    begin
      Bad := True; { Assume fail }
      if isNanOrInf(n1) xor IsNanOrInf(n2) then
        Msg := Format('%s fails: %g %g', [Name, n1, n2])
      else if Abs(n1-n2) > tol then
        Msg := Format('%s out of tolerance: %g %g %g (%g)', [Name, n1, n2, n1-n2, n1/n2])
      else
        Bad := False; { OK }
      if Abs(n1-n2) > Result then
        Result := Abs(n1-n2);
      if Bad then
        Fail(Msg);
    end;
  var
    i: Integer;
  begin
    if not CheckStr('Class', A.ClassName, B.ClassName) then
      Exit;
    CheckStr('Name', A.Name, B.Name);
    CheckStr('Comment', A.Comment, B.Comment);
    if CheckInt('nSettings', A.nSettings, B.nSettings) then
      for i := 1 to A.nSettings do begin
        CheckInt(A.SettingNames[i], A.Settings[i], B.Settings[i]);
        CheckStr('Setting name', A.SettingNames[i], B.SettingNames[i]);
      end;
    if CheckInt('nParameters', A.nParameters, B.nParameters) then
      for i := 1 to A.nParameters do begin
        CheckNo(A.ParameterNames[i], A.Parameters[i], B.Parameters[i], Tolerance);
        CheckStr('Parameter name', A.ParameterNames[i], B.ParameterNames[i]);
      end;
    CheckInt('Component Count', A.ComponentCount, B.ComponentCount);
    for i := 0 to A.ComponentCount - 1 do
      CompareNode(A.Components[i] as TMNode, B.Components[i] as TMNode);
  end;
var
  IMix: TMixture;
begin
  IMix := ImportLmatrixFile(S);
  IMix.CheckProperties;
  Mixture.CheckProperties;
  { Copy some values across, as these will nevr be identical }
  IMix.Name := Mixture.Name;
  CompareNode(Mixture, IMix);
  IMix.Free;
end;

end.
