unit DushinskyFrm;
{$I DFS.inc}{Copyright (C) 2008 C M Western; see commandline.pas for license}
interface

uses
  SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, WhatsThis, Vib, MNode, Grids, NGrid, StdCtrls, ExtCtrls, AlEdit,
  EditExtr, NewSpin, ScrlCtrl, Graphs, Split, GraphI, GraphM;

type
  TDushinskyForm = class(THelpForm)
    Grid: TNumberGrid;
    CalculateCheckBox: TCheckBox;
    Split: TSplit;
    TopPanel: TPanel;
    G: TGraphD;
    InnerPanel: TPanel;
    MagnificationLabel: TLabel;
    PlotPlaneGroup: TRadioGroup;
    MagnificationSpin: TFixedSpin;
    ShowTRCheckBox: TCheckBox;
    procedure FormClose(Sender: TObject; var AnAction: TCloseAction);
    procedure GridChanged(Sender: TObject; Col, Row: Integer);
    procedure CalculateCheckBoxClick(Sender: TObject);
    procedure PlotPlaneGroupClick(Sender: TObject);
    procedure GridSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure GridMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    TransitionMoment: TMultidimensionalFCF;
    procedure MNodeChange(Sender: TMnode; Change: TMNodeChange; Index: Integer);
    procedure PopulateForm(ACol: Integer = -1);
  public
    destructor Destroy; override;
  end;

function ConnectDushinskyForm(ATransitionMoment: TMultidimensionalFCF): TComponent;

implementation

uses MathSup, Global, Mixture, LmatrixFrm;

{$IFNDEF FPC}
{$R *.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

function ConnectDushinskyForm(ATransitionMoment: TMultidimensionalFCF): TComponent;
var
  Form: TDushinskyForm;
  i: Integer;
begin
  for i := 0 to Screen.CustomFormCount - 1 do
    if Screen.CustomForms[i] is TDushinskyForm then
      if TDushinskyForm(Screen.CustomForms[i]).TransitionMoment = ATransitionMoment then begin
        ReShow(Screen.CustomForms[i]);
        Result := Screen.CustomForms[i];
        Exit;
      end;
  Form := TDushinskyForm.Create(Application);
  Result := Form;
  Form.TransitionMoment := ATransitionMoment;
  Form.PopulateForm;
  Form.Show;
  AddMNodeNotification({$IFDEF FPC}@{$ENDIF}Form.MNodeChange);
end;

{ TDushinskyForm }

procedure TDushinskyForm.CalculateCheckBoxClick(Sender: TObject);
begin
  with TransitionMoment do begin
    Calculate := CalculateCheckBox.Checked;
    if Calculate then
      PopulateForm;
  end;
end;

destructor TDushinskyForm.Destroy;
begin
  RemoveMNodeNotification({$IFDEF FPC}@{$ENDIF}MNodeChange);
  inherited;
end;

procedure TDushinskyForm.FormClose(Sender: TObject; var AnAction: TCloseAction);
begin
  AnAction := caFree;
end;

procedure TDushinskyForm.GridChanged(Sender: TObject; Col, Row: Integer);
begin
  if (Row > 0) and not TransitionMoment.Calculate then begin
    if Col = 1 then
      TransitionMoment.Mode(Row-1).Displacement := Grid[Row,Col]
//    else if Col > 1 then
//      TransitionMoment.S[Row,Col-1] := Grid[Row,Col];
  end;
end;

procedure TDushinskyForm.GridMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  PopulateForm;
end;

procedure TDushinskyForm.GridSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
begin
  PopulateForm(ACol);
end;

procedure TDushinskyForm.MNodeChange(Sender: TMnode; Change: TMNodeChange; Index: Integer);
begin
  if Sender = TransitionMoment then
    case Change of
      mnDestroy: Free;
      mnParameterChange, mnSettingChange, mnPropNameChange, mnNameChange:
        PopulateForm;
    end;
end;

procedure TDushinskyForm.PlotPlaneGroupClick(Sender: TObject);
begin
  PopulateForm;
end;

procedure TDushinskyForm.PopulateForm(ACol: Integer);
var
  Bra, Ket: TElectronicState;
  m, i, nModes, nRealModes: Integer;
  xMin, xMax, yMin, yMax: Double;
  LimitsSet: Boolean;
  Points, L: TDPointArray;
  s: string;
  d: Double;
begin
  if csLoading in ComponentState then
    Exit;
  if Acol < 0 then
    ACol := Grid.Col;
  TransitionMoment.PrepareMatrices(False,False,ShowTRCheckBox.Checked);
  Caption := 'Dushinsky transformation for ' + TransitionMoment.UniqueName;
  Bra := TransitionMoment.Bra as TElectronicState;
  Ket := TransitionMoment.Ket as TElectronicState;
  nModes := TransitionMoment.S.RowCount;
  nRealModes := Bra.Molecule.nModes;
  Grid.RowCount := nModes+1;
  Grid.ColCount := nModes+2;
  for m := 0 to nModes - 1 do begin
    if m < nRealModes then begin
      s := Bra.Mode(m).Name;
      d := TransitionMoment.d[m+1];
    end else begin
      d := 0;
      s := 'R'+Char(Ord('x')+m-nRealModes);
      if m >= nModes-3 then
        s := 'T'+Char(Ord('x')+m-nRealModes-3);
    end;
    Grid.Lbl[m+1] := s;
    Grid.Title[m+2] := s;
    Grid.Value[m+1,1] := d;
    for i := 0 to nModes-1 do begin
      Grid.Value[m+1,i+2] := TransitionMoment.S[m+1,i+1];
    end;
  end;
  if TransitionMoment.Calculate then
    Grid.Options := Grid.Options - [goEditing]
  else
    Grid.Options := Grid.Options + [goEditing];
  CalculateCheckBox.Checked := TransitionMoment.Calculate;
  if Bra.Molecule.nNuclei = 0 then begin
    Split.OneWindowVisible(Split.TopLeftWindow);
    PlotPlaneGroup.Enabled := False;
    MagnificationSpin.Enabled := False;
  end else begin
    PlotPlaneGroup.Enabled := True;
    MagnificationSpin.Enabled := True;
    Split.MakeBothVisible;
    G.Start;
    LimitsSet := False;
    for i := 0 to 1 do begin
      PlotAtoms(G, Bra, PlotPlaneGroup.ItemIndex, ACol-2, i=0, i=1, False, False, False, ShowTRCheckBox.Checked, MagnificationSpin.Value, 3,
                Points, L, xMin, xMax, yMin, yMax, LimitsSet);
      PlotAtoms(G, Ket, PlotPlaneGroup.ItemIndex, ACol-2, i=0, False, True, True, False, ShowTRCheckBox.Checked, MagnificationSpin.Value, 4,
                Points, L, xMin, xMax, yMin, yMax, LimitsSet);
    end;
    G.Finish;
  end;
end;

end.
