unit ExpBitmap;
{$I DFS.inc}{Copyright (C) 2005 C M Western; see commandline.pas for license}
interface

uses {$IFDEF MSWINDOWS} Windows,  {$ENDIF}
     {$IFNDEF CONSOLE} Graphs, GraphRange, {$ENDIF}
     Units, Classes, Contnrs, MNode, Mixture, Graphics, Overlay, pgDOM, GraphM;

type

  { TBitmapPlot }

  TBitmapPlot = class(TPlot)
  private
    FPicture: TPicture;
    function GetPicture: TPicture;
  protected
    function GetHMin: Double; override;
    function GetHMax: Double; override;
    function GetIMin: Double; override;
    function GetIMax: Double; override;
    procedure PictureChanged(Sender: TObject);
  public
    LastGraphic: TGraphic;
    LastSettings: TXMLDocument;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ClearSelection; override;
    procedure Compact;
    function Empty: Boolean; override;
    function FindPeak(AFmin, AFmax: Double; out Position: Double; PlotUnits: TResonanceUnits): Boolean; override;
    function LimitsInRange(Simulation: TSimulation; var AImin, AImax: Double; PlotIntensityUnits: TIntensityUnits):Boolean; override;
    procedure Invert; override;
    procedure LoadFromFile(FileName: string; DefaultUnits: TResonanceUnits; AlwaysPrompt: Boolean; DefaultIntensityUnits: TIntensityUnits); override;
    function nDF: Integer;
    procedure ScaleParameters(UFrom, UTo: TResonanceUnits); override;
    procedure SelectFrequencyRange(AFmin, AFmax: Double); override;
    {$IFNDEF CONSOLE}
    function Show(G: TGraphM; Simulation: TSimulation; PlotIntensityUnits: TIntensityUnits; PlotRange: TPlotRange; var PlotRanges: TPlotRanges; OwnAxis: Boolean; ShowName: Boolean; ForceAxis: Boolean = False):Boolean; override;
    procedure CopyToClipboard(WithLinkedMNodes: Boolean = False; MNodeList: TObjectList = nil); override;
    {$ENDIF}
    function Resolution: Double; override;
    function ReadProperties(Node: TDOMNode; CheckUnread: Boolean = True):Integer; override;
    function WriteProperties(Parent: TDOMNode; AllParameters: Boolean = True; WithChildren: Boolean = True): TDOMElement; override;
    function HelpOnSetting(AIndex: Integer): string; override;
    function HelpOnParameter(AIndex: Integer): string; override;
    procedure SaveForUndo; override;
    procedure Undo; override;
    function CanUndo: Boolean; override;
    procedure Crop(Rect: TRect);
    procedure ScanPlateSpectrum(DeltaX: Integer); overload;
    procedure ScanPlateSpectrum(Angle: Double; Bounds: array of TPoint); overload;
  published
    property Picture: TPicture read GetPicture;
    property Frequency1: Double index 2 read GetParameter write SetParameter;
    property Frequency2: Double index 3 read GetParameter write SetParameter;
    property BaselineIntensity: Double index 4 read GetParameter write SetParameter;
    property MaximumIntensity:  Double index 5 read GetParameter write SetParameter;
    property PlateAngle:        Double index 6 read GetParameter write SetParameter;

    property Xpixel1: Integer index 6 read GetSetting write SetSetting;
    property Xpixel2: Integer index 7 read GetSetting write SetSetting;
    property BaselinePixel: Integer index 8 read GetSetting write SetSetting;
    property MaximumPixel:  Integer index 9 read GetSetting write SetSetting;
    property TopLeftX:      Integer index 10 read GetSetting write SetSetting;
    property TopLeftY:      Integer index 11 read GetSetting write SetSetting;
    property TopRightX:     Integer index 12 read GetSetting write SetSetting;
    property TopRightY:     Integer index 13 read GetSetting write SetSetting;
    property BottomLeftX:   Integer index 14 read GetSetting write SetSetting;
    property BottomLeftY:   Integer index 15 read GetSetting write SetSetting;
    property BottomRightX:  Integer index 16 read GetSetting write SetSetting;
    property BottomRightY:  Integer index 17 read GetSetting write SetSetting;

  end;

implementation

{$IFDEF VCL_}
{$DEFINE GRAPHICEX}
{$ENDIF}

uses {$IFNDEF CONSOLE} { Main, }{$ENDIF}
     {$IFDEF VCL_}
       {$IFDEF GRAPHICEX}
         GraphicEx,
       {$ENDIF}
       GIFImage, PNGImage, JPeg,
     {$ENDIF}
     {$IFDEF LCL_} {$IFNDEF MSWINDOWS} CompatMsg, {$ENDIF}
                   LCLType, FPImage, IntfGraphics, FPReadPNG, FPWritePNG, FPReadPNM, FPReadGif, LCLIntf, {$ENDIF}
     {$IFDEF LCL_} FPImgCmn, PNGComn, ZStream, GraphType, {$ENDIF}
     Math, MathSup, Huge, SysUtils, Clipbrd, Progress, Plate, ExpPlot;

{ TBitmapPlot }

constructor TBitmapPlot.Create(AOwner: TComponent);
begin
  SavedProperties := SavedProperties + ',Frequency2=1000,MaximumIntensity=1,PlateAngle=90';
  inherited;
end;

destructor TBitmapPlot.Destroy;
begin
  if Assigned(FPicture) then
    PictureChanged(Self);
  FreeAndNil(FPicture);
  FreeAndNil(LastGraphic);
  FreeAndNil(LastSettings);
  inherited;
end;

function TBitmapPlot.GetHMin: Double;
begin
  if not Empty then
    Result := Frequency1 - Xpixel1 * (Frequency2-Frequency1)/(Xpixel2-Xpixel1)
  else
    Result := Frequency1;
  Result := AdjustFrequency(Result);
end;

function TBitmapPlot.GetHMax: Double;
begin
  if not Empty then
    Result := Frequency2 + (Picture.Width-Xpixel2)*(Frequency2-Frequency1)/(Xpixel2-Xpixel1)
  else
    Result := Frequency2;
  Result := AdjustFrequency(Result);
end;

function TBitmapPlot.GetIMin: Double;
begin
  if Assigned(Picture) and (BaselinePixel <> MaximumPixel) then
    Result := BaselineIntensity - BaselinePixel * (MaximumIntensity-BaselineIntensity)/(MaximumPixel-BaselinePixel)
  else
    Result := BaselineIntensity;
end;

function TBitmapPlot.GetIMax: Double;
begin
  if Assigned(Picture) and (BaselinePixel <> MaximumPixel) then
    Result := MaximumIntensity + (Picture.Height- MaximumPixel) * (MaximumIntensity-BaselineIntensity)/(MaximumPixel-BaselinePixel)
  else
    Result := MaximumIntensity;
end;

function TBitmapPlot.GetPicture: TPicture;
begin
  if not Assigned(FPicture) then begin
    FPicture := TPicture.Create;
    FPicture.OnChange := {$IFDEF FPC}@{$ENDIF}PictureChanged;
  end;
  Result := FPicture;
end;

{$IFNDEF CONSOLE}
procedure TBitmapPlot.CopyToClipboard(WithLinkedMNodes: Boolean = False; MNodeList: TObjectList = nil);
var
  AFormat: Word;
  {$IFDEF MSWINDOWS}
  AData: Cardinal;
  APalette: HPALETTE;
  {$ENDIF}
begin
  Clipboard.Open;
  try
    if not Assigned(MNodeList) then begin
      Picture.SaveToClipboardFormat(AFormat{$IFDEF VCL_} , AData, APalette {$ENDIF});
      {$IFDEF VCL_}
      if APalette <> 0 then
        Clipboard.SetAsHandle(CF_PALETTE, APalette);
      Clipboard.SetAsHandle(AFormat, AData);
      {$ENDIF}
    end;
    inherited CopyToClipboard(WithLinkedMNodes, MNodeList);
  finally
    Clipboard.Close;
  end;
end;

function TBitmapPlot.Resolution: Double;
begin
  Result := 0;
end;

function TBitmapPlot.Show(G: TGraphM; Simulation: TSimulation; PlotIntensityUnits: TIntensityUnits;  PlotRange: TPlotRange; var PlotRanges: TPlotRanges;  OwnAxis: Boolean; ShowName: Boolean; ForceAxis: Boolean):Boolean;
var
  x0, x1, y0, y1, xi, xx, yi, yx: Double;
  OwnRange: Boolean;
begin
  Result := False;
  if Empty then Exit;
  OwnRange := OwnAxis or IsArbitraryStyle(PlotIntensityUnits)
              or IsArbitraryStyle(IntensityUnits);
  G.MapArg(x0, x1, y0, y1);
  Result := LimitsInRange(Simulation, yi, yx, PlotIntensityUnits);
  if OwnRange then begin
    PlotRange.yi := yi;
    PlotRange.yx := yx;
  end;
  MapFromPlotRange(G, PlotRange);
  xi := Convert(HMin,Units,Simulation.PlotUnits);
  xx := Convert(HMax,Units,Simulation.PlotUnits);
  if Result then
    G.DrawGraphic(xi, xx, Imin, Imax, Picture.Graphic);
  G.Map(x0, x1, y0, y1);
  // Was: AddPlotRange(PlotRanges, GraphRange.PlotRange(G, 0, 1, Imin, Imax));
  AddPlotRange(PlotRanges, PlotRange);
  Result := inherited Show(G, Simulation, PlotIntensityUnits, PlotRange, PlotRanges, OwnAxis, ShowName)
            or Result;
end;
{$ENDIF}

procedure TBitmapPlot.PictureChanged(Sender: TObject);
begin
  PerformMNodeNotification(Self, mnGraphicChanged, -1);
end;

function TBitmapPlot.LimitsInRange(Simulation: TSimulation; 
  var AImin, AImax: Double;
  PlotIntensityUnits: TIntensityUnits): Boolean;
var
  xi, xx: Double;
  AFmin, AFmax: Double;
begin
  AImin := ConvertIntensity(BaselineIntensity,(HMin+HMax)/2,Units,IntensityUnits,Simulation.PlotUnits,PlotIntensityUnits,True);
  AImax := ConvertIntensity(MaximumIntensity,(HMin+HMax)/2,Units,IntensityUnits,Simulation.PlotUnits,PlotIntensityUnits,True);
  xi := Convert(HMin,Units,Simulation.PlotUnits);
  xx := Convert(HMax,Units,Simulation.PlotUnits);
  AFmin := Simulation.Fmin-Simulation.Foffset;
  AFmax := Simulation.Fmax-Simulation.Foffset;
  InOrder(AFmin, AFmax);
  InOrder(xi, xx);
  Result := (xi <= AFmax) and (AFmin <= xx); 
end;

procedure TBitmapPlot.LoadFromFile(FileName: string; DefaultUnits: TResonanceUnits; AlwaysPrompt: Boolean; DefaultIntensityUnits: TIntensityUnits);
var
  NewPicture: TPicture;
  FormatUsed: Word;
  ActiveClipboard: TClipboard;
  ImageSize: Double;
  {$IFDEF MSWINDOWS}
  APalette: HPALETTE;
  AHandle: THandle;
  {$ENDIF}
  {$IFDEF SHOWCLIP}
  procedure ShowIt(C: TClipboard);
  var
    i: Integer;
  begin
    WriteLn(Log, C.FormatCount, ':');
    for i := 0 to C.FormatCount-1 do
      WriteLn(Log, C.Formats[i]);
    WriteLn(Log);
  end;
  {$ENDIF}
  function TryClipboard(Clip: TClipboard):Boolean;
  var
    i: Integer;
  begin
    ActiveClipboard := CLip;
    if Clip.HasFormat(CF_BITMAP) then
      FormatUsed := CF_BITMAP { Prefer bitmap }
    else
      for i := 0 to Clip.FormatCount-1 do begin
        // WriteLn(Log, Clip.Formats[i]);
        if TPicture.SupportsClipboardFormat(Clip.Formats[i]) then begin
          FormatUsed := Clip.Formats[i];
          Break;
        end;
      end;
    Result := FormatUsed <> CF_TEXT;
  end;

begin
  Name := ExtractFileName(FileName);
  if Name = '' then begin
    Name := IntToStr(Owner.ComponentCount);
    FDefaultName := True;
  end;
  Comment := FileName;
  NewPicture := TPicture.Create;
  try
    if FileName = '' then begin
      {$IFDEF SHOWCLIP}
        ShowIt(Clipboard(ctPrimarySelection));
        ShowIt(Clipboard(ctSecondarySelection));
        ShowIt(Clipboard(ctClipboard));
      {$ENDIF}
      FormatUsed := CF_TEXT; { Flag no valid formats found }
      if TryClipboard(Clipboard)
         {$IFNDEF MSWINDOWS}
         or TryClipboard(Clipboard(ctPrimarySelection))
         or TryClipboard(Clipboard(ctSecondarySelection))
         {$ENDIF}
        then begin
          { Take from clipboard }
          {$IFDEF VCL_}
            if Clipboard.HasFormat(CF_PALETTE) then
              APalette := Clipboard.GetAsHandle(CF_PALETTE)
            else
              APalette := 0;
            Clipboard.Open;
            try
              AHandle := Clipboard.GetAsHandle(FormatUsed);
              if AHandle = 0 then
                RaiseLastOSError;
              NewPicture.LoadFromClipboardFormat(FormatUsed, AHandle, APalette);
            finally
              Clipboard.Close;
            end;
          {$ELSE}
            NewPicture.LoadFromClipboardFormatID(ActiveClipboard.ClipboardType,FormatUsed);
          {$ENDIF}
      end else
        {$IFDEF XBASED}
        if not LoadBitmapFromClipboard(NewPicture.Bitmap) then
        { TODO -cLCLcarbon : Implement}
        {$ENDIF}
          raise EReportException.Create('No valid picture formats on clipboard');
    end else
      NewPicture.LoadFromFile(FileName);
    with Picture.Bitmap do begin
      {$IFDEF VCL_}
      if NewPicture.Graphic is TMetafile then begin
        { TODO: may need to experiment with Width and height
          - defaults can be too small and thus loose information }
//        with TMetafile(NewPicture.Graphic) do begin
//          WriteLn(Log,Inch);
//          WriteLn(Log,MMHeight);
//          WriteLn(Log,MMWidth);
//        end;
        Width := NewPicture.Width;
        Height := NewPicture.Height;
        Canvas.Draw(0,0,NewPicture.Graphic);
      end else
      {$ENDIF}
        Assign(NewPicture.Graphic);
      { Heuristic - transparent images cause problems if very large }
      ImageSize := (Width*1.0)*Height;
      if PixelFormat <> pf1bit then
        ImageSize := ImageSize*8;
      if not (PixelFormat in [pf1bit, pf4bit, pf8bit]) then
        ImageSize := ImageSize*3;
      if ImageSize < 127e3 then begin
        Transparent := True;
        TransparentColor := clWhite;
      end;
    end;
    Xpixel1 := 0;
    Xpixel2 := Picture.Width;
    Frequency2 := Picture.Width;
    BaselinePixel := 0;
    MaximumPixel := Picture.Height;
    Units := DefaultUnits;
    IntensityUnits := DefaultIntensityUnits;
    if (IntensityUnits <> Arbitrary) and Assigned(Owner) then
      (Owner as TOverlays).IntensityUnits := Normalized;
    Modified := False;
  finally
    NewPicture.Free;
  end;
end;

function TBitmapPlot.nDF: Integer;
begin
  Result := Picture.Width;
end;

function TBitmapPlot.CanUndo: Boolean;
begin
  Result := Assigned(LastGraphic);
end;

procedure TBitmapPlot.ClearSelection;
begin
  { TODO : TBitmapPlot.ClearSelection }
end;

function TBitmapPlot.FindPeak(AFmin, AFmax: Double; out Position: Double; PlotUnits: TResonanceUnits): Boolean;
begin
  Position := (AFmin+AFmax)/2; { Rough and ready estimate }
  Result := True;
end;

procedure TBitmapPlot.Invert;
var
  T: Integer;
begin
  T := BaselinePixel;
  BaselinePixel := MaximumPixel;
  MaximumPixel := T;
end;

procedure TBitmapPlot.SelectFrequencyRange(AFmin, AFmax: Double);
begin
  { TODO: TBitmapPlot.SelectFrequencyRange }
end;

type

  TEncodedStream = class(TMemoryStream)
    nChars: Integer;
    function Write(const Buffer; Count: Longint): Longint; override;
    procedure WritePlain(s: string);
  end;

procedure TEncodedStream.WritePlain(s: string);
begin
  inherited Write(s[1], Length(s));
end;

function TEncodedStream.Write(const Buffer; Count: Integer): Longint;
var
  p: PChar;
  by: string[2];
begin
  p := @Buffer;
  Result := 0;
  while Count > 0 do begin
    by := IntToHex(Ord(p^), 2);
    inherited Write(by[1], 2);
    Inc(p);
    Dec(Count);
    Inc(Result);
    Inc(nChars, 2);
    if (nChars > 80) then begin
      by := sLineBreak;
      inherited Write(by[1], Length(sLineBreak));
      nChars := 0;
    end;
  end;
end;

function TBitmapPlot.ReadProperties(Node: TDOMNode; CheckUnread: Boolean):Integer;
var
  Child: TDOMNode;
  Stream: TStringStream;
  GraphicClassName: string;
  GraphicClass: TPersistentClass;
  Graphic: TGraphic;
  s, d: PChar;
  by: string[3];
  {$IFDEF FPC}
  Reader: TFPCustomImageReader;
  TBit: TBitmap;
  IntfImg: TLazIntfImage;
  Bitmap, MaskBitmap: HBitmap;
  MStream: TMemoryStream;
  RawImage: TRawImage;
  {$ENDIF}
  {$IFDEF GRAPHICEX}
  i: Integer;
  ExtList: TStringList;
  {$ENDIF}
begin
  Result := inherited ReadProperties(Node, False);
  Child := Node.FirstChild;
  while Assigned(Child) do begin
    if Child is TDOMCDATASection then begin
      Stream := TStringStream.Create(TDOMCDATASection(Child).Data);
      {$IFDEF FPC}
      Reader := nil;
      {$ENDIF}
      Graphic := nil;
      try
        d := @Stream.DataString[1];
        s := d;
        while not (s^ in [#10, #13]) do
          Inc(s);
        GraphicClassName := Copy(Stream.DataString,1,s-d);
        SetLength(by,3);
        by[1] := '$';
        while s^ <> #0 do begin
          while s^ in [#10,#13] do Inc(s);
          if s^ = #0 then Break;
          by[2] := s^;
          Inc(s);
          by[3] := s^;
          Inc(s);
          d^ := Char(StrToInt(by));
          Inc(d);
        end;
        Stream.Size := d - @Stream.DataString[1];
        GraphicClass := GetClass(GraphicClassName);
        if GraphicClass = nil then
          if GraphicClassName = 'TBitmap' then
            GraphicClass := TBitmap;
          if GraphicClassName = 'TJPEGImage' then
            GraphicClass := TJPEGImage;
          {$IFDEF VCL_}
            {$IFDEF GRAPHICEX}
              ExtList := TStringList.Create;
              try
                FileFormatList.GetExtensionList(ExtList);
                for i := 0 to ExtList.Count-1 do
                  if FileFormatList.GraphicFromExtension(ExtList[i]).ClassName = GraphicClassName then begin
                    GraphicClass := FileFormatList.GraphicFromExtension(ExtList[i]);
                    Break;
                  end;
              finally
                ExtList.Free;
              end;
            {$ENDIF}
            if GraphicClassName = 'TGIFImage' then
              GraphicClass := TGIFImage;
            if GraphicClassName = 'TPNGObject'then
                GraphicClass := TPNGGraphic;
          {$ELSE}
            if GraphicClassName = 'TPNGObject' then begin
              GraphicClass := TBitmap;
              Reader := TFPReaderPNG.Create;
            end;
          {$ENDIF}
        if (GraphicClass = nil) or not GraphicClass.InheritsFrom(TGraphic) then
          raise Exception.Create('Unknown Graphic type:'+GraphicClassName);
        Graphic := TGraphicClass(GraphicClass).Create;
        {$IFDEF FPC}
        if Assigned(Reader) then begin
          IntfImg:=TLazIntfImage.Create(0,0);
          try
            {$IFDEF OLDIMAGE}
            { Revision r11861 of Lazarus changed IntfImg; this is the old version }
            IntfImg.GetDescriptionFromDevice(0);
            IntfImg.LoadFromStream(Stream,Reader);
            IntfImg.CreateBitmap(Bitmap, MaskBitmap, False);
            IntfImg.GetRawImage(RawImage);
            { Loose transparency information, which seems to cause problems }
            { TODO: Fix LCL so that transparent bitmaps can be stretched }
            RawImage.Mask := nil;
            if not CreateBitmapFromRawImage(RawImage,Bitmap,MaskBitmap,False) then
              raise Exception.Create('Failed to create bitmaps');
            {$ELSE}
            IntfImg.DataDescription := GetDescriptionFromDevice(0);
            IntfImg.LoadFromStream(Stream,Reader);
            IntfImg.CreateBitmaps(Bitmap, MaskBitmap, False);
            {$ENDIF}
            with Graphic as TBitmap do begin
              Handle := Bitmap;
              MaskHandle := MaskBitmap;
              //MStream := TMemoryStream.Create;
              //try
                //SaveToStream(MStream);
                //MStream.Position := 0;
                //LoadFromStream(MStream);
              //finally
                //MStream.Free;
              //end;
            end;
          finally
            IntfImg.Free;
          end;
        end else
        {$ENDIF}
          Graphic.LoadFromStream(Stream);
        Picture.Bitmap.Assign(Graphic);
        {$IFDEF FPC}
        // TOpenBitmap(Picture.Bitmap).FreeSaveStream;
        // Picture.Bitmap.SaveToFile('/tmp/wibble.bmp');
        // Picture.Bitmap.LoadFromFile('/tmp/wibble.bmp');
        // Tbit := TBitmap.Create;
        // Tbit.Height := Picture.Bitmap.Height;
        // Tbit.Width := Picture.Bitmap.Width;
        // Tbit.PixelFormat := pf24bit;
        // TBit.Canvas.Brush.Color := clblack;
        // TBit.Canvas.FillRect(0,0,Tbit.Width,TBit.Height);
        // StretchBlt(TBit.Canvas.Handle,0,0,Tbit.Width,TBit.Height,Picture.Bitmap.Canvas.Handle,0,0,Tbit.Width,TBit.Height,SRCAND);
        // TBit.Canvas.Draw(0,0,Picture.Graphic);
        // TBit.SaveToFile('/tmp/wibble1.bmp');
        {$ELSE}
        Picture.Bitmap.Transparent := True;
        Picture.Bitmap.TransparentColor := clWhite;
        {$ENDIF}
        ReadPeaks(Child.NextSibling);
        Node.RemoveChild(Child);
        Break;
      finally
        Stream.Free;
        Graphic.Free;
      end;
    end;
    Child := Child.NextSibling;
  end;
  if CheckUnread then
    Result := Result + nUnreadProperties(Node);
end;

{$DEFINE USEPNG}
function TBitmapPlot.WriteProperties(Parent: TDOMNode; AllParameters: Boolean; WithChildren: Boolean): TDOMElement;
var
  CDATASection: TDOMCDATASection;
  Stream: TEncodedStream;
  s: string;
  {$IFDEF USEPNG}
  {$IFNDEF FPC}
  Png: TPNGObject;
  {$ELSE}
  IntfImg: TLazIntfImage;
  Writer: TFPWriterPNG;
  MS: TMemoryStream;
  {$ENDIF}
  {$ENDIF}
begin
  Result := inherited WriteProperties(Parent, AllParameters, WithChildren);
  if not Assigned(Picture.Graphic) then
    Exit;
  Stream := TEncodedStream.Create;
  try
    {$IFNDEF USEPNG}
    Stream.WritePlain(Picture.Graphic.ClassType.ClassName+sLineBreak);
    Picture.Graphic.SaveToStream(Stream);
    {$ELSE}
    {$IFDEF FPC}
    Stream.WritePlain('TPNGObject'+sLineBreak);
    IntfImg:=TLazIntfImage.Create(0,0);
    try
      Writer := TFPWriterPNG.Create;
      Writer.Indexed := False;
      Writer.WordSized := False;
      IntfImg.LoadFromBitmap((Picture.Graphic as TBitmap).Handle,(Picture.Graphic as TBitmap).MaskHandle);
      { Only require copy because TFPCustomImageWriter.ImageWrite clears stream }
      MS := TMemoryStream.Create;
      try
        IntfImg.SaveToStream(MS,Writer);
        MS.Position := 0;
        Stream.CopyFrom(MS, MS.Size);
      finally
        MS.Free;
      end;
    finally
      Writer.Free;
      IntfImg.Free;
    end;
    {$ELSE}
    Png := TPNGObject.Create;
    try
      Png.Assign(Picture.Graphic);
      Stream.WritePlain(Png.ClassType.ClassName+sLineBreak);
      Png.SaveToStream(Stream);
    finally
      Png.Free;
    end;
    {$ENDIF}
    {$ENDIF}
    Stream.WritePlain(sLineBreak);
    SetString(s, PChar(Stream.Memory), Stream.Position);
    CDATASection := Parent.OwnerDocument.CreateCDATASection(s);
    Result.AppendChild(CDATASection);
    WritePeaks(Result);
  finally
    Stream.Free;
  end;
end;

const
  BitmapPlotSettingNames: array[6..9] of string = (
'Horizontal Pixel at Frequency1',
'Horizontal Pixel at Frequency2',
'Vertical Pixel corresponding to BaselineIntensity',
'Vertical Pixel corresponding to MaximumIntensity'
);

function TBitmapPlot.HelpOnSetting(AIndex: Integer): string;
begin
  if AIndex > High(BitmapPlotSettingNames) then
    Result := 'Bound of active region of plate'
  else if AIndex >= Low(BitmapPlotSettingNames) then
    Result := BitmapPlotSettingNames[AIndex]
  else
    Result := inherited HelpOnSetting(AIndex);
end;

const
 BitmapPlotParameterNames: array [2..6] of string = (
'Frequency at Xpixel1.',
'Frequency at Xpixel2.',
'Intensity at BaselinePixel - normally 0.',
'Intensity at MaximumPixel - normally 1.',
'Angle of lines on plate with respect to vertical'
);

function TBitmapPlot.HelpOnParameter(AIndex: Integer): string;
begin
  if (AIndex >= Low(BitmapPlotParameterNames))
     and (AIndex <= High(BitmapPlotParameterNames)) then
    Result := BitmapPlotParameterNames[AIndex]
  else
    Result := inherited HelpOnParameter(AIndex);
end;

procedure TBitmapPlot.ScaleParameters(UFrom, UTo: TResonanceUnits);
begin
  FrequencyOffset := ConvertWidth(FrequencyOffset, (Frequency1+Frequency2)/2, UFrom, UTo);
  Frequency1 := Convert(Frequency1, UFrom, UTo);
  Frequency2 := Convert(Frequency2, UFrom, UTo);
end;

function TBitmapPlot.Empty: Boolean;
begin
  Result := not Assigned(Picture) or (Xpixel1 = Xpixel2);
end;

procedure TBitmapPlot.Compact;
var
  S, D: TBitmap;
  x, y, v: Integer;
  c: TColor;
begin
  if not (Picture.Graphic is TBitmap) then begin
    S := TBitmap.Create;
    S.Assign(Picture.Graphic);
    Picture.Graphic := S;
  end else
    S := Picture.Bitmap;
  D := TBitmap.Create;
  PushProgress('Compacting',Picture.Width-1);
  try
    D.Monochrome := True;
    D.Width := Picture.Width;
    D.Height := Picture.Height;
    for x := 0 to D.Width-1 do begin
      for y := 0 to D.Height-1 do begin
        c := S.Canvas.Pixels[x,y];
        v := (c and 255) + ((c shr 8) and 255) + ((c shr 16) and 255);
        if v >= 256+128 then
          D.Canvas.Pixels[x,y] := clWhite
        else
          D.Canvas.Pixels[x,y] := clBlack;
      end;
      SetProgress(x);
    end;
    PopProgress;
    D.Transparent := True;
    D.TransparentColor := clWhite;
    SaveForUndo;
    Picture.Graphic := D;
  except
    D.Free;
    PopProgress;
  end;
end;

procedure TBitmapPlot.SaveForUndo;
begin
  FreeAndNil(LastGraphic);
  if Assigned(Picture.Graphic) then begin
    LastGraphic := TGraphicClass(Picture.Graphic.ClassType).Create;
    LastGraphic.Assign(Picture.Graphic);
    Picture.Graphic := nil;
    LastSettings := TXMLDocument.Create;
    WriteProperties(LastSettings);
  end;
end;

procedure TBitmapPlot.Undo;
begin
  if Assigned(LastGraphic) then
   try
    if Assigned(LastSettings) then
      ReadProperties(LastSettings.FirstChild);
    Picture.Graphic := LastGraphic;
    CheckProperties;
   finally
    FreeAndNil(LastGraphic);
    FreeAndNil(LastSettings);
  end;
end;

procedure TBitmapPlot.Crop(Rect: TRect);
var
  D: TBitmap;
  x, dx, y, dy: Integer;
  procedure DoShift;
  begin
    { In separate procedure as a work around for Delphi bug }
    if (Xpixel1 = 0) and (Xpixel2 = LastGraphic.Width)
       and (BaselinePixel = 0) and (MaximumPixel = LastGraphic.Height)
       and (BaselineIntensity = 0) and (MaximumIntensity = 1)
       and (Frequency1 = 0) and (Frequency2 = Xpixel2) then begin
      { Scaling not changed from default, so keep default scaling }
      Xpixel2 := D.Width;
      Frequency2 := Xpixel2;
      MaximumPixel := D.Height;
    end else begin
      { Preserve Scaling }
      Xpixel1 := Xpixel1 - x;
      Xpixel2 := Xpixel2 - x;
      BaselinePixel := Self.BaselinePixel - Min(Rect.Top,Rect.Bottom);
      MaximumPixel := MaximumPixel - Min(Rect.Top,Rect.Bottom);
    end;
  end;
begin
  x := Max(Min(Rect.Left,Rect.Right),0);
  dx := Min(Picture.Width-x, Abs(Rect.Right - Rect.Left));
  y := Max(Min(Rect.Top,Rect.Bottom),0);
  dy := Min(Picture.Height-y, Abs(Rect.Bottom - Rect.Top));
  if (dx <= 0) or (dy <= 0) then
    Exit;
  SaveForUndo;
  D := TBitmap.Create;
  try
    if LastGraphic is TBitmap then begin
      D.Monochrome := TBitmap(LastGraphic).Monochrome;
      D.PixelFormat := TBitmap(LastGraphic).PixelFormat;
    end;
    if LastGraphic.Transparent then begin
      D.Transparent := True;
      D.TransparentColor := clWhite;
    end;
    D.Width := dx;
    D.Height := dy;
    DoShift;
    D.Canvas.Draw(-x,-y,LastGraphic);
    Picture.Graphic := D;
  finally
    D.Free;
  end;
end;

procedure TBitmapPlot.ScanPlateSpectrum(Angle: Double; Bounds: array of TPoint);
var
  D: TBitmap;
  ExpPlot: TExperimentalPlot;
  i: Integer;
  Threshold: Double;
begin
  ExpPlot := TExperimentalPlot.Create(Owner);
  try
    ExpPlot.Name := 'Digitized from ' + Name;
    ExpPlot.Units := Units;
    D := TBitmap.Create;
    try
      D.Assign(Picture.Graphic);
      if Owner is TOverlays then
        Threshold := TOverlays(Owner).PeakThreshold
      else
        Threshold := 0.5;
      Plate.ScanPlateSpectrum(D,Angle,Bounds,Threshold,ExpPlot.Frequency,ExpPlot.Intensity);
      for i := 0 to High(ExpPlot.Frequency) do
        ExpPlot.Frequency[i] := Frequency1 - (Xpixel1-ExpPlot.Frequency[i]) * (Frequency2-Frequency1)/(Xpixel2-Xpixel1);
    finally
      D.Free;
    end;
  except
    ExpPlot.Free;
    raise;
  end;
end;

procedure TBitmapPlot.ScanPlateSpectrum(DeltaX: Integer);
var
  D: TBitmap;
  R: TRect;
  ExpPlot: TExperimentalPlot;
  i, nDFp, nPixels, Threshold: Integer;
  v: Double;
begin
  ExpPlot := TExperimentalPlot.Create(Owner);
  try
    ExpPlot.Name := 'Digitized from ' + Name;
    ExpPlot.Units := Units;
    D := TBitmap.Create;
    try
      D.Assign(Picture.Graphic);
      if Owner is TOverlays then
        Threshold := Round(TOverlays(Owner).PeakThreshold * D.Height)
      else
        Threshold := D.Height div 2;
      R := Rect(0,0,-DeltaX,D.Height-1);
      if DeltaX < 0 then
        OffsetRect(R,DeltaX,0);
      nDFp := D.Width + Abs(DeltaX);
      SetLength(ExpPlot.Frequency, nDFp);
      SetLength(ExpPlot.Intensity, nDFp);
      i := 0;
      while (R.Left < D.Width) or (R.Right < D.Width) do begin
        v := PixelSum(R,D,nPixels);
        if nPixels > Threshold then begin
          ExpPlot.Frequency[i] := Frequency1 - (Xpixel1-R.Left) * (Frequency2-Frequency1)/(Xpixel2-Xpixel1);
          ExpPlot.Intensity[i] := v/nPixels;
          Inc(i);
        end;
        Inc(R.Left);
        Inc(R.Right);
      end;
      SetLength(ExpPlot.Frequency, i);
      SetLength(ExpPlot.Intensity, i);
    finally
      D.Free;
    end;
  except
    ExpPlot.Free;
    raise;
  end;
end;

initialization
  TBitmapPlot.RegisterClass(TOverlays);
end.
