unit LevelPlotter;
{$I DFS.inc}{Copyright (C) 2005-9 C M Western; see commandline.pas for license}
interface

uses {$IFNDEF CONSOLE}LevelFrm,{$ENDIF}
     Classes, GraphM, MNode, Mixture, Global, pgDOM, Gmarks;

type

  { TLevelPlotter }

  TLevelPlotter = class({$IFNDEF CONSOLE}TLevelForm{$ELSE}TMNode{$ENDIF})
  private
    FirstPoint: Boolean;
    LogHeaderShown: Boolean;
    SymmetryIndex, ASelectIndex, BSelectIndex, IgnoreSpinsIndex,
    XQuantumNumberIndex, LabelsStyleIndex: Integer;
    DoneCreating: Boolean;
    {$IFDEF CONSOLE}
    PlottedPoints: TPlottedPoints;
    nPlottedPoints: Integer;
    FActiveObject: string;
    FSymmetryList: TStringList;
    CurrentMNode: TMNode;
    procedure SetActiveObject(const AValue: string);
    {$ENDIF}
    procedure CheckIndices;
  public
    EMin, EMax: Double;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure PlotManifold(G: TGraphM; Manifold: TManifold; State: TState; ToLog, ShowMolecule, ShowManifold: Boolean);
    procedure PlotItem(G: TGraphM; MNode:TMNode; Log: Boolean);
    function GetSetting(AIndex: Integer): Integer; {$IFNDEF CONSOLE} override; {$ENDIF}
    procedure SetSetting(AIndex: Integer; const AValue: Integer); {$IFNDEF CONSOLE} override; {$ENDIF}
    function GetSettingsNames(AIndex: Integer; out LowestValue: Integer): PPChar; override;
    {$IFDEF CONSOLE}
    function ReadProperties(Node: TDOMNode; CheckUnread: Boolean = True):Integer; override;
    property ActiveObject: string read FActiveObject write SetActiveObject;
    {$ENDIF}
  published
    { Note that if the form is edited, the settings numbers must be updated to reflect the creation order on the form }
//    property Left: Integer index 1 read GetSetting write SetSetting; { 0 }
//    property Top: Integer index 2 read GetSetting write SetSetting; { 0 }
//    property Width: Integer index 3 read GetSetting write SetSetting; { 0 }
//    property Height: Integer index 4 read GetSetting write SetSetting; { 0 }
//    property Visible: Integer index 5 read GetFlag write SetSetting; { False }
//    property ActiveObject: Integer index 6 read GetSetting write SetSetting; {  }
    property CalcBoth: Boolean       index 7 read GetFlag write SetFlag; { True }
    property CalcLine: Boolean       index 8 read GetFlag write SetFlag; { False }
    property CalcPoints: Boolean     index 9 read GetFlag write SetFlag; { False }
    property JoinHighest: Boolean    index 10 read GetFlag write SetFlag; { False }
    property TrackQuanta: Boolean    index 11 read GetFlag write SetFlag; { True }
    property TrackState: Boolean     index 12 read GetFlag write SetFlag; { True }
    property ExternalField: Boolean  index 13 read GetFlag write SetFlag; { False }
    property AltPlot: Boolean        index 14 read GetFlag write SetFlag; { False }
    property ASelect: Integer        index 15 read GetSetting write SetSetting; { all }
    property BSelect: Integer        index 16 read GetSetting write SetSetting; { all }
    property Symmetry: Integer       index 17 read GetSetting write SetSetting; { all }
    property XMax: Integer           index 18 read GetSetting write SetSetting; { 40 }
    property XMin: Integer           index 19 read GetSetting write SetSetting; { 0 }
    property XQuantumNumber: Integer index 20 read GetSetting write SetSetting; { J }
    property JoinObs: Boolean        index 21 read GetFlag write SetFlag; { True }
    property ObsLower: Boolean       index 22 read GetFlag write SetFlag; { False }
    property ObsUpper: Boolean       index 23 read GetFlag write SetFlag; { False }
    property IgnoreSpins: Integer    index 24 read GetSetting write SetSetting; { all }
    property ExcludePerturbations: Boolean index 25 read GetFlag write SetFlag; { False }
    property LabelsStyle: Integer    index 26 read GetSetting write SetSetting; {  }
    property FieldSamples: Integer   index 27 read GetSetting write SetSetting; { 20 }
    property ESweep: Boolean         index 28 read GetFlag write SetFlag; { True }
    property BSweep: Boolean         index 29 read GetFlag write SetFlag; { False }
    property Ymax: Double            index 1 read GetParameter write SetParameter; {  1.00000000000000E+0004 }
    property Ymin: Double            index 2 read GetParameter write SetParameter; {  0.00000000000000E+0000 }
    property Reduction: Double       index 3 read GetParameter write SetParameter; {  0.00000000000000E+0000 }
    property ErrorMultiplier: Double index 4 read GetParameter write SetParameter; {  1.00000000000000E+0000 }
    property MinField: Double        index 5 read GetParameter write SetParameter; {  0.00000000000000E+0000 }
    property MaxField: Double        index 6 read GetParameter write SetParameter; {  1.00000000000000E+0004 }
    property ShiftMult: Double       index 7 read GetParameter write SetParameter; {  1.00000000000000E+0000 }
  end;

  TLabelPos = (lbNone, lbLeftOut, lbLeftIn, lbRightIn, lbRightOut, lbStart, lbEnd);

function ReadLevelPlotter(Mixture: TMixture):TLevelPlotter;

const
  LabelsStyleNames: array [0..7] of PChar = ('None', 'LeftOut', 'LeftIn', 'RightIn', 'RightOut', 'Start', 'End', nil);

implementation

uses SysUtils, Math, MathSup, Graphics, GStyle, QuantumNumber, QuantumMath, MNodeColour,
     Obs, Worker, Progress, SymUtils, StaticField, UCompat;

function ReadLevelPlotter(Mixture: TMixture):TLevelPlotter;
var
  Node: TDOMElement;
begin
  {$IFDEF CONSOLE}
    Result := TLevelPlotter.Create(Mixture);
    { Read TLevelPlotter object saved as a FormSettings object }
    if Assigned(Mixture.LoadedFormSettings) then begin
      Node := Mixture.LoadedFormSettings.FirstChild as TDOMElement;
      while Assigned(Node) do begin
        if ( CompareText(Node.NodeName,'FormSettings') = 0 )
           and ( CompareText(Node.GetAttribute('Name'),'LevelWindow') = 0) then begin
             Result.ReadProperties(Node,False);
             Break;
           end;
        Node := Node.NextSibling as TDOMElement;
      end;
    end;
  {$ELSE}
    Result := LevelForm as TLevelPlotter;
  {$ENDIF}
end;

{ TLevelPlotter }

{$IFDEF CONSOLE}
function TLevelPlotter.ReadProperties(Node: TDOMNode; CheckUnread: Boolean): Integer;
begin
  FActiveObject := ReadValue(Node,'ActiveObject','');
  Result:=inherited ReadProperties(Node, CheckUnread);
end;

procedure TLevelPlotter.SetActiveObject(const AValue: string);
begin
  FreeAndNil(FSymmetryList);
  FActiveObject := AValue;
  CurrentMNode := nil; { TODO: Fix }
  FSymmetryList := MakeSymmetryList(CurrentMNode);
end;
{$ENDIF}

constructor TLevelPlotter.Create(AOwner: TComponent);
begin
  {$IFDEF CONSOLE}
  SavedProperties := SavedProperties
    + ',CalcBoth=True,TrackQuanta=True,TrackState=True,ESweep=True,BSweep=False,'
    + ',ASelect=all,BSelect=all,Symmetry=all,XMax=40,XQuantumNumber=J,JoinObs=True,IgnoreSpins=all'
    + ',FieldSamples=20,Ymax=1e4,ErrorMultiplier=1,MaxField=1e4,ShiftMult=1';
  {$ENDIF}
  inherited Create(AOwner);
  CheckIndices;
  DoneCreating := True;
end;

destructor TLevelPlotter.Destroy;
begin
  {$IFDEF CONSOLE}
  FreeAndNil(FSymmetryList);
  {$ENDIF}
  inherited Destroy;
end;

procedure TLevelPlotter.PlotManifold(G: TGraphM; Manifold: TManifold; State: TState; ToLog, ShowMolecule, ShowManifold: Boolean);
var
  J: TQuantumNumber;
  ASelecting, BSelecting, IncludePerturbations: Boolean;
  MarkStyle: TMarkStyle;
  ShowPoints, ShowLines: Boolean;
  FirstPointOnCurve, i, nErrors: Integer;
  Matches: array of Integer;
  LastColour: TColor;
  TIndex, MaxTIndex: Integer;
  LastName: string;
  Simulation: TSimulation;
  Overflow: Exception;

  function XQno(Ev: TEigenValues; EvNo: Integer):TQuantumNumber;
  begin
    Result := Ev.Qno(EvNo,TQnoType(XQuantumNumber));
  end;

  function AdjustY(y: Double; J: TQuantumNumber): Double;
  begin
    Result := y - Reduction*J*(J+2)/4;
  end;

  function IsSelected(Ev: TEigenValues; EvNo: Integer):Boolean;
  begin
    Result := False;
    if (Symmetry >= 0) and (Symmetry <> Ev.Manifold.Molecule.JDeAdjustedSym(Ev.DominantBasis[EvNo].F,Ev.DominantBasis[EvNo].Sym)) then
      Exit;
    if ASelecting and (Ev.DominantBasis[EvNo].Qno(qtA) <> ASelect) then
      Exit;
    if BSelecting and (Ev.DominantBasis[EvNo].Qno(qtB) <> BSelect) then
      Exit;
    Result := True;
  end;

  procedure PlotPoint(Ev: TEigenValues; EvNo: Integer; AX, E: Double); overload;
  begin
    if Manifold.nStates > 0 then begin
      if LastName = '' then
        LastName := Ev.DominantBasis[EvNo].State.Name;
      LastColour := Ev.DominantBasis[EvNo].State.FindColour;
    end else begin
      if LastName = '' then
        LastName := Manifold.Name;
      LastColour := Manifold.FindColour;
    end;
    if LastColour = clHide then
      Exit
    else if LastColour = clNone then
      LastColour := clBlack;
    if FirstPoint then begin
      Emin := E;
      EMax := E;
      FirstPoint := False;
    end else begin
      if E > EMax then EMax := E;
      if E < EMin then EMin := E;
    end;
    if Length(PlottedPoints) <= nPlottedPoints then
      SetLength(PlottedPoints, nPlottedPoints+10000);
    with PlottedPoints[nPlottedPoints] do begin
      X := AX;
      Y := E;
      Inc(nPlottedPoints);
      Index := -EvNo;
      Extra := PtrInt(Ev);
    end;
  end;

  procedure PlotPoint(Ev: TEigenValues; EvNo: Integer); overload;
  var
    QX: TQuantumNumber;
    E: Double;
  begin
    QX := XQno(Ev,EvNo);
    E := AdjustY(Ev.Energy[EvNo], QX); { TODO: Make this a choice? Last release had Ev.Qno(EvNo,qtJ) }
    if IsSelected(Ev,EvNo) then
      PlotPoint(Ev,EvNo,QX/2,E);
  end;

  procedure PlotPoints(From: Integer); { Plot buffered points }
  var
    i: Integer;
    Ev: TEigenValues;
  begin
    if ShowPoints then
      for i := From to nPlottedPoints-1 do
        with PlottedPoints[i] do begin
          Ev := TEigenValues(Extra);
          if Manifold.nStates > 0 then begin
            LastColour := Ev.DominantBasis[-Index].State.FindColour;
          end else
            LastColour := Manifold.FindColour;
          if LastColour = clNone then
            LastColour := clBlack;
          MarkStyle.Colour := LastColour;
          if AltPlot then
            MarkStyle.MarkType := MarkTypeFor(Ev.Sym)
          else
            MarkStyle.MarkType := MarkTypeFor(Ev.Manifold.Molecule.JDeAdjustedSym(Ev.DominantBasis[-Index].F,Ev.DominantBasis[-Index].Sym));
          G.SymbolT(X, Y, MarkStyle);
        end;
  end;

  procedure PlotCurve(Lbl: string; LabelPos:TLabelPos);
  var
    LabelFound: Boolean;
    LX, LY: Double;
    i: Integer;
  begin
    if ShowLines and (nPlottedPoints-FirstPointOnCurve > 1) then begin
      { Record first visible point for LabelStyle = Start }
      LabelFound := not (LabelPos in [lbStart,lbEnd]);
      G.StyleP(LastColour, G.LineStyle.Width, G.LineStyle.Pattern);
      for i := FirstPointOnCurve to nPlottedPoints-1 do begin
        if i = FirstPointOnCurve then
          G.ClipPoint(PlottedPoints[i].X, PlottedPoints[i].Y)
        else
          G.ClipJoin(PlottedPoints[i].X, PlottedPoints[i].Y);
        if not LabelFound or (LabelPos = lbEnd) and (G.OutCode(PlottedPoints[i].X, PlottedPoints[i].Y) = 0) then begin
          LabelFound := True;
          LX := PlottedPoints[i].X-G.cx/6;
          LY := PlottedPoints[i].Y-G.cy/3;
        end;
      end;
      if LabelFound and (LabelPos in [lbStart,lbEnd]) then
        G.TextJ(LX, LY, taRightJustify, Lbl);
    end;
  end;

  function PlotObsPoint(Manifoldo: TManifold; Jo: TQuantumNumber; Symo: Shortint; Indexo: Integer;
                         Manifoldc: TManifold; Jc: TQuantumNumber; Symc: Shortint; Indexc: Integer;
                         E: Double; Number: Integer; Upper: Boolean):Boolean;
  var
    ECalc: Double;
    XQ: TQuantumNumber;
    Ev: TEigenvalues;
  begin
    Result := nErrors > 50;
    XQ := Jo;
    if (Manifoldo = Manifold) and (Jo >= XMin) and (Jo <= XMax) then
      with Observations[Number] do begin
        LastColour := Colour;
        { TODO -cField: Add field search here }
        if Assigned(Manifoldc) then begin
          { Find "other" end of transition }
          Indexc := FindEigenvaluesAndIndex(Ev, Manifoldc, Jc, Symc, Indexc, Name,
                                            Upper, LineNumber, FileName, RemoveSpins, QuantumNumberFormat, ExField);
          if Indexc <= 0 then begin
            Inc(nErrors);
            Exit;
          end;
          E := E + Ev.E[Indexc];
          if LastColour = clNone then
            LastColour := Ev.DominantBasis[Indexc].State.FindColour;
        end else if ManifoldU = nil then
          E := -E; { Lower level only }
        if LastColour = clHide then
          Exit
        else if LastColour = clNone then
          LastColour := clBlack;
        MarkStyle.Colour := LastColour;
        if AltPlot then
          MarkStyle.MarkType := MarkTypeFor(Symo)
        else
          MarkStyle.MarkType := MarkTypeFor(Manifoldo.Molecule.JDeAdjustedSym(Jo,Symo));
        if JoinObs then begin
          Indexo := FindEigenvaluesAndIndex(Ev, Manifoldo, Jo, Symo, Indexo, Name,
                                            not Upper, LineNumber, FileName, RemoveSpins, QuantumNumberFormat, ExField);
          if Indexo <= 0 then
            Inc(nErrors)
          else begin
            if not IsSelected(Ev,Indexo) then
              Exit;
            LastColour := Ev.DominantBasis[Indexo].State.FindColour;
            if LastColour = clHide then
              Exit
            else if LastColour = clNone then
              LastColour := clBlack;
            XQ := XQno(Ev,Indexo);
            Ecalc := Ev.E[Indexo];
            E := (E-Ecalc)*ErrorMultiplier + Ecalc;
            Ecalc := Adjusty(Ecalc,XQ);
          end;
        end else
          Indexo := -1;
        E := Adjusty(E,XQ); { TODO: May not adjust on correct quantum number if Join Obs not set }
        if Length(PlottedPoints) <= nPlottedPoints then
          SetLength(PlottedPoints, nPlottedPoints+10000);
        with PlottedPoints[nPlottedPoints] do begin
          X := XQ/2;
          Y := E;
          Index := Number;
          G.SymbolT(X, E, MarkStyle);
        end;
        Inc(nPlottedPoints);
        if Indexo >0 then begin
          G.StyleP(LastColour, G.LineStyle.Width, G.LineStyle.Pattern);
          G.ClipPoint(XQ/2, E);
          G.ClipJoin(XQ/2, Ecalc);
        end;
      end;
  end;

  procedure CheckHeaderShown;
  var
    Mixture: TMixture;
    SL: Integer;
  begin
    if not LogHeaderShown then begin
      { Header for list }
      Mixture := Manifold.Mixture;
      SL := Max(Mixture.LongestSymLength,4);
      WriteLn(Log, 'Energy Level List');
      if ShowMolecule then
        Write(Log, Copy('Molecule',1,Length(Mixture.LongestMoleculeName)):Length(Mixture.LongestMoleculeName), ' ');
      if ShowManifold then
        Write(Log, 'M':Length(Mixture.LongestManifoldName), ' ');
      if ExternalField then
        Write(Log, '   M ')
      else
        Write(Log, '   J ');
      Write(Log, 'Sym':SL, '   # ');
      if ExternalField then
        Write(Log,'Field(V/m)');
      Write(Log,      '       Energy ');
      if Mixture.ShowEstUnc then
        Write(Log, 'Est Unc   ');
      if Manifold.HasWidth then
        Write(Log, 'Width     ');
      Write(Log, '   g ');
      Write(Log, 'Population ');
      WriteLn(Log, 'Label');
      LogHeaderShown := True;
    end;
  end;

  procedure ShowState(Ev: TEigenvalues; i:Integer);
  var
    Mixture: TMixture;
    SL, Offset: Integer;
  begin
    Mixture := Manifold.Mixture;
    SL := Max(Mixture.LongestSymLength,4);
    if ShowMolecule then
      Write(Log, Ev.Manifold.Molecule.Name:Length(Mixture.LongestMoleculeName), ' ');
    if ShowManifold then
      Write(Log, Ev.Manifold.Name:Length(Mixture.LongestManifoldName), ' ');
    Write(Log, Manifold.QuantumNumberToStr(Ev.J):4, ' ', Manifold.Molecule.SymName(Ev.DominantBasis[i].Sym):SL, i:4);
    if ExternalField then
      if ESweep then
        Write(Log, DtosMA(Ev.ExField.E,10,-4))
      else
        Write(Log, DtosMA(Ev.ExField.B,10,-4));
    Write(Log, Ev.E[i]:14:Manifold.Precision);
    if Mixture.ShowEstUnc then
      Write(Log, DtosMA(EstErr(nil,Ev,-1,i),10,-4));
    if Manifold.HasWidth then
      Write(Log, DtosMA(Ev.Widths[i],10,-4));
    Write(Log,Ev.StatisticalWeight(i)*(Ev.J+1):5);
    Write(Log,DtosMA(Ev.StatisticalWeight(i)*(Ev.J+1)*Manifold.Population(Ev,i,Simulation,Overflow,0)/Manifold.Molecule.PartitionFunction(Simulation,Overflow),11,-1));
    Write(Log, ' ', Ev.DominantBasis[i].StateLabel(Mixture.QuantumNumberFormat));
    if Mixture.ShowDerivatives and (Length(Ev.Derivatives) > 0) then begin
      if Manifold.Count = 1 then
        Offset := Length(Manifold.State[0].FullHeirarchicName(False,True))+2
      else
        Offset := Length(Manifold.FullHeirarchicName(False,True))+2;
      for SL := 0 to High(Ev.Derivatives) do
        with Ev.Derivatives[SL] do
          Write(Log, ' ', Copy(MNode.FullParameterName(Index),Offset,255), DtosMA(Values[i], 12, -1) );
    end;
    WriteLn(Log);
  end;

  procedure ShowLog;
  var
    Sym, AdjSym, i, Pass: Integer;
    Ev: TEigenValues;
  begin
    CheckHeaderShown;
    for Pass := 0 to 1 do begin
      J := Manifold.FindJmin(XMin);
      while J <= XMax do begin
        for Sym := 0 to Manifold.nSym-1 do
          if (Symmetry < 0) or (Symmetry = Sym) then begin
            AdjSym := Manifold.Molecule.JAdjustedSym(J, Sym);
            Ev := Manifold.Eigenvalues(J, AdjSym, [meHrot, meWidth, meAllDerivatives, meFloatedDerivatives], Simulation.ExField, True, plNone, State, IncludePerturbations);
            if Pass = 1 then { Two passes needed to ensure derivatives all calculated if needed }
              for i := 1 to Ev.Size do
                ShowState(Ev,i);
          end;
        Inc(J, 2);
      end;
    end;
    WriteLn(Log);
  end;

  procedure ShowOneState(Sym: Integer; MatchState: TState);
  var
    EvNo, MaxEvNo, i, AdjSym: Integer;
    IndexL, IndexU, nMatch, IndexT: Integer; { Numbers filtered by State }
    Match: Boolean;
    Ev: TEigenValues;
  begin
    EvNo := 1;
    MaxEvNo := 0;
    LastName := '';
    repeat
      FirstPointOnCurve := nPlottedPoints; { No points plotted yet }
      if Simulation.HasFields then
        J := -Manifold.FindJmax(XMax,-IgnoreSpins)
      else
        J := Manifold.FindJmin(XMin,-IgnoreSpins);
      while J <= XMax do begin
        AdjSym := Manifold.Molecule.JAdjustedSym(J, Sym);
        Ev := Manifold.Eigenvalues(J, AdjSym, [], Simulation.ExField, True, plNone, State, IncludePerturbations, -IgnoreSpins);
        if Assigned(MatchState) or (TIndex >= 0) then begin
          { Find the number matching this state }
          nMatch := 0;
          for i := 1 to Ev.Size do begin
            Match := not Assigned(MatchState) or (Ev.DominantBasis[i].State = MatchState);
            if TIndex >= 0 then begin
              IndexT := Ev.DominantBasis[i].TrackIndex;
              if IndexT > MaxTIndex then
                MaxTIndex := IndexT;
              Match := Match and (TIndex = IndexT);
            end;
            if Match then begin
              Inc(nMatch);
              if nMatch > Length(Matches) then
                SetLength(Matches, nMatch+20);
              Matches[nMatch-1] := i;
            end;
          end;
          if EvNo <= nMatch then begin
            IndexL := Matches[EvNo-1];
            IndexU := Matches[nMatch-EvNo];
          end;
        end else begin
          nMatch := Ev.Size;
          IndexL := EvNo;
          IndexU := Ev.Size-EvNo+1;
        end;
        if nMatch > MaxEvNo then
          MaxEvNo := nMatch;
        if nMatch >= EvNo then
          if JoinHighest then
            PlotPoint(Ev,IndexU)
          else
            PlotPoint(Ev,IndexL);
        Inc(J, 2);
      end;
      PlotCurve(LastName,TLabelPos(LabelsStyle));
      // PlotPoints(FirstPointOnCurve);
      Inc(EvNo);
    until EvNo > MaxEvNo;
  end;

  procedure ShowStateLabels;
  var
    i: Integer;
  begin
    for i := 0 to Manifold.nStates-1 do
      with Manifold.State[i] do
        if (Origin >= G.Ymin) and (Origin <= G.Ymax) then
          case TLabelPos(LabelsStyle) of
            lbLeftOut: G.TextJ(G.Xmin-G.cx/6, Origin-G.cy/3, taRightJustify, Name);
            lbLeftIn: G.TextJ(G.Xmin+G.cx*1.5, Origin, taLeftJustify,  Name);
            lbRightIn: G.TextJ(G.Xmax, Origin, taRightJustify, Name);
            lbRightOut: G.TextJ(G.Xmax, Origin, taLeftJustify, Name);
          end;
  end;

  procedure SweepField;
  var
    i, j, iZeroField, iEff: Integer;
    Evs: array of TEigenvalues;
    Fields: array of Double;
    ExFields: array of TStaticFields;
    M: TQuantumNumber;
    E, Mult: Double;
    Lbl: string;
    k: Integer;
  begin
    { Potentially memory intensive, as all eigenvalues at all fields cached }
    FirstPoint := True;
    if ToLog then
      CheckHeaderShown;
    Mult := ShiftMult;
    M := XMin;
    if SmallestProjection(M) <> Manifold.HalfSpin(-IgnoreSpins) then
      Inc(M);
    PushProgress('M',XMax div 2);
    try
      while M <= XMax do begin
        SetLength(Evs, FieldSamples);
        SetLength(Fields, FieldSamples);
        SetLength(ExFields, FieldSamples);
        SetProgress(M div 2);
        { Choose fields }
        iZeroField := -1;
        for i := 0 to High(Fields) do begin
          Fields[i] := MinField + (MaxField-MinField)*i/High(Fields);
          if (iZeroField < 0) or (Abs(Fields[i]) < Abs(Fields[iZeroField]))  then
            iZeroField := i;
        end;
        { Shift one of the points to close to zero, if zero included in plot range }
        if MaxField * MinField <= 0 then begin
          { Shift not quite to zero to make life easier for state tracking }
          Fields[iZeroField] := (MaxField-MinField)/High(Fields)/10000;
          if Fields[iZeroField] = 0 then
            Exit; { calculation at zero field breaks, as symmetry -1 is then invalid }
        end else begin
          iZeroField := 0; { No zero field point, so use LHS as reference }
          Mult := 1;
        end;
        for i := 0 to High(Fields) do begin
          ExFields[i] := Simulation.ExField;
          if ESweep then
            ExFields[i].E := Fields[i]
          else
            ExFields[i].B := Fields[i];
          { Start parallel calculation }
          Evs[i] := Manifold.Eigenvalues(M, -1, [], ExFields[i], False, plNone, State, IncludePerturbations, -IgnoreSpins);
        end;
        PushProgress('Field',High(Fields));
        try
          for i := 0 to High(Fields) do begin
            SetProgress(i);
            Evs[i].Make(True,[meHrot]);
          end;
        finally
          PopProgress;
        end;
        for i := 1 to Evs[0].Size do
          if IsSelected(Evs[Max(0,iZeroField)],i) then begin
            FirstPointOnCurve := nPlottedPoints;
            Lbl := Evs[Max(0,iZeroField)].DominantLabel(i,GlobalMixture.QuantumNumberFormat);
            { TODO: Match states with largest overlap at adjacent fields if TrackStates.Checked }
            for j := 0 to High(Fields) do begin
              iEff := i;
              if TrackQuanta then begin
                { Look for match in state label }
                if Lbl <> Evs[j].DominantLabel(i,GlobalMixture.QuantumNumberFormat) then
                  for k := 1 to Evs[0].Size do
                    if (j <> k) and (Lbl = Evs[j].DominantLabel(k,GlobalMixture.QuantumNumberFormat)) then begin
                      iEff := k;
                      Break;
                    end;
              end;
              E := Evs[j].Energy[iEff];
              if iZeroField >= 0 then
                E := (E - Evs[iZeroField].Energy[i])*Mult + Evs[iZeroField].Energy[i];
              PlotPoint(Evs[j],iEff,Fields[j],E);
              if ToLog then
                ShowState(Evs[j],iEff);
            end;
            PlotCurve(Lbl,TLabelPos(LabelsStyle));
            if ToLog then
              Writeln(Log);
          end;
          Inc(M,2);
        end;
    finally
      PopProgress;
    end;
  end;

var
  Sym, LastSym, FirstPointInManifold: Integer;
begin
  nErrors := 0;
  Manifold.CheckProperties;
  MarkStyle := TMarkStyle.Create;
  MarkStyle.DotAtCentre := True;
  IncludePerturbations := not ExcludePerturbations;
  ShowPoints := CalcPoints or CalcBoth;
  ShowLines := CalcLine or CalcBoth;
  ASelecting := ASelect <> Low(Integer);
  BSelecting := BSelect <> Low(Integer);
  Simulation := Manifold.Mixture.Simulations[0];
  Overflow := nil;
  FirstPointInManifold := nPlottedPoints;
  if (ShowPoints or ShowLines) and (Manifold.nStates > 0) then
    if ExternalField then
      SweepField
    else begin
      if Manifold.Jmax > 0 then
        Xmax := Min(Xmax, Manifold.Jmax);
      if ToLog then
        ShowLog;
      if TrackQuanta then begin
        TIndex := 0;
        MaxTIndex := 0;
      end else begin
        TIndex := -1;
        MaxTIndex := -1;
      end;
      if Simulation.HasFields then
        LastSym := 0
      else
        LastSym := Manifold.nSym-1;
      repeat
        for Sym := 0 to LastSym do
          if (Symmetry < 0) or (Symmetry = Sym) then
            if TrackState and (Manifold.nStates > 1) then
              for i := 0 to Manifold.nStates-1 do
                ShowOneState(Sym,Manifold.State[i])
            else
              ShowOneState(Sym,nil);
        Inc(TIndex);
      until TIndex > MaxTIndex;
      if LabelsStyle <> 0 then
        ShowStateLabels;
    end;
  PlotPoints(FirstPointInManifold); { Plot points last so that filled circles work }

  { Now plot observed energy levels }
  if ObsUpper then
    for i := 0 to High(Observations) do
      with Observations[i] do
        if not (ObsType in [obIntensity,obParameter])
               and PlotObsPoint(ManifoldU, JU, SymU, IndexU, ManifoldL, JL, SymL, IndexL, Frequency, i, True) then
          Break;
  if ObsLower then
    for i := 0 to High(Observations) do
      with Observations[i] do
        if not (ObsType in [obIntensity,obParameter])
               and PlotObsPoint(ManifoldL, JL, SymL, IndexL, ManifoldU, JU, SymU, IndexU, -Frequency, i, False) then
          Break;
  MarkStyle.Free;
  if nErrors > 0 then
    G.Title1(Format('%d problems with observations found', [nErrors]));
  if Assigned(Overflow) then begin
    WriteLn(Log, Overflow.Message + ' - fail in partition function');
    OverFlow.Free;
  end;
end;

procedure TLevelPlotter.PlotItem(G: TGraphM; MNode: TMNode; Log: Boolean);

  procedure PlotMolecule(Molecule: TMolecule; ShowMolecule:Boolean; ShowPartitionFunction: Boolean);
  var
    i: Integer;
    Overflow: Exception;
    DummyArray: TDoubleArray;
  begin
    if ShowPartitionFunction then begin
      if Molecule.Mixture.Simulations[0].Tspin >= 0 then
        SetLength(DummyArray, Molecule.nSpinSpecies)
      else
        SetLength(DummyArray,0);
      LogHeaderShown := Molecule.ActivePartitionFunction.CalculatePartitionFunction(Molecule, Molecule.Mixture.Simulations[0], Overflow, DummyArray, True,
                                                            ShowMolecule, not LogHeaderShown) > 0;
      Overflow.Free; { Problem shown in log, so do not repeat }
    end else
      for i := 0 to Molecule.nManifolds-1 do
        PlotManifold(G, Molecule.Manifold[i], nil, Log, ShowMolecule, Molecule.nManifolds > 1);
  end;
  procedure PlotSpecies(Species: TSpecies; ShowMolecule:Boolean; ShowPartitionFunction: Boolean);
  var
    i: Integer;
  begin
    for i := 0 to Species.Count-1 do
      PlotMolecule(Species.Isotopomer[i], ShowMolecule or (Species.Count > 1), ShowPartitionFunction);
  end;
  procedure PlotMixture(Mixture: TMixture; ShowPartitionFunction: Boolean);
  var
    i: Integer;
  begin
    for i := 0 to Mixture.nSpecies-1 do
      PlotSpecies(Mixture.Species[i], Mixture.nMolecules > 1, ShowPartitionFunction);
  end;
  procedure PlotState(State: TState);
  begin
    if State.Manifold.nStates = 1 then
      PlotManifold(G, State.Manifold, nil, Log, False, False)
    else
      PlotManifold(G, State.Manifold, State, Log, False, False);
  end;
  procedure Plot(ShowPartitionFunction: Boolean);
  begin
    LogHeaderShown := False;
    if MNode is TMixture then
      PlotMixture(TMixture(MNode), ShowPartitionFunction)
    else if MNode is TSpecies then
      PlotSpecies(TSpecies(MNode), False, ShowPartitionFunction)
    else if MNode is TMolecule then
      PlotMolecule(TMolecule(MNode), False, ShowPartitionFunction)
    else if MNode is TManifold then
      if ShowPartitionFunction then
        PlotMolecule(TManifold(MNode).Molecule, False, ShowPartitionFunction)
      else
        PlotManifold(G, TManifold(MNode), nil, Log, False, False)
    else if MNode is TState then
      if ShowPartitionFunction then
        PlotMolecule(TState(MNode).Molecule, False, ShowPartitionFunction)
      else
        PlotState(TState(MNode));
  end;

var
  XAxis: string;
begin
  MNode.CheckProperties;
  MNode.RootNode.FixupProperties.Free; { Check Mixture also to acount for e.g. spin changes }

  if ExternalField then
    if ESweep then
      XAxis := 'Electric Field / Vm-1'
    else
      XAxis := 'Magnetic Field / T'
  else
    XAxis := QnoTypeNames[XQuantumNumber];

  nPlottedPoints := 0;
  G.Start;
  try
    { Plot some Levels }
    if ExternalField then
      G.Map(MinField, MaxField, Ymin, Ymax)
    else
      G.Map(XMin/2, XMax/2, Ymin, Ymax);
    G.Title2(SymmetryKey(MNode, AltPlot));
    FirstPoint := True;
    if Reduction <> 0 then
      G.Scales(XAxis, '(Energy - ' + FloatToStr(Reduction) + QnoTypeNames[XQuantumNumber] + '(' + QnoTypeNames[XQuantumNumber] + '+1))/' + (MNode.RootNode as TMixture).UnitsName)
    else
      G.Scales(XAxis, 'Energy/' + (MNode.RootNode as TMixture).UnitsName);
    Plot(False);
    if Log and not ExternalField then
      Plot(True);
  finally
    G.Finish;
  end;
  SetLength(PlottedPoints,nPlottedPoints);
end;

function TLevelPlotter.GetSetting(AIndex: Integer):Integer;
var
  s: string;
begin
  if not DoneCreating then
    CheckIndices;
  if AIndex = SymmetryIndex then begin
    if not TryStrToSymmetry(Result,SettingToStr(AIndex),FSymmetryList) then
      raise EReportException.Create('Bad symmetry:'+SettingToStr(AIndex));
  end else if ((AIndex = IgnoreSpinsIndex) or (AIndex = ASelectIndex) or (AIndex = BSelectIndex)) then begin
    s := SettingToStr(AIndex);
    if LowerCase(s) = 'all' then
      if AIndex = IgnoreSpinsIndex then
        Result := 0
      else
        Result := Low(Integer)
    else
      Result := inherited GetSetting(AIndex);
  end else
    Result := inherited GetSetting(AIndex);
end;

procedure TLevelPlotter.SetSetting(AIndex: Integer; const AValue: Integer);
begin
  if not DoneCreating then
    CheckIndices;
  if AIndex = SymmetryIndex then
    SetSettingFromStr(AIndex,SymmetryToStr(AValue,FSymmetryList))
  else if (AIndex = IgnoreSpinsIndex) and (GetSetting(AIndex) = 0) then
    SetSettingFromStr(AIndex,'all')
  else if ((AIndex = ASelectIndex) or (AIndex = BSelectIndex)) and (AValue <= -9999) then
    SetSettingFromStr(AIndex,'all')
  else
    inherited SetSetting(AIndex, AValue);
end;

function TLevelPlotter.GetSettingsNames(AIndex: Integer; out LowestValue: Integer): PPChar;
begin
  Result := inherited GetSettingsNames(AIndex, LowestValue);
  if AIndex = XQuantumNumberIndex then
    Result := @QnoTypeNames[0]
  else if AIndex = LabelsStyleIndex then
    Result := @LabelsStyleNames[0];
end;

procedure TLevelPlotter.CheckIndices;
begin
  {$IFNDEF CONSOLE}
  with FormSettings do begin
  {$ENDIF}
    if SymmetryIndex <= 0 then
      { Can't be set in Create, because needed in intialisation }
      SymmetryIndex := IndexOfSetting('Symmetry');
    if ASelectIndex <= 0 then
      ASelectIndex  :=  IndexOfSetting('ASelect');
    if BSelectIndex <= 0 then
      BSelectIndex  := IndexOfSetting('BSelect');
    if IgnoreSpinsIndex <= 0 then
      IgnoreSpinsIndex := IndexOfSetting('IgnoreSpins');
    if XQuantumNumberIndex <= 0 then
      XQuantumNumberIndex := IndexOfSetting('XQuantumNumber');
    if LabelsStyleIndex <= 0 then
      LabelsStyleIndex := IndexOfSetting('LabelsStyle');
  {$IFNDEF CONSOLE}
  end;
  {$ENDIF}
end;

end.
