unit LmatrixFrm;
{$I DFS.inc}{Copyright (C) 2008 C M Western; see commandline.pas for license}
interface

uses
  SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, WhatsThis, Vib, Grids, NGrid, MNode, ExtCtrls, ScrlCtrl, Graphs,
  Split, StdCtrls, AlEdit, EditExtr, NewSpin, GraphM, MathSup, GraphI, Menus;

type

  TLMatrixForm = class(THelpForm)
    Grid: TNumberGrid;
    Split: TSplit;
    G: TGraphD;
    Panel: TPanel;
    PlotPlaneGroup: TRadioGroup;
    InnerPanel: TPanel;
    MagnificationLabel: TLabel;
    MagnificationSpin: TFixedSpin;
    MassWeighted: TCheckBox;
    OperateButton: TButton;
    OperatePopupMenu: TPopupMenu;
    PrintGeometryItem: TMenuItem;
    FixCentreofMassItem: TMenuItem;
    OrthogonalizeItem: TMenuItem;
    ShowTRCheckBox: TCheckBox;
    NormalizeItem: TMenuItem;
    procedure GridChanged(Sender: TObject; Col, Row: Integer);
    procedure FormClose(Sender: TObject; var AnAction: TCloseAction);
    procedure PlotPlaneGroupClick(Sender: TObject);
    procedure GridSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure MagnificationSpinChange(Sender: TObject);
    procedure GridMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PrintGeometryItemClick(Sender: TObject);
    procedure FixCentreofMassItemClick(Sender: TObject);
    procedure OrthogonalizeItemClick(Sender: TObject);
    procedure OperateButtonClick(Sender: TObject);
    procedure NormalizeItemClick(Sender: TObject);
  private
    State: TElectronicState;
    procedure MNodeChange(Sender: TMnode; Change: TMNodeChange; Index: Integer);
  public
    destructor Destroy; override;
    procedure PopulateForm(ACol:Integer = -1);
  end;

function ConnectLmatrixFrm(AState: TElectronicState): TComponent;
procedure PlotAtoms(G: TGraphM; State:TElectronicState; PlotPlane, Mode: Integer;
                    FindLimits, PlotAxes, PlotLastPoints, AvoidDoubleLabels, MassWeighted, ShowTR: Boolean;
                    Magnification: Double; nMark: Integer; var LastPoints, Labels: TDPointArray;
                    var xMin, xMax, yMin, yMax: Double; var LimitsSet: Boolean);

implementation

uses {$IFDEF LCL_} LResources, {$ENDIF}
     Global, Mixture, Math, GStyle, Gmarks, MatrixVec;

{$IFNDEF FPC}
{$R *.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

function ConnectLmatrixFrm(AState: TElectronicState): TComponent;
var
  Form: TLMatrixForm;
  i: Integer;
begin
  for i := 0 to Screen.CustomFormCount - 1 do
    if Screen.CustomForms[i] is TLMatrixForm then
      if TLMatrixForm(Screen.CustomForms[i]).State = AState then begin
        ReShow(Screen.CustomForms[i]);
        Result := Screen.CustomForms[i];
        Exit;
      end;
  Form := TLMatrixForm.Create(Application);
  Result := Form;
  Form.State := AState;
  AddMNodeNotification({$IFDEF FPC}@{$ENDIF}Form.MNodeChange);
  Form.Show;
  Form.PopulateForm;
end;

{ TLMatrixForm }

procedure TLMatrixForm.OperateButtonClick(Sender: TObject);
begin
  OperatePopupMenu.PopupComponent := OperateButton;
  OperatePopupMenu.Popup(Mouse.CursorPos.X, Mouse.CursorPos.Y);
end;

destructor TLMatrixForm.Destroy;
begin
  RemoveMNodeNotification({$IFDEF FPC}@{$ENDIF}MNodeChange);
  inherited;
end;

procedure TLMatrixForm.FixCentreofMassItemClick(Sender: TObject);
begin
  State.FixCentreOfMass(Sender);
end;

procedure TLMatrixForm.FormClose(Sender: TObject; var AnAction: TCloseAction);
begin
  AnAction := caFree;
end;

procedure TLMatrixForm.GridChanged(Sender: TObject; Col, Row: Integer);
var
  V: Double;
  p: Integer;
begin
  if Row > 0 then
    if Col = 1 then
      with State.Nucleus((Row-1) div 3) do
        SetParameter(IndexOfParameter('x') + (Row-1) mod 3,Grid[Row,Col])
    else if (Col > 1) and (Col-2 < State.Molecule.nModes) then
      with State.Mode(Col-2) do begin
        V := Grid[Row,Col];
        if not MassWeighted.Checked then
          V := V / Sqrt((State.Nucleus((Row-1) div 3) as TMolecularNucleus).Mass);
        p := FirstDynamicParameter+Row-1;
        { Test to avoid rounding errors looking like a change }
        if not SameValue(V,GetParameter(p)) then
          SetParameter(p,V);
      end;
end;

procedure TLMatrixForm.GridMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  PopulateForm;
end;

procedure TLMatrixForm.GridSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
begin
  PopulateForm(ACol);
end;

procedure TLMatrixForm.MagnificationSpinChange(Sender: TObject);
begin
  PopulateForm;
end;

procedure TLMatrixForm.MNodeChange(Sender: TMnode; Change: TMNodeChange; Index: Integer);
begin
  case Change of
    mnDestroy: if Sender = State then Free;
    mnParameterChange, mnSettingChange, mnPropNameChange, mnNameChange:
      PopulateForm;
  end;
end;

procedure TLMatrixForm.NormalizeItemClick(Sender: TObject);
begin
  State.Normalize(Sender);
end;

procedure TLMatrixForm.OrthogonalizeItemClick(Sender: TObject);
begin
  State.Orthogonalize(Sender);
end;

procedure TLMatrixForm.PlotPlaneGroupClick(Sender: TObject);
begin
  PopulateForm;
end;

procedure PlotAtoms(G: TGraphM; State:TElectronicState; PlotPlane, Mode: Integer;
                    FindLimits, PlotAxes, PlotLastPoints, AvoidDoubleLabels, MassWeighted, ShowTR: Boolean;
                    Magnification: Double; nMark: Integer; var LastPoints, Labels: TDPointArray;
                    var xMin, xMax, yMin, yMax: Double; var LimitsSet: Boolean);
var
  nLabel: Integer;
  chDist: Double;
  procedure AddLabel(x, y: Double; Lab: string);
  begin
    Inc(nLabel);
    if not AvoidDoubleLabels then begin
      { Keep track of label positions }
      if High(Labels) <= nLabel then
        SetLength(Labels, nLabel+10);
      Labels[nLabel].X := x;
      Labels[nLabel].Y := y;
    end else
      if (nLabel < Length(Labels))
         and (Sqr(Labels[nLabel].X-x)+Sqr(Labels[nLabel].Y-y) < chDist) then
        Exit;
    G.Text(x,y,Lab);
  end;
  procedure Limit(x,y:Double);
  begin
    if not FindLimits then
      Exit;
    if not LimitsSet then begin
//      xMin := x;
//      xMax := x;
//      yMin := y;
//      yMax := y;
      xMin := -0.5; xMax := -xMin;
      yMin := xMin; yMax := -yMin;
      LimitsSet := True;
    end;
    xMin := Min(xMin,x);
    xMax := Max(xMax,x);
    yMin := Min(yMin,y);
    yMax := Max(yMax,y);
  end;
var
  i, ix, iy, M, M1, M2: Integer;
  Gx, Gy, Dx, Dy: Double;
  cx, cy, keff, Q: Double;
  P: TDPoint;
  A: TColumnVector;
begin
  case PlotPlane of
    0: begin ix := 0; iy := 1; end;
    1: begin ix := 1; iy := 2; end;
    2: begin ix := 0; iy := 2; end;
  end;
  if PlotAxes then begin
    G.MapLin(xMin-(Xmax-xMin)/10,xMax+(xMax-xMin)/10,yMin-(yMax-yMin)/10,yMax+(yMax-yMin)/10);
    with State.Nucleus(0) do
      G.Scales(ParameterNames[IndexOfParameter('x')+ix],ParameterNames[IndexOfParameter('x')+iy]);
  end;
  nLabel := -1;
  G.Chlen(cx,cy);
  chDist := Sqr(cx)+Sqr(cy);
  if FindLimits then
    State.PrepareMatrices(False,True)
  else begin
    if nMark = 4 then
      G.LineStyle.Pattern := lpDot;
    { Show principla axes }
    A := TColumnVector.Create(3);
    for i := 1 to 3 do begin
      A.Zero;
      A[i] := 100;
      A.PreMultiply(State.Itrans);
      Dx := A[ix+1];
      Dy := A[iy+1];
      G.ClipPoint(-Dx,-Dy);
      G.ClipJoin(Dx, Dy);
      M1 := G.OutCode(0,0);
      M2 := G.OutCode(Dx,Dy);
      if (M1 and M2) = 0 then begin
        { Some of axis visible }
        G.ClpPnt(Dx,Dy,0,0,M2);
        AddLabel(Dx+cx/2,Dy+cy/2,Char(Ord('a')+i-1));
      end;
    end;
    A.Free;
    G.LineStyle.Pattern := lpSolid;
  end;
  if not PlotLastPoints then
    SetLength(LastPoints, State.Molecule.nNuclei);
  for i := 0 to State.Molecule.nNuclei -1 do begin
    with State.Nucleus(i) do begin
      Gx := GetParameter(IndexOfParameter('x')+ix);
      Gy := GetParameter(IndexOfParameter('x')+iy);
    end;
    Limit(Gx,Gy);
    M1 := 0;
    M2 := Mode;
    if FindLimits then
      M2 := State.adm.ColCount-1
    else if Mode >= 0 then
      M1 := Mode;
    for M := M1 to M2 do begin
      if (M < State.Molecule.nModes) and (Abs(State.Mode(M).Omega) > 1e-5) then begin
        { Force constant in cm-1 Angstrom^-2 amu }
        keff := 2*Pi*ReduceFrequency*Sqr(State.Mode(M).Omega);
        { Value of normal co-ordinate at zero point energy }
        Q := Sqrt(State.Mode(M).Omega/keff);
      end else
        { Keep sensible scale for translations and rotationa }
        Q := 1/400/ReduceFrequency;
      if not MassWeighted then
        Q := Q*Sqrt((State.Nucleus(i) as TMolecularNucleus).Mass);
      Dx := Gx + State.adm[3*i+ix+1,M+1]*Magnification*Q;
      Dy := Gy + State.adm[3*i+iy+1,M+1]*Magnification*Q;
      Limit(Dx,Dy);
    end;
    if not FindLimits then begin
      G.Symbol(Gx,Gy,nMark);
      AddLabel(Gx+cx,Gy-cy,State.Nucleus(i).Name);
      if Mode >= 0 then begin
        G.Arrow(Gx,Gy,Dx,Dy,arHead);
        P.X := Dx;
        P.Y := Dy;
      end else begin
        P.X := Gx;
        P.Y := Gy;
      end;
      if PlotLastPoints then begin
        G.LineStyle.Pattern := lpDot;
        with LastPoints[i] do
          G.Point(X, Y);
        G.Join(P.X, P.Y);
        G.LineStyle.Pattern := lpSolid;
      end else
        LastPoints[i] := P;
    end;
  end;
end;

procedure TLMatrixForm.PopulateForm(ACol:Integer = -1);
var
  i, j, b, Pass: Integer;
  Nucleus: TNucleus;
  xMin, xMax, yMin, yMax: Double;
  LimitsSet: Boolean;
  P, L: TDPointArray;
  function MassWeight(V: Double; nNucleus: Integer):Double;
  begin
    if MassWeighted.Checked then
      Result := V
    else
      Result := V * Sqrt((State.Nucleus(nNucleus) as TMolecularNucleus).Mass);
  end;
begin
  if csLoading in ComponentState then
    Exit;
  if Acol < 0 then
    ACol := Grid.Col;
  Caption := 'L matrix for ' + State.UniqueName;
  State.PrepareMatrices(False,ShowTRCheckBox.Checked);
  Grid.RowCount := State.Molecule.nNuclei*3+1;
  Grid.ColCount := State.adm.ColCount+2;
  if State.Molecule.nNuclei = 0 then
    Exit;
  { Equilibrium geometries }
  for i := 0 to State.Molecule.nNuclei - 1 do begin
    Nucleus := State.Nucleus(i);
    for j := 0 to 2 do begin
      Grid.Lbl[i*3+j+1] := Nucleus.Name + ' ' + Char(Ord('x')+j);
      Grid.Value[i*3+j+1,1] := Nucleus.GetParameter(Nucleus.IndexOfParameter('x')+j)
    end;
  end;
  for i := 0 to State.Molecule.nModes - 1 do begin
    Grid.Title[i+2] := State.Mode(i).Name;
    with State.Mode(i) do
      for j := 0 to 3*State.Molecule.nNuclei - 1 do
        Grid.Value[j+1,i+2] := MassWeight(GetParameter(FirstDynamicParameter+j),j div 3);
  end;
  if ShowTRCheckBox.Checked then begin
    { Rotations }
    b := State.Molecule.nModes;
    for i := 0 to 2 do
      if State.Ixx[i] > 1e-10 then begin
        Grid.Title[b+2] := 'R'+Char(Ord('x')+i);
        for j := 0 to 3*State.Molecule.nNuclei - 1 do
          Grid.Value[j+1,b+2] := MassWeight(State.adm[j+1,State.Molecule.nModes+i+1],j div 3);
        Inc(b);
      end;
    { Translations }
    b := 3*State.Molecule.nNuclei-3;
    for i := 0 to 2 do begin
      Grid.Title[b+2] := 'T'+Char(Ord('x')+i);
      for j := 0 to 3*State.Molecule.nNuclei - 1 do
        Grid.Value[j+1,b+2] := MassWeight(State.adm[j+1,b+1],j div 3);
      Inc(b);
    end;
  end;
  LimitsSet := False;
  G.Start;
  for pass := 0 to 1 do
    PlotAtoms(G, State, PlotPlaneGroup.ItemIndex, ACol-2, pass=0, pass=1, False, False, MassWeighted.Checked, ShowTRCheckBox.Checked,
              MagnificationSpin.Value, 3, P, L, xMin, xMax, yMin, yMax, LimitsSet);
  G.Finish;
end;

procedure TLMatrixForm.PrintGeometryItemClick(Sender: TObject);
begin
  State.PrintMolecularInfo(Sender);
end;

end.
