unit LogUnit;
{$I DFS.inc}{Copyright (C) 2005 C M Western; see commandline.pas for license}
interface

uses
  {$IFDEF MSWINDOWS} Windows, {$ENDIF}
  Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, ActnList,
  {$IFNDEF LCL_} FileNameEdit, {$ELSE} EditBtn, {$ENDIF}
  Dialogs, ExtCtrls, ScrlCtrl, Log, Buttons, StdCtrls, AlEdit, EditExtr,
  NewSpin, MNode, SettingsForm, Split, Graphs, PlotOC, ComCtrls, Menus,
  Obs, Mixture, Settings;

type

  { TLogForm }

  TLogForm = class(TSettingsForm)
    Log: TLog;
    TopPanel: TPanel;
    ObsFileName: TFileNameEdit;
    FitSpeed: TSpeedButton;
    EditButton: TSpeedButton;
    ClearButton: TSpeedButton;
    CopyButton: TSpeedButton;
    SaveDialog: TSaveDialog;
    SaveLogButton: TSpeedButton;
    UndoFitButton: TSpeedButton;
    RedoFitButton: TSpeedButton;
    ActionList: TActionList;
    UndoFitAction: TAction;
    RedoFitAction: TAction;
    FitType: TComboBox;
    OutputComboBox: TComboBox;
    procedure ObsFileNameChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FitSpeedClick(Sender: TObject);
    procedure EditButtonClick(Sender: TObject);
    procedure ClearButtonClick(Sender: TObject);
    procedure CopyButtonClick(Sender: TObject);
    procedure LogFormDestroy(Sender: TObject);
    procedure SaveLogButtonClick(Sender: TObject);
    procedure UndoFitActionExecute(Sender: TObject);
    procedure UndoFitActionUpdate(Sender: TObject);
    procedure RedoFitActionExecute(Sender: TObject);
    procedure RedoFitActionUpdate(Sender: TObject);
    procedure FitSpeedMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure OutputComboBoxChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    {$IFDEF VCL_}
    procedure WMDropFiles(var Message: TMessage); message WM_DROPFILES;
    {$ENDIF}
  protected
    procedure HandleControl(Control: TControl; var PName, AValue, PHint: string; var Save: Boolean; var SettingType: TSettingType; Write: Boolean); override;
    procedure FormDropFiles(Sender: TObject; const FileNames: array of String);
    procedure MNodeChange(Sender: TMNode; Change: TMNodeChange; Index: Integer); override;
  public
    procedure ArrangeForm(Screen: TRect); override;
    procedure LoadObservations(const AName: string = ''; ObsType: TObsType = obLine; TakeWeight: TTakeWeight = tkObs);
    procedure DoFit(const FileName: string; JustResiduals: Boolean; TakeWeight: TTakeWeight);
  end;

var
  LogForm: TLogForm;

implementation

uses
   {$IFDEF LCL_} Types, {$ENDIF}
   TextAssign, Fitter, Huge, ClipBrd, WCompat, SlaveEdit, LineFrm, Worker,
   Global, Main, QuantumNumber, FormUtils, CalFrm, Math, CommandLine;

{$IFNDEF FPC}
{$R *.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

procedure TLogForm.FormCreate(Sender: TObject);
begin
  DragAcceptFiles(Self, True);
  {$IFNDEF VCL_}
  OnDropFiles := @FormDropFiles;
  {$ENDIF}
  GlobalMixture.GetEnumNames(GlobalMixture.IndexOfSetting('PrintLevel'),OutputComboBox.Items);
  OutputComboBox.ItemIndex := Ord(GlobalMixture.PrintLevel);
end;

procedure TLogForm.HandleControl(Control: TControl; var PName, AValue,
  PHint: string; var Save: Boolean; var SettingType: TSettingType;
  Write: Boolean);
begin
  if Control = OutputComboBox then
    Exit;
  inherited;
  if not Write and (Control = ObsFileName) and (AValue <> '') then
    AValue := ExtractRelativePath(GetCurrentDir+PathDelim, ExpandFileName(AValue));
end;

procedure TLogForm.EditButtonClick(Sender: TObject);
begin
  LaunchEditor(ObsFileName.FileName);
end;

procedure TLogForm.LoadObservations(const AName: string; ObsType: TObsType; TakeWeight: TTakeWeight);
var
  f: TStream;
  SName: string;
begin
  if AName = '' then
    SName := ObsFileName.FileName
  else
    SName := AName;
  if SName = '' then
    SName := '<clipboard>';
  if CompareText(SName, '<clipboard>') = 0 then begin
    f := THugeMemoryStream.Create;
    THugeMemoryStream(f).SetFromClipboard(CF_TEXT);
  end else begin
    MainForm.MRUFileList.AddItem(ExpandFileName(SName));
    f := TFileStream.Create(SName, fmOpenRead or fmShareDenyWrite);
  end;
  try
    Obs.LoadObservations(GlobalMixture, f, SName, ObsType, TakeWeight);
  finally
    f.Free;
  end;
  MainForm.LastObsWasLineForm := False;
end;

procedure TLogForm.DoFit(const FileName: string; JustResiduals: Boolean; TakeWeight: TTakeWeight);
begin
  case FitType.ItemIndex of
    0: MainForm.DoLineFit({$IFDEF FPC}@{$ENDIF}LoadObservations,FileName,obLine,JustResiduals,TakeWeight);
    1: MainForm.ContourFit(JustResiduals,FileName, ftContour);
    2: MainForm.DoLineFit({$IFDEF FPC}@{$ENDIF}LoadObservations,FileName,obIntensity,JustResiduals,TakeWeight);
    3: MainForm.DoLineFit({$IFDEF FPC}@{$ENDIF}LoadObservations,FileName,obELower,JustResiduals,TakeWeight);
    4: MainForm.DoLineFit({$IFDEF FPC}@{$ENDIF}LoadObservations,FileName,obEUpper,JustResiduals,TakeWeight);
    5: CalForm.FitButtonClick(nil);
    6: MainForm.DoLineFit({$IFDEF FPC}@{$ENDIF}LoadObservations,FileName,obCDLower,JustResiduals,TakeWeight);
    7: MainForm.DoLineFit({$IFDEF FPC}@{$ENDIF}LoadObservations,FileName,obCDUpper,JustResiduals,TakeWeight);
    8: MainForm.ContourFit(JustResiduals,FileName, ftOverlaySum);
  end;
end;

procedure TLogForm.FitSpeedClick(Sender: TObject);
begin
  if not SaveAllWindows then Exit;
  DoFit(ObsFileName.FileName, False, tkObs);
end;

procedure TLogForm.FitSpeedMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if mbRight = Button then
    DoFit('<clipboard>', False, tkObs);
end;

procedure TLogForm.ArrangeForm(Screen: TRect);
var
  R, MainRect: TRect;
begin
  inherited;
  MainRect := ScreenBounds(MainForm);
  { Arrange below main form }
  R.Left := MainRect.Left;
  R.Right := MainRect.Right;
  R.Top := Min(MainRect.Bottom, Screen.Bottom-100);
  R.Bottom := Screen.Bottom;
  if LineForm.Visible then begin
    { Overlap with lineform }
    Inc(R.Top, LineForm.Grid.ClientToScreen(Point(0,0)).Y-ScreenBounds(LineForm).Top);
    Inc(R.Left,25);
  end;
  SetScreenBounds(Self,R);
end;

procedure TLogForm.ClearButtonClick(Sender: TObject);
begin
  Flush(Worker.Log);
  Log.Clear;
end;

procedure TLogForm.CopyButtonClick(Sender: TObject);
begin
  if not Log.AreaSelected then
    Log.SelectAll;
  Log.CopyToClipboard
end;

procedure TLogForm.LogFormDestroy(Sender: TObject);
begin
  DisconnectLogs;
end;

procedure TLogForm.MNodeChange(Sender: TMNode; Change: TMNodeChange; Index: Integer);
begin
  inherited;
  if Visible and (Change = mnSettingChange) and (GlobalMixture <> nil) and (Sender = GlobalMixture)
     and (Index = GlobalMixture.IndexOfSetting('PrintLevel')) then
    OutputComboBox.ItemIndex := Ord(GlobalMixture.PrintLevel);
end;

procedure TLogForm.SaveLogButtonClick(Sender: TObject);
begin
  if SaveDialog.Execute then
    Log.Lines.SaveToFile(SaveDialog.FileName);
end;

procedure TLogForm.UndoFitActionExecute(Sender: TObject);
begin
  GlobalMixture.UndoFit;
end;

procedure TLogForm.UndoFitActionUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled := (GlobalMixture <> nil) and GlobalMixture.CanUndoFit;
end;

procedure TLogForm.RedoFitActionExecute(Sender: TObject);
begin
  GlobalMixture.RedoFit;
end;

procedure TLogForm.RedoFitActionUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled := (GlobalMixture <> nil) and GlobalMixture.CanRedoFit;
end;

{$IFDEF VCL_}
procedure TLogForm.WMDropFiles(var Message: TMessage);
begin
  HandleWMDropFiles(Message, FormDropFiles);
end;
{$ENDIF}

procedure TLogForm.ObsFileNameChange(Sender: TObject);
begin
  MainForm.LastObsWasLineForm := False;
end;

procedure TLogForm.OutputComboBoxChange(Sender: TObject);
begin
  if (csLoading in ComponentState) or not Visible or (GlobalMixture = nil) then
    Exit;
  GlobalMixture.PrintLevel := TPrintLevel(OutputComboBox.ItemIndex);
end;

procedure TLogForm.FormDropFiles(Sender: TObject;const FileNames: array of String);
begin
  if Length(FileNames) > 0 then
    ObsFileName.FileName := FileNames[0];
end;

procedure TLogForm.FormShow(Sender: TObject);
begin
  OutputComboBox.ItemIndex := Ord(GlobalMixture.PrintLevel);
  {$IFDEF LCL_}
  EditButton.BorderSpacing.Left := EditButton.Height;
  {$ENDIF}
end;

end.

