unit Main;
{$I DFS.inc}
{
PGOPHER - a program for simulating rotational structure
Copyright (C) 2005 C M Western
School of Chemistry, University of Bristol, Bristol, UK
For further information please see http://pgopher.chm.bris.ac.uk/
or contact help-pgopher@bristol.ac.uk

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
}

interface

uses {$IFDEF MSWINDOWS} Windows, {$ENDIF}
     SysUtils, Variants, Classes, Graphics, Controls, Forms,
     Dialogs, PlotFrm, ExtCtrls, ScrlCtrl, Graphs, ComCtrls, Buttons, ToolWin,
     StdCtrls, GPenDlg, PrintPre, ExActn, ActnList, ProjAbout, MRUFList, Menus,
     ToolBarEx, Split, Mixture, MNode, Obs, Overlay, TextStrm, MatrixVec, AlEdit,
     SampleMols, Settings, GraphI, GraphM, Units;

{$IFDEF DFS_DELPHI_10_UP}
  {$DEFINE SHOWMEMUSAGE}
{$ELSE}
  {$IFDEF FASTMM4} {$DEFINE SHOWMEMUSAGE} {$ENDIF}
{$ENDIF}

type

  TMixtureReader = function (Stream: TTextStream):TMixture;

  { TMainForm }

  TMainForm = class(TPlotForm)
    MainMenu: TMainMenu;
    FileMenu: TMenuItem;
    NewItem: TMenuItem;
    EmptyMixtureItem: TMenuItem;
    NewLinearMoleculeItem: TMenuItem;
    OpenItem: TMenuItem;
    ImportItem: TMenuItem;
    OldPgopherItem: TMenuItem;
    OldD3hItem: TMenuItem;
    CALPGMfileItem: TMenuItem;
    BinaryfileItem: TMenuItem;
    ExportItem: TMenuItem;
    ExportLineListItem: TMenuItem;
    ExportPlotItem: TMenuItem;
    LoadOverlayItem: TMenuItem;
    MergeItem: TMenuItem;
    N6: TMenuItem;
    SaveItem: TMenuItem;
    SaveAsItem: TMenuItem;
    N2: TMenuItem;
    PrintItem: TMenuItem;
    PrintSetupItem: TMenuItem;
    FileExitSeparator: TMenuItem;
    ExitItem: TMenuItem;
    EditMenu: TMenuItem;
    CopyItem: TMenuItem;
    PasteItem: TMenuItem;
    CopyPictureItem: TMenuItem;
    StylesItem: TMenuItem;
    PasteOverlay1Item: TMenuItem;
    PlotMenu: TMenuItem;
    LeftItem: TMenuItem;
    RightItem: TMenuItem;
    CompressItem: TMenuItem;
    ExpandItem: TMenuItem;
    PlotSelectionItem: TMenuItem;
    AllItem: TMenuItem;
    N3: TMenuItem;
    SimulateItem: TMenuItem;
    NextPlotstyleItem: TMenuItem;
    ShowPartsItem: TMenuItem;
    ShowSumItem: TMenuItem;
    FortratItem: TMenuItem;
    ShowFortratItem: TMenuItem;
    UpperStateItem: TMenuItem;
    UnitsItem: TMenuItem;
    WavenumbersItem: TMenuItem;
    MHzItem: TMenuItem;
    KelvinItem: TMenuItem;
    eVItem: TMenuItem;
    nmVacuumItem: TMenuItem;
    nmAirItem: TMenuItem;
    N5: TMenuItem;
    MagnifyPlotItem: TMenuItem;
    ShrinkplotItem: TMenuItem;
    ResetZoomItem: TMenuItem;
    OverlaysMenu: TMenuItem;
    PasteOverlayItem: TMenuItem;
    PasteAnotherItem: TMenuItem;
    SelectRegionItem: TMenuItem;
    ClearSelectionItem: TMenuItem;
    InvertItem: TMenuItem;
    ClearOverlayItem: TMenuItem;
    OverlayPropertiesItem: TMenuItem;
    N1: TMenuItem;
    DeleteLastPeakItem: TMenuItem;
    ClearAllPeaksItem: TMenuItem;
    ViewMenu: TMenuItem;
    ConstantsItem: TMenuItem;
    LineListItem: TMenuItem;
    LogItem: TMenuItem;
    OverlaysItem: TMenuItem;
    StatesItem: TMenuItem;
    TransitionsItem: TMenuItem;
    LevelsItem: TMenuItem;
    SpectrumItem: TMenuItem;
    TimingItem: TMenuItem;
    ClearLogItem: TMenuItem;
    ClearCacheItem: TMenuItem;
    ClearObservedItem: TMenuItem;
    HelpMenu: TMenuItem;
    IndexItem: TMenuItem;
    SearchItem: TMenuItem;
    ProceduresItem: TMenuItem;
    WhatIsItem: TMenuItem;
    UsingHelpItem: TMenuItem;
    AboutItem: TMenuItem;
    DumpItem: TMenuItem;
    OpenDialog: TOpenDialog;
    ImportDialog: TOpenDialog;
    ExportLineListDialog: TSaveDialog;
    MRUFileList: TdfsMRUFileList;
    OpenOverlayDialog: TOpenDialog;
    Project: TProject;
    ActionList: TActionList;
    ExCopy: TExCopy;
    ExPaste: TExPaste;
    GraphPrintPreview: TGraphPrintPreview;
    GraphPenDialog: TGraphPenDialog;
    SaveDialog: TSaveDialog;
    CalibrateItem: TMenuItem;
    N4: TMenuItem;
    ArrangeFormsItem: TMenuItem;
    CalibrationItem: TMenuItem;
    I2Item: TMenuItem;
    ThreadsItem: TMenuItem;
    AsymmetricTopItem: TMenuItem;
    VelocityItem: TMenuItem;
    ResidualsItem: TMenuItem;
    SaveArrangementItem: TMenuItem;
    SavePositionsItem: TMenuItem;
    FileOptionsItem: TMenuItem;
    SaveSettingsItem: TMenuItem;
    LoadPositionsItem: TMenuItem;
    LoadSettingsItem: TMenuItem;
    OtherItem: TMenuItem;
    StandardItem: TMenuItem;
    OnTopItem: TMenuItem;
    IndependentItem: TMenuItem;
    SaveOverlayItem: TMenuItem;
    SaveOverlayAsItem: TMenuItem;
    SpevViewfileItem: TMenuItem;
    JB95FileItem: TMenuItem;
    CopyTopPlotItem: TMenuItem;
    TopPlotItem: TMenuItem;
    SymmetricTopItem: TMenuItem;
    HitranFileItem: TMenuItem;
    STROTABFileItem: TMenuItem;
    PlotOptionsItem: TMenuItem;
    FontItem: TMenuItem;
    VariableCharSizeItem: TMenuItem;
    LimitLineSegmentsItem: TMenuItem;
    LmatrixImportITEM: TMenuItem;
    VibrationalSpectrumItem: TMenuItem;
    PromptOnImportItem: TMenuItem;
    OnImportItem: TMenuItem;
    DefaulttoArbitraryUnitsItem: TMenuItem;
    procedure I2ItemClick(Sender: TObject);
    procedure ArrangeFormsItemClick(Sender: TObject);
    procedure CalibrateItemClick(Sender: TObject);
    procedure ShowSumButtonClick(Sender: TObject);
    procedure SaveAsItemClick(Sender: TObject);
    procedure SaveItemClick(Sender: TObject);
    procedure DumpItemClick(Sender: TObject);
    procedure AboutItemClick(Sender: TObject);
    procedure UsingHelpItemClick(Sender: TObject);
    procedure WhatIsItemClick(Sender: TObject);
    procedure ProceduresItemClick(Sender: TObject);
    procedure SearchItemClick(Sender: TObject);
    procedure IndexItemClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure ClearObservedItemClick(Sender: TObject);
    procedure ClearCacheItemClick(Sender: TObject);
    procedure ClearLogItemClick(Sender: TObject);
    procedure TimingItemClick(Sender: TObject);
    procedure SpectrumItemClick(Sender: TObject);
    procedure LevelsItemClick(Sender: TObject);
    procedure TransitionsItemClick(Sender: TObject);
    procedure StatesItemClick(Sender: TObject);
    procedure LogItemClick(Sender: TObject);
    procedure LineListItemClick(Sender: TObject);
    procedure ConstantsItemClick(Sender: TObject);
    procedure ClearAllPeaksItemClick(Sender: TObject);
    procedure DeleteLastPeakItemClick(Sender: TObject);
    procedure OverlayPropertiesItemClick(Sender: TObject);
    procedure ClearOverlayItemClick(Sender: TObject);
    procedure InvertItemClick(Sender: TObject);
    procedure ClearSelectionItemClick(Sender: TObject);
    procedure SelectRegionItemClick(Sender: TObject);
    procedure WavenumbersItemClick(Sender: TObject);
    procedure ShowFortratItemClick(Sender: TObject);
    procedure ShowSumItemClick(Sender: TObject);
    procedure ShowPartsItemClick(Sender: TObject);
    procedure PasteAnotherItemClick(Sender: TObject);
    procedure UpperStateItemClick(Sender: TObject);
    procedure PasteOverlayItemClick(Sender: TObject);
    procedure StylesItemClick(Sender: TObject);
    procedure CopyPictureItemClick(Sender: TObject);
    procedure PrintSetupItemClick(Sender: TObject);
    procedure PrintItemClick(Sender: TObject);
    procedure ExitItemClick(Sender: TObject);
    procedure MRUFileListMRUItemClick(Sender: TObject; AFilename: string);
    procedure MRUFileListGetDisplayName(Sender: TObject; AFilename: string;
      var ADisplayName: string);
    procedure LoadOverlayItemClick(Sender: TObject);
    procedure ExportPlotItemClick(Sender: TObject);
    procedure ExportLineListItemClick(Sender: TObject);
    procedure BinaryfileItemClick(Sender: TObject);
    procedure CALPGMfileItemClick(Sender: TObject);
    procedure OldD3hItemClick(Sender: TObject);
    procedure OldPgopherItemClick(Sender: TObject);
    procedure OpenItemClick(Sender: TObject);
    procedure NewLinearMoleculeItemClick(Sender: TObject);
    procedure EmptyMixtureItemClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure AsymmetricTopItemClick(Sender: TObject);
    procedure ResidualsItemClick(Sender: TObject);
    procedure SaveArrangementItemClick(Sender: TObject);
    procedure IndependentItemClick(Sender: TObject);
    procedure SaveOverlayItemClick(Sender: TObject);
    procedure SaveOverlayAsItemClick(Sender: TObject);
    procedure SpevViewfileItemClick(Sender: TObject);
    procedure JB95FileItemClick(Sender: TObject);
    procedure CopyTopPlotItemClick(Sender: TObject);
    procedure TopPlotItemClick(Sender: TObject);
    procedure SymmetricTopItemClick(Sender: TObject);
    procedure HitranFileItemClick(Sender: TObject);
    procedure STROTABFileItemClick(Sender: TObject);
    procedure VariableCharSizeItemClick(Sender: TObject);
    procedure PlotOptionsItemClick(Sender: TObject);
    procedure FontItemClick(Sender: TObject);
    procedure LimitLineSegmentsItemClick(Sender: TObject);
    procedure LmatrixImportITEMClick(Sender: TObject);
    procedure VibrationalSpectrumItemClick(Sender: TObject);
  private
    FMixture: TMixture;
    MixtureName: string;
    procedure SetMixture(AMixture: TMixture);
    procedure ChangeSimulationSetting(AIndex: Integer; Sender: TObject);
    procedure UpdateToolbar;
    function LastPlot(WithPeak: Boolean = False):TPlot;
    {$IFDEF SHOWMEMUSAGE }
    procedure MemoryItemCLick(Sender:TObject);
    {$ENDIF}
    procedure nCPUsItemCLick(Sender:TObject);
    procedure SetName(AFileName: string);
    procedure HandleException(Sender: TObject; E: Exception);
    procedure AddThreadItems;
  protected
    procedure MNodeChange(Sender: TMNode; Change: TMNodeChange; Index: Integer); override;
  public
    Overlays: TOverlays;
    LastObsWasLineForm: Boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure ArrangeForm(Screen: TRect); override;

    { File Handling }
    procedure AddOverlay(FileName: string; Merge:Boolean; DefaultUnits: TResonanceUnits);
    procedure AddSampleMixture(SampleProc: TSampleProc);
    function ClearOverlays(Prompt: Boolean):Boolean;
    procedure ImportFile(DefaultProc: TMixtureReader; DefaultMerge: Boolean = False); overload;
    procedure ImportFile(DefaultProc: TMixtureReader; AFileName: string; Merge: Boolean); overload;
    procedure LoadAnyObservations;
    procedure LoadFile(AFileName: string; Merge: Boolean);
    procedure LoadGeneralFile(AFileName: string; Merge, PreferOverlay: Boolean; DefaultUnits: TResonanceUnits);
    function OKToChange(TheMixture: Boolean; TheOverlays: Boolean): Boolean;
    function MakePlotForm(ASimulation: TSimulation): TPlotForm;
    procedure SaveMixtureTo(AFileName: string);

    { Fitting }
    procedure ContourFit(JustResiduals: Boolean; AFileName: string; FitType: TFitType);
    procedure ShowContour(Spectra: TOverlays; ASimulation: TColumnVector);
    procedure ShowOverlaySum(Spectra: TOverlays; ASimulation: TColumnVector);
    procedure DoLineFit(ObservationsLoader: TObservationReaderProc; const Source: string; ObsType: TObsType; JustResiduals: Boolean = False; TakeWeight: TTakeWeight = tkObs);

    { Plotting }
    procedure ConsiderReplot(All: Boolean = False; PurgeLineSelection: Boolean = True);
    procedure DoPlot(All: Boolean = False; PurgeLineSelection: Boolean = True);
    procedure ShowSimulation; override;

    property Mixture: TMixture read FMixture write SetMixture;
  end;

var
  MainForm: TMainForm;

implementation

uses {$IFDEF LCL_} LResources, LCLType, LCLProc, HelpIntfs, LazHelpHTML, {$ENDIF}
     {$IFDEF USE_LAZCHMHELP} LazChmHelp, {$ENDIF} { Define to use lazarus .chm viewer; not reccomended at the moment as it does not handle greek letters such as &Pi; }
     {$IFDEF VCL_} PHtmlHelpViewer, {$ENDIF}
     {$IFDEF RecyclerMM} FRMMSnapShot, {$ENDIF}
     {$IFDEF SHOWMEMUSAGE} FastMMUsageTracker, {$ENDIF}
     {$IFDEF HELPHELP} MakeHelp, {$ENDIF}
     Math, Global, Hitran, ImportACDD, PgopherImport, D3hImport, ImportCALPGM,
     ImportJB95, ImportSTROTAB, ImportLmatrix, UCompat, GStyle,
     LineExportOpt, ExpPlotLoader, ExpPlot, ReadBinary, ChildForm, ExpCan,
     ExportG, ImportSpecView, Clipbrd, OverlaySumFitter, Constrain,
     LogUnit, Progress, Fitter, LineFrm, OvrlyFrm, ConstFrm, Worker,
     StateInfo, TranInfo, LevelFrm, Profile, ProfileForm, WhatsThis, WCompat, QuantumSpin,
     CalFrm, Atomic, CalSource, Sysinfo, PGWork, FixForm, ResidFrm, SettingsForm;

{$IFDEF VCL_}
{$R version.res}
{$ENDIF}

{$IFNDEF FPC}
{$R *.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

{ Helpers }

procedure TMainForm.SetMixture(AMixture: TMIxture);
var
  i: Integer;
  SaveSettingsInfo: TSaveSettingsInfo;
begin
  if AMixture <> Mixture then begin
    FreeAndNil(FMixture);
    if not (csDestroying in ComponentState) then
      Application.ProcessMessages; { Allow time for forms to disappear }
    ForAllSettingsForms({$IFDEF FPC}@{$ENDIF}ClearSettings,nil);
    FMixture := AMixture;
    if Assigned(AMixture) then begin
      FSimulation := Mixture.Simulations[0];
      FSimulation.Name := 'Simulation';
      if Mixture.nSimulations > 1 then
        for i := 1 to Mixture.nSimulations - 1 do
          MakePlotForm(Mixture.Simulations[i]);
      SetGlobalQuantumNumberFormat(Mixture.QuantumNumberFormat);
      with SaveSettingsInfo do begin
        DOMDocument := Mixture.LoadedFormSettings;
        SavePositions := LoadPositionsItem.Checked;
        SaveSettings := LoadSettingsItem.Checked;
      end;
      ForAllSettingsForms({$IFDEF FPC}@{$ENDIF}FinishLoading,TObject(@SaveSettingsInfo));
      Mixture.ClearFormSettings;
    end else
      FSimulation := nil;
    PerformMNodeNotification(Mixture, mnNewBaseNode, 0);
  end;
end;

procedure TMainForm.ChangeSimulationSetting(AIndex: Integer; Sender: TObject);
begin
  if Assigned(Simulation) then
    Simulation.SetFlag(AIndex, (Sender as TSpeedButton).Down);
  ShowSimulation;
end;

procedure TMainForm.UpdateToolbar;
var
  i: Integer;
  procedure UpdateEditControl(Edit: TCustomEdit; Value: Double);
  begin
    if not Edit.HandleAllocated or not Edit.Focused then begin
      Edit.Text := FloatToStr(Value); { TODO: Trim to displayable number of figures? }
      Edit.Modified := False;
    end;
  end;
begin
  if not Assigned(Mixture) then
    Exit; { Mixture being changed }
  with Mixture.Simulations[0] do begin
    UpdateEditControl(MainForm.Temperature, Temperature);
    UpdateEditControl(MainForm.Gaussian, Gaussian);
    UpdateEditControl(MainForm.Lorentzian, Lorentzian);
    UpdateEditControl(MainForm.Foffset, Foffset);
    ShowSumButton.Down := ShowSum;
    ShowPartsButton.Down := ShowParts;
    ShowFortratButton.Down := ShowFortrat;
    ShowSumItem.Checked := ShowSum;
    ShowPartsItem.Checked := ShowParts;
    ShowFortratItem.Checked := ShowFortrat;
    UpperStateItem.Checked := UseUpper;
    for i := 0 to UnitsItem.Count-1 do
      if UnitsItem.Items[i].Tag = Ord(PlotUnits) then
        UnitsItem.Items[i].Checked := True;
    CopyTopPlotItem.Enabled := ShowFortrat;
    TopPlotItem.Enabled := ShowFortrat;
  end;
end;

function TMainForm.LastPlot(WithPeak: Boolean): TPlot;
var
  i: Integer;
begin
  Result := nil;
  if not Assigned(Overlays) then Exit;
  for i := 0 to Overlays.Count - 1 do
    if Overlays[i] = FLastPlot then
      Break
    else if i = Overlays.Count - 1 then
      FLastPlot := nil;
  if Assigned(FLastPlot) and (not WithPeak or ( FLastPlot.nPeaks > 0 )) then begin
    Result := FLastPlot;
    Exit;
  end;
  for i := Overlays.Count - 1 downto 0 do
    if not WithPeak or ( Overlays[i].nPeaks > 0 ) then begin
      Result := Overlays[i];
      Exit;
    end;
end;

constructor TMainForm.Create(AOwner: TComponent);
{$IFDEF LCL_}
var
  HelpFileDir, lHelpExe: string;
  UseChmFile: Boolean;
{$ENDIF}
begin
  inherited;
  Tag := 0;
  FMixture := TMixture.Create(nil);
  FSimulation := Mixture.Simulations[0];
  UpdateToolbar;
  DefaultFormSettings := MakeFormSettings(nil);
  {$IFDEF LCL_}
  HelpFileDir := ExtractFilePath(ParamStr(0));
  lHelpExe := HelpFileDir+'lhelp';
  if DirectoryExists(HelpFileDir+'Help') then
    HelpFileDir := HelpFileDir+'Help'
  else if DirectoryExists(HelpFileDir + '..' + DirectorySeparator + 'Help') then
    HelpFileDir := HelpFileDir + '..' + DirectorySeparator + 'Help'
  else
    HelpFileDir := 'Help';
  HelpFileDir := ExpandFileName(HelpFileDir);
  if not FileExists(lHelpExe) then
    lHelpExe := HelpFileDir + DirectorySeparator + 'lhelp';
  UseChmFile :=
  {$IFDEF USE_LAZCHMHELP}
    FileExists(lHelpExe) and FileExists(HelpFileDir + DirectorySeparator + 'pgopher.chm');
  {$ELSE}
    False;
  {$ENDIF}
  with TSingleHTMLHelpDatabase.Create(Self) do begin
    AutoRegister := True;
    KeywordPrefix := 'Pgopher/';
    if not UseChmFile then
      DefaultBaseURL := 'file://'+HelpFileDir
    else
      DefaultBaseURL := 'pgopher.chm://';
  end;
  if not UseChmFile then
    with THTMLBrowserHelpViewer.Create(Self) do
      AutoRegister := True;
  {$IFDEF USE_LAZCHMHELP}
  if UseChmFile then
    with TChmHelpViewer.Create(Self) do begin
      AutoRegister := True;
      HelpEXE := lHelpExe;
      HelpLabel := 'pgopherxxx';
      HelpFilesPath := HelpFileDir + DirectorySeparator;
    end;
  {$ENDIF}
  Application.OnException := {$IFDEF FPC}@{$ENDIF}HandleException;
  {$IFNDEF OLDLAZARUS}
  Application.Flags := Application.Flags + [AppNoExceptionMessages];
  {$ENDIF}
  {$ENDIF}
  HasBeenArranged := True; { Take initial default }
  FixAllForms := False; { Only adjust the main form }
end;

procedure TMainForm.FormCreate(Sender: TObject);
{$IFDEF SHOWMEMUSAGE }
var
  MenuItem: TMenuItem;
  {$ENDIF}
begin
  inherited;
  {$IFDEF LCL}
  Application.OnHint := @StatusBarHint;
  {$ENDIF}
  {$IFDEF SHOWMEMUSAGE }
  MenuItem := TMenuItem.Create(Self);
  MenuItem.Caption := 'Memory';
  MenuItem.OnClick := MemoryItemCLick;
  ViewMenu.Insert(ViewMenu.IndexOf(TimingItem)+1, MenuItem);
  {$ENDIF}
  ToolBar.DisableAlign;
  OpenButton.Left := 0;
  ToolBar.EnableAlign;
  StuffCalibrationMenu(CalibrationItem,{$IFDEF FPC}@{$ENDIF}I2ItemClick);
  {$IFNDEF HELPHELP}
//  DumpItem.Visible := False;
  {$ENDIF}
  {$IFNDEF MSWINDOWS}
  WhatIsItem.Visible := False;
  {$ENDIF}
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  inherited;
  AddThreadItems; { Add here so that Checked item reflects number of processes }
end;

procedure TMainForm.FontItemClick(Sender: TObject);
begin
  G.ChooseFont(nil);
  OC.Font := G.Font;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := CanClose and OKToChange(True,True);
end;

function TMainForm.OKToChange(TheMixture, TheOverlays: Boolean): Boolean;
var
  s, e: string;
  procedure Check(MNode: TStreamableMNode; DefaultName: string);
  begin
    if Assigned(MNode) and MNode.Modified then begin
      Result := False;
      if s <> '' then begin
        s := s + ' and ';
        e := ' have';
      end;
      if MNode.Name = '' then
        s := s + DefaultName
      else
        s := s + MNode.Name;
    end;
  end;
begin
  Result := True;
  s := '';
  e := ' has';
  if TheMixture then
    Check(Mixture, 'Mixture');
  if TheOverlays then
    Check(Overlays, 'Overlay');
  if not Result then begin
    Result := MessageDlg(s+e+' changed: discard?', mtWarning, mbOkCancel, 0) = mrOK ;
    if Result then begin
      if TheMixture and Assigned(Mixture) then
        Mixture.Modified := False;
      if TheOverlays and Assigned(Overlays) then
        Overlays.Modified := False;
    end;
  end;
end;

destructor TMainForm.Destroy;
begin
  Application.OnException := nil;
  ClearOverlays(False);
  SetMixture(nil);
  inherited;
end;

procedure TMainForm.SetName(AFileName: string);
var
  i: Integer;
  NewCaption: string;
begin
  DoDirSeparators(AFileName);
  MixtureName := AFileName;
  OpenDialog.FileName := MixtureName;
  if AFileName = '' then
    NewCaption := 'PGOPHER'
  else begin
    MixtureName := ExpandFileName(AFileName);
    SetCurrentDir(ExtractFilePath(AFileName)); { TODO: This is not that useful, as default directory for windows open dialog is from most recent open/save dialog }
    NewCaption := ExtractFileName(AFileName) + ' - PGOPHER';
    MRUFileList.AddItem(MixtureName);
    MRUFileList.Save;
  end;
  for i := 0 to Screen.FormCount-1 do
    if Screen.Forms[i] is TPlotForm then
      if Screen.Forms[i] <> Self then
        TPlotForm(Screen.Forms[i]).Caption := IntToStr(Screen.Forms[i].Tag) + ' - ' + NewCaption
      else
        TPlotForm(Screen.Forms[i]).Caption := NewCaption;
  Application.Title := NewCaption;
end;

procedure TMainForm.HandleException(Sender: TObject; E: Exception);
begin
{$IFDEF LCL_}
  if IsExpectedException(E.ClassType) then
    Application.MessageBox(PChar(E.Message), 'Pgopher', MB_OK + MB_ICONERROR)
  else
    try
      {$IFNDEF OLDLAZARUS}
      Application.Flags := Application.Flags - [AppNoExceptionMessages];
      if ExceptObject is Exception then begin
        DebugLn('TApplication.HandleException ',Exception(ExceptObject).Message);
      end else begin
        DebugLn('TApplication.HandleException Strange Exception ');
      end;
      DumpExceptionBackTrace;
      {$ENDIF}
      Application.ShowException(E);
    finally
      {$IFNDEF OLDLAZARUS}
      Application.Flags := Application.Flags + [AppNoExceptionMessages];
      {$ENDIF}
    end;
{$ENDIF}
end;

{ Plotting }

procedure TMainForm.ShowSimulation;
var
  i: Integer;
begin
  ShowSimulation(Mixture, Overlays);
  { Update other screens - should use same linelist }
  for i := 0 to Screen.FormCount-1 do
    if (Screen.Forms[i] <> Self) and (Screen.Forms[i] is TPlotForm) then
      TPlotForm(Screen.Forms[i]).ShowSimulation(Mixture, Overlays);
end;

procedure TMainForm.DoPlot(All, PurgeLineSelection: Boolean);
var
  i: Integer;
begin
  if PurgeLineSelection and Assigned(Mixture.LineList) and Mixture.LineList.Selected then
    Mixture.LineList := nil;
  if All then begin
    if AllWanted then
      FindLimits;
    for i := 0 to Screen.FormCount-1 do
      if Screen.Forms[i] is TPlotForm then
        with TPlotForm(Screen.Forms[i]) do
          if AllWanted then
            FindLimits;
  end;
  AllWanted := False;
  ShowSimulation;
end;

procedure TMainForm.ConsiderReplot(All, PurgeLineSelection: Boolean);
begin
  if not Assigned(Mixture) or Mixture.AutoReplot then begin
    ForAllSettingsForms({$IFDEF FPC}@{$ENDIF}DoUpdateActions,nil); { Force update of Child Windows }
    AllWanted := True;
    DoPlot(All,PurgeLineSelection);
  end;
end;

{ File Handling }

procedure TMainForm.AddOverlay(FileName: string; Merge: Boolean; DefaultUnits: TResonanceUnits);
var
  DefaultIntensityUnits: TIntensityUnits;
begin
  if not Merge and not OKToChange(False,True) then
    Abort;
  try
    BeginUpdate;
    if DefaulttoArbitraryUnitsItem.Checked then
      DefaultIntensityUnits := Arbitrary
    else
      DefaultIntensityUnits := Normalized;
    AddToOverlay(Overlays, FileName, Merge, DefaultUnits, PromptonImportItem.Checked, DefaultIntensityUnits);
  finally
    EndUpdate(True);
  end;
  if FileName = '' then
    Exit { Skip }
  else if Filename[Length(FileName)] = '>' then
    MRUFileList.AddItem(FileName)
  else
    MRUFileList.AddItem(ExpandFileName(FileName));
  MRUFileList.Save;
end;

function TMainForm.ClearOverlays(Prompt: Boolean): Boolean;
begin
  if Prompt and not OKToChange(False,True) then begin
    Result := False;
    Exit;
  end;
  Result := True;
  BeginUpdate;
  try
    FreeAndNil(Overlays);
  finally
    EndUpdate(True);
  end;
end;

procedure TMainForm.ImportFile(DefaultProc: TMixtureReader; DefaultMerge: Boolean);
begin
  if CloseQuery and ImportDialog.Execute then
    ImportFile(DefaultProc, ImportDialog.FileName, DefaultMerge or MergeItem.Checked);
end;

procedure TMainForm.ImportFile(DefaultProc: TMixtureReader; AFileName: string; Merge: Boolean);
var
  Stream: TFreeFormatStream;
  FStream: TFileStream;
  Line: string;
  NewMixture: TMixture;
  MemoryStream: TMemoryStream;
begin
  SetCurrentDir(ExtractFileDir(AFileName));
  FStream := TFileStream.Create(AFileName, fmOpenRead or fmShareDenyWrite);
  Stream := nil;
  try
    Stream := TFreeFormatStream.Create(FStream);
    Line := LowerCase(Stream.RestOfLine);
    Stream.Position := 0;
    if Pos('this is a d3h ', Line) > 0 then
      DefaultProc := {$IFDEF FPC}@{$ENDIF}ImportD3hFile
    else if Pos(' pgopher ', Line) > 0 then
      DefaultProc := {$IFDEF FPC}@{$ENDIF}ImportPgopherFile
    else if Pos('idn', Line) > 0 then
      DefaultProc := {$IFDEF FPC}@{$ENDIF}ImportACDDFile;
    NewMixture := DefaultProc(Stream);
    if Merge then
      try
        MemoryStream := TMemoryStream.Create;
        try
          BeginUpdate;
          NewMixture.SaveToStream(MemoryStream);
          MemoryStream.Position := 0;
          Mixture.MergeFromStream(MemoryStream);
        finally
          EndUpdate(True);
          MemoryStream.Free;
        end;
      except
        NewMixture.Free;
      end
    else begin
      SetMixture(NewMixture);
      SetName('');
      Mixture.Comment := 'Imported from ' + AFileName;
      Mixture.CheckProperties;
      Mixture.Modified := False;
    end;
  finally
    Stream.Free;
    FStream.Free;
  end;
end;

procedure TMainForm.I2ItemClick(Sender: TObject);
var
  MenuItem: TMenuItem;
  s: string;
  P: Integer;
begin
  MenuItem := Sender as TMenuItem;
  s := MenuItem.Caption;
  P := Pos('&', s);
  if P > 0 then
    Delete(s,P,1);
  AddOverlay('<' + s + ' Calibration>',True,cm1);
end;

procedure TMainForm.LoadAnyObservations;
begin
  if LastObsWasLineForm and LineForm.HasObservations then
    LineForm.LoadObservations('',obLine,tkObs)
  else
    LogForm.LoadObservations;
end;

procedure TMainForm.LoadFile(AFileName: string; Merge: Boolean);
var
  NewMixture: TMixture;
  Stream: TFileStream;
begin
  if Merge and Assigned(Mixture) and (Mixture.nMolecules > 0) then begin
    Stream := TFileStream.Create(AFileName, fmOpenRead or fmShareDenyWrite);
    try
      BeginUpdate;
      Mixture.MergeFromStream(Stream, AFileName);
      SetName('');
    finally
      EndUpdate(True);
      Stream.Free;
    end;
  end else begin
    NewMixture := TMixture.Create(nil);
    try
      NewMixture.LoadFromFile(AFilename);
      SetMixture(NewMixture);
      SetName(AFileName);
    except
      NewMixture.Free;
      raise;
    end;
  end;
  ClearObservations;
end;

procedure TMainForm.LoadGeneralFile(AFileName: string; Merge, PreferOverlay: Boolean; DefaultUnits: TResonanceUnits);
var
  Ext: string;
begin
  Ext := LowerCase(ExtractFileExt(AFileName));
  case FileType(AFileName, PreferOverlay) of
    fiParameters:
      if Merge or OKToChange(True,False) then
        if (Ext = '.pgo') or (Ext = '.xml') then
          LoadFile(AFileName,Merge)
        else if Ext = '.svw' then
          ImportFile({$IFDEF FPC}@{$ENDIF}ImportSpecViewFile, AFileName, Merge)
        else if Ext = '.jbw' then begin
          ImportFile({$IFDEF FPC}@{$ENDIF}ImportJB95File, AFileName, Merge);
          AddOverlay(AFileName, Merge, DefaultUnits); { JBW files can also contain an overlay }
        end else if Ext = '.sta' then
          ImportFile({$IFDEF FPC}@{$ENDIF}ImportSTROTABFile, AFileName, Merge)
        else
          ImportFile({$IFDEF FPC}@{$ENDIF}ImportCALPGMFile, AFileName, Merge);
    fiObservations: LogForm.ObsFileName.Text := AFileName;
    fiOverlay: AddOverlay(AFileName, Merge, DefaultUnits);
  end;
end;

procedure TMainForm.AddThreadItems;
var
  MenuItem: TMenuItem;
begin
  while ThreadsItem.Count < Max(Max(4, 4*nCPUs),nWorkers) do begin
    MenuItem := TMenuItem.Create(Self);
    if ThreadsItem.Count = 0 then
      MenuItem.Caption := 'Single Process'
    else
      MenuItem.Caption := Format('%d Processes', [ThreadsItem.Count+1]);
    MenuItem.Tag := ThreadsItem.Count+1;
    MenuItem.OnClick := {$IFDEF FPC}@{$ENDIF}nCPUsItemCLick;
    MenuItem.AutoCheck := True;
    MenuItem.RadioItem := True;
    MenuItem.GroupIndex := 45;
    ThreadsItem.Insert(ThreadsItem.Count,MenuItem);
  end;
  ThreadsItem.Items[Max(nWorkers,1)-1].Checked := True;
end;

procedure TMainForm.ArrangeForm(Screen: TRect);
var
  R: TRect;
begin
  inherited;
  R := ScreenBounds(Self);
  OffsetRect(R,Screen.Left-R.Left,Screen.Top-R.Top);
  SetScreenBounds(Self,R);
end;

procedure TMainForm.ArrangeFormsItemClick(Sender: TObject);
var
  i: Integer;
  Rect: TRect;
begin
  Rect := ScreenAreaFor(Application.MainForm);
  ArrangeForm(Rect);
  for i := 0 to Screen.FormCount-1 do
    if Screen.Forms[i] is THelpForm then
      with THelpForm(Screen.Forms[i]) do
       if Visible then
         ArrangeForm(Rect)
       else
         HasBeenArranged := False;
end;

{ Fitting }

procedure TMainForm.ContourFit(JustResiduals: Boolean; AFileName: string; FitType: TFitType);
var
  i: Integer;
begin
  if not Assigned(Overlays) then
    raise EReportException.Create('Overlays required');
  if BeginCalculation then Exit;
  PushProgress('Fit', Mixture.FitCycles);
  try
    if Overlays.ContourFile then begin
      PushProgress('Loading');
      try
        LoadObservations(Mixture,nil,AFileName,obContour);
      finally
        PopProgress;
      end;
    end else
      ClearObservations;
    for i := 1 to Mixture.FitCycles do begin
      SetProgress(i);
      Mixture.PrepareForFit(FitType, Overlays);
      if Mixture.PrintLevel > plMinimal then
        Mixture.PrettyPrint(Worker.Log, Mixture.PrintLevel);
      try
        if FitType = ftContour then
          FitContour(Mixture, Overlays, {$IFDEF FPC}@{$ENDIF}ShowContour, Mixture.FittingParameters, JustResiduals)
        else begin
          FitOverlaySum(Mixture, Overlays, {$IFDEF FPC}@{$ENDIF}ShowOverlaySum, Mixture.FittingParameters, JustResiduals);
          OverlayForm.ShowCurrentPlot;
        end;
        Mixture.PushParameters(FitType, True); { Save current set for redo fit }
      except
        Mixture.DiscardFit;
        raise;
      end;
    end;
    PopProgress;
  finally
    EndCalculation;
    PopProgress;
  end;
end;

procedure TMainForm.ShowContour(Spectra: TOverlays; ASimulation: TColumnVector);
var
  i, j, l: Integer;
  v, f: Double;
  dmin, dmax: Double; { Limits of obs-calc }
  omin, omax: Double;
  XLabel: string;
begin
  SetLength(FortratPlotRanges,0);
  SetLength(SpectrumPlotRanges,0);
  Split.Visible := True;
  Split.MakeBothVisible;
  { Determine Limits on obs-calc, obs }
  l := 1;
  for i := 0 to Spectra.nPlots-1 do
   if Spectra[i] is TExperimentalPlot then
    with TExperimentalPlot(Spectra[i]) do
      if not Hide then
        for j := StartPoint to EndPoint do begin
          // WAS: v := (Intensity[j]-Baseline)*Scale;
          v := Intensity[j];
          if l = 1 then begin
            omin := v;
            omax := v;
          end else begin
            if v < omin then omin := v;
            if v > omax then omax := v;
          end;
          v := v - ASimulation[l];
          if l = 1 then begin
            dmin := v;
            dmax := v;
          end else begin
            if v < dmin then dmin := v;
            if v > dmax then dmax := v;
          end;
          Inc(l);
        end;
  if l = 1 then Exit; { No points }
  { Plot obs - calc }
  OC.Start;
  OC.Map(Mixture.Simulations[0].Fmin, Mixture.Simulations[0].Fmax, dmin, dmax);
  OC.Axes('', 'Obs-Calc');
  l := 1;
  for i := 0 to Spectra.nPlots-1 do
   if Spectra[i] is TExperimentalPlot then
    with TExperimentalPlot(Spectra[i]) do
      if not Hide then
        for j := StartPoint to EndPoint do begin
          f := Convert(AdjustFrequency(Frequency[j]),Units,Mixture.Simulations[0].PlotUnits);
          // WAS: v := (Intensity[j]-Baseline)*Scale - Simulation[l];
          v := Intensity[j] - ASimulation[l];
          if l = 1 then
            OC.ClipPoint(f, v)
          else
            OC.ClipJoin(f, v);
          Inc(l);
        end;
  OC.Finish;
  G.Start;
  G.Map(Mixture.Simulations[0].Fmin, Mixture.Simulations[0].Fmax, omin, omax);
  XLabel := Mixture.UnitsTypeName + '/' + NameOfUnit(Mixture.Simulations[0].PlotUnits);
  G.Axis(Mixture.Simulations[0].Fmin, Mixture.Simulations[0].Fmax, omin, true, XLabel);
  G.Axis(G.Ymin,G.Ymax,G.Xmin,False,'');
  l := 1;
  for i := 0 to Spectra.nPlots-1 do
   if Spectra[i] is TExperimentalPlot then
    with TExperimentalPlot(Spectra[i]) do
      if not Hide then
        for j := StartPoint to EndPoint do begin
          f := Convert(AdjustFrequency(Frequency[j]),Units,Mixture.Simulations[0].PlotUnits);
          v := ASimulation[l];
          if l = 1 then
            G.ClipPoint(f, v)
          else
            G.ClipJoin(f, v);
          Inc(l);
        end;
  G.Style(1);
  l := 1;
  for i := 0 to Spectra.nPlots-1 do
   if Spectra[i] is TExperimentalPlot then
    with TExperimentalPlot(Spectra[i]) do
      if not Hide then
        for j := StartPoint to EndPoint do begin
          f := Convert(AdjustFrequency(Frequency[j]),Units,Mixture.Simulations[0].PlotUnits);
          // WAS: v := (Intensity[j]-Baseline)*Scale;
          v := Intensity[j];
          if l = 1 then
            G.ClipPoint(f, v)
          else
            G.ClipJoin(f, v);
          Inc(l);
        end;
  G.Finish;
end;

procedure TMainForm.ShowOverlaySum(Spectra: TOverlays; ASimulation: TColumnVector);
var
  i, j, l: Integer;
  v, f: Double;
  dmin, dmax: Double; { Limits of obs-calc }
  omin, omax: Double;
  XLabel: string;
begin
  SetLength(FortratPlotRanges,0);
  SetLength(SpectrumPlotRanges,0);
  Split.Visible := True;
  Split.MakeBothVisible;
  { Determine Limits on obs-calc, obs }
  l := 1;
  for i := 0 to Spectra.nPlots-1 do
   if Spectra[i] is TExperimentalPlot then
    with TExperimentalPlot(Spectra[i]) do
      if ComponentType = Target then
        for j := StartPoint to EndPoint do begin
          { TODO: should probably select frequency range here also }
          v := Intensity[j];
          if l = 1 then begin
            omin := v;
            omax := v;
          end else begin
            if v < omin then omin := v;
            if v > omax then omax := v;
          end;
          v := v - ASimulation[l];
          if l = 1 then begin
            dmin := v;
            dmax := v;
          end else begin
            if v < dmin then dmin := v;
            if v > dmax then dmax := v;
          end;
          Inc(l);
        end;
  if l = 1 then Exit; { No points }
  { Plot obs - calc }
  OC.Start;
  OC.Map(Mixture.Simulations[0].Fmin, Mixture.Simulations[0].Fmax, dmin, dmax);
  OC.Axes('', 'Obs-Calc');
  l := 1;
  for i := 0 to Spectra.nPlots-1 do
   if Spectra[i] is TExperimentalPlot then
    with TExperimentalPlot(Spectra[i]) do
      if ComponentType = Target then begin
        if Colour <> clNone then
          OC.StyleP(Colour, OC.LineStyle.Width, lpSolid)
        else
          OC.Style(1);
        for j := StartPoint to EndPoint do begin
          f := Convert(AdjustFrequency(Frequency[j]),Units,Mixture.Simulations[0].PlotUnits);
          v := Intensity[j] - ASimulation[l];
          if j = StartPoint then
            OC.ClipPoint(f, v)
          else
            OC.ClipJoin(f, v);
          Inc(l);
        end;
      end;
  OC.Finish;
  G.Start;
  G.Map(Mixture.Simulations[0].Fmin, Mixture.Simulations[0].Fmax, omin, omax);
  XLabel := Mixture.UnitsTypeName + '/' + NameOfUnit(Mixture.Simulations[0].PlotUnits);
  G.Axis(Mixture.Simulations[0].Fmin, Mixture.Simulations[0].Fmax, omin, true, XLabel);
  G.Axis(G.Ymin,G.Ymax,G.Xmin,False,'');
  l := 1;
  for i := 0 to Spectra.nPlots-1 do
   if Spectra[i] is TExperimentalPlot then
    with TExperimentalPlot(Spectra[i]) do
      if ComponentType = Target then begin
        if Colour <> clNone then
          G.StyleP(Colour, G.LineStyle.Width, lpDot)
        else
          G.Style(1);
        for j := StartPoint to EndPoint do begin
          f := Convert(AdjustFrequency(Frequency[j]),Units,Mixture.Simulations[0].PlotUnits);
          v := ASimulation[l];
          if j = StartPoint then
            G.ClipPoint(f, v)
          else
            G.ClipJoin(f, v);
          Inc(l);
        end;
      end;
  G.Style(1);
  for i := 0 to Spectra.nPlots-1 do
   if Spectra[i] is TExperimentalPlot then
    with TExperimentalPlot(Spectra[i]) do
      if ComponentType = Target then begin
        if Colour <> clNone then
          G.StyleP(Colour, G.LineStyle.Width, lpSolid)
        else
          G.Style(1);
        for j := StartPoint to EndPoint do begin
          f := Convert(AdjustFrequency(Frequency[j]),Units,Mixture.Simulations[0].PlotUnits);
          v := Intensity[j];
          if j = StartPoint then
            G.ClipPoint(f, v)
          else
            G.ClipJoin(f, v);
        end;
      end;
  G.Finish;
end;

procedure TMainForm.DoLineFit(ObservationsLoader: TObservationReaderProc;
  const Source: string; ObsType: TObsType; JustResiduals: Boolean;
  TakeWeight: TTakeWeight);
var
  i, Prec: Integer;
  FitStatus: string;
begin
  Mixture.CheckProperties;
  if BeginCalculation then Exit;
  PushProgress('Fitting');
  try
    PushProgress('Loading');
    try
      ObservationsLoader(Source, ObsType, TakeWeight);
    finally
      PopProgress;
    end;
    if not JustResiduals then
      for i := 1 to Mixture.FitCycles do begin
        Mixture.PrepareForFit(ftLine,nil);
        if Mixture.PrintLevel > plMinimal then
          Mixture.PrettyPrint(Worker.Log, Mixture.PrintLevel);
        try
          SetProgress(i, Mixture.FitCycles);
          if ObsType = obIntensity then
            Prec := -1
          else
            Prec := Mixture.Precision;
          FitStatus := FitLines(Mixture.FittingParameters, Observations, Prec, Mixture);
          Mixture.PushParameters(ftLine, True); { Save current set for redo fit }
        except
          Mixture.DiscardFit;
          raise;
        end;
      end
    else
      FitStatus := '';
    ResidualForm.PlotResiduals(FitStatus);
  finally
    EndCalculation;
    PopProgress;
  end;
  ConsiderReplot;
end;

{ Menu Handling }

procedure TMainForm.AddSampleMixture(SampleProc: TSampleProc);
var
  Mix: TMixture;
begin
  if OKToChange(True,False) then begin
    Mix := TMixture.Create(nil);
    if Assigned(SampleProc) then
      SampleProc(Mix);
    SetMixture(Mix);
    SetName('');
    if Assigned(SampleProc) then begin
      DoPlot;
      if Mix.Simulations[0].Fmin = 0 then
        AllButtonClick(nil);
    end;
    Mixture.Modified := False;
  end;
end;

procedure TMainForm.EmptyMixtureItemClick(Sender: TObject);
begin
  AddSampleMixture(nil);
end;

procedure TMainForm.NewLinearMoleculeItemClick(Sender: TObject);
begin
  AddSampleMixture({$IFDEF FPC}@{$ENDIF}AddSampleLinearMolecule);
end;

procedure TMainForm.AsymmetricTopItemClick(Sender: TObject);
begin
  AddSampleMixture({$IFDEF FPC}@{$ENDIF}AddSampleAsymmetricTop);
end;

procedure TMainForm.SymmetricTopItemClick(Sender: TObject);
begin
  AddSampleMixture({$IFDEF FPC}@{$ENDIF}AddSampleSymmetricTop);
end;

procedure TMainForm.VibrationalSpectrumItemClick(Sender: TObject);
begin
  AddSampleMixture({$IFDEF FPC}@{$ENDIF}AddSampleVibratingMolecule);
end;

procedure TMainForm.OpenItemClick(Sender: TObject);
var
  i: Integer;
  Files: array of string;
begin
  if CloseQuery and OpenDialog.Execute then begin
    SetLength(Files, OpenDialog.Files.Count);
    for i := 0 to OpenDialog.Files.Count - 1 do
      Files[i] := OpenDialog.Files[i];
    FormDropFiles(nil, Files);
  end;
end;

procedure TMainForm.OldPgopherItemClick(Sender: TObject);
begin
  ImportDialog.DefaultExt := '.dat';
  ImportDialog.FilterIndex := 1;
  ImportFile({$IFDEF FPC}@{$ENDIF}ImportPgopherFile);
end;

procedure TMainForm.OldD3hItemClick(Sender: TObject);
begin
  ImportDialog.DefaultExt := '.dat';
  ImportDialog.FilterIndex := 1;
  ImportFile({$IFDEF FPC}@{$ENDIF}ImportD3hFile);
end;

procedure TMainForm.CALPGMfileItemClick(Sender: TObject);
begin
  ImportDialog.DefaultExt := '.par';
  ImportDialog.FilterIndex := 2;
  ImportDialog.Filter := StringReplace(ImportDialog.Filter, 'HITRAN', 'CALPGM', []);
  ImportFile({$IFDEF FPC}@{$ENDIF}ImportCALPGMFile);
end;

procedure TMainForm.HitranFileItemClick(Sender: TObject);
begin
  ImportDialog.DefaultExt := '.par';
  ImportDialog.FilterIndex := 2;
  ImportDialog.Filter := StringReplace(ImportDialog.Filter, 'CALPGM', 'HITRAN', []);
  ImportFile({$IFDEF FPC}@{$ENDIF}ImportHITRANFile);
end;

procedure TMainForm.SpevViewfileItemClick(Sender: TObject);
begin
  ImportDialog.DefaultExt := '.svw';
  ImportDialog.FilterIndex := 3;
  ImportFile({$IFDEF FPC}@{$ENDIF}ImportSpecViewFile);
end;

procedure TMainForm.JB95FileItemClick(Sender: TObject);
begin
  ImportDialog.DefaultExt := '.jbw';
  ImportDialog.FilterIndex := 4;
  ImportFile({$IFDEF FPC}@{$ENDIF}ImportJB95File);
end;

procedure TMainForm.STROTABFileItemClick(Sender: TObject);
begin
  ImportDialog.DefaultExt := '.sta';
  ImportDialog.FilterIndex := 5;
  ImportFile({$IFDEF FPC}@{$ENDIF}ImportSTROTABFile);
end;

procedure TMainForm.LmatrixImportITEMClick(Sender: TObject);
begin
  ImportDialog.DefaultExt := '.out';
  ImportDialog.FilterIndex := 6;
  ImportFile({$IFDEF FPC}@{$ENDIF}ImportLmatrixFile);
end;

procedure TMainForm.BinaryfileItemClick(Sender: TObject);
var
  ExperimenalPlot: TExperimentalPlot;
  Stream: TStream;
  BinaryFileLoader: TBinaryFileLoader;
  DefaultIntensityUnits: TIntensityUnits;
begin
  if not MergeItem.Checked then begin
    if not ClearOverlays(False) then
      Exit;
    Overlays := TOverlays.Create(nil);
    PerformMNodeNotification(Overlays, mnNewBaseNode, 0);
  end;
  ImportDialog.DefaultExt := '.dat';
  ImportDialog.FilterIndex := 3;
  if ImportDialog.Execute then begin
    Stream := TFileStream.Create(ImportDialog.FileName, fmOpenRead or fmShareDenyWrite);
    try
      ExperimenalPlot := TExperimentalPlot.Create(Overlays);
      try
        ExperimenalPlot.Name := ExtractFileName(ImportDialog.FileName);
        ExperimenalPlot.Comment := 'Imported from ' + ImportDialog.FileName;
        if DefaulttoArbitraryUnitsItem.Checked then
          DefaultIntensityUnits := Arbitrary
        else
          DefaultIntensityUnits := Normalized;
        BeginUpdate;
        BinaryFileLoader := TBinaryFileLoader.Create(TPGOPHERPlotLoaderConnector.Create(ExperimenalPlot,ImportDialog.FileName),Stream,'',Simulation.PlotUnits,DefaultIntensityUnits);
        try
          if not BinaryFileLoader.Load(Worker.Log, False) then
            FreeAndNil(ExperimenalPlot);
        finally
          BinaryFileLoader.Free;
          EndUpdate(True);
        end;
      except
        ExperimenalPlot.Free;
      end;
    finally
      Stream.Free;
    end;
  end;
end;

procedure TMainForm.ExportLineListItemClick(Sender: TObject);
var
  F: TextFile;
  i: Integer;
  Threshold: Double;
  var Problem: Exception;
  ASimulation: TSimulation;
  PrintLevel: TPrintLevel;
  Ext: string;
begin
  if not Assigned(Mixture) then
    Exit;
  Problem := nil;
  ASimulation := Mixture.Simulations[0];
  if not Assigned(Mixture.LineList) then
    ASimulation.MakeLineList(Problem);
  if not Assigned(Mixture.LineList) then
    Exit;
  if Mixture.Name <> '' then
    ExportLineListDialog.FileName := ChangeFileExt(Mixture.Name, '');
  if ExportLineListDialog.Execute and (LineExportOptions.ShowModal = mrOK) then begin
    AssignFile(F, ExportLineListDialog.FileName);
    Ext := LowerCase(ExtractFileExt(ExportLineListDialog.FileName));
    if Ext = '.csv' then
      PrintLevel := plCSV
    else if Ext = '.txt' then
      PrintLevel := plText
    else if Ext = '.bin' then
      PrintLevel := plBinary
    else
      PrintLevel := plLineList;
    try
      ReWrite(F);
      if (PrintLevel = plLineList) and (Mixture.PrintLevel > plMinimal) then
        Mixture.PrettyPrint(F,Mixture.PrintLevel);
      if Assigned(Problem) then
        WriteLn(F,Problem.Message);
      with Mixture.LineList do begin
        Threshold := LargestIntensity*LineExportOptions.ThresholdSpin.Value;
        HeadingPrinted := False;
        for i := 0 to nLines-1 do
          ShowLine(i,F,PrintLevel,Threshold,LineExportOptions.FitFormatCheck.Checked,Problem);
        ShowFooter(F,PrintLevel);
      end;
    finally
      CloseFile(F);
    end;
  end;
  Problem.Free;
end;

procedure TMainForm.ExportPlotItemClick(Sender: TObject);
begin
  G.ExecuteExport;
end;

procedure TMainForm.TopPlotItemClick(Sender: TObject);
begin
  OC.ExecuteExport;
end;

procedure TMainForm.LoadOverlayItemClick(Sender: TObject);
var
  i: Integer;
begin
  OpenOverlayDialog.Filter :=
    'Data File (*.dat)|*.dat|Overlay File (*.ovr)|*.ovr|'
    +PlotLoaderFilter
    +GraphicFilter(TGraphic)
    +'|All Files (*.*)|'+AllFilesMask;
  if OpenOverlayDialog.Execute then begin
    for i := 0 to OpenOverlayDialog.Files.Count-1 do
      AddOverlay(OpenOverlayDialog.Files[i], MergeItem.Checked or (i > 0), Simulation.PlotUnits);
    ConsiderReplot(Mixture.nSpecies = 0);
  end;
end;

procedure TMainForm.SaveItemClick(Sender: TObject);
begin
  if MixtureName = '' then
    SaveAsItemClick(nil)
  else
    SaveMixtureTo(MixtureName);
end;

procedure TMainForm.SaveMixtureTo(AFileName: string);
var
  SaveSetingsInfo: TSaveSettingsInfo;
begin
  Mixture.ClearFormSettings;
  if SavePositionsItem.Checked or SaveSettingsItem.Checked then begin
    with SaveSetingsInfo do begin
      DOMDocument := Mixture.LoadedFormSettings;
      SavePositions := SavePositionsItem.Checked;
      SaveSettings := SaveSettingsItem.Checked;
    end;
    ForAllSettingsForms({$IFDEF FPC}@{$ENDIF}SaveSettings, TObject(@SaveSetingsInfo));
  end;
  Mixture.SaveToFile(AFileName);
  Mixture.ClearFormSettings;
end;

procedure TMainForm.SaveOverlayAsItemClick(Sender: TObject);
begin
  OverlayForm.SaveAsButtonClick(Overlays);
end;

procedure TMainForm.SaveOverlayItemClick(Sender: TObject);
begin
  OverlayForm.SaveButtonClick(Overlays);
end;

procedure TMainForm.SaveArrangementItemClick(Sender: TObject);
begin
  ChildForm.BeginUpdate;
  ForAllSettingsForms({$IFDEF FPC}@{$ENDIF}ShowInMixture, nil);
  ChildForm.EndUpdate;
end;

procedure TMainForm.SaveAsItemClick(Sender: TObject);
begin
  if not Assigned(Mixture) then Exit;
  SaveDialog.FileName := MixtureName;
  if SaveDialog.Execute then begin
    if ExtractFileExt(SaveDialog.FileName) = '.ovr' then
      raise EReportException.Create('.ovr files are used to save overlaid spectral data; please use Save Overlay menu item for these');
    SaveMixtureTo(SaveDialog.FileName);
    SetName(SaveDialog.FileName);
  end;
end;

procedure TMainForm.PrintItemClick(Sender: TObject);
begin
  GraphPrintPreview.Show;
end;

procedure TMainForm.PrintSetupItemClick(Sender: TObject);
begin
  GraphPrintPreview.PrinterSetup;
end;

procedure TMainForm.MRUFileListGetDisplayName(Sender: TObject;
  AFilename: string; var ADisplayName: string);
var
  p: Integer;
begin
  if ADisplayName[Length(ADisplayName)] = '>' then
    Exit; { Calibration or clipboard item }
  p := Pos(' ', ADisplayName);
  if p > 0 then
    ADisplayName := Copy(ADisplayName, 1, p)
  else
    ADisplayName := '';
  ADisplayName := ADisplayName + ExtractRelativepath(GetCurrentDir+PathDelim, AFilename);
end;

procedure TMainForm.MRUFileListMRUItemClick(Sender: TObject; AFilename: string);
begin
  if LowerCase(ExtractFileExt(AFilename)) = '.pgo' then begin
    if OKToChange(True,False) then
      LoadFile(AFilename,MergeItem.Checked);
  end else
    LoadGeneralFile(AFileName, MergeItem.Checked, False, Simulation.PlotUnits);
  ConsiderReplot(Mixture.nSpecies = 0);
end;

procedure TMainForm.nCPUsItemCLick(Sender: TObject);
begin
  StartWorkers((Sender as TComponent).Tag,TPgopherWorker);
end;

procedure TMainForm.ExitItemClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.CopyPictureItemClick(Sender: TObject);
begin
  G.CopyToClipboard;
end;

procedure TMainForm.CopyTopPlotItemClick(Sender: TObject);
begin
  OC.CopyToClipboard;
end;

procedure TMainForm.StylesItemClick(Sender: TObject);
begin
  GraphPenDialog.Graph := LastGraph;
  GraphPenDialog.Execute;
end;

procedure TMainForm.ShowPartsItemClick(Sender: TObject);
begin
  ShowPartsButton.Down := not ShowPartsButton.Down;
  ChangeSimulationSetting(ShowPartsIndex, ShowPartsButton);
end;

procedure TMainForm.ShowSumItemClick(Sender: TObject);
begin
  ShowSumButton.Down := not ShowSumButton.Down;
  ChangeSimulationSetting(ShowSumIndex, ShowSumButton);
end;

procedure TMainForm.ShowFortratItemClick(Sender: TObject);
begin
  ShowFortratButton.Down := not ShowFortratButton.Down;
  ChangeSimulationSetting(ShowFortratIndex, ShowFortratButton);
end;

procedure TMainForm.UpperStateItemClick(Sender: TObject);
begin
  Mixture.Simulations[0].UseUpper := not Mixture.Simulations[0].UseUpper;
  ConsiderReplot;
end;

procedure TMainForm.WavenumbersItemClick(Sender: TObject);
begin
  if not Assigned(Mixture) then
    Exit;
  Mixture.Simulations[0].ChangePlotUnits(TResonanceUnits((Sender as TComponent).Tag));
  UpdateToolbar;
  ConsiderReplot;
end;

procedure TMainForm.PasteOverlayItemClick(Sender: TObject);
begin
  if not MergeItem.Checked then
    if not ClearOverlays(True) then
      Exit;
  PasteAnotherItemClick(Sender);
end;

procedure TMainForm.PlotOptionsItemClick(Sender: TObject);
begin
  VariableCharSizeItem.Checked := not (grFixedCharSize in G.GOptions);
  LimitLineSegmentsItem.Checked := not (grUnlimitedSegmentLength in G.GOptions);
end;

procedure TMainForm.PasteAnotherItemClick(Sender: TObject);
var
{$IFDEF MSWINDOWS}
  Files: TStringArray;
{$ELSE}
  Lines: TStringList;
  s: string;
{$ENDIF}
  i: Integer;
  DefaultUnits: TResonanceUnits;
begin
  if not (Sender is TPlotForm) then
    Sender := Self;
  if Assigned(Sender) then
    DefaultUnits := (Sender as TPlotForm).Simulation.PlotUnits
  else
    DefaultUnits := cm1;
  {$IFDEF MSWINDOWS}
  if Clipboard.HasFormat(CF_HDROP) then begin
    Clipboard.Open;
    try
      Files := ConvertHDROP(Clipboard.GetAsHandle(CF_HDROP));
    finally
      Clipboard.Close;
    end;
    for i := 0 to High(Files) do
      LoadGeneralFile(Files[i],True,True,DefaultUnits);
    ConsiderReplot(GlobalMixture.nSpecies = 0);
    Exit;
  end;
  {$ELSE}
  if ClipboardHasText then begin
    { Look for text simply containing a list of file names one per line }
    { Nautilus seems to provide this format at least when copying file(s) to the clipboard }
    s := Clipboard.AsText;
    Lines := TStringList.Create;
    try
      Lines.Text := s;
      { Check all lines to be blank, or valid file names }
      for i := 0 to Lines.Count-1 do begin
        s := Trim(Lines[i]);
        if Length(s) > 0 then begin
          if (s[1] <> DirectorySeparator) or not(FileExists(s)) then begin
            Lines.Clear;
            Break;
          end;
        end;
      end;
      if Lines.Count > 0 then begin
        for i := 0 to Lines.Count-1 do
          LoadGeneralFile(Trim(Lines[i]),True,True,DefaultUnits);
        ConsiderReplot(GlobalMixture.nSpecies = 0);
        Exit;
      end;
    finally
      Lines.Free;
    end;
  end;
  {$ENDIF}
  AddOverlay('',True,DefaultUnits);
  ShowSimulation;
end;

procedure TMainForm.SelectRegionItemClick(Sender: TObject);
var
  i: Integer;
begin
  if not Assigned(Overlays) or not LastGraph.AreaSelected then Exit;
  for i := 0 to Overlays.Count-1 do
    Overlays[i].SelectFrequencyRange(LastGraph.SelectedMathArea.Left, LastGraph.SelectedMathArea.Right);
  ConsiderReplot(False,False);
end;

procedure TMainForm.ClearSelectionItemClick(Sender: TObject);
var
  i: Integer;
begin
  if not Assigned(Overlays) then Exit;
  for i := 0 to Overlays.Count-1 do
    Overlays[i].ClearSelection;
  ConsiderReplot(False,False);
end;

procedure TMainForm.InvertItemClick(Sender: TObject);
begin
  if Assigned(Overlays) then
    Overlays.Invert;
  ConsiderReplot(False,False);
end;

procedure TMainForm.ClearOverlayItemClick(Sender: TObject);
begin
  if ClearOverlays(True) then
    ShowSimulation;
end;

procedure TMainForm.OverlayPropertiesItemClick(Sender: TObject);
begin
  ReShow(OverlayForm);
  OverlayForm.BaseNode := Overlays;
end;

procedure TMainForm.CalibrateItemClick(Sender: TObject);
begin
  ReShow(CalForm);
  CalForm.BaseNode := Overlays;
  if Sender <> nil then
    ConsiderReplot(False,False);
end;

procedure TMainForm.DeleteLastPeakItemClick(Sender: TObject);
var
  Plot: TPlot;
begin
  Plot := LastPlot(True);
  if Assigned(Plot) then begin
    Plot.DeletePeak(Plot.nPeaks-1);
    ConsiderReplot(False,False);
  end;
end;

procedure TMainForm.ClearAllPeaksItemClick(Sender: TObject);
var
  i: Integer;
begin
  ClearObservations;
  if Assigned(Overlays) then
    for i := 0 to Overlays.Count-1 do
      Overlays[i].ClearPeaks;
  if Assigned(Sender) then
    ConsiderReplot(False,False);
end;

procedure TMainForm.ConstantsItemClick(Sender: TObject);
begin
  if not Assigned(Mixture) then Exit;
  ReShow(ConstantsForm);
  ConstantsForm.BaseNode := Mixture;
end;

procedure TMainForm.LineListItemClick(Sender: TObject);
begin
  ReShow(LineForm);
end;

procedure TMainForm.LogItemClick(Sender: TObject);
begin
  ReShow(LogForm);
end;

procedure TMainForm.StatesItemClick(Sender: TObject);
begin
  ReShow(StatesForm);
  StatesForm.BaseNode := Mixture;
end;

procedure TMainForm.TransitionsItemClick(Sender: TObject);
begin
  ReShow(TransitionForm);
  TransitionForm.BaseNode := Mixture;
end;

procedure TMainForm.LevelsItemClick(Sender: TObject);
begin
  ReShow(LevelForm);
  LevelForm.BaseNode := Mixture;
end;

procedure TMainForm.SpectrumItemClick(Sender: TObject);
var
  PlotForm: TPlotForm;
begin
  if not Assigned(Mixture) then
    Exit;
  PlotForm := MakePlotForm(nil);
  PlotForm.ShowSimulation(Mixture, Overlays);
end;

procedure TMainForm.TimingItemClick(Sender: TObject);
begin
  WriteTimingInfo(Worker.Log);
  ReShow(ProfilerForm);
end;

{$IFDEF SHOWMEMUSAGE }
procedure TMainForm.MemoryItemCLick(Sender: TObject);
begin
  {$IFDEF RecyclerMM}
  RMMSnapShot.Show;
  {$ENDIF}
  ShowFastMMUsageTracker;
end;
{$ENDIF}

procedure TMainForm.ClearLogItemClick(Sender: TObject);
begin
  LogForm.ClearButtonClick(nil);
end;

procedure TMainForm.ClearCacheItemClick(Sender: TObject);
begin
  Mixture.ClearCache;
end;

procedure TMainForm.ClearObservedItemClick(Sender: TObject);
begin
  ClearObservations;
  ConsiderReplot(False,False);
end;

procedure TMainForm.IndependentItemClick(Sender: TObject);
begin
  SetCurrentWindowManagerMode(TWindowManagerMode((Sender as TComponent).Tag));
end;

procedure TMainForm.IndexItemClick(Sender: TObject);
begin
  {$IFDEF VCL_}
  Application.HelpSystem.ShowTableOfContents;
  {$ELSE}
  Application.HelpKeyword('Index');
  {$ENDIF}
end;

procedure TMainForm.SearchItemClick(Sender: TObject);
begin
  {$IFDEF VCL_}
  PHtmlHelpViewer.ShowHtmlSearch;
  {$ELSE}
  Application.HelpKeyword('aindex');
  {$ENDIF}
end;

procedure TMainForm.ProceduresItemClick(Sender: TObject);
begin
  Application.HelpKeyword('Procedures');
end;

procedure TMainForm.ResidualsItemClick(Sender: TObject);
begin
  ReShow(ResidualForm);
end;

procedure TMainForm.WhatIsItemClick(Sender: TObject);
begin
  SetInHelp(True);
end;

procedure TMainForm.UsingHelpItemClick(Sender: TObject);
begin
  Application.HelpKeyword('Using Help');
end;

procedure TMainForm.VariableCharSizeItemClick(Sender: TObject);
begin
  if grFixedCharSize in G.GOptions then
    G.GOptions := G.GOptions - [grFixedCharSize]
  else
    G.GOptions := G.GOptions + [grFixedCharSize];
  OC.GOptions := G.GOptions;
end;

procedure TMainForm.LimitLineSegmentsItemClick(Sender: TObject);
begin
  if grUnlimitedSegmentLength in G.GOptions then
    G.GOptions := G.GOptions - [grUnlimitedSegmentLength]
  else
    G.GOptions := G.GOptions + [grUnlimitedSegmentLength];
  OC.GOptions := G.GOptions;
end;

procedure TMainForm.AboutItemClick(Sender: TObject);
begin
  Project.Execute;
end;

procedure TMainForm.DumpItemClick(Sender: TObject);
begin
  {$IFDEF HELPHELP} MakeSkeletonHelp(Screen.ActiveForm); {$ENDIF}
end;

procedure TMainForm.ShowSumButtonClick(Sender: TObject);
begin
  ChangeSimulationSetting(TComponent(Sender).Tag, Sender);
end;

function TMainForm.MakePlotForm(ASimulation: TSimulation): TPlotForm;
begin
  Result := TPlotForm.Create(Application);
  if Assigned(ASimulation) then begin
    Result.Simulation := ASimulation;
    Simulation.Tag := Result.Tag;
  end else
    Result.InitializeFromSimulation(Simulation);
  Result.Caption := IntToStr(Result.Tag) + ' - ' + Caption;
  Result.Show;
end;

procedure TMainForm.MNodeChange(Sender: TMNode; Change: TMNodeChange;
  Index: Integer);
begin
  if Change = mnDestroy then begin
    if Assigned(Mixture) then
      Mixture.ClearUndo;
    ClearObservations;
  end;
  inherited;
  if Sender = nil then Exit;
  if Sender = Mixture then begin
    if (Change = mnSettingChange) and (Index = QuantumNumberFormatIndex) then
      SetGlobalQuantumNumberFormat(Mixture.QuantumNumberFormat);
    if Change = mnNewBaseNode then
      UpdateToolbar;
  end;
  if not Assigned(FSimulation) or (Sender <> Simulation) then Exit;
  if ( (Change = mnParameterChange)
        and (Index in [TemperatureIndex, GaussianIndex, LorentzianIndex, FoffsetIndex]) )
     or (Change = mnSettingChange)
        and (Index in [ShowSumIndex, ShowPartsIndex, ShowFortratIndex, ShowFortratIndex+1]) then
  UpdateToolbar;
end;

end.

