unit NuclearInfo;
{$I DFS.inc}{Copyright (C) 2008 C M Western; see commandline.pas for license}
{ Data from http://physics.nist.gov/PhysRefData/Compositions/index.html }

interface

type
  TNuclearInfo = record
    Symbol: string;
    AtomicNumber: Integer;
    MassNumber: Integer;
    Mass: Double;
    Abundance: Double;
    StandardWeight: Double;
  end;

function FindNucleus(ApproximateMass: Double; Name: string = ''):TNuclearInfo;
function FindExactMass(ApproximateMass: Double; Name: string = ''): Double;
function FindMostAbundantNucleus(Name: string): TNuclearInfo; overload;
function FindMostAbundantNucleus(AtomicNumber: Integer): TNuclearInfo; overload;

implementation

uses SysUtils;

const
Nuclei: array [0..348] of TNuclearInfo = (
(Symbol:'H'; AtomicNumber:1; MassNumber:1; Mass:1.007825; Abundance:99.9885; StandardWeight:1.00794),
(Symbol:'H'; AtomicNumber:1; MassNumber:2; Mass:2.014102; Abundance:0.0115; StandardWeight:1.00794),
(Symbol:'H'; AtomicNumber:1; MassNumber:3; Mass:3.016049; Abundance:0; StandardWeight:1.00794),
(Symbol:'D'; AtomicNumber:1; MassNumber:2; Mass:2.014102; Abundance:0.0115; StandardWeight:2.014102),
(Symbol:'T'; AtomicNumber:1; MassNumber:3; Mass:3.016049; Abundance:0; StandardWeight:3.016049),
(Symbol:'He'; AtomicNumber:2; MassNumber:3; Mass:3.016029; Abundance:0.000137; StandardWeight:4.002602),
(Symbol:'He'; AtomicNumber:2; MassNumber:4; Mass:4.002603; Abundance:99.99986; StandardWeight:4.002602),
(Symbol:'Li'; AtomicNumber:3; MassNumber:6; Mass:6.015122; Abundance:7.59; StandardWeight:6.941),
(Symbol:'Li'; AtomicNumber:3; MassNumber:7; Mass:7.016004; Abundance:92.41; StandardWeight:6.941),
(Symbol:'Be'; AtomicNumber:4; MassNumber:9; Mass:9.012182; Abundance:100; StandardWeight:9.012182),
(Symbol:'B'; AtomicNumber:5; MassNumber:10; Mass:10.01294; Abundance:19.9; StandardWeight:10.811),
(Symbol:'B'; AtomicNumber:5; MassNumber:11; Mass:11.00931; Abundance:80.1; StandardWeight:10.811),
(Symbol:'C'; AtomicNumber:6; MassNumber:12; Mass:12; Abundance:98.93; StandardWeight:12.0107),
(Symbol:'C'; AtomicNumber:6; MassNumber:13; Mass:13.00335; Abundance:1.07; StandardWeight:12.0107),
(Symbol:'C'; AtomicNumber:6; MassNumber:14; Mass:14.00324; Abundance:0; StandardWeight:12.0107),
(Symbol:'N'; AtomicNumber:7; MassNumber:14; Mass:14.00307; Abundance:99.632; StandardWeight:14.00674),
(Symbol:'N'; AtomicNumber:7; MassNumber:15; Mass:15.00011; Abundance:0.368; StandardWeight:14.00674),
(Symbol:'O'; AtomicNumber:8; MassNumber:16; Mass:15.99491; Abundance:99.757; StandardWeight:15.9994),
(Symbol:'O'; AtomicNumber:8; MassNumber:17; Mass:16.99913; Abundance:0.038; StandardWeight:15.9994),
(Symbol:'O'; AtomicNumber:8; MassNumber:18; Mass:17.99916; Abundance:0.205; StandardWeight:15.9994),
(Symbol:'F'; AtomicNumber:9; MassNumber:19; Mass:18.9984; Abundance:100; StandardWeight:18.9984),
(Symbol:'Ne'; AtomicNumber:10; MassNumber:20; Mass:19.99244; Abundance:90.48; StandardWeight:20.1797),
(Symbol:'Ne'; AtomicNumber:10; MassNumber:21; Mass:20.99385; Abundance:0.27; StandardWeight:20.1797),
(Symbol:'Ne'; AtomicNumber:10; MassNumber:22; Mass:21.99139; Abundance:9.25; StandardWeight:20.1797),
(Symbol:'Na'; AtomicNumber:11; MassNumber:23; Mass:22.98977; Abundance:100; StandardWeight:22.98977),
(Symbol:'Mg'; AtomicNumber:12; MassNumber:24; Mass:23.98504; Abundance:78.99; StandardWeight:24.305),
(Symbol:'Mg'; AtomicNumber:12; MassNumber:25; Mass:24.98584; Abundance:10; StandardWeight:24.305),
(Symbol:'Mg'; AtomicNumber:12; MassNumber:26; Mass:25.98259; Abundance:11.01; StandardWeight:24.305),
(Symbol:'Al'; AtomicNumber:13; MassNumber:27; Mass:26.98154; Abundance:100; StandardWeight:26.98154),
(Symbol:'Si'; AtomicNumber:14; MassNumber:28; Mass:27.97693; Abundance:92.2297; StandardWeight:28.0855),
(Symbol:'Si'; AtomicNumber:14; MassNumber:29; Mass:28.97649; Abundance:4.6832; StandardWeight:28.0855),
(Symbol:'Si'; AtomicNumber:14; MassNumber:30; Mass:29.97377; Abundance:3.0872; StandardWeight:28.0855),
(Symbol:'P'; AtomicNumber:15; MassNumber:31; Mass:30.97376; Abundance:100; StandardWeight:30.97376),
(Symbol:'S'; AtomicNumber:16; MassNumber:32; Mass:31.97207; Abundance:94.93; StandardWeight:32.066),
(Symbol:'S'; AtomicNumber:16; MassNumber:33; Mass:32.97146; Abundance:0.76; StandardWeight:32.066),
(Symbol:'S'; AtomicNumber:16; MassNumber:34; Mass:33.96787; Abundance:4.29; StandardWeight:32.066),
(Symbol:'S'; AtomicNumber:16; MassNumber:36; Mass:35.96708; Abundance:0.02; StandardWeight:32.066),
(Symbol:'Cl'; AtomicNumber:17; MassNumber:35; Mass:34.96885; Abundance:75.78; StandardWeight:35.4527),
(Symbol:'Cl'; AtomicNumber:17; MassNumber:37; Mass:36.9659; Abundance:24.22; StandardWeight:35.4527),
(Symbol:'Ar'; AtomicNumber:18; MassNumber:36; Mass:35.96755; Abundance:0.3365; StandardWeight:39.948),
(Symbol:'Ar'; AtomicNumber:18; MassNumber:38; Mass:37.96273; Abundance:0.0632; StandardWeight:39.948),
(Symbol:'Ar'; AtomicNumber:18; MassNumber:40; Mass:39.96238; Abundance:99.6003; StandardWeight:39.948),
(Symbol:'K'; AtomicNumber:19; MassNumber:39; Mass:38.96371; Abundance:93.2581; StandardWeight:39.0983),
(Symbol:'K'; AtomicNumber:19; MassNumber:40; Mass:39.964; Abundance:0.0117; StandardWeight:39.0983),
(Symbol:'K'; AtomicNumber:19; MassNumber:41; Mass:40.96183; Abundance:6.7302; StandardWeight:39.0983),
(Symbol:'Ca'; AtomicNumber:20; MassNumber:40; Mass:39.96259; Abundance:96.941; StandardWeight:40.078),
(Symbol:'Ca'; AtomicNumber:20; MassNumber:42; Mass:41.95862; Abundance:0.647; StandardWeight:40.078),
(Symbol:'Ca'; AtomicNumber:20; MassNumber:43; Mass:42.95877; Abundance:0.135; StandardWeight:40.078),
(Symbol:'Ca'; AtomicNumber:20; MassNumber:44; Mass:43.95548; Abundance:2.086; StandardWeight:40.078),
(Symbol:'Ca'; AtomicNumber:20; MassNumber:46; Mass:45.95369; Abundance:0.004; StandardWeight:40.078),
(Symbol:'Ca'; AtomicNumber:20; MassNumber:48; Mass:47.95253; Abundance:0.187; StandardWeight:40.078),
(Symbol:'Sc'; AtomicNumber:21; MassNumber:45; Mass:44.95591; Abundance:100; StandardWeight:44.95591),
(Symbol:'Ti'; AtomicNumber:22; MassNumber:46; Mass:45.95263; Abundance:8.25; StandardWeight:47.867),
(Symbol:'Ti'; AtomicNumber:22; MassNumber:47; Mass:46.95176; Abundance:7.44; StandardWeight:47.867),
(Symbol:'Ti'; AtomicNumber:22; MassNumber:48; Mass:47.94795; Abundance:73.72; StandardWeight:47.867),
(Symbol:'Ti'; AtomicNumber:22; MassNumber:49; Mass:48.94787; Abundance:5.41; StandardWeight:47.867),
(Symbol:'Ti'; AtomicNumber:22; MassNumber:50; Mass:49.94479; Abundance:5.18; StandardWeight:47.867),
(Symbol:'V'; AtomicNumber:23; MassNumber:50; Mass:49.94716; Abundance:0.25; StandardWeight:50.9415),
(Symbol:'V'; AtomicNumber:23; MassNumber:51; Mass:50.94396; Abundance:99.75; StandardWeight:50.9415),
(Symbol:'Cr'; AtomicNumber:24; MassNumber:50; Mass:49.94605; Abundance:4.345; StandardWeight:51.9961),
(Symbol:'Cr'; AtomicNumber:24; MassNumber:52; Mass:51.94051; Abundance:83.789; StandardWeight:51.9961),
(Symbol:'Cr'; AtomicNumber:24; MassNumber:53; Mass:52.94065; Abundance:9.501; StandardWeight:51.9961),
(Symbol:'Cr'; AtomicNumber:24; MassNumber:54; Mass:53.93888; Abundance:2.365; StandardWeight:51.9961),
(Symbol:'Mn'; AtomicNumber:25; MassNumber:55; Mass:54.93805; Abundance:100; StandardWeight:54.93805),
(Symbol:'Fe'; AtomicNumber:26; MassNumber:54; Mass:53.93961; Abundance:5.845; StandardWeight:55.845),
(Symbol:'Fe'; AtomicNumber:26; MassNumber:56; Mass:55.93494; Abundance:91.754; StandardWeight:55.845),
(Symbol:'Fe'; AtomicNumber:26; MassNumber:57; Mass:56.9354; Abundance:2.119; StandardWeight:55.845),
(Symbol:'Fe'; AtomicNumber:26; MassNumber:58; Mass:57.93328; Abundance:0.282; StandardWeight:55.845),
(Symbol:'Co'; AtomicNumber:27; MassNumber:59; Mass:58.9332; Abundance:100; StandardWeight:58.9332),
(Symbol:'Ni'; AtomicNumber:28; MassNumber:58; Mass:57.93535; Abundance:68.0769; StandardWeight:58.6934),
(Symbol:'Ni'; AtomicNumber:28; MassNumber:60; Mass:59.93079; Abundance:26.2231; StandardWeight:58.6934),
(Symbol:'Ni'; AtomicNumber:28; MassNumber:61; Mass:60.93106; Abundance:1.1399; StandardWeight:58.6934),
(Symbol:'Ni'; AtomicNumber:28; MassNumber:62; Mass:61.92835; Abundance:3.6345; StandardWeight:58.6934),
(Symbol:'Ni'; AtomicNumber:28; MassNumber:64; Mass:63.92797; Abundance:0.9256; StandardWeight:58.6934),
(Symbol:'Cu'; AtomicNumber:29; MassNumber:63; Mass:62.9296; Abundance:69.17; StandardWeight:63.546),
(Symbol:'Cu'; AtomicNumber:29; MassNumber:65; Mass:64.92779; Abundance:30.83; StandardWeight:63.546),
(Symbol:'Zn'; AtomicNumber:30; MassNumber:64; Mass:63.92915; Abundance:48.63; StandardWeight:65.39),
(Symbol:'Zn'; AtomicNumber:30; MassNumber:66; Mass:65.92604; Abundance:27.9; StandardWeight:65.39),
(Symbol:'Zn'; AtomicNumber:30; MassNumber:67; Mass:66.92713; Abundance:4.1; StandardWeight:65.39),
(Symbol:'Zn'; AtomicNumber:30; MassNumber:68; Mass:67.92485; Abundance:18.75; StandardWeight:65.39),
(Symbol:'Zn'; AtomicNumber:30; MassNumber:70; Mass:69.92533; Abundance:0.62; StandardWeight:65.39),
(Symbol:'Ga'; AtomicNumber:31; MassNumber:69; Mass:68.92558; Abundance:60.108; StandardWeight:69.723),
(Symbol:'Ga'; AtomicNumber:31; MassNumber:71; Mass:70.92471; Abundance:39.892; StandardWeight:69.723),
(Symbol:'Ge'; AtomicNumber:32; MassNumber:70; Mass:69.92425; Abundance:20.84; StandardWeight:72.61),
(Symbol:'Ge'; AtomicNumber:32; MassNumber:72; Mass:71.92208; Abundance:27.54; StandardWeight:72.61),
(Symbol:'Ge'; AtomicNumber:32; MassNumber:73; Mass:72.92346; Abundance:7.73; StandardWeight:72.61),
(Symbol:'Ge'; AtomicNumber:32; MassNumber:74; Mass:73.92118; Abundance:36.28; StandardWeight:72.61),
(Symbol:'Ge'; AtomicNumber:32; MassNumber:76; Mass:75.9214; Abundance:7.61; StandardWeight:72.61),
(Symbol:'As'; AtomicNumber:33; MassNumber:75; Mass:74.9216; Abundance:100; StandardWeight:74.9216),
(Symbol:'Se'; AtomicNumber:34; MassNumber:74; Mass:73.92248; Abundance:0.89; StandardWeight:78.96),
(Symbol:'Se'; AtomicNumber:34; MassNumber:76; Mass:75.91921; Abundance:9.37; StandardWeight:78.96),
(Symbol:'Se'; AtomicNumber:34; MassNumber:77; Mass:76.91991; Abundance:7.63; StandardWeight:78.96),
(Symbol:'Se'; AtomicNumber:34; MassNumber:78; Mass:77.91731; Abundance:23.77; StandardWeight:78.96),
(Symbol:'Se'; AtomicNumber:34; MassNumber:80; Mass:79.91652; Abundance:49.61; StandardWeight:78.96),
(Symbol:'Se'; AtomicNumber:34; MassNumber:82; Mass:81.9167; Abundance:8.73; StandardWeight:78.96),
(Symbol:'Br'; AtomicNumber:35; MassNumber:79; Mass:78.91834; Abundance:50.69; StandardWeight:79.904),
(Symbol:'Br'; AtomicNumber:35; MassNumber:81; Mass:80.91629; Abundance:49.31; StandardWeight:79.904),
(Symbol:'Kr'; AtomicNumber:36; MassNumber:78; Mass:77.92039; Abundance:0.35; StandardWeight:83.8),
(Symbol:'Kr'; AtomicNumber:36; MassNumber:80; Mass:79.91638; Abundance:2.28; StandardWeight:83.8),
(Symbol:'Kr'; AtomicNumber:36; MassNumber:82; Mass:81.91348; Abundance:11.58; StandardWeight:83.8),
(Symbol:'Kr'; AtomicNumber:36; MassNumber:83; Mass:82.91414; Abundance:11.49; StandardWeight:83.8),
(Symbol:'Kr'; AtomicNumber:36; MassNumber:84; Mass:83.91151; Abundance:57; StandardWeight:83.8),
(Symbol:'Kr'; AtomicNumber:36; MassNumber:86; Mass:85.91061; Abundance:17.3; StandardWeight:83.8),
(Symbol:'Rb'; AtomicNumber:37; MassNumber:85; Mass:84.91179; Abundance:72.17; StandardWeight:85.4678),
(Symbol:'Rb'; AtomicNumber:37; MassNumber:87; Mass:86.90918; Abundance:27.83; StandardWeight:85.4678),
(Symbol:'Sr'; AtomicNumber:38; MassNumber:84; Mass:83.91343; Abundance:0.56; StandardWeight:87.62),
(Symbol:'Sr'; AtomicNumber:38; MassNumber:86; Mass:85.90926; Abundance:9.86; StandardWeight:87.62),
(Symbol:'Sr'; AtomicNumber:38; MassNumber:87; Mass:86.90888; Abundance:7; StandardWeight:87.62),
(Symbol:'Sr'; AtomicNumber:38; MassNumber:88; Mass:87.90561; Abundance:82.58; StandardWeight:87.62),
(Symbol:'Y'; AtomicNumber:39; MassNumber:89; Mass:88.90585; Abundance:100; StandardWeight:88.90585),
(Symbol:'Zr'; AtomicNumber:40; MassNumber:90; Mass:89.9047; Abundance:51.45; StandardWeight:91.224),
(Symbol:'Zr'; AtomicNumber:40; MassNumber:91; Mass:90.90565; Abundance:11.22; StandardWeight:91.224),
(Symbol:'Zr'; AtomicNumber:40; MassNumber:92; Mass:91.90504; Abundance:17.15; StandardWeight:91.224),
(Symbol:'Zr'; AtomicNumber:40; MassNumber:94; Mass:93.90632; Abundance:17.38; StandardWeight:91.224),
(Symbol:'Zr'; AtomicNumber:40; MassNumber:96; Mass:95.90828; Abundance:2.8; StandardWeight:91.224),
(Symbol:'Nb'; AtomicNumber:41; MassNumber:93; Mass:92.90638; Abundance:100; StandardWeight:92.90638),
(Symbol:'Mo'; AtomicNumber:42; MassNumber:92; Mass:91.90681; Abundance:14.84; StandardWeight:95.94),
(Symbol:'Mo'; AtomicNumber:42; MassNumber:94; Mass:93.90509; Abundance:9.25; StandardWeight:95.94),
(Symbol:'Mo'; AtomicNumber:42; MassNumber:95; Mass:94.90584; Abundance:15.92; StandardWeight:95.94),
(Symbol:'Mo'; AtomicNumber:42; MassNumber:96; Mass:95.90468; Abundance:16.68; StandardWeight:95.94),
(Symbol:'Mo'; AtomicNumber:42; MassNumber:97; Mass:96.90602; Abundance:9.55; StandardWeight:95.94),
(Symbol:'Mo'; AtomicNumber:42; MassNumber:98; Mass:97.90541; Abundance:24.13; StandardWeight:95.94),
(Symbol:'Mo'; AtomicNumber:42; MassNumber:100; Mass:99.90748; Abundance:9.63; StandardWeight:95.94),
(Symbol:'Tc'; AtomicNumber:43; MassNumber:97; Mass:96.90636; Abundance:0; StandardWeight:98),
(Symbol:'Tc'; AtomicNumber:43; MassNumber:98; Mass:97.90722; Abundance:0; StandardWeight:98),
(Symbol:'Tc'; AtomicNumber:43; MassNumber:99; Mass:98.90625; Abundance:0; StandardWeight:98),
(Symbol:'Ru'; AtomicNumber:44; MassNumber:96; Mass:95.9076; Abundance:5.54; StandardWeight:101.07),
(Symbol:'Ru'; AtomicNumber:44; MassNumber:98; Mass:97.90529; Abundance:1.87; StandardWeight:101.07),
(Symbol:'Ru'; AtomicNumber:44; MassNumber:99; Mass:98.90594; Abundance:12.76; StandardWeight:101.07),
(Symbol:'Ru'; AtomicNumber:44; MassNumber:100; Mass:99.90422; Abundance:12.6; StandardWeight:101.07),
(Symbol:'Ru'; AtomicNumber:44; MassNumber:101; Mass:100.9056; Abundance:17.06; StandardWeight:101.07),
(Symbol:'Ru'; AtomicNumber:44; MassNumber:102; Mass:101.9043; Abundance:31.55; StandardWeight:101.07),
(Symbol:'Ru'; AtomicNumber:44; MassNumber:104; Mass:103.9054; Abundance:18.62; StandardWeight:101.07),
(Symbol:'Rh'; AtomicNumber:45; MassNumber:103; Mass:102.9055; Abundance:100; StandardWeight:102.9055),
(Symbol:'Pd'; AtomicNumber:46; MassNumber:102; Mass:101.9056; Abundance:1.02; StandardWeight:106.42),
(Symbol:'Pd'; AtomicNumber:46; MassNumber:104; Mass:103.904; Abundance:11.14; StandardWeight:106.42),
(Symbol:'Pd'; AtomicNumber:46; MassNumber:105; Mass:104.9051; Abundance:22.33; StandardWeight:106.42),
(Symbol:'Pd'; AtomicNumber:46; MassNumber:106; Mass:105.9035; Abundance:27.33; StandardWeight:106.42),
(Symbol:'Pd'; AtomicNumber:46; MassNumber:108; Mass:107.9039; Abundance:26.46; StandardWeight:106.42),
(Symbol:'Pd'; AtomicNumber:46; MassNumber:110; Mass:109.9052; Abundance:11.72; StandardWeight:106.42),
(Symbol:'Ag'; AtomicNumber:47; MassNumber:107; Mass:106.9051; Abundance:51.839; StandardWeight:107.8682),
(Symbol:'Ag'; AtomicNumber:47; MassNumber:109; Mass:108.9048; Abundance:48.161; StandardWeight:107.8682),
(Symbol:'Cd'; AtomicNumber:48; MassNumber:106; Mass:105.9065; Abundance:1.25; StandardWeight:112.411),
(Symbol:'Cd'; AtomicNumber:48; MassNumber:108; Mass:107.9042; Abundance:0.89; StandardWeight:112.411),
(Symbol:'Cd'; AtomicNumber:48; MassNumber:110; Mass:109.903; Abundance:12.49; StandardWeight:112.411),
(Symbol:'Cd'; AtomicNumber:48; MassNumber:111; Mass:110.9042; Abundance:12.8; StandardWeight:112.411),
(Symbol:'Cd'; AtomicNumber:48; MassNumber:112; Mass:111.9028; Abundance:24.13; StandardWeight:112.411),
(Symbol:'Cd'; AtomicNumber:48; MassNumber:113; Mass:112.9044; Abundance:12.22; StandardWeight:112.411),
(Symbol:'Cd'; AtomicNumber:48; MassNumber:114; Mass:113.9034; Abundance:28.73; StandardWeight:112.411),
(Symbol:'Cd'; AtomicNumber:48; MassNumber:116; Mass:115.9048; Abundance:7.49; StandardWeight:112.411),
(Symbol:'In'; AtomicNumber:49; MassNumber:113; Mass:112.9041; Abundance:4.29; StandardWeight:114.818),
(Symbol:'In'; AtomicNumber:49; MassNumber:115; Mass:114.9039; Abundance:95.71; StandardWeight:114.818),
(Symbol:'Sn'; AtomicNumber:50; MassNumber:112; Mass:111.9048; Abundance:0.97; StandardWeight:118.71),
(Symbol:'Sn'; AtomicNumber:50; MassNumber:114; Mass:113.9028; Abundance:0.66; StandardWeight:118.71),
(Symbol:'Sn'; AtomicNumber:50; MassNumber:115; Mass:114.9033; Abundance:0.34; StandardWeight:118.71),
(Symbol:'Sn'; AtomicNumber:50; MassNumber:116; Mass:115.9017; Abundance:14.54; StandardWeight:118.71),
(Symbol:'Sn'; AtomicNumber:50; MassNumber:117; Mass:116.903; Abundance:7.68; StandardWeight:118.71),
(Symbol:'Sn'; AtomicNumber:50; MassNumber:118; Mass:117.9016; Abundance:24.22; StandardWeight:118.71),
(Symbol:'Sn'; AtomicNumber:50; MassNumber:119; Mass:118.9033; Abundance:8.59; StandardWeight:118.71),
(Symbol:'Sn'; AtomicNumber:50; MassNumber:120; Mass:119.9022; Abundance:32.58; StandardWeight:118.71),
(Symbol:'Sn'; AtomicNumber:50; MassNumber:122; Mass:121.9034; Abundance:4.63; StandardWeight:118.71),
(Symbol:'Sn'; AtomicNumber:50; MassNumber:124; Mass:123.9053; Abundance:5.79; StandardWeight:118.71),
(Symbol:'Sb'; AtomicNumber:51; MassNumber:121; Mass:120.9038; Abundance:57.21; StandardWeight:121.76),
(Symbol:'Sb'; AtomicNumber:51; MassNumber:123; Mass:122.9042; Abundance:42.79; StandardWeight:121.76),
(Symbol:'Te'; AtomicNumber:52; MassNumber:120; Mass:119.904; Abundance:0.09; StandardWeight:127.6),
(Symbol:'Te'; AtomicNumber:52; MassNumber:122; Mass:121.903; Abundance:2.55; StandardWeight:127.6),
(Symbol:'Te'; AtomicNumber:52; MassNumber:123; Mass:122.9043; Abundance:0.89; StandardWeight:127.6),
(Symbol:'Te'; AtomicNumber:52; MassNumber:124; Mass:123.9028; Abundance:4.74; StandardWeight:127.6),
(Symbol:'Te'; AtomicNumber:52; MassNumber:125; Mass:124.9044; Abundance:7.07; StandardWeight:127.6),
(Symbol:'Te'; AtomicNumber:52; MassNumber:126; Mass:125.9033; Abundance:18.84; StandardWeight:127.6),
(Symbol:'Te'; AtomicNumber:52; MassNumber:128; Mass:127.9045; Abundance:31.74; StandardWeight:127.6),
(Symbol:'Te'; AtomicNumber:52; MassNumber:130; Mass:129.9062; Abundance:34.08; StandardWeight:127.6),
(Symbol:'I'; AtomicNumber:53; MassNumber:127; Mass:126.9045; Abundance:100; StandardWeight:126.9045),
(Symbol:'Xe'; AtomicNumber:54; MassNumber:124; Mass:123.9059; Abundance:0.09; StandardWeight:131.29),
(Symbol:'Xe'; AtomicNumber:54; MassNumber:126; Mass:125.9043; Abundance:0.09; StandardWeight:131.29),
(Symbol:'Xe'; AtomicNumber:54; MassNumber:128; Mass:127.9035; Abundance:1.92; StandardWeight:131.29),
(Symbol:'Xe'; AtomicNumber:54; MassNumber:129; Mass:128.9048; Abundance:26.44; StandardWeight:131.29),
(Symbol:'Xe'; AtomicNumber:54; MassNumber:130; Mass:129.9035; Abundance:4.08; StandardWeight:131.29),
(Symbol:'Xe'; AtomicNumber:54; MassNumber:131; Mass:130.9051; Abundance:21.18; StandardWeight:131.29),
(Symbol:'Xe'; AtomicNumber:54; MassNumber:132; Mass:131.9042; Abundance:26.89; StandardWeight:131.29),
(Symbol:'Xe'; AtomicNumber:54; MassNumber:134; Mass:133.9054; Abundance:10.44; StandardWeight:131.29),
(Symbol:'Xe'; AtomicNumber:54; MassNumber:136; Mass:135.9072; Abundance:8.87; StandardWeight:131.29),
(Symbol:'Cs'; AtomicNumber:55; MassNumber:133; Mass:132.9054; Abundance:100; StandardWeight:132.9055),
(Symbol:'Ba'; AtomicNumber:56; MassNumber:130; Mass:129.9063; Abundance:0.106; StandardWeight:137.327),
(Symbol:'Ba'; AtomicNumber:56; MassNumber:132; Mass:131.9051; Abundance:0.101; StandardWeight:137.327),
(Symbol:'Ba'; AtomicNumber:56; MassNumber:134; Mass:133.9045; Abundance:2.417; StandardWeight:137.327),
(Symbol:'Ba'; AtomicNumber:56; MassNumber:135; Mass:134.9057; Abundance:6.592; StandardWeight:137.327),
(Symbol:'Ba'; AtomicNumber:56; MassNumber:136; Mass:135.9046; Abundance:7.854; StandardWeight:137.327),
(Symbol:'Ba'; AtomicNumber:56; MassNumber:137; Mass:136.9058; Abundance:11.232; StandardWeight:137.327),
(Symbol:'Ba'; AtomicNumber:56; MassNumber:138; Mass:137.9052; Abundance:71.698; StandardWeight:137.327),
(Symbol:'La'; AtomicNumber:57; MassNumber:138; Mass:137.9071; Abundance:0.09; StandardWeight:138.9055),
(Symbol:'La'; AtomicNumber:57; MassNumber:139; Mass:138.9063; Abundance:99.91; StandardWeight:138.9055),
(Symbol:'Ce'; AtomicNumber:58; MassNumber:136; Mass:135.9071; Abundance:0.185; StandardWeight:140.116),
(Symbol:'Ce'; AtomicNumber:58; MassNumber:138; Mass:137.906; Abundance:0.251; StandardWeight:140.116),
(Symbol:'Ce'; AtomicNumber:58; MassNumber:140; Mass:139.9054; Abundance:88.45; StandardWeight:140.116),
(Symbol:'Ce'; AtomicNumber:58; MassNumber:142; Mass:141.9092; Abundance:11.114; StandardWeight:140.116),
(Symbol:'Pr'; AtomicNumber:59; MassNumber:141; Mass:140.9076; Abundance:100; StandardWeight:140.9076),
(Symbol:'Nd'; AtomicNumber:60; MassNumber:142; Mass:141.9077; Abundance:27.2; StandardWeight:144.24),
(Symbol:'Nd'; AtomicNumber:60; MassNumber:143; Mass:142.9098; Abundance:12.2; StandardWeight:144.24),
(Symbol:'Nd'; AtomicNumber:60; MassNumber:144; Mass:143.9101; Abundance:23.8; StandardWeight:144.24),
(Symbol:'Nd'; AtomicNumber:60; MassNumber:145; Mass:144.9126; Abundance:8.3; StandardWeight:144.24),
(Symbol:'Nd'; AtomicNumber:60; MassNumber:146; Mass:145.9131; Abundance:17.2; StandardWeight:144.24),
(Symbol:'Nd'; AtomicNumber:60; MassNumber:148; Mass:147.9169; Abundance:5.7; StandardWeight:144.24),
(Symbol:'Nd'; AtomicNumber:60; MassNumber:150; Mass:149.9209; Abundance:5.6; StandardWeight:144.24),
(Symbol:'Pm'; AtomicNumber:61; MassNumber:145; Mass:144.9127; Abundance:0; StandardWeight:145),
(Symbol:'Pm'; AtomicNumber:61; MassNumber:147; Mass:146.9151; Abundance:0; StandardWeight:145),
(Symbol:'Sm'; AtomicNumber:62; MassNumber:144; Mass:143.912; Abundance:3.07; StandardWeight:150.36),
(Symbol:'Sm'; AtomicNumber:62; MassNumber:147; Mass:146.9149; Abundance:14.99; StandardWeight:150.36),
(Symbol:'Sm'; AtomicNumber:62; MassNumber:148; Mass:147.9148; Abundance:11.24; StandardWeight:150.36),
(Symbol:'Sm'; AtomicNumber:62; MassNumber:149; Mass:148.9172; Abundance:13.82; StandardWeight:150.36),
(Symbol:'Sm'; AtomicNumber:62; MassNumber:150; Mass:149.9173; Abundance:7.38; StandardWeight:150.36),
(Symbol:'Sm'; AtomicNumber:62; MassNumber:152; Mass:151.9197; Abundance:26.75; StandardWeight:150.36),
(Symbol:'Sm'; AtomicNumber:62; MassNumber:154; Mass:153.9222; Abundance:22.75; StandardWeight:150.36),
(Symbol:'Eu'; AtomicNumber:63; MassNumber:151; Mass:150.9198; Abundance:47.81; StandardWeight:151.964),
(Symbol:'Eu'; AtomicNumber:63; MassNumber:153; Mass:152.9212; Abundance:52.19; StandardWeight:151.964),
(Symbol:'Gd'; AtomicNumber:64; MassNumber:152; Mass:151.9198; Abundance:0.2; StandardWeight:157.25),
(Symbol:'Gd'; AtomicNumber:64; MassNumber:154; Mass:153.9209; Abundance:2.18; StandardWeight:157.25),
(Symbol:'Gd'; AtomicNumber:64; MassNumber:155; Mass:154.9226; Abundance:14.8; StandardWeight:157.25),
(Symbol:'Gd'; AtomicNumber:64; MassNumber:156; Mass:155.9221; Abundance:20.47; StandardWeight:157.25),
(Symbol:'Gd'; AtomicNumber:64; MassNumber:157; Mass:156.924; Abundance:15.65; StandardWeight:157.25),
(Symbol:'Gd'; AtomicNumber:64; MassNumber:158; Mass:157.9241; Abundance:24.84; StandardWeight:157.25),
(Symbol:'Gd'; AtomicNumber:64; MassNumber:160; Mass:159.9271; Abundance:21.86; StandardWeight:157.25),
(Symbol:'Tb'; AtomicNumber:65; MassNumber:159; Mass:158.9253; Abundance:100; StandardWeight:158.9253),
(Symbol:'Dy'; AtomicNumber:66; MassNumber:156; Mass:155.9243; Abundance:0.06; StandardWeight:162.5),
(Symbol:'Dy'; AtomicNumber:66; MassNumber:158; Mass:157.9244; Abundance:0.1; StandardWeight:162.5),
(Symbol:'Dy'; AtomicNumber:66; MassNumber:160; Mass:159.9252; Abundance:2.34; StandardWeight:162.5),
(Symbol:'Dy'; AtomicNumber:66; MassNumber:161; Mass:160.9269; Abundance:18.91; StandardWeight:162.5),
(Symbol:'Dy'; AtomicNumber:66; MassNumber:162; Mass:161.9268; Abundance:25.51; StandardWeight:162.5),
(Symbol:'Dy'; AtomicNumber:66; MassNumber:163; Mass:162.9287; Abundance:24.9; StandardWeight:162.5),
(Symbol:'Dy'; AtomicNumber:66; MassNumber:164; Mass:163.9292; Abundance:28.18; StandardWeight:162.5),
(Symbol:'Ho'; AtomicNumber:67; MassNumber:165; Mass:164.9303; Abundance:100; StandardWeight:164.9303),
(Symbol:'Er'; AtomicNumber:68; MassNumber:162; Mass:161.9288; Abundance:0.14; StandardWeight:167.26),
(Symbol:'Er'; AtomicNumber:68; MassNumber:164; Mass:163.9292; Abundance:1.61; StandardWeight:167.26),
(Symbol:'Er'; AtomicNumber:68; MassNumber:166; Mass:165.9303; Abundance:33.61; StandardWeight:167.26),
(Symbol:'Er'; AtomicNumber:68; MassNumber:167; Mass:166.932; Abundance:22.93; StandardWeight:167.26),
(Symbol:'Er'; AtomicNumber:68; MassNumber:168; Mass:167.9324; Abundance:26.78; StandardWeight:167.26),
(Symbol:'Er'; AtomicNumber:68; MassNumber:170; Mass:169.9355; Abundance:14.93; StandardWeight:167.26),
(Symbol:'Tm'; AtomicNumber:69; MassNumber:169; Mass:168.9342; Abundance:100; StandardWeight:168.9342),
(Symbol:'Yb'; AtomicNumber:70; MassNumber:168; Mass:167.9339; Abundance:0.13; StandardWeight:173.04),
(Symbol:'Yb'; AtomicNumber:70; MassNumber:170; Mass:169.9348; Abundance:3.04; StandardWeight:173.04),
(Symbol:'Yb'; AtomicNumber:70; MassNumber:171; Mass:170.9363; Abundance:14.28; StandardWeight:173.04),
(Symbol:'Yb'; AtomicNumber:70; MassNumber:172; Mass:171.9364; Abundance:21.83; StandardWeight:173.04),
(Symbol:'Yb'; AtomicNumber:70; MassNumber:173; Mass:172.9382; Abundance:16.13; StandardWeight:173.04),
(Symbol:'Yb'; AtomicNumber:70; MassNumber:174; Mass:173.9389; Abundance:31.83; StandardWeight:173.04),
(Symbol:'Yb'; AtomicNumber:70; MassNumber:176; Mass:175.9426; Abundance:12.76; StandardWeight:173.04),
(Symbol:'Lu'; AtomicNumber:71; MassNumber:175; Mass:174.9408; Abundance:97.41; StandardWeight:174.967),
(Symbol:'Lu'; AtomicNumber:71; MassNumber:176; Mass:175.9427; Abundance:2.59; StandardWeight:174.967),
(Symbol:'Hf'; AtomicNumber:72; MassNumber:174; Mass:173.94; Abundance:0.16; StandardWeight:178.49),
(Symbol:'Hf'; AtomicNumber:72; MassNumber:176; Mass:175.9414; Abundance:5.26; StandardWeight:178.49),
(Symbol:'Hf'; AtomicNumber:72; MassNumber:177; Mass:176.9432; Abundance:18.6; StandardWeight:178.49),
(Symbol:'Hf'; AtomicNumber:72; MassNumber:178; Mass:177.9437; Abundance:27.28; StandardWeight:178.49),
(Symbol:'Hf'; AtomicNumber:72; MassNumber:179; Mass:178.9458; Abundance:13.62; StandardWeight:178.49),
(Symbol:'Hf'; AtomicNumber:72; MassNumber:180; Mass:179.9465; Abundance:35.08; StandardWeight:178.49),
(Symbol:'Ta'; AtomicNumber:73; MassNumber:180; Mass:179.9475; Abundance:0.012; StandardWeight:180.9479),
(Symbol:'Ta'; AtomicNumber:73; MassNumber:181; Mass:180.948; Abundance:99.988; StandardWeight:180.9479),
(Symbol:'W'; AtomicNumber:74; MassNumber:180; Mass:179.9467; Abundance:0.12; StandardWeight:183.84),
(Symbol:'W'; AtomicNumber:74; MassNumber:182; Mass:181.9482; Abundance:26.5; StandardWeight:183.84),
(Symbol:'W'; AtomicNumber:74; MassNumber:183; Mass:182.9502; Abundance:14.31; StandardWeight:183.84),
(Symbol:'W'; AtomicNumber:74; MassNumber:184; Mass:183.9509; Abundance:30.64; StandardWeight:183.84),
(Symbol:'W'; AtomicNumber:74; MassNumber:186; Mass:185.9544; Abundance:28.43; StandardWeight:183.84),
(Symbol:'Re'; AtomicNumber:75; MassNumber:185; Mass:184.953; Abundance:37.4; StandardWeight:186.207),
(Symbol:'Re'; AtomicNumber:75; MassNumber:187; Mass:186.9558; Abundance:62.6; StandardWeight:186.207),
(Symbol:'Os'; AtomicNumber:76; MassNumber:184; Mass:183.9525; Abundance:0.02; StandardWeight:190.23),
(Symbol:'Os'; AtomicNumber:76; MassNumber:186; Mass:185.9538; Abundance:1.59; StandardWeight:190.23),
(Symbol:'Os'; AtomicNumber:76; MassNumber:187; Mass:186.9557; Abundance:1.96; StandardWeight:190.23),
(Symbol:'Os'; AtomicNumber:76; MassNumber:188; Mass:187.9558; Abundance:13.24; StandardWeight:190.23),
(Symbol:'Os'; AtomicNumber:76; MassNumber:189; Mass:188.9581; Abundance:16.15; StandardWeight:190.23),
(Symbol:'Os'; AtomicNumber:76; MassNumber:190; Mass:189.9584; Abundance:26.26; StandardWeight:190.23),
(Symbol:'Os'; AtomicNumber:76; MassNumber:192; Mass:191.9615; Abundance:40.78; StandardWeight:190.23),
(Symbol:'Ir'; AtomicNumber:77; MassNumber:191; Mass:190.9606; Abundance:37.3; StandardWeight:192.217),
(Symbol:'Ir'; AtomicNumber:77; MassNumber:193; Mass:192.9629; Abundance:62.7; StandardWeight:192.217),
(Symbol:'Pt'; AtomicNumber:78; MassNumber:190; Mass:189.9599; Abundance:0.014; StandardWeight:195.078),
(Symbol:'Pt'; AtomicNumber:78; MassNumber:192; Mass:191.961; Abundance:0.782; StandardWeight:195.078),
(Symbol:'Pt'; AtomicNumber:78; MassNumber:194; Mass:193.9627; Abundance:32.967; StandardWeight:195.078),
(Symbol:'Pt'; AtomicNumber:78; MassNumber:195; Mass:194.9648; Abundance:33.832; StandardWeight:195.078),
(Symbol:'Pt'; AtomicNumber:78; MassNumber:196; Mass:195.9649; Abundance:25.242; StandardWeight:195.078),
(Symbol:'Pt'; AtomicNumber:78; MassNumber:198; Mass:197.9679; Abundance:7.163; StandardWeight:195.078),
(Symbol:'Au'; AtomicNumber:79; MassNumber:197; Mass:196.9666; Abundance:100; StandardWeight:196.9666),
(Symbol:'Hg'; AtomicNumber:80; MassNumber:196; Mass:195.9658; Abundance:0.15; StandardWeight:200.59),
(Symbol:'Hg'; AtomicNumber:80; MassNumber:198; Mass:197.9668; Abundance:9.97; StandardWeight:200.59),
(Symbol:'Hg'; AtomicNumber:80; MassNumber:199; Mass:198.9683; Abundance:16.87; StandardWeight:200.59),
(Symbol:'Hg'; AtomicNumber:80; MassNumber:200; Mass:199.9683; Abundance:23.1; StandardWeight:200.59),
(Symbol:'Hg'; AtomicNumber:80; MassNumber:201; Mass:200.9703; Abundance:13.18; StandardWeight:200.59),
(Symbol:'Hg'; AtomicNumber:80; MassNumber:202; Mass:201.9706; Abundance:29.86; StandardWeight:200.59),
(Symbol:'Hg'; AtomicNumber:80; MassNumber:204; Mass:203.9735; Abundance:6.87; StandardWeight:200.59),
(Symbol:'Tl'; AtomicNumber:81; MassNumber:203; Mass:202.9723; Abundance:29.524; StandardWeight:204.3833),
(Symbol:'Tl'; AtomicNumber:81; MassNumber:205; Mass:204.9744; Abundance:70.476; StandardWeight:204.3833),
(Symbol:'Pb'; AtomicNumber:82; MassNumber:204; Mass:203.973; Abundance:1.4; StandardWeight:207.2),
(Symbol:'Pb'; AtomicNumber:82; MassNumber:206; Mass:205.9744; Abundance:24.1; StandardWeight:207.2),
(Symbol:'Pb'; AtomicNumber:82; MassNumber:207; Mass:206.9759; Abundance:22.1; StandardWeight:207.2),
(Symbol:'Pb'; AtomicNumber:82; MassNumber:208; Mass:207.9766; Abundance:52.4; StandardWeight:207.2),
(Symbol:'Bi'; AtomicNumber:83; MassNumber:209; Mass:208.9804; Abundance:100; StandardWeight:208.9804),
(Symbol:'Po'; AtomicNumber:84; MassNumber:209; Mass:208.9824; Abundance:0; StandardWeight:209),
(Symbol:'Po'; AtomicNumber:84; MassNumber:210; Mass:209.9829; Abundance:0; StandardWeight:209),
(Symbol:'At'; AtomicNumber:85; MassNumber:210; Mass:209.9871; Abundance:0; StandardWeight:210),
(Symbol:'At'; AtomicNumber:85; MassNumber:211; Mass:210.9875; Abundance:0; StandardWeight:210),
(Symbol:'Rn'; AtomicNumber:86; MassNumber:211; Mass:210.9906; Abundance:0; StandardWeight:222),
(Symbol:'Rn'; AtomicNumber:86; MassNumber:220; Mass:220.0114; Abundance:0; StandardWeight:222),
(Symbol:'Rn'; AtomicNumber:86; MassNumber:222; Mass:222.0176; Abundance:0; StandardWeight:222),
(Symbol:'Fr'; AtomicNumber:87; MassNumber:223; Mass:223.0197; Abundance:0; StandardWeight:223),
(Symbol:'Ra'; AtomicNumber:88; MassNumber:223; Mass:223.0185; Abundance:0; StandardWeight:226),
(Symbol:'Ra'; AtomicNumber:88; MassNumber:224; Mass:224.0202; Abundance:0; StandardWeight:226),
(Symbol:'Ra'; AtomicNumber:88; MassNumber:226; Mass:226.0254; Abundance:0; StandardWeight:226),
(Symbol:'Ra'; AtomicNumber:88; MassNumber:228; Mass:228.0311; Abundance:0; StandardWeight:226),
(Symbol:'Ac'; AtomicNumber:89; MassNumber:227; Mass:227.0277; Abundance:0; StandardWeight:227),
(Symbol:'Th'; AtomicNumber:90; MassNumber:230; Mass:230.0331; Abundance:0; StandardWeight:232.0381),
(Symbol:'Th'; AtomicNumber:90; MassNumber:232; Mass:232.0381; Abundance:100; StandardWeight:232.0381),
(Symbol:'Pa'; AtomicNumber:91; MassNumber:231; Mass:231.0359; Abundance:100; StandardWeight:231.0359),
(Symbol:'U'; AtomicNumber:92; MassNumber:233; Mass:233.0396; Abundance:0; StandardWeight:238.0289),
(Symbol:'U'; AtomicNumber:92; MassNumber:234; Mass:234.0409; Abundance:0.0055; StandardWeight:238.0289),
(Symbol:'U'; AtomicNumber:92; MassNumber:235; Mass:235.0439; Abundance:0.72; StandardWeight:238.0289),
(Symbol:'U'; AtomicNumber:92; MassNumber:236; Mass:236.0456; Abundance:0; StandardWeight:238.0289),
(Symbol:'U'; AtomicNumber:92; MassNumber:238; Mass:238.0508; Abundance:99.2745; StandardWeight:238.0289),
(Symbol:'Np'; AtomicNumber:93; MassNumber:237; Mass:237.0482; Abundance:0; StandardWeight:237),
(Symbol:'Np'; AtomicNumber:93; MassNumber:239; Mass:239.0529; Abundance:0; StandardWeight:237),
(Symbol:'Pu'; AtomicNumber:94; MassNumber:238; Mass:238.0496; Abundance:0; StandardWeight:244),
(Symbol:'Pu'; AtomicNumber:94; MassNumber:239; Mass:239.0522; Abundance:0; StandardWeight:244),
(Symbol:'Pu'; AtomicNumber:94; MassNumber:240; Mass:240.0538; Abundance:0; StandardWeight:244),
(Symbol:'Pu'; AtomicNumber:94; MassNumber:241; Mass:241.0568; Abundance:0; StandardWeight:244),
(Symbol:'Pu'; AtomicNumber:94; MassNumber:242; Mass:242.0587; Abundance:0; StandardWeight:244),
(Symbol:'Pu'; AtomicNumber:94; MassNumber:244; Mass:244.0642; Abundance:0; StandardWeight:244),
(Symbol:'Am'; AtomicNumber:95; MassNumber:241; Mass:241.0568; Abundance:0; StandardWeight:243),
(Symbol:'Am'; AtomicNumber:95; MassNumber:243; Mass:243.0614; Abundance:0; StandardWeight:243),
(Symbol:'Cm'; AtomicNumber:96; MassNumber:243; Mass:243.0614; Abundance:0; StandardWeight:247),
(Symbol:'Cm'; AtomicNumber:96; MassNumber:244; Mass:244.0627; Abundance:0; StandardWeight:247),
(Symbol:'Cm'; AtomicNumber:96; MassNumber:245; Mass:245.0655; Abundance:0; StandardWeight:247),
(Symbol:'Cm'; AtomicNumber:96; MassNumber:246; Mass:246.0672; Abundance:0; StandardWeight:247),
(Symbol:'Cm'; AtomicNumber:96; MassNumber:247; Mass:247.0703; Abundance:0; StandardWeight:247),
(Symbol:'Cm'; AtomicNumber:96; MassNumber:248; Mass:248.0723; Abundance:0; StandardWeight:247),
(Symbol:'Bk'; AtomicNumber:97; MassNumber:247; Mass:247.0703; Abundance:0; StandardWeight:247),
(Symbol:'Bk'; AtomicNumber:97; MassNumber:249; Mass:249.075; Abundance:0; StandardWeight:247),
(Symbol:'Cf'; AtomicNumber:98; MassNumber:249; Mass:249.0748; Abundance:0; StandardWeight:251),
(Symbol:'Cf'; AtomicNumber:98; MassNumber:250; Mass:250.0764; Abundance:0; StandardWeight:251),
(Symbol:'Cf'; AtomicNumber:98; MassNumber:251; Mass:251.0796; Abundance:0; StandardWeight:251),
(Symbol:'Cf'; AtomicNumber:98; MassNumber:252; Mass:252.0816; Abundance:0; StandardWeight:251),
(Symbol:'Es'; AtomicNumber:99; MassNumber:252; Mass:252.083; Abundance:0; StandardWeight:252),
(Symbol:'Fm'; AtomicNumber:100; MassNumber:257; Mass:257.0951; Abundance:0; StandardWeight:257),
(Symbol:'Md'; AtomicNumber:101; MassNumber:256; Mass:256.094; Abundance:0; StandardWeight:258),
(Symbol:'Md'; AtomicNumber:101; MassNumber:258; Mass:258.0984; Abundance:0; StandardWeight:258),
(Symbol:'No'; AtomicNumber:102; MassNumber:259; Mass:259.101; Abundance:0; StandardWeight:259),
(Symbol:'Lr'; AtomicNumber:103; MassNumber:262; Mass:262.1097; Abundance:0; StandardWeight:262),
(Symbol:'Rf'; AtomicNumber:104; MassNumber:261; Mass:261.1087; Abundance:0; StandardWeight:261),
(Symbol:'Db'; AtomicNumber:105; MassNumber:262; Mass:262.1141; Abundance:0; StandardWeight:262),
(Symbol:'Sg'; AtomicNumber:106; MassNumber:266; Mass:266.1219; Abundance:0; StandardWeight:266),
(Symbol:'Bh'; AtomicNumber:107; MassNumber:264; Mass:264.1247; Abundance:0; StandardWeight:264),
(Symbol:'Hs'; AtomicNumber:108; MassNumber:269; Mass:269.1341; Abundance:0; StandardWeight:269),
(Symbol:'Mt'; AtomicNumber:109; MassNumber:268; Mass:268.1388; Abundance:0; StandardWeight:268),
(Symbol:'Uun'; AtomicNumber:110; MassNumber:271; Mass:271.1461; Abundance:0; StandardWeight:271),
(Symbol:'Uuu'; AtomicNumber:111; MassNumber:272; Mass:272.1535; Abundance:0; StandardWeight:272)
);

procedure TrimNumber(var Name: string);
var
  i: Integer;
begin
  { Trim trailing number and other junk from nucleus name }
  i := 0;
  while (i < Length(Name)) and (i < 3) do begin
    if not (Name[i+1] in ['A'..'Z', 'a'..'z']) then
      Break;
    Inc(i);
  end;
  SetLength(Name, i);
end;

function FindNucleus(ApproximateMass: Double; Name: string = ''):TNuclearInfo;
var
  i: Integer;
begin
  TrimNumber(Name);
  Result.Mass := -1;
  for i := 0 to High(Nuclei) do
    if Abs(ApproximateMass-Nuclei[i].MassNumber) < 0.5 then
      if (Name = '') or (CompareText(Name,Nuclei[i].Symbol)=0) then begin
        if Result.Mass > 0 then begin
          Result.Mass := -2;
          Exit; { Too many matches }
        end;
        Result := Nuclei[i];
      end;
end;

function FindExactMass(ApproximateMass: Double; Name: string = ''): Double;
var
  Nuc: TNuclearInfo;
begin
  if ApproximateMass <= 0 then
    Nuc := FindMostAbundantNucleus(Name)
  else
    Nuc := FindNucleus(ApproximateMass, Name);
  Result := Nuc.Mass;
end;

function FindMostAbundantNucleus(Name: string): TNuclearInfo;
var
  i: Integer;
begin
  TrimNumber(Name);
  Result.Abundance := -1;
  for i := 0 to High(Nuclei) do
    if (CompareText(Name,Nuclei[i].Symbol)=0) and (Nuclei[i].Abundance > Result.Abundance) then
      Result := Nuclei[i];
  if Result.Abundance < 0 then
    Result.Mass := -1; { Not found }
end;

function FindMostAbundantNucleus(AtomicNumber: Integer): TNuclearInfo;
var
  i: Integer;
begin
  Result.Abundance := -1;
  for i := 0 to High(Nuclei) do
    if (AtomicNumber = Nuclei[i].AtomicNumber) and (Nuclei[i].Abundance > Result.Abundance) then
      Result := Nuclei[i];
  if Result.Abundance < 0 then
    Result.Mass := -1; { Not found }
end;

end.
