unit OverlaySumFitter;
{$I DFS.inc}{Copyright (C) 2010 C M Western; see commandline.pas for license}
interface

uses Mixture, Overlay, Fitter;

type
  TComponentType = (None, Component, Floating, Target);

function FitOverlaySum(Mixture: TMixture; Spectra: TOverlays; ShowContour: OnShowContour; var Parameters: TParameters; JustResiduals: Boolean):string;

implementation

uses SysUtils, MatrixVec, ExpPlot, MNode, Constrain, Units;

function FitOverlaySum(Mixture: TMixture; Spectra: TOverlays; ShowContour: OnShowContour; var Parameters: TParameters; JustResiduals: Boolean):string;
  var
    Sources: array of TExperimentalPlot;
  procedure CalculateObs(v: TColumnVector);
  var
    i, j, k, l, p: Integer;
    Sum, f, vi, vf: Double;
  begin
    l := 1;
    for i := 0 to Spectra.nPlots-1 do
      if Spectra[i] is TExperimentalPlot then
        with TExperimentalPlot(Spectra[i]) do
          if ComponentType = Target then
            for j := StartPoint to EndPoint do begin
              Sum := 0;
              f := AdjustFrequency(Frequency[j]);
              for k := 0 to High(Sources) do begin
                vf := Convert(f,Units,Sources[k].Units);
                vi := Sources[k].IntensityAt(vf,p);
                vi := ConvertIntensity(vi,vf,Sources[k].Units,Sources[k].IntensityUnits,Units,IntensityUnits);
                Sum := Sum + GetParameter(FirstDynamicParameter+k)*vi;
              end;
              { Logically we should be comparing to the values in the observed TExperimentalPlot.
                However the calculated values would not then depend on the scaling parameters in
                the observed spectrum, so we apply the reverse transform to the calculated values }
              v[l] := Sum/Scale + Baseline; { Inverse transform }
              Inc(l);
            end;
  end;
var
  f: TFitter;
  nObs: Integer;
  i, j, l: Integer;
  vo, vn, x, y: Double;
begin
  nObs := 0;
  { Find total number of observed points }
  for i := 0 to Spectra.nPlots-1 do
    if Spectra[i] is TExperimentalPlot then
      with TExperimentalPlot(Spectra[i]) do
        case ComponentType of
          None: ;
          Component, OverlaySumFitter.Floating:
            begin
              SetLength(Sources,Length(Sources)+1);
              Sources[High(Sources)] := TExperimentalPlot(Spectra[i]);
            end;
          Target: Inc(nObs, EndPoint-StartPoint+1);
        end;

  f := TFitter.Create(Length(Parameters), nObs);
  { Because of inverse scaling used in CalculateObs (see comment)
    resdiduals are inversely proportional to spectrum scale factors,
    so flag as such }
  for i := 0 to High(Parameters) do
    with Parameters[i] do
      if (MNode is TExperimentalPlot) and (Index > 0) and (MNode.ParameterNames[Index] = 'Scale') then
        Inverse := True;
  try
    CalculateObs(f.omc);
    if Assigned(ShowContour) then
      ShowContour(Spectra, f.omc);
    if JustResiduals then Exit;
    {  Calculate Derivatives wrt each parameter }
    { TODO: Optimize for simple derivatives }
    { Note that f.weights is only used as workspace, and all weights are taken as 1 }
    for i := 0 to High(Parameters) do begin
      vo := Parameters[i].Value;
      if Parameters[i].Increment <= 0 then begin
        vn := vo * 1.0001;
        if vn = 0 then
          vn := 0.001; { Random Guess }
      end else
        vn := vo + Parameters[i].Increment;
      Parameters[i].MNode.SetParameter(Parameters[i].Index, vn);
      ApplyConstraints;
      try
        Parameters[i].MNode.CheckProperties;
        CalculateObs(f.weights);
      finally
        Parameters[i].MNode.SetParameter(Parameters[i].Index, vo);
        ApplyConstraints;
        Parameters[i].MNode.CheckProperties;
      end;
      for j := 1 to nObs do
        f.a[j,i+1] := (f.omc[j] - f.weights[j])/(vn-vo);
    end;
    FreeAndNil(f.Weights);
    l := 1; { Finally work out observed - calculated }
    for i := 0 to Spectra.nPlots-1 do
     if Spectra[i] is TExperimentalPlot then
      with TExperimentalPlot(Spectra[i]) do
        if ComponentType = Target then
          for j := StartPoint to EndPoint do begin
            // Note Baseline and Scale applied to calculated, not observed
            f.omc[l] := f.omc[l] - Intensity[j];
            Inc(l);
          end;
    Result := f.DoLeastSQ(Parameters, Mixture.Precision, Mixture);
  finally
    f.Free;
  end;
end;

end.
