unit Progress;
{$I DFS.inc}{Copyright (C) 2005 C M Western; see commandline.pas for license}
interface

{$IFNDEF CONSOLE}
uses
  {$IFDEF MSWINDOWS} Windows, {$ENDIF}
  SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Buttons, Worker;

type

  { TProgressForm }

  TProgressForm = class(TForm)
    WasCalculating: TLabel;
    ProgressBar: TProgressBar;
    Button: TButton;
    Calculating: TLabel;
    ValueLabel: TLabel;
    procedure ButtonClick(Sender: TObject);
  end;

var
  ProgressForm: TProgressForm;
{$ENDIF}

procedure CheckForAbort;

procedure ResetProgress;
procedure PushProgress(Calculating: string; AFinalPosition: Integer = -1);
procedure PopProgress;
procedure SetProgress(Position: Integer; AFinalPosition: Integer = -1);

implementation

uses {$IFNDEF CONSOLE} {$IFDEF LCL_} LResources, {$ENDIF} {$ENDIF}
     Clock;

{$IFNDEF CONSOLE}
{$IFNDEF FPC}
{$R *.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

var
  Level: Integer = -1;
  FinalPosition: array of Integer;
  CalcName: array of string;

procedure TProgressForm.ButtonClick(Sender: TObject);
begin
  Aborted := True;
end;

{$ENDIF}

var
  NextUpdate: Int64;
  ShortDelay: Integer;

procedure ResetProgress;
begin
  {$IFNDEF CONSOLE}
  Aborted := False;
  ProgressForm.Hide;
  Level := -1;
  {$ENDIF}
end;

procedure CheckForAbort;
begin
  {$IFNDEF CONSOLE}
  if Aborted then begin
    ResetProgress;
    Abort;
  end;
  {$ENDIF}
end;

procedure PushProgress(Calculating: string; AFinalPosition: Integer = -1);
begin
  {$IFNDEF CONSOLE}
  Inc(Level);
  if Level > High(FinalPosition) then begin
    SetLength(FinalPosition, Level+10);
    SetLength(CalcName, Level+10);
  end;
  FinalPosition[Level] := AFinalPosition;
  CalcName[Level] := Calculating;
  if Level = 0 then begin
    ShortDelay := Round(ClockFrequency/5);
    NextUpdate := Round(ClockFrequency) + Clocks();
  end;
  CheckForAbort;
  {$ENDIF}
end;

procedure PopProgress;
begin
  {$IFNDEF CONSOLE}
  Dec(Level);
  if Level < 0 then
    ResetProgress;
  {$ENDIF}
end;

procedure SetProgress(Position: Integer; AFinalPosition: Integer = -1);
var
  CurrentClock: Int64;
begin
  {$IFNDEF CONSOLE}
  if Level < 0 then
    Exit;
  CheckForAbort;
  if AFinalPosition > 0 then
    FinalPosition[Level] := AFinalPosition;
  if Position > FinalPosition[Level] then
    FinalPosition[Level] := FinalPosition[Level] + FinalPosition[Level] div 10 + 2;
  if FinalPosition[Level] > 0 then begin
    CurrentClock := Clocks();
    if CurrentClock > NextUpdate then begin
      NextUpdate := CurrentClock + ShortDelay;
      if not ProgressForm.Visible then
        ProgressForm.Show;
      if Screen.ActiveForm <> ProgressForm then
        ProgressForm.BringToFront;
      ProgressForm.Calculating.Caption := CalcName[Level];
      if Level > 0 then
        ProgressForm.WasCalculating.Caption := CalcName[Level-1]
      else
        ProgressForm.WasCalculating.Caption := '';
      ProgressForm.ProgressBar.Max := FinalPosition[Level];
      ProgressForm.ProgressBar.Position := Position;
      ProgressForm.ValueLabel.Caption := IntToStr(Position);
      Application.ProcessMessages;
    end;
  end;
  {$ENDIF}
end;

end.
