unit ResidFrm;
{$I DFS.inc}{Copyright (C) 2005 C M Western; see commandline.pas for license}
interface

uses
  {$IFDEF MSWINDOWS} Windows, {$ENDIF}
  SysUtils, Variants, Classes, Graphics, Controls, Forms, ActnList,
  Dialogs, ExtCtrls, ScrlCtrl, Log, Buttons, StdCtrls, AlEdit, EditExtr,
  NewSpin, MNode, SettingsForm, Split, Graphs, PlotOC, ComCtrls, Menus,
  Obs, Mixture, GraphI, GraphM;

type

  { TLogForm }

  TResidualForm = class(TSettingsForm)
    G: TGraphD;
    OCPanel: TPanel;
    YAxis: TComboBox;
    Label2: TLabel;
    Label3: TLabel;
    XAxis: TComboBox;
    Upper: TCheckBox;
    ReloadButton: TSpeedButton;
    ZoomButton: TSpeedButton;
    ZoomOutButton: TSpeedButton;
    ResetZoomButton: TSpeedButton;
    StatusBar: TStatusBar;
    GPopupMenu: TPopupMenu;
    EditPointItem: TMenuItem;
    LoadAll: TCheckBox;
    ShowTransitionItem: TMenuItem;
    procedure UpperClick(Sender: TObject);
    procedure PopupMenuPopup(Sender: TObject);
    procedure EditPointItemClick(Sender: TObject);
    procedure GMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure ResetZoomButtonClick(Sender: TObject);
    procedure ZoomOutButtonClick(Sender: TObject);
    procedure ZoomButtonClick(Sender: TObject);
    procedure YAxisChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ReloadButtonClick(Sender: TObject);
    procedure ShowTransitionItemClick(Sender: TObject);
  private
    FResidualPlotter: TResidualPlotter;
    procedure SetResidualPlotter(const Value: TResidualPlotter);
  public
    procedure ArrangeForm(Screen: TRect); override;
    procedure PlotResiduals(FitStatus: string);
    procedure DiscardResidualPlotter(const Value: TResidualPlotter);
    procedure Loaded; override;
    property ResidualPlotter: TResidualPlotter read FResidualPlotter write SetResidualPlotter;
  end;

var
  ResidualForm: TResidualForm;

implementation

uses TextAssign, Fitter, Huge, ClipBrd, WCompat, SlaveEdit,
     Global, Main, QuantumNumber, QuantumMath, FormUtils, CalFrm, Math, CommandLine,
     LogUnit, Types;

{$IFNDEF FPC}
{$R *.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

procedure TResidualForm.EditPointItemClick(Sender: TObject);
begin
  if Assigned(ResidualPlotter) then
    ResidualPlotter.EditLastPoint
end;

procedure TResidualForm.ShowTransitionItemClick(Sender: TObject);
begin
  if Assigned(ResidualPlotter) then
    ResidualPlotter.ShowLastPoint
end;

procedure TResidualForm.DiscardResidualPlotter(const Value: TResidualPlotter);
begin
  { Called when residual plotter being deleted }
  if FResidualPlotter = Value then begin
    FResidualPlotter := nil;
    UpperClick(nil);
  end
end;

procedure TResidualForm.ArrangeForm(Screen: TRect);
var
  R, MainRect: TRect;
begin
  inherited;
  { Bottom left corner }
  MainRect := ScreenBounds(MainForm);
  R := Rect(MainRect.Right, MainRect.Bottom, Screen.Right, Screen.Bottom);
  with R do begin
    if Right - Left < 200 then
      Left := Right - 200;
    if Bottom-Top < 200 then
      Top := Bottom - 200;
  end;
  SetScreenBounds(Self,R);
end;

procedure TResidualForm.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FResidualPlotter);
end;

procedure TResidualForm.SetResidualPlotter(const Value: TResidualPlotter);
begin
  FreeAndNil(FResidualPlotter);
  FResidualPlotter := Value;
  UpperClick(nil);
end;

procedure TResidualForm.UpperClick(Sender: TObject);
begin
  if ResidualPlotter is TObsNumberPlotter then
    TObsNumberPlotter(ResidualPlotter).Upper := Upper.Checked;
  YAxisChange(Sender);
end;

procedure TResidualForm.PlotResiduals(FitStatus: string);
begin
  ResidualPlotter := TObsNumberPlotter.Create(FitStatus);
  if ResidualPlotter.LastPlotFailed then begin
    ReShow(ResidualForm);
    raise EReportException.Create('Failure in plotting residuals - see log for details');
  end;
end;

procedure TResidualForm.PopupMenuPopup(Sender: TObject);
begin
  EditPointItem.Enabled := Assigned(ResidualPlotter) and (ResidualPlotter.LastLineNumber >= 0);
  ShowTransitionItem.Enabled := EditPointItem.Enabled;
end;

procedure TResidualForm.YAxisChange(Sender: TObject);
begin
  if Assigned(ResidualPlotter) then begin
    ResidualPlotter.XType := TOCXStyle(XAxis.ItemIndex);
    ResidualPlotter.YType := TOCStyle(YAxis.ItemIndex);
    ResidualPlotter.Plot(G);
  end else
    G.PlotChanged;
end;

procedure TResidualForm.ZoomButtonClick(Sender: TObject);
begin
  G.ZoomSelected(True);
end;

procedure TResidualForm.ZoomOutButtonClick(Sender: TObject);
begin
  G.ZoomOut;
end;

procedure TResidualForm.ReloadButtonClick(Sender: TObject);
var
  TakeWeight: TTakeWeight;
begin
  if not SaveAllWindows then Exit;
  if LoadAll.Checked then
    TakeWeight := tkAll
  else
    TakeWeight := tkObs;
  LogForm.DoFit(LogForm.ObsFileName.FileName, True, TakeWeight);
end;

procedure TResidualForm.ResetZoomButtonClick(Sender: TObject);
begin
  G.ResetZoom;
end;

procedure TResidualForm.GMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  if Assigned(ResidualPlotter) then begin
    ResidualPlotter.LastLineNumber := -1;
    ShowGraphInfo(G, StatusBar, ResidualPlotter.PlottedPoints, {$IFDEF FPC}@{$ENDIF}ResidualPlotter.OnGetPointInfo);
  end;
end;

procedure TResidualForm.Loaded;
var
  i: Integer;
begin
  for i := 0 to High(QnoTypeNames)-1 do
    XAxis.Items.Add(QnoTypeNames[i]);
  G.StuffPopupMenu(GPopupMenu);
  inherited;
end;

end.

