unit Skeleton;

interface

procedure SkeletonFunction(Param2: string);

implementation

uses SysUtils, MNode, HundA, Mixture, Overlay, JPLCat, Vib,
     ExpBitmap, ExpPlot, OAsymT, SymT, Worker, Global, UCompat, PartFunc, OscFields;

var
  IndFile: Text;
  Context: string;

procedure WriteSkeleton(MNodeClass: TMNodeClass; StandAlone: Boolean = True);
var
  Name, LocalContext: string;
  MNode: TMNode;
  i: Integer;
begin
  MNode := nil;
  if (GlobalMixture <> nil) and not StandAlone then
    MNode := GlobalMixture.ComponentWithName('*',MNodeClass,nil,True);
  if not Assigned(MNode) and StandAlone then
    try
      MNode := MNodeClass.Create(nil);
    except
      Exit;
    end;
  if not Assigned(MNode) then
    Exit;
  Name := Copy(MNode.ClassType.ClassName,2,255);
  AssignFile(Output,'skel\/' + LowerCase(Name) + '.htm');
  Rewrite(Output);
  Writeln('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">');
  Writeln('<html>');
  Writeln('<head>');
  Writeln('<title>', Name, '</title>');
  Writeln('<body>');
  Writeln('<h1>', Name, '</h1>');
  if MNode.nSettings > 0 then begin
    Writeln('<h2>Settings</h2>');
    Writeln('<table border="1" cellpadding="2" cellspacing="2"><tbody>');
    for i := 1 to MNode.nSettings do begin
      Writeln('<tr><td><a name="', MNode.SettingNames[i], '"></a>', MNode.SettingNames[i], '</td><td>', MNode.HelpOnSetting(i),'</td>');
    end;
    Writeln('</tbody></table>');
  end;
  if MNode.nParameters > 0 then begin
    Writeln('<h2>Parameters</h2>');
    Writeln('<table border="1" cellpadding="2" cellspacing="2"><tbody>');
    for i := 1 to MNode.nParameters do begin
      Writeln('<tr><td><a name="', MNode.ParameterNames[i], '"></a>', MNode.ParameterNames[i], '</td><td>', MNode.HelpOnParameter(i),'</td>');
    end;
    Writeln('</tbody></table>');
  end;
  Writeln('</body>');
  Writeln('</html>');
  CloseFile(Output);
  {$IFDEF OFF}
  WriteLn(IndFile, #9'<LI> <OBJECT type="text/sitemap">');
  WriteLn(IndFile, #9#9'<param name="Name" value="', Name, '">');
  WriteLn(IndFile, #9#9'<param name="Local" value="', LowerCase(Name), '">');
  WriteLn(IndFile, #9'</OBJECT>');

  for i := 1 to MNode.nSettings do begin
    WriteLn(IndFile, #9'<LI> <OBJECT type="text/sitemap">');
    WriteLn(IndFile, #9#9'<param name="Name" value="', MNode.SettingNames[i], '">');
    WriteLn(IndFile, #9#9'<param name="Local" value="', LowerCase(Name), '.htm#', MNode.SettingNames[i], '">');
    WriteLn(IndFile, #9'</OBJECT>');
  end;
  for i := 1 to MNode.nParameters do begin
    WriteLn(IndFile, #9'<LI> <OBJECT type="text/sitemap">');
    WriteLn(IndFile, #9#9'<param name="Name" value="', MNode.ParameterNames[i], '">');
    WriteLn(IndFile, #9#9'<param name="Local" value="', LowerCase(Name), '.htm#', MNode.ParameterNames[i], '">');
    WriteLn(IndFile, #9'</OBJECT>');
  end;
  {$ENDIF}
  LocalContext := Context;
  if LocalContext = '' then begin
    LocalContext := Name;
    for i := Length(Name) downto 1 do
      if LocalContext[i] in ['A'..'Z'] then
        Insert(' ',LocalContext,i);
    LocalContext := LocalContext + #9'Main'
  end;
  WriteLn(IndFile,Name,#9,LowerCase(Name),'.htm'#9,LocalContext);
  for i := 1 to MNode.nSettings do
    WriteLn(IndFile,MNode.SettingNames[i],#9,LowerCase(Name),'.htm#',MNode.SettingNames[i],#9,LocalContext);
  for i := 1 to MNode.nParameters do
    WriteLn(IndFile,MNode.ParameterNames[i],#9,LowerCase(Name),'.htm#',MNode.ParameterNames[i],#9,LocalContext);
end;
{$IFDEF OFF}
procedure MakeSymTables;
var
  Mixture: TMIxture;
  Species: TSpecies;
  Molecule: TAsymmetricMolecule;
  Manifold: TAsymmetricManifold;
  State: TAsymmetricTop;
  PointGroup: TPointGroup;
  Sym: Integer;
  zAx, xAx: TabcAxis;
  Rep: TRepresentation;
begin
  Mixture := TMixture.Create(nil);
  Species := TSpecies.Create(Mixture);
  Molecule := TAsymmetricMolecule.Create(Species);
  Manifold := TAsymmetricManifold.Create(Molecule);
  State := TAsymmetricTop.Create(Manifold);
  for PointGroup := Low(PointGroup) to High(PointGroup) do begin
    Molecule.PointGroup := PointGroup;
    for Rep := Low(TRepresentation) to High(TRepresentation) do begin
      Molecule.Representation := Rep;
      for zAx := aAxis to cAxis do begin
        for xAx := cAxis downto aAxis do begin
          if zAx <> xAx then begin
            for Sym := 0 to Molecule.LastSym do begin
              Molecule.C2zAxis := zAx;
              Molecule.C2xAxis := xAx;
              State.Symmetry := Sym;
              { TODO: Molecule.PseudoC2v := True; }
              Mixture.CheckProperties;
              State.SymmetryTable(Output);
            end;
            if Molecule.PointGroup in [C1, Ci, C2, Cs, C2h] then Break;
          end;
        end;
        if Molecule.PointGroup in [C1, Ci] then Break;
      end;
    end;
  end;
end;

procedure MakeAsymLineLists(DirName: string);
var
  Mixture: TMixture;
  Species: TSpecies;
  Molecule: TAsymmetricMolecule;
  ManifoldL: TAsymmetricManifold;
  StateL, StateZ, StateX, StateY: TAsymmetricTop;
  PointGroup: TPointGroup;
  Sym: Integer;
  zAx, xAx: TabcAxis;
  Rep: TRepresentation;
  FileName: string;
begin
  Mixture := GlobalMixture;
  Species := Mixture.Species[0];
  Molecule := Species.Isotopomer[0] as TAsymmetricMolecule;
  ManifoldL := Molecule.Manifold[0] as TAsymmetricManifold;
  StateL := ManifoldL.State[0] as TAsymmetricTop;
  StateZ := Molecule.Manifold[1].State[0] as TAsymmetricTop;
  StateX := Molecule.Manifold[2].State[0] as TAsymmetricTop;
  StateY := Molecule.Manifold[3].State[0] as TAsymmetricTop;
  for PointGroup := Low(PointGroup) to High(PointGroup) do begin
    Molecule.PointGroup := PointGroup;
    for Rep := Low(TRepresentation) to High(TRepresentation) do begin
      Molecule.Representation := Rep;
      for zAx := aAxis to cAxis do begin
        for xAx := cAxis downto aAxis do begin
          if zAx <> xAx then begin
            for Sym := 0 to Molecule.LastSym do begin
              Molecule.C2zAxis := zAx;
              Molecule.C2xAxis := xAx;
              StateL.Symmetry := Sym;
              case PointGroup of
                C1: begin StateZ.Symmetry := Sym; StateX.Symmetry := Sym; StateY.Symmetry := Sym; end;
                Ci: begin StateZ.Symmetry := Sym xor 1; StateX.Symmetry := Sym xor 1; StateY.Symmetry := Sym xor 1; end;
                C2: begin StateZ.Symmetry := Sym; StateX.Symmetry := Sym xor 1; StateY.Symmetry := Sym xor 1; end;
                Cs: begin StateZ.Symmetry := Sym xor 1; StateX.Symmetry := Sym; StateY.Symmetry := Sym; end;
                D2: begin StateZ.Symmetry := Sym xor 1; StateX.Symmetry := Sym xor 3; StateY.Symmetry := Sym xor 2; end;
                C2v: begin StateZ.Symmetry := Sym; StateX.Symmetry := Sym xor 2; StateY.Symmetry := Sym xor 3; end;
                C2h: begin StateZ.Symmetry := Sym xor 2; StateX.Symmetry := Sym xor 3; StateY.Symmetry := Sym xor 3; end;
                D2h: begin StateZ.Symmetry := Sym xor 5; StateX.Symmetry := Sym xor 7; StateY.Symmetry := Sym xor 6; end;
              end;
              { TODO: Molecule.PseudoC2v := True; }
              Mixture.CheckProperties;
              FileName := DirName + PathDelim
                          + Molecule.SettingToStr(Molecule.IndexOfSetting('PointGroup')) + '_'
                          + Molecule.SettingToStr(Molecule.IndexOfSetting('Representation')) + '_'
                          + Molecule.SettingToStr(Molecule.IndexOfSetting('C2zAxis')) + '_'
                          + Molecule.SettingToStr(Molecule.IndexOfSetting('C2xAxis')) + '_'
                          + StateL.SettingToStr(StateL.IndexOfSetting('Symmetry'));
              case FileName[Length(FileName)] of
                '''': FileName[Length(FileName)] := 'p';
                '"': begin FileName[Length(FileName)] := 'p'; FileName := FileName +'p'; end;
              end;
              Mixture.SaveToFile(FileName+'.pgo');
              WriteLineList(Mixture, FileName);
            end;
            if Molecule.PointGroup in [C1, Ci, C2, Cs, C2h] then Break;
          end;
        end;
        if Molecule.PointGroup in [C1, Ci] then Break;
      end;
    end;
  end;
end;
{$ENDIF}

procedure WriteLineList(Mixture: TMixture; Name: string);
var
  Problem: Exception;
  F: Text;
  i, n0: Integer;
begin
  Problem := nil;
  Mixture.Simulations[0].MakeLineList(Problem);
  if Assigned(Problem) then
    raise Problem;
  if not Assigned(Mixture.LineList) then
    Exit;
  n0 := 0;
  with Mixture.LineList do
    for i := 0 to nLines-1 do begin
      if Line[i].EigenvaluesL.Sym = 0 then
        Inc(n0);
      if Line[i].EigenvaluesU.Sym = 0 then
        Inc(n0);
    end;
  WriteLn(Log, Name:30, Mixture.LineList.nLines:5, n0:5);
  if n0 = 0 then begin
    WriteLn(Log, '%PGO% >>dotests.out --testpgo ', Name, '.pgo nolines.out');
    raise Exception.Create('Help');
  end;
//  WriteLn(Log, '%PGO% >>dotests.out --testpgo ', Name, '.pgo ', Name, '.out');
  AssignFile(F, Name + '.out');
  try
    ReWrite(F);
    with Mixture.LineList do begin
      HeadingPrinted := False;
      for i := 0 to nLines-1 do begin
        ShowLine(i,F,' ',Mixture.Simulations[0].OThreshold,False,Problem);
        if Assigned(Problem) then
          raise Problem;
      end;
      ShowFooter(F,' ');
    end;
  finally
    CloseFile(F);
  end;
  Problem.Free;
end;

procedure MakeSymLineLists(DirName: string; UseAlt: Boolean);
var
  Mixture: TMixture;
  Species: TSpecies;
  Molecule: TSymmetricMolecule;
  ManifoldL, ManifoldU: TSymmetricManifold;
  TransitionMoment: TSphericalTransitionMoment;
  StateL, StateU: TSymmetricTop;
  PointGroup: TPointGroup;
  FileName, s: string;
  SymL: Integer;
  SymU: Integer;
  Component: Integer;
  i: Integer;
begin
  Mixture := GlobalMixture;
  Species := Mixture.Species[0];
  Molecule := Species.Isotopomer[0] as TSymmetricMolecule;
  Molecule.AlternateBasis := UseAlt;
  ManifoldL := Molecule.Manifold[0] as TSymmetricManifold;
  ManifoldU := Molecule.Manifold[1] as TSymmetricManifold;
  StateL := ManifoldL.State[0] as TSymmetricTop;
  StateU := ManifoldU.State[0] as TSymmetricTop;
  TransitionMoment := Molecule.GetTransitionMoments(0).TransitionMoment[0] as TSphericalTransitionMoment;
  for PointGroup := C3h to High(PointGroup) do begin
    Molecule.PointGroup := PointGroup;
    for i := WTI to WTI+MaxnStatWeight - 1 do
      Molecule.Settings[i] := 11+i-WTI;
    for Component := 0 to 1 do begin
      for SymL := 0 to Molecule.LastSym do begin
        StateL.Symmetry := SymL;
        for SymU := SymL to Molecule.LastSym do begin
          StateU.Symmetry := SymU;
          TransitionMoment.Component := 2*Component;
          Mixture.CheckProperties;
          FileName := DirName + PathDelim
                      + Molecule.SettingToStr(Molecule.IndexOfSetting('PointGroup'));
          if Molecule.AlternateBasis then
            FileName := FileName + 'Alt';
          FileName := FileName + '_'
                      + StateU.SettingToStr(StateU.IndexOfSetting('Symmetry')) + '_'
                      + StateL.SettingToStr(StateL.IndexOfSetting('Symmetry')) + '_'
                      + IntToStr(Component);
          for i := 1 to Length(FileName) do
            case FileName[i] of
              '''': FileName[i] := 'p';
              '"': FileName[i] := 'd';
            end;
          Mixture.SaveToFile(FileName+'.pgo');
          s := FileName;
          if Molecule.VibronicallyAllowed(TransitionMoment) = vaYes then
            WriteLineList(Mixture, FileName)
          else
            s := 'nolines';
//          WriteLn('%PGO% >>dotests.out --testpgo ', FileName, '.pgo ', s, '.out');
        end;
      end;
    end;
  end;
end;

procedure WriteHelpTemplate;
begin
  // Run against skeleton.pgo for objects that need a parent
  // --misc Help/scripts/skeleton.pgo dummy
  AssignFile(IndFile,'skel/base.hhk');
  Rewrite(IndFile);
//
  Context := '';
  WriteSkeleton(TPolarization);
  WriteSkeleton(TInterpolatedPartitionFunction);
  WriteSkeleton(TMixture);
  WriteSkeleton(TSpecies);
  WriteSkeleton(TCartesianTransitionMoment);
  WriteSkeleton(TManifold);
  WriteSkeleton(TMolecule);
  WriteSkeleton(TPerturbation);
  WriteSkeleton(TSpecies);
  WriteSkeleton(TSphericalTransitionMoment);
  WriteSkeleton(TNuclearMagneticMoment);
  WriteSkeleton(TState);
  WriteSkeleton(TTransitionMoment);
  WriteSkeleton(TTransitionMoments);
  WriteSkeleton(TNucleus);
  WriteSkeleton(TOverlays);
  WriteSkeleton(TPlot);
  WriteSkeleton(TBitmapPlot);
  WriteSkeleton(TExperimentalPlot);
//
  Context := 'Asymmetric Top';
  WriteSkeleton(TAsymmetricTop,False);
  WriteSkeleton(TAsymmetricTopPerturbation);
// No unique settings  WriteSkeleton(TAsymmetricManifold);
  WriteSkeleton(TAsymmetricMolecule);
  WriteSkeleton(TAsymmetricTopNucleus);
  WriteSkeleton(TAsymmetricTopOrbitalTransitionMoment);
  WriteSkeleton(TAsymmetricTopRotationalTransitionMoment);
//
  Context := 'Symmetric Top';
  WriteSkeleton(TSymmetricTop);
  WriteSkeleton(TSymmetricTopPerturbation);
// TODO:  WriteSkeleton(TSymmetricTopNucleus);
// No unique settings  WriteSkeleton(TSymmetricManifold);
  WriteSkeleton(TSymmetricMolecule);
//

  Context := 'Linear Molecule';
  WriteSkeleton(TLinear);
  WriteSkeleton(TLinearPerturbation);
// No unique settings  WriteSkeleton(TLinearManifold);
  WriteSkeleton(TLinearMolecule);
  WriteSkeleton(TLinearNucleus);
  WriteSkeleton(TOrbitalTransitionMoment);
  WriteSkeleton(TSpinTransitionMoment);
  WriteSkeleton(TRotationalTransitionMoment);
//

// Vib
  Context := 'Vibrating Molecule';
  WriteSkeleton(TVibratingMolecule);
// No unique settings  WriteSkeleton(TVibrationalManifold);
  WriteSkeleton(TElectronicState,False);
  WriteSkeleton(TVibrationalMode,False);
  WriteSkeleton(TMolecularNucleus,False);
  WriteSkeleton(TVibrationalPerturbation);
  WriteSkeleton(TMultidimensionalFCF);
  WriteSkeleton(TVibrationalModeChange,False);

  CloseFile(IndFile);
end;

procedure SkeletonFunction(Param2: string);
begin
  // MakeAsymLineLists(Param2);
  // MakeSymTables;
  MakeSymLineLists(Param2,True);
  // MakennTable(StrToInt(Param2),GlobalMixture.Species[0].Molecule[0]);
  // WriteHelpTemplate;
end;

initialization
//  MakeSymTables;
//  WriteHelpTemplate;
end.
