unit SymUtils;
{$I DFS.inc}{Copyright (C) 2005-9 C M Western; see commandline.pas for license}

interface

uses Classes, MNode, GMarks;

{ Display symmetry key of given MNode; pass Mixture for overall key }
function MarkTypeFor(Sym: Integer):TMarkType;
function SymmetryKey(MNode: TMNode; ToggleJAdjust: Boolean):string;

function MakeSymmetryList(MNode: TMNode): TStringList;
function TryStrToSymmetry(var Symmetry: Integer; TheText: string; SymmetryList: TStringList):Boolean;
function SymmetryToStr(Symmetry: Integer; SymmetryList: TStringList):string;

implementation

uses SysUtils, Math, Mixture;

function MarkTypeFor(Sym: Integer): TMarkType;
begin
  if Ord(mtPlusCircle) + Sym > Ord(LastMark) then
    Result := TMarkType(Sym - (Ord(LastMark) - Ord(mtPlusCircle)) + Ord(FirstMark))
  else
    Result := TMarkType(Sym + Ord(mtPlusCircle))
end;

function SymmetryKey(MNode: TMNode; ToggleJAdjust: Boolean):string;
var
  Molecule: TMolecule;
  Mixture: TMixture;
  Node: TComponent;
  sym, i, j, LastSym, LastSymShown: Integer;
  s1, s2: string;
  NumbersOnly: Boolean;
  DoJAdjust: Boolean;
begin
  Node := MNode;
  Molecule := nil;
  while Assigned(Node) do begin
    if Node is TMolecule then
      Molecule := Node as TMolecule;
    Node := Node.Owner;
  end;
  if Assigned(Molecule) then begin
    { Work out symmetry key for this molecule }
    Result := 'Symmetry';
    DoJAdjust := Molecule.JAdjustSym xor ToggleJAdjust;
    if DoJAdjust then begin
      for i := 0 to Molecule.nNuclei - 1 do
        DoJAdjust := DoJAdjust and (Molecule.Nuclei[i].Spin = 0);
      if DoJAdjust then
        Result := Result + ' (Jeven/Jodd)';
    end;
    Result := Result + ': ';
    for sym := 0 to Molecule.LastSym do begin
      Result := Result + #1 + Char(Ord(MarkTypeFor(sym))) + #0':';
      if DoJAdjust then begin
        s1 := Molecule.JAdjustedSymName(0,sym);
        s2 := Molecule.JAdjustedSymName(2,Molecule.JAdjustedSym(2,Sym));
        if s2 <> s1 then
          s1 := s1 + '/' + s2;
        Result := Result + s1 + ' (';
        s1 := Molecule.SymName(sym);
        s2 := Molecule.SymName(Molecule.JAdjustedSym(2,Sym));
        if s1 <> s2 then
          Result := Result + s1 + '/' + s2
        else
          Result := Result + s1;
        Result := Result + ')'
      end else
        Result := Result + Molecule.SymName(sym);
      Result := Result + ' ';
    end;
  end else begin
    { Check to see of all Molecules have same symmetry name }
    Result := '';
    LastSym := 0;
    LastSymShown := -1;
    NumbersOnly := False;
    Mixture := MNode.RootNode as TMixture;
    for i := 0 to Mixture.nSpecies-1 do
      with Mixture.Species[i] do
        for j := 0 to Count-1 do begin
          LastSym := Max(Isotopomer[j].LastSym,LastSym);
          if not NumbersOnly then
            if Result = '' then
              Result := SymmetryKey(Isotopomer[j],ToggleJAdjust)
            else if Result <> SymmetryKey(Isotopomer[j],ToggleJAdjust) then begin
              { Symmetry keys different - force numeric display of symmetries }
              NumbersOnly := True;
              Result := '';
            end;
          if NumbersOnly then begin
            if Result = '' then
              Result := 'Symmetry:';
            while LastSymShown < LastSym do begin
              Inc(LastSymShown);
              Result := Result + #1 + Char(Ord(MarkTypeFor(LastSymShown))) + #0':'
                               + IntToStr(LastSymShown) + ' ';
            end;
          end;
        end;
  end;
end;

function MakeSymmetryList(MNode: TMNode): TStringList;
var
  i, j, k: Integer;
  Other: TStringList;
begin
  { Return a list of symmetry names for given MNode; returns nil if
    names not in common }
  Result := nil; { Default is no list }
  if not Assigned(MNode) then
    Exit;
  while not (MNode is TMolecule) and Assigned(MNode.Owner) do
    MNode := MNode.Owner as TMNode;
  if MNode is TMolecule then
    with TMolecule(MNode) do begin
      { Easy - just return list of symmetry names }
      Result := TStringList.Create;
      for i := 0 to LastSym do
        Result.Add(JAdjustedSymName(0,i))
    end
  else
    with MNode as TMixture do
      for i := 0 to nSpecies-1 do
        with Species[i] do
          for j := 0 to ComponentCount-1 do
            if Assigned(Result) then begin
              Other := MakeSymmetryList(Isotopomer[j]);
              try
                if Other.Count = Result.Count then
                  for k := 0 to Result.Count - 1 do
                    if Other[k] <> Result[k] then begin
                      FreeAndNil(Result);
                      Exit;
                    end;
              finally
                Other.Free;
              end
            end else
              Result := MakeSymmetryList(Isotopomer[j])
end;

function TryStrToSymmetry(var Symmetry: Integer; TheText: string; SymmetryList: TStringList):Boolean;
var
  i: Integer;
begin
  Result := False;
  if SameText(TheText, 'all') then begin
    Symmetry := -1;
    Result := True;
  end else if Assigned(SymmetryList) then
    for i := 0 to SymmetryList.Count - 1 do
      if CompareText(TheText, SymmetryList[i]) = 0 then begin
        Symmetry := i;
        Result := True;
        Exit;
      end;
end;

function SymmetryToStr(Symmetry: Integer; SymmetryList: TStringList):string;
begin
  if Symmetry = -1 then
    Result := 'all'
  else if Assigned(SymmetryList) and (Symmetry >= 0) and (Symmetry < SymmetryList.Count) then
    Result := SymmetryList[Symmetry]
  else
    Result := '?';
end;

end.
