unit TranInfo;
{$I DFS.inc}{Copyright (C) 2005 C M Western; see commandline.pas for license}
interface

uses
  { Windows, Messages, } SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ChildForm, StdCtrls, AlEdit, EditExtr, NewSpin, ExtCtrls, ComCtrls,
  MNode, Split, Grids, NGrid, Buttons, QuantumSpin, QuantumNumber;

type
  TTransitionForm = class(TChildForm)
    LabelPanel: TPanel;
    Label1: TLabel;
    JLabel: TLabel;
    Label3: TLabel;
    UpperObject: TComboBox;
    UpperJ: TQuantumSpin;
    UpperSymmetry: TSpinBox;
    LowerObject: TComboBox;
    LowerSymmetry: TSpinBox;
    LowerJ: TQuantumSpin;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    ActiveObject: TComboBox;
    OrgPanel: TPanel;
    OriginalCaption: TLabel;
    OrgGrid: TNumberGrid;
    Split: TSplit;
    TranPanel: TPanel;
    Label8: TLabel;
    TranGrid: TNumberGrid;
    Square: TCheckBox;
    Add: TRadioButton;
    Label9: TLabel;
    Replace: TRadioButton;
    Skip: TRadioButton;
    PlotLines: TCheckBox;
    Label10: TLabel;
    JChange: TQuantumSpin;
    Perturbations: TCheckBox;
    OriginalLabel: TLabel;
    TransformedLabel: TLabel;
    SimulateButton: TSpeedButton;
    ExcitationType: TComboBox;
    Label11: TLabel;
    ExcitationRank: TQuantumSpin;
    Label12: TLabel;
    UpperIndex: TSpinBox;
    LowerIndex: TSpinBox;
    AllButton: TSpeedButton;
    OrgHonlLondon: TCheckBox;
    TranHonlLondon: TCheckBox;
    procedure OrgGridGetCellLabel(Sender: TObject; Col, Row: Integer;
      var s: string; var IsText: Boolean);
    procedure AllButtonClick(Sender: TObject);
    procedure UpperJChange(Sender: TObject);
    procedure LowerSymmetrySetIntegerValue(Sender: TObject;
      NewValue: Integer; var NewText: String);
    procedure UpperSymmetryGetIntegerValue(Sender: TObject;
      var NewValue: Integer; TheText: String; var StringValid: Boolean);
    procedure JChangeGetIntegerValue(Sender: TObject;
      var NewValue: Integer; TheText: String; var StringValid: Boolean);
    procedure JChangeSetIntegerValue(Sender: TObject;
      NewValue: Integer; var NewText: String);
    procedure SimulateButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure UpperJIncrement(Sender: TObject; var NewValue: Integer; LastValue,
      n: Integer);
  private
    function ReadJSpinBox(SpinBox: TQuantumSpin):TQuantumNumber;
  protected
    procedure FormConstrainedResize(Sender: TObject; var MinWidth,
      MinHeight, MaxWidth, MaxHeight: Integer);
    procedure ParameterChanged(MNode: TMNode; Index: Integer); override;
    procedure SettingChanged(MNode: TMNode; Index: Integer); override;
    function StuffObject(const Name1, Name2: string; MNode: TMNode; var ParentNode: TTreeNode): Integer; override;
    procedure ShowTransitions(OnlyIfAuto: Boolean; ShowAll:Boolean{ = False});
  public
    procedure Loaded; override;
    function BaseNodeClass: TClass; override;
    procedure ShowItem(MNode: TMNode); override;
    procedure StuffSelector(MNode: TMNode); override;
  end;

var
  TransitionForm: TTransitionForm;

implementation

uses Math, Mixture, MatrixVec, Types,
     Global, LineFrm, Units, DRintensity, Main, FormUtils,
     SettingsForm, Profile, ComplexMatrix;

{$IFNDEF FPC}
{$R *.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

var
  DeleteUnwantedStatesTime: string = 'DeleteUnwantedStates';

{ TTransitionForm }

function TTransitionForm.BaseNodeClass: TClass;
begin
  Result := TMixture;
end;

procedure TTransitionForm.ParameterChanged(MNode: TMNode; Index: Integer);
begin
  ComboBoxChange(ActiveObject);
end;

function TTransitionForm.ReadJSpinBox(SpinBox: TQuantumSpin): TQuantumNumber;
begin
  Result := SpinBox.Value;
  if Result <= SpinBox.MinValue then
    Result := Low(Integer);
end;

procedure TTransitionForm.SettingChanged(MNode: TMNode; Index: Integer);
begin
  ComboBoxChange(ActiveObject);
end;

procedure TTransitionForm.ShowItem(MNode: TMNode);
var
  Molecule: TMolecule;
  TransitionMoments: TTransitionMoments;
  i: Integer;
  Change: TQuantumNumber;
  procedure StuffManifold(Manifold: TManifold; ComboBox: TComboBox);
  var
    i: Integer;
  begin
    if (Manifold.nStates > 0) and (ComboBox.Items.IndexOfObject(Manifold.State[0]) >= 0) then
      Exit;
    if Manifold.nStates <> 1 then begin
      if ComboBox.Items.IndexOfObject(Manifold) >= 0 then { Note External molecule has no states }
        Exit;
      ComboBox.AddItem(Manifold.UniqueName, Manifold);
    end;
    for i := 0 to Manifold.nStates-1 do
      ComboBox.AddItem(Manifold.State[i].UniqueName, Manifold.State[i]);
  end;
  procedure StuffTransitionMoments(TransitionMoments: TTransitionMoments);
  var
    Bra, Ket: TManifold;
  begin
    Bra := TransitionMoments.Bra as TManifold;
    Ket := TransitionMoments.Ket as TManifold;
    if Bra.Initial then
      StuffManifold(Bra, LowerObject)
    else
      StuffManifold(Bra, UpperObject);
    if Ket.Initial then
      StuffManifold(Ket, LowerObject)
    else
      StuffManifold(Ket, UpperObject);
    if Bra.Initial and Ket.Initial then begin
      { Ensure upper state box contains all sensible final states }
      StuffManifold(Ket, UpperObject);
      StuffManifold(Bra, UpperObject);
    end;
  end;
  procedure CheckForMultipleManifolds(ComboBox: TComboBox; JSpinBox: TQuantumSpin; SymSpinBox: TSpinBox);
  var
    i: Integer;
    nManifolds: Integer;
    CurManifold, Manifold: TManifold;
  begin
    nManifolds := 0;
    CurManifold := nil;
    for i := 0 to ComboBox.Items.Count-1 do begin
      if ComboBox.Items.Objects[i] is TManifold then
        Manifold := TManifold(ComboBox.Items.Objects[i])
      else if ComboBox.Items.Objects[i] is TState then
        Manifold := TState(ComboBox.Items.Objects[i]).Manifold
      else
        raise Exception.Create('CheckForMultipleManifolds');
      if Manifold <> CurManifold then begin
        CurManifold := Manifold;
        Inc(nManifolds);
      end;
    end;
    if nManifolds = 0 then
      Exit;
    if nManifolds > 1 then
      ComboBox.Items.InsertObject(0, '(All)', nil);
    ComboBox.ItemIndex := 0;
    if ComboBox.Items.Objects[0] is TManifold then begin
      Manifold := TManifold(ComboBox.Items.Objects[0]);
      { Force symmetry in range }
      UpdateSymmetrySpin(SymSpinBox,Manifold);
    end else if ComboBox.Items.Objects[0] is TState then begin
      Manifold := TState(ComboBox.Items.Objects[0]).Manifold;
      UpdateSymmetrySpin(SymSpinBox,Manifold);
    end else
      UpdateSymmetrySpin(SymSpinBox,Manifold.Mixture);
    i := ReadJSpinBox(JSpinBox); { Save previous value to preserve 'all' status }
    { Force correct even/oddness of J }
    if Manifold.Mixture.Simulations[0].HasFields then
      JSpinBox.MinValue := -JSpinBox.MaxValue - Manifold.HalfSpin
    else
      JSpinBox.MinValue := Manifold.HalfSpin - 2;
    if i = Low(Integer) then
      i := JSpinBox.MinValue;
    i := (Max(i,JSpinBox.MinValue) and not 1) or Manifold.HalfSpin;
    JSpinBox.Value := i;
  end;
begin
  if not Assigned(MNode) then Exit;
  BeginUpdate;
  if MNode is TMolecule then begin
    Molecule := TMolecule(MNode);
    if Molecule.nTransitionMoments = 1 then
      TransitionMoments := Molecule.GetTransitionMoments(0)
    else
      TransitionMoments := nil;
  end else begin
    if MNode is TTransitionMoment then
      TransitionMoments := MNode.Owner as TTransitionMoments
    else
      TransitionMoments := MNode as TTransitionMoments;
    Molecule := TransitionMoments.Molecule;
  end;
  UpperObject.Clear;
  LowerObject.Clear;
  if Assigned(TransitionMoments) then
    StuffTransitionMoments(TransitionMoments)
  else if Molecule.nTransitionMoments = 0 then { No transition moments - appropriate for PopDist }
    for i := 0 to Molecule.nManifolds - 1 do
      StuffManifold(Molecule.Manifold[i],LowerObject)
  else
    for i := 0 to Molecule.nTransitionMoments-1 do
      StuffTransitionMoments(Molecule.GetTransitionMoments(i));
  CheckForMultipleManifolds(UpperObject, UpperJ, UpperSymmetry);
  CheckForMultipleManifolds(LowerObject, LowerJ, LowerSymmetry);
  UpdateComboBox(UpperObject);
  UpdateComboBox(LowerObject);
  FComboBoxUpdated := False;
  { Update possible changes using MaxRank, preserving 'all' if required }
  Change := JChange.Value;
  if Change = JChange.MinValue then
    Change := -100;
  JChange.MinValue := -Molecule.MaxRank-2;
  JChange.MaxValue := Molecule.MaxRank;
  JChange.Value := Max(Min(Change,JChange.MaxValue), JChange.MinValue);
  UpperIndex.Value := UpperIndex.Value; { Force display of all if required }
  LowerIndex.Value := LowerIndex.Value;
  CurrentMNode := MNode;
  ShowTransitions(True,False);
  EndUpdate(False);
end;

procedure TTransitionForm.ShowTransitions(OnlyIfAuto: Boolean; ShowAll:Boolean);
var
  Molecule: TMolecule;
  TransitionMoments: TTransitionMoments;
  TransitionMoment: TTransitionMoment;
  ManifoldU, ManifoldL: TManifold;
  StateU, StateL: TState;
  EigenValuesU, EigenValuesL: TEigenvalues;
  SymU, SymL: Integer;
  LineList: TLineList;
  AddLines, NoUpper: Boolean;
  DeltaJ: TQuantumNumber;
  Excitation: TExcitation;
  Simulation: TSimulation;
  function Extract(var Manifold: TManifold; var State: TState; var Sym: Integer; ComboBox: TComboBox;
                   JSpinBox: TQuantumSpin; SymSpinBox: TSpinBox; OtherJ, DeltaJ: TQuantumNumber): TEigenvalues;
  var
    MNode: TMNode;
    J: TQuantumNumber;
    Matrices: TComplexMatrices;
    I: Integer;
  begin
    Result := nil;
    State := nil;
    Manifold := nil;
    Sym := -1;
    if ComboBox.Items.Count = 0 then
      Exit;
    MNode := ComboBox.Items.Objects[ComboBox.ItemIndex] as TMNode;
    if MNode is TManifold then
      Manifold := TManifold(MNode)
    else if MNode is TState then begin
      State := TState(MNode);
      Manifold := State.Manifold;
    end;
    Sym := SymSpinBox.Value;
    if not Simulation.HasFields and (Sym < 0) then Exit;
    J := ReadJSpinBox(JSpinBox);
    if (J = Low(Integer)) and (OtherJ <> Low(Integer)) and (Abs(DeltaJ) < 100) then
      J := OtherJ + DeltaJ;
    if (J = Low(Integer)) or (not Simulation.HasFields and (J < 0)) then
      Exit;
    if Assigned(Manifold) then begin
      if Odd(J+Manifold.HalfSpin) then Exit;
      Result := TEigenvalues.Create(Manifold, J, Manifold.Molecule.JAdjustedSym(J,Sym), Simulation.ExField, True, State, 0);
      if Assigned(Result) and (Result.Size > 0) then
        try
          Result.MakeHamiltonian(Matrices, Manifold.Molecule.BlockMatrix, Manifold.Mixture.BasisOrder); { TODO: Make checkbox }
          Result.Diagonalize(Matrices, True, Manifold.EigenSearch, Manifold.LimitSearch);
        except
          for I := 0 to High(Matrices) do
            Matrices[I].Free;
          Result.Free;
          raise;
        end
      else
        FreeAndNil(Result);
    end;
  end;
  procedure ShowMatrix(D: TComplexMatrix; Grid: TNumberGrid; ShowBasis, Square: Boolean; Scale: Double);
  var
    i: Integer;
    function SLabel(i: Integer; Ev: TEigenValues): string;
    begin
      if ShowBasis then
        Result := Ev.Basis[i].BasisLabel
      else
        Result := Ev.DominantBasis[i].StateLabel(Ev.Manifold.Mixture.QuantumNumberFormat);
    end;
  begin
    Grid.RowCount := Max(EigenValuesL.Size,1)+1;
    Grid.ColCount := Max(EigenValuesU.Size,1)+1;
    for i := 1 to EigenValuesU.Size do begin
      Grid.Title[i] := SLabel(i, EigenValuesU);
      if EigenValuesL.Size = 0 then
        Grid.Valid[1, i] := False;
    end;
    if EigenValuesL.Size = 0 then
      Grid.Lbl[1] := ' ';
    for i := 1 to EigenValuesL.Size do begin
      Grid.Lbl[i] := SLabel(i, EigenValuesL);
      if EigenValuesU.Size = 0 then
        Grid.Valid[i, 1] := False;
    end;
    ShowComplexMatrixInGrid(D, Grid, 0, Grid.Width < OrgPanel.Width, Square, False, 0, Scale);
    if EigenValuesU.Size = 0 then
      Grid.Title[1] := ' ';
    if (EigenValuesU.Sym < 0) or (EigenValuesL.Sym < 0) then
      Grid.Caption := Format('<%s||%s>', [EigenValuesL.Manifold.UniqueName, EigenValuesU.Manifold.UniqueName])
    else
      Grid.Caption := Format('<%s||%s>', [Molecule.BothSymNames(EigenValuesL.J, EigenvaluesL.Sym),
                                          Molecule.BothSymNames(EigenValuesU.J, EigenvaluesU.Sym)]);
    Grid.AutoAdjustColumn(0);
  end;
  procedure ShowMatrices;
  var
    D, F: TComplexMatrix;
    s: string;
    Factor: Double;
  begin
    F := nil;
    D := nil;
    if (EigenValuesL.Size > 0) and (EigenValuesU.Size >0)
       and (EigenValuesL.HasFields
            or (EigenValuesL.StatisticalWeight(1) = EigenValuesU.StatisticalWeight(1)))
     then
      if Assigned(TransitionMoment)
         or (Assigned(TransitionMoments) and (TransitionMoments.nRanks = 1)) then begin
        D := TransitionMoments.OriginalTransitionMatrix(EigenValuesL, EigenValuesU, TransitionMoments.MaxRank, TransitionMoment, True, True, True, Simulation);
        F := D;
        OrgHonlLondon.Enabled := True;
        TranHonlLondon.Enabled := True;
        if Assigned(TransitionMoment) then
          Factor := Sqrt(TransitionMoment.Rank+1)
        else
          Factor := Sqrt(TransitionMoments.TransitionMoment[0].Rank+1); { Must be all same rank }
      end else if Assigned(TransitionMoments) then
        D := TransitionMoments.TransitionMatrix(EigenValuesL, EigenValuesU, True, False, nil, Simulation);
    try
      ShowMatrix(F, OrgGrid, True, False, IfThen(OrgHonlLondon.Checked,Factor,1));
      s := '< Lower ';
      if EigenValuesL.Size > 0 then
        s := s + EigenValuesL.Basis[1].BasisLabelKey;
      s := s + ' |T| Upper ';
      if EigenValuesU.Size > 0 then
        s := s + EigenValuesU.Basis[1].BasisLabelKey;
      s := s + ' >';
      OriginalLabel.Caption := s;
      OriginalCaption.Caption := 'Original Transition Matrix';
      if Assigned(F) and (D.Size > 0) then begin
        TransitionMoments.TransformTransitionMatrix(D, EigenValuesL, EigenValuesU);
        Square.Enabled := True;
        s := '< Lower ';
        if EigenValuesL.Size > 0 then
          s := s + EigenValuesL.Basis[1].StateLabelKey;
        s := s + ' |T| Upper ';
        if EigenValuesU.Size > 0 then
          s := s + EigenValuesU.Basis[1].StateLabelKey;
        s := s + ' >';
        TransformedLabel.Caption := s;
      end else begin
        Square.Enabled := False;
        Square.Checked := True;
      end;
      ShowMatrix(D, TranGrid, False, Square.Checked, IfThen(TranHonlLondon.Enabled and TranHonlLondon.Checked,Factor,1));
    finally
      D.Free;
    end;
  end;
  procedure CalculateMatrices; { If transition selection sufficiently limited,
                                 show transition matrixes }
  var
    DeltaJ: TQuantumNumber;
    procedure ShowMatrix(Eigenvalues: TEigenvalues; Name: string);
    begin
      Split.OneWindowVisible(OrgPanel);
      Split.Visible := False;
      OrgGrid.Caption := '';
      OrgGrid.OnGetCellLabel := {$IFDEF FPC}@{$ENDIF}OrgGridGetCellLabel;
      OriginalCaption.Caption := Name + ShowEigenvaluesInGrid(Eigenvalues, OrgGrid, OriginalLabel);
    end;
  begin
    DeltaJ := JChange.Value;
    if DeltaJ = JChange.MinValue then
      DeltaJ := -100;
    OrgHonlLondon.Enabled := False;
    TranHonlLondon.Enabled := False;
    EigenvaluesU := Extract(ManifoldU, StateU, SymU, UpperObject, UpperJ, UpperSymmetry, ReadJSpinBox(LowerJ), DeltaJ);
    try
      EigenvaluesL := Extract(ManifoldL, StateL, SymL, LowerObject, LowerJ, LowerSymmetry, ReadJSpinBox(UpperJ), -DeltaJ);
      try
        if Assigned(EigenvaluesU) and Assigned(EigenvaluesL) then begin
          if not Assigned(TransitionMoments) then
            TransitionMoments := Molecule.FindTransitionMoments(EigenValuesU.Manifold, EigenValuesL.Manifold);
          Split.Visible := True;
          OrgGrid.OnGetCellLabel := nil;
          ShowMatrices;
        end else if Assigned(EigenValuesU) then
          ShowMatrix(EigenValuesU, 'Upper')
        else if Assigned(EigenValuesL) then
          ShowMatrix(EigenValuesL, 'Lower')
        else begin
          OrgGrid.Blank;
          TranGrid.Blank;
        end;
      finally
        EigenValuesL.Free;
      end;
    finally
      EigenValuesU.Free;
    end;
  end;

  procedure DeleteUnwantedStates;
  begin { Remove unselected states }
    if (LowerIndex.Value <= 0) and (UpperIndex.Value <= 0) then
      Exit;
    { TODO: This needs a rethink - Upper/Lower selection uses Manifold.Initial elsewhere }
    { TODO: This test will fail for overlapping manifolds }
    if not (Assigned(ManifoldL) and Assigned(ManifoldU)) then
      raise EReportException.Create('Manifold must be selected for eigenvalue selection');
    if ManifoldL.LowestStateOrigin > ManifoldU.LowestStateOrigin then
      LineList.DeleteSelected(LowerIndex.Value, UpperIndex.Value)
    else
      LineList.DeleteSelected(UpperIndex.Value,LowerIndex.Value);
  end;
var
  Problem: Exception;
  r: Integer;
begin
  if (csLoading in ComponentState) or not Visible then
    Exit;
  Caption := 'Selected Transitions';
  if CurrentMNode = nil then Exit;
  TransitionMoment := nil;
  if CurrentMNode is TMolecule then begin
    Molecule := TMolecule(CurrentMNode);
    if Molecule.nTransitionMoments = 1 then
      TransitionMoments := Molecule.GetTransitionMoments(0)
    else
      TransitionMoments := nil;
  end else begin
    if CurrentMNode is TTransitionMoment then begin
      TransitionMoment := TTransitionMoment(CurrentMNode);
      TransitionMoments := CurrentMNode.Owner as TTransitionMoments;
    end else
      TransitionMoments := CurrentMNode as TTransitionMoments;
    Molecule := TransitionMoments.Molecule;
  end;
  Simulation := Molecule.Mixture.Simulations[0];
  if Simulation.HasFields then
    JLabel.Caption := '&M'
  else
    JLabel.Caption := '&J';
  try
    if Assigned(BaseNode) then
      BaseNode.CheckProperties;
    JChange.CheckCurrentValue;
    ExcitationRank.CheckCurrentValue;
    UpperJ.CheckCurrentValue;
    UpperSymmetry.CheckCurrentValue;
    UpperIndex.CheckCurrentValue;
    LowerJ.CheckCurrentValue;
    LowerSymmetry.CheckCurrentValue;
    LowerIndex.CheckCurrentValue;
  except
    on E: Exception do begin
      OriginalLabel.Caption := E.Message;
      Exit;
    end;
  end;
  Caption := CurrentMNode.UniqueName + ' - ' + Caption;
  BeginUpdate;
  Molecule.CheckProperties;
  Molecule.RootNode.FixupProperties.Free; { Check Mixture also to acount for e.g. spin changes }
  try
    if OnlyIfAuto and not Molecule.Mixture.AutoReplot then
      Exit;
    CalculateMatrices;
    NoUpper := UpperObject.Items.Count = 0;
    if ((ReadJSpinBox(LowerJ) <> Low(Integer)) and Odd(LowerJ.Value + Molecule.HalfSpin))
       or (not NoUpper and ((ReadJSpinBox(UpperJ) <> Low(Integer)) and Odd(UpperJ.Value + Molecule.HalfSpin))) then
      Exit; { Invalid J }
    AddLines := Add.Checked or Replace.Checked;
    if AddLines or PlotLines.Checked then begin
      if Replace.Checked then
        LineForm.DeleteUnassignedClick(Self);
      LineList := TLineList.Create(Simulation.PlotUnits,IsPopDist(Simulation.IntensityUnits));
      try
        DeltaJ := JChange.Value;
        if NoUpper or (DeltaJ = JChange.MinValue) then
          DeltaJ := -10000;
        Excitation := RandomExcitation;
        Excitation.Rank := ExcitationRank.Value;
        Excitation.Simple := ExcitationType.ItemIndex <= 0;
        if not Excitation.Simple then begin
          { TODO : Make more general }
          Excitation.DeltaJ := ExcitationType.ItemIndex div 2;
          if Odd(ExcitationType.ItemIndex) then
            Excitation.DeltaJ := -Excitation.DeltaJ;
        end;
        Problem := nil;
        Molecule.AddToSpectrum(LineList, plNone, Excitation, Problem, Simulation,
                               ReadJSpinBox(LowerJ), SymL,
                               ReadJSpinBox(UpperJ), SymU,
                               ManifoldL, ManifoldU,
                               DeltaJ, TransitionMoments, TransitionMoment,
                               StateL, StateU, Perturbations.Checked);
        if Assigned(Problem) then begin
          Caption := Problem.Message;
          Problem.Free;
        end;
        TStart(DeleteUnwantedStatesTime);
        DeleteUnwantedStates;
        TEnd(DeleteUnwantedStatesTime);
        if AddLines and (ReadJSpinBox(LowerJ) = Low(Integer)) and (ReadJSpinBox(UpperJ) = Low(Integer)) and (LineList.nLines > 20) then begin
          r := MessageDlg(Format('Add %d lines to line list?', [LineList.nLines]), mtConfirmation, [mbYes, mbNo, mbNoToAll], 0);
          AddLines := r = mrYes;
          if r = mrNoToAll then
            Skip.Checked := True;
        end;
        if AddLines then begin
          LineList.FindLines(0, 1e99, LineList.Units);
          LineForm.ShowLines(Molecule.Mixture.Simulations[0],False);
          SetFocus;
        end;
        LineList.Selected := True;
      except
        LineList.Free;
        raise;
      end;
      if PlotLines.Checked then begin
        Molecule.Mixture.LineList := LineList;
        if ShowAll then
          MainForm.FindLimits;
        MainForm.ShowSimulation;
      end else
        FreeAndNil(LineList);
    end;
  finally
    EndUpdate(False);
  end;
end;

function TTransitionForm.StuffObject(const Name1, Name2: string; MNode: TMNode; var ParentNode: TTreeNode): Integer;
begin
  Result := 0;
  if (MNode is TTransitionMoment)
     or ((MNode is TTransitionMoments) { and (TTransitionMoments(MNode).Count > 1)} ) { ExternalMolecule transition moments have no states }
     or ((MNode is TMolecule) and (TMolecule(MNode).nTransitionMoments <>1)) then begin
    ActiveObject.AddItem(Name1 + Name2, MNode);
    Result := ActiveObject.Items.Count-1;
  end;
end;

procedure TTransitionForm.StuffSelector(MNode: TMNode);
begin
  {$IFNDEF LCL_}
  OnConstrainedResize := FormConstrainedResize;
  {$ENDIF}
  UpperObject.Clear;
  LowerObject.Clear;
  StuffSelectorHelper(MNode,ActiveObject);
end;

procedure TTransitionForm.UpperJChange(Sender: TObject);
begin
  if Lockout > 0 then Exit;
  ShowTransitions(True,False);
end;

procedure TTransitionForm.UpperJIncrement(Sender: TObject; var NewValue: Integer; LastValue, n: Integer);
var
  Mv: Integer;
begin
  Mv := (Sender as TSpinBox).MinValue;
  if (Mv < -2) and (NewValue <= Mv+2) then
    NewValue := (2 - (Mv and 1))*n; { Stepping from all when negative values are allowed starts from 0 }
  if Odd(NewValue+Mv) then
    Inc(NewValue); { Force reasonable spin }
end;

procedure TTransitionForm.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  SaveComboBox(UpperObject);
  SaveComboBox(LowerObject);
end;

procedure TTransitionForm.FormConstrainedResize(Sender: TObject;
  var MinWidth, MinHeight, MaxWidth, MaxHeight: Integer);
begin
  MinHeight := ClientToScreen(Point(0,0)).Y - Top + LabelPanel.Height;
end;

procedure TTransitionForm.Loaded;
begin
  inherited;
  UpdateSymmetrySpin(UpperSymmetry, nil);
  UpdateSymmetrySpin(LowerSymmetry, nil);
  OrgGrid.CopyLabels := True;
  TranGrid.CopyLabels := True;
end;

procedure TTransitionForm.LowerSymmetrySetIntegerValue(Sender: TObject; NewValue: Integer; var NewText: String);
begin
  with Sender as TSpinBox do
    if NewValue <= MinValue + Increment div 2 then
      NewText := 'all';
end;

procedure TTransitionForm.UpperSymmetryGetIntegerValue(Sender: TObject;
  var NewValue: Integer; TheText: String; var StringValid: Boolean);
begin
  if SameText(TheText, 'all') or (TheText = '') then begin
    NewValue := (Sender as TSpinBox).MinValue;
    StringValid := True;
  end;
end;

procedure TTransitionForm.JChangeGetIntegerValue(
  Sender: TObject; var NewValue: Integer; TheText: String;
  var StringValid: Boolean);
var
  Change: Integer;
begin
  if SameText(TheText, 'all') or (TheText = '') then begin
    NewValue := (Sender as TSpinBox).MinValue;
    StringValid := True;
  end else if (Length(TheText) = 1) then begin
    Change := (Ord(LowerCase(TheText)[1])-Ord('q'))*2;
    if Abs(Change) <= 18 then begin
      NewValue := Change;
      StringValid := True;
    end;
  end;
end;

procedure TTransitionForm.JChangeSetIntegerValue(
  Sender: TObject; NewValue: Integer; var NewText: String);
begin
  if NewValue = (Sender as TSpinBox).MinValue then
    NewText := 'all'
  else
    NewText := Char((NewValue div 2) + Ord('Q'));
end;

procedure TTransitionForm.SimulateButtonClick(Sender: TObject);
begin
  ShowTransitions(False,False);
end;

procedure TTransitionForm.AllButtonClick(Sender: TObject);
begin
  ShowTransitions(False,True);
end;

procedure TTransitionForm.OrgGridGetCellLabel(Sender: TObject; Col,
  Row: Integer; var s: string; var IsText: Boolean);
begin
  if Row > 0 then
    if Col = 0 then
      s := IntToStr(Row)
    else if Col = 1 then begin
      s := OrgGrid.Lbl[Row];
      IsText := True;
    end;
end;

end.
