#!/bin/bash
# Create pgopher.app package for Mac OS X
# 
# Based on script from:
# Copyright (C) 2006 Jochen Kpper

#set -x
#set -v

CPU=`fpc -iTP`
OS=`fpc -iTO`
appname=pgopher

appfile=$HOME/$appname/$CPU-$OS/$appname

extraname=slave
extrafile=../../slave/$CPU-$OS/$extraname

appfolder=/tmp/$appname.app
macosfolder=$appfolder/Contents/MacOS
plistfile=$appfolder/Contents/Info.plist

libraries=

# Find Fink/Ports libraries
function addlibrariesfor () {
  l=`otool -L $1 | awk "{print \\$1}" | grep -v "/usr/" | grep -v "/System/" | grep -v \:`
  for lib in $l; do
    if [[ "$libraries" != *\ $lib\ * ]] ; then
      echo "adding $lib"
      libraries="$libraries $lib "
    fi
  done
}

addlibrariesfor $appfile
addlibrariesfor $extrafile

oldlibraries=
while [ "$libraries" != "$oldlibraries" ] ; do
  oldlibraries=$libraries
  for ll in $libraries; do
    addlibrariesfor $ll
  done
done

if ! [ -e $appfile ]; then
    echo "$appfile does not exist"
elif [ -e $appfolder ]; then
    echo "$appfolder already exists"
else
    mkdir $appfolder
    mkdir $appfolder/Contents
    mkdir $appfolder/Contents/Frameworks
    mkdir $appfolder/Contents/MacOS
    mkdir $appfolder/Contents/Resources

    # copy executable
    install -c -m 755 $appfile $macosfolder/$appname
    chmod 755 $macosfolder/$appname

    # copy slave
    install -c -m 755 $extrafile $macosfolder/$extraname
    chmod 755 $macosfolder/$extraname

    # copy dynamic libraries
    for library in $libraries; do
        lib=`basename $library`
        install -c -m 755 $library $appfolder/Contents/Frameworks/$lib
        # change dylib search path in executable and libraries
        install_name_tool -change $library @executable_path/../Frameworks/$lib $macosfolder/$appname
        install_name_tool -change $library @executable_path/../Frameworks/$lib $macosfolder/$extraname
        for lbry in $libraries; do
            lb=`basename $lbry`
#            echo install_name_tool -change $lbry @executable_path/../Frameworks/$lb $appfolder/Contents/Frameworks/$lib
            install_name_tool -change $lbry @executable_path/../Frameworks/$lb $appfolder/Contents/Frameworks/$lib
        done
    done

    # Create PkgInfo file.
    echo "APPL????" >$appfolder/Contents/PkgInfo

    # Create information property list file (Info.plist).
    cp Info.plist $plistfile

    # Icon
    cp ../pgopher.icns $appfolder/Contents/Resources/

    cd /tmp
    zip -r pgopher-MacOSX-$CPU.app.zip `basename $appfolder`
fi
