unit AirVac;
{$I DFS.inc}{ Copyright (C) 2005 C M Western; see UCompat.pas for license}

interface

function Refrac(s:Double):Double; { Refractive index of air at s (in cm-1) }
function Air(a:Double):Double; { Return air wavelength/nm given vacuum wavenumber/cm-1 }
function Vac(a:Double):Double; { Return vacuum wavenumber/cm-1 given air wavelength/nm }
function nmcm1(a:Double):Double;   { Interconvert between nm and cm-1 (in vacuum), limiting large values }
function nQuartz(nm:Double):Double; { Refractive index of quartz at given wavelength/nm }

const
  { Limit possible cm-1 and nm range to physically reasonable values }
  Max_nm_cm1: Double = 1e20;

implementation

{ TODO: Add reference for source of the formula }

function Refrac(s:Double):Double; { Refractive index of air at s (in cm-1) }
var
  n: Double;
begin
  n := 1e-8*s*s;
  n := 8342.13 + 2406030./(130-n) + 15997/(38.9-n);
  Result := 1 + 1e-8*n;
end;

function nmcm1(a:Double):Double;
begin
  { Interconvert between nm and cm-1 (in vacuum), limiting large values }
  if Abs(a) < 1e7/Max_nm_cm1 then
    if a < 0 then
      Result := -Max_nm_cm1
    else
      Result := Max_nm_cm1
  else
    Result := 1e7/a;
end;

function Air(a:Double):Double;
begin
  if Abs(a) < 1e7/Max_nm_cm1 then
    Result := nmcm1(a)
  else
    Result := 1e7/a/Refrac(a);
end;

function Vac(a:Double):Double;
begin
  if Abs(a) < 1e7/Max_nm_cm1 then
    Result := nmcm1(a)
  else begin
    Result := refrac(1e7/a);
    Result := refrac(1e7/a/Result);
    Result := refrac(1e7/a/Result);
    Result := refrac(1e7/a/Result);
    Result := 1e7/a/Result;
  end;
end;

{ Values from:
  "The Refractive Index of Quartz" W. R. C. Coode-Adams
  Proceedings of the Royal Society of London. Series A, 117, 209 (1927)
  Stable URL: http://www.jstor.org/stable/94781 }
function nQuartz(nm:Double):Double; { Refractive index of quartz at given wavelength/nm }
begin
  nm := Sqr(nm/1000);
  Result := 3.53445 + 0.008067/(nm-0.0127493) + 0.002682/(nm-0.000974) + 127.2/(nm-108);
  Result := Sqrt(Result);
end;

end.
