unit MathSup;
{$I DFS.inc}{ Copyright (C) 2005 C M Western; see UCompat.pas for license}
interface

type
  { Complex type - compatible with fpc type, but define our own so it works with Delphi also }
  Complex = record
    re : real;
    im : real;
  end;
  PComplex = ^Complex;
const
  z0: Complex = (re:0; im:0);
  z1: Complex = (re:1; im:0);

function SameValue(const za,zb: Complex; const Epsilon: Double = 0):Boolean; overload;
procedure CMul(out r: Complex; const z: Complex; const a: Double); overload;
procedure CMul(out r: Complex; const za, zb: Complex); overload;
procedure CMul(var z: Complex; const a: Double); overload;
procedure CAdd(out r: Complex; const za, zb: Complex); overload;  inline;
procedure CAdd(var r: Complex; const za: Complex); overload; inline;
function SqrMod(const z: Complex):Double;
function IsZero(const z: Complex):Boolean; overload;
function IsEqual(const za, zb: Complex):Boolean;

type
  TSingleArray = array of Single;
  TSingleArrayArray = array of TSingleArray;
  TDoubleArray = array of Double;
  TDoubleArrayArray = array of TDoubleArray;
  TIntegerArray = array of Integer; { Needed to avoid some type problems }
  TShortIntArray = array of Shortint;
  TCardinalArray = array of Cardinal;
  TInt64Array = array of Int64;
  TComplexArray = array of Complex;
  TBooleanArray = array of Boolean;

  TDPoint = record
    X: Double;
    Y: Double;
  end;
  TDPointArray = array of TDPoint;

  TDRect = record
    case Integer of
      0: (Left, Top, Right, Bottom: Double);
      1: (TopLeft, BottomRight: TDPoint);
  end;

{$IFNDEF DFS_COMPILER_6_UP}
 {$IFNDEF FPC}
  {$DEFINE OLD_NAN}
 {$ENDIF}
{$ENDIF}

{$IFDEF OLD_NAN}
{ function IsNan(var v:Double):Boolean; }
{$ENDIF}
{$IFNDEF FPC}
{$DEFINE FPC_HAS_TYPE_EXTENDED}
{$ENDIF}
{$IFDEF FPC_HAS_TYPE_EXTENDED}
procedure SetNan(out v:Extended); overload;
{$ENDIF}
procedure SetNan(out v:Double); overload;
procedure SetNan(out v:Single); overload;
function DtosErr(Value: Double;        { The value to print out }
                 Error: Double;        { its error }
                 nErrDigits: Integer;  { The number of digits of the error to print;
                                         <0 for normal error display but with given siginficant figures in Error }
                 MaxPrec: Integer = -1 { The maximum precision to print }
                ):string;
function DtosM(v:Double; Width, Fixed: Integer):string;
function DtosMA(v:Double; Width, Fixed: Integer):string;
function AtLeast(Value, Spaces: Integer):string; overload;
function AtLeast(Value: Double; Spaces, Places: Integer):string; overload;
function IsNanOrInf(const v:Double): Boolean;
function RoundToIncrement(Value, Increment: Extended): Extended;
function RoundedStr(Value, Increment: Double): string;
{$IFDEF FPC_HAS_TYPE_EXTENDED}
function TryStrToFloatEx(const S: string; out Value: Extended):Boolean; overload;
{$ENDIF}
function TryStrToFloatEx(const S: string; out Value: Double):Boolean; overload;
function StrToFloatEx(const S: string):Extended;

procedure Limits(const a: array of Double; var Mi, Mx: Double; First:Integer = 0; Last: Integer = Low(Integer)); overload;
procedure Limits(const a: array of Integer; var Mi, Mx: Integer; First:Integer = 0; Last: Integer = Low(Integer)); overload;
procedure Limits(const a: array of Cardinal; var Mi, Mx: Cardinal; First:Integer = 0; Last: Integer = Low(Integer)); overload;

procedure InOrder(var Mi, Mx: Double);  overload;
procedure InOrder(var Mi, Mx: Integer); overload;
function Within(const v, Mi, Mx: Double): Boolean; overload;
function Within(const v, Mi, Mx: Integer): Boolean; overload;

{ Constants to assist in forcing evalution as Double }
const
  One:Double = 1;
  Two:Double = 2;
  Three:Double = 3;
  Four:Double = 4;
  Six:Double = 6;

implementation

uses Math, SysUtils;
{
#define nan8(v) ( (*((short *)&(v)+3) & 0x7ff0) == 0x7ff0 )
#define nan4(v) ( (*((short *)&(v)+1) & 0x7f80) == 0x7f80 )
}
{$IFNDEF VER80}
{$IFDEF OLD_NAN}
type
  PWord = ^Word;
{$ENDIF}

{$IFDEF OLD_NAN}
{ This test is actually IsNan() or IsInfinite()
function IsNan(var v:Double):Boolean;
begin
  Result := ( PWord(PChar(@v)+6)^ and $7ff0) = $7ff0;
end; }
{$ENDIF}

{$IFDEF FPC_HAS_TYPE_EXTENDED}
procedure SetNan(out v:Extended);
begin
  {$IFDEF FPC}
  {$IFDEF FPC_BIG_ENDIAN}
    PWord(PChar(@v))^ := $7fff; { TODO : Revert when next line works }
    PWord(PChar(@v)+2)^ := $7fff; { TODO : Revert when next line works }
  {$ELSE}
    PWord(PChar(@v)+8)^ := $7fff; { TODO : Revert when next line works }
    PWord(PChar(@v)+6)^ := $7fff; { TODO : Revert when next line works }
  {$ENDIF}
  {$ELSE}
  v := Nan;
  {$ENDIF}
end;
{$ENDIF}

function IsNanOrInf(const v:Double): Boolean;
begin
  {$IFDEF FPC_BIG_ENDIAN}
    Result := ( PWord(PChar(@v))^ and $7ff0 ) = $7ff0;
  {$ELSE}
    Result := ( PWord(PChar(@v)+6)^ and $7ff0 ) = $7ff0;
  {$ENDIF}
end;

procedure SetNan(out v:Double);
begin
  {$IFDEF OLD_NAN}
  PWord(PChar(@v)+6)^ := $7ff1;
  {$ELSE}
  {$IFDEF FPC}
  {$IFDEF FPC_BIG_ENDIAN}
    PWord(PChar(@v))^ := $7ff1; { TODO : Revert when next line works }
  {$ELSE}
    PWord(PChar(@v)+6)^ := $7ff1; { TODO : Revert when next line works }
  {$ENDIF}
  {$ELSE}
  v := Nan;
  {$ENDIF}
  {$ENDIF}
end;

procedure SetNan(out v:Single);
begin
  {$IFDEF OLD_NAN}
  PWord(PChar(@v)+2)^ := $7f90;
  {$ELSE}
  {$IFDEF FPC}
  {$IFDEF FPC_BIG_ENDIAN}
    PWord(PChar(@v))^ := $7f90; { TODO : Revert when next line works }
  {$ELSE}
    PWord(PChar(@v)+2)^ := $7f90; { TODO : Revert when next line works }
  {$ENDIF}
  {$ELSE}
  v := Nan;
  {$ENDIF}
  {$ENDIF}
end;

{$ELSE}
function IsNanOrInf(var v:Double):Boolean; assembler;
asm
  les bx, v
  mov ax,es:[bx+6]
  and ax,$7ff0
  sub ax,$7ff0
  je @l1
  mov ax,-1
@l1:
  inc ax
end;

procedure SetNan(var v:Double); assembler;
asm
  les bx, v
  mov ax,$7ff0
  mov es:[bx+6],ax
end;
{$ENDIF}

function SameValue(const za,zb: Complex; const Epsilon: Double = 0):Boolean;
begin
  Result := Math.SameValue(za.re,zb.re,Epsilon) and Math.SameValue(za.im,za.im,Epsilon);
end;

procedure CMul(out r: Complex; const z: Complex; const a: Double);
begin
  r.re := z.re * a;
  r.im := z.im * a;
end;

procedure CMul(out r: Complex; const za, zb: Complex); overload;
var
  T: Double; { Allow r in same place as za or zb }
begin
  T := za.re*zb.re - za.im*zb.im;
  r.im := za.im*zb.re + za.re*zb.im;
  r.re := T;
end;

procedure CAdd(var r: Complex; const za: Complex);
begin
  r.re := r.re + za.re;
  r.im := r.im + za.im;
end;

procedure CMul(var z: Complex; const a: Double); overload;
begin
  z.re := z.re*a;
  z.im := z.im*a;
end;

procedure CAdd(out r: Complex; const za, zb: Complex); inline;
begin
  r.re := za.re + zb.re;
  r.im := za.im + zb.im;
end;

function SqrMod(const z: Complex):Double;
begin
  Result := z.re*z.re + z.im*z.im;
end;

function IsZero(const z: Complex):Boolean;
begin
  Result := (z.re = 0) and (z.im = 0);
end;

function IsEqual(const za, zb: Complex):Boolean;
begin
  Result := (za.re = zb.re) and (za.im = zb.im);
end;

function TidyFloatToStr(Value: Extended): string;
var
  p: Integer;
begin
  Result := FloatToStr(Value);
  p := Pos('.E', Result);
  if p > 0 then
    Delete(Result, p, 1);
end;

{ Display the error in a number in units of the last significant figure, such as 0.132(45) }
function DtosErr(Value: Double;        { The value to print out }
                 Error: Double;        { its error }
                 nErrDigits: Integer;  { The number of digits of the error to print;
                                         <0 for normal error display but with given siginficant figures in Error }
                 MaxPrec: Integer = -1 { The maximum precision to print }
                ):string;
var
  Scale: Extended;
  PowerOfTen, nDigits: Integer;
  Body, Err, Exponent: string;
  Negative: Boolean;
begin
  if MaxPrec < 0 then
    MaxPrec := 15;
  if nErrDigits < -9 then
    nErrDigits := -9
  else if nErrDigits > 9 then
    nErrDigits := 9
  else if nErrDigits = 0 then
    nErrDigits := 1;
  PowerOfTen := MaxInt;
  if not IsNanOrInf(Error) and (Error <> 0) then begin
    PowerOfTen := Floor(Log10(Abs(Error))) - Abs(nErrDigits) + 1;
    Scale := IntPower(10,PowerOfTen);
  end;
  if (PowerOfTen = MaxInt) or (Abs(Value/Scale) >= High(Int64)) then begin
    { Error is smaller than would show in MaxPrec digits, so limit number of digits displayed }
    if (PowerOfTen = MaxInt) or (Value = 0) then begin
      Scale := 1; { Problem - force simple display }
      nErrDigits := -1;
    end else begin
      PowerOfTen := Floor(Log10(Abs(Value)))-MaxPrec+1;
      Scale := IntPower(10,PowerOfTen);
    end;
  end;
  Body := Trim(IntToStr(Round(Value/Scale))); { Number with correct number of significant figures as integer }
  nDigits := Length(Body);
  if Body[1] = '-' then
    Dec(nDigits);
  if (nDigits > MaxPrec) and (PowerOfTen <> MaxInt) then begin
    Inc(PowerOfTen, nDigits-MaxPrec);
    Scale := IntPower(10,PowerOfTen);
    Body := Trim(IntToStr(Round(Value/Scale))); { Number with correct number of significant figures as integer }
  end;
  if nErrDigits > 0 then begin
    Negative := Body[1] = '-';
    if Negative then
      Delete(Body,1,1);
    if IsNanOrInf(Error) then
      Err := FloatToStr(Error)
    else
      Err := Trim(IntToStr(Round(Error/Scale)));  { Error string as integer }
    Exponent := '';
    if PowerOfTen > 0 then begin
      if Length(Body) > 1 then begin
        Inc(PowerOfTen, Length(Body)-1);
        Insert('.',Body,2);
      end;
      Exponent :=  'e' + IntToStr(PowerOfTen);
    end else if PowerOfTen = 0 then
      { }
    else if Length(Body) + PowerOfTen > 0 then
      Insert('.',Body,Length(Body) + PowerOfTen+1)
    else if Length(Body) + PowerOfTen > -2 then
      Body := '0.' + StringOfChar('0',-(Length(Body)+PowerOfTen)) + Body
    else begin
      Exponent := 'e' + IntToStr(Length(Body) - 1 + PowerOfTen);
      Insert('.',Body,2);
    end;
    if Body[Length(Body)] = '.' then
      Delete(Body,length(Body),1);
    Result := Body + '(' + Err + ')' + Exponent;
    if Negative then
      Result := '-' + Result;
  end else begin
    if Scale <> 1 then
      Result := TidyFloatToStr(Round(Value/Scale)*Scale) + ' '
    else
      Result := TidyFloatToStr(Value) + ' ';
    if IsNanOrInf(Error) then
      Result := Result + TidyFloatToStr(Error)
    else
      Result := Result + TidyFloatToStr(Round(Error/Scale)*Scale)
  end;
end;

function DtosMA(v:Double; Width, Fixed: Integer):string;
{ String representation of v in Width characters if possible. The result
  will always include a leading space.
  If Fixed < 0:
    Designed to work with normally left aligned columns,
    so any additional spaces required are added on the right.
  If Fixed >= 0:
    Designed to work with normally right aligned columns,
    so any additional spaces required are added on the left.
}
begin
  Result := ' ' + DtosM(v, Width-1, Fixed);
  if Length(Result) < Width then
    if Fixed < 0 then
      Result := Result + StringOfChar(' ', Width-Length(Result))
    else
      Result := StringOfChar(' ', Width-Length(Result)) + Result; 
end;

function dtosm(v:Double; Width, Fixed: Integer):string;
{ Produce string representation of d.
   - the result will contain no more than width characters; if width <=0
     then the longest sensible
   - if fixed < 0, g format is used, with the precision
     adjusted to not overfill the string
   - if fixed >= 0, f format is used with fixed figures after
     the point if possible.
}
var
  p, LastDigit: Integer;
  {$IFNDEF USEOLD}
  DecPlaces, NegativeOne, Exponent, MinFixedWidth, MinFloatingWidth, DPpos: Integer;
  ExponentStr: string;
  {$ELSE}
  n: Integer;
  {$ENDIF}
  function DoRounding(Var dtosm: string): Boolean;
  var
    RoundUp: Boolean;
    RoundDigit, Q, E: Integer;
  begin
    { Round Result string at LastDigit }
    Result := False;
    if LastDigit > Length(dtosm) then
      LastDigit := Length(dtosm);
    RoundDigit := LastDigit + 1;
    if dtosm[LastDigit] = '.' then
      Dec(LastDigit);
    RoundUp := (RoundDigit <= Length(dtosm)) and not (dtosm[RoundDigit] in [ #0, 'E']);
    if RoundUp and (dtosm[RoundDigit] = '.') then
      Inc(RoundDigit);
    RoundUp := RoundUp and (RoundDigit <= Length(dtosm)) and (dtosm[RoundDigit] >= '5');
    if RoundUp and (dtosm[RoundDigit] = '5') then begin
      { Explicit Test for 5000000... }
      { Point to first character after 5 that is not '0' }
      Inc(RoundDigit);
      while (RoundDigit < Length(dtosm)) and (dtosm[RoundDigit] = '0') do
        Inc(RoundDigit);
      if (RoundDigit >= Length(dtosm)) or (dtosm[RoundDigit] = 'E') then
        { Round to nearest or even }
        RoundUp := Odd(Ord(dtosm[LastDigit]) - Ord('0'));
    end;
    Q := LastDigit;
    while RoundUp and (Q > 1)do begin
      RoundUp := dtosm[Q] = '9';
      if RoundUp then begin
        dtosm[Q] := '0';
        Dec(Q);
        if dtosm[Q] = '.' then
          Dec(Q);
        if Q = 1 then begin
          System.Insert('1', dtosm, 2);
          Inc(LastDigit);
          Result := True;
        end;
      end else
        Inc(dtosm[Q]);
    end;
    if dtosm[1] <> '-' then
      dtosm := Copy(dtosm, 2, LastDigit-1)
    else
      dtosm := Copy(dtosm, 1, LastDigit);
    Q := Pos('.', dtosm);
    if Q > 0 then begin
      { Remove trailing zeros after point }
      E := Length(dtosm);
      while (E > Q) and (dtosm[E] = '0') do
        Dec(E);
      if E = Q then
        System.Delete(dtosm, Q, Length(dtosm))
      else begin
        System.Delete(dtosm, E+1, Length(dtosm));
        dtosm[Q] := DecimalSeparator;
      end;
    end;
  end;
begin
  if Width <= 0 then Width := 40;
  if (Fixed >= 0) or IsNanOrInf(v) then begin
    { Try fixed if required }
    Result := FloatToStrF(v, ffFixed, 30, Fixed);
    if (Length(Result) <= Width) or IsNanOrInf(v) then Exit;
  end;
  {$IFNDEF USEOLD}
  NegativeOne := 0;
  if V < 0 then
    NegativeOne := 1;
  Str(V:23, Result);
  { Want exactly one character (space or sign) before digits }
  { Str does not seem to be entirely consistent }
  p := 1;
  while (p < Length(Result)) and (Result[p] = ' ') do
    Inc(p);
  if Result[p] in ['0'..'9'] then
    Dec(p);
  if p <= 0 then
    raise Exception.Create('DTOSM')
  else if p > 1 then
    Delete(Result, 1, p-1);
  p := Pos('E', Result);
  if p <= 0 then
    raise Exception.Create('DTOSM1');
  Exponent := StrToInt(Copy(Result, P+1, Length(Result)));
  if Exponent > 0 then
    MinFixedWidth := NegativeOne + Exponent + 1
  else
    MinFixedWidth := NegativeOne + -Exponent + 1;
  { Consider Floating format }
  ExponentStr := IntToStr(Exponent);
  MinFloatingWidth := NegativeOne + Length(ExponentStr) + 2;
  if (Exponent >= -4) and (Exponent < 10) and ((MinFixedWidth <= Width) or (MinFixedWidth <= MinFloatingWidth)) then begin
    { Number of decimal places possible in a fixed point representation }
    DPpos := 3 + Exponent;
    if DPpos < 3 then begin
      Result := Result[1] + '.' + StringOfChar('0', -Exponent-1) + Result[2] + Copy(Result, 4, P-4);
    end else if DPPos > 3 then
      Result := Copy(Result, 1, 2) + Copy(Result, 4, DPPos-3) + '.' + Copy(Result, DPpos+1, P-DPPos-1)
    else if DPPos = 3 then
      System.Delete(Result, P, Length(Result)); { Just delete exponent }
    P := Pos('.', Result);
    LastDigit := Width + 1 - NegativeOne;
    if LastDigit < 2 - Exponent then
      { 2 - Exponent is the position of the leading digit in numbers like 0.001 }
      LastDigit := 2 - Exponent;
    if LastDigit < P then
      LastDigit := P; { Fixed has been forced, so must show all figures before point }
    DoRounding(Result);
  end else begin
    { Floating }
    { Exponential format }
    LastDigit := P-1;
    while Result[LastDigit] = '0' do
      Dec(LastDigit);
    SetLength(Result, LastDigit);
    { Characters available for mantissa after point }
    DecPlaces := Width - Length(ExponentStr) - NegativeOne - 3;
    { Round string to required number of Places }
    if DecPlaces > 0 then
      LastDigit := DecPlaces + 3
    else
      { Single digit is a special case }
      LastDigit := 2;
    if DoRounding(Result) then begin
      { 9.999... wrapping up by a power of 10 }
      Inc(Exponent);
      if v < 0 then
        Result := Result[1]+'1e'+IntToStr(Exponent)
      else
        Result := '1e'+IntToStr(Exponent);
      Exit;
    end;
    Result := Result + 'e' + ExponentStr
  end;
  {$ELSE}
  n := Width - 1;
  if not IsNanOrInf(v) and (v < 0) then Dec(n);
  if n < 1 then n := 1;
  if n > 15 then n := 15;
  Result := FloatToStrF(v, ffGeneral, n, 0);
  if Length(Result) > Width then begin
    { Determine precision displayed }
    p := 1;
    n := 0;
    while (p <= Length(Result)) and not (Result[p] in ['1'..'9']) do Inc(p);
    while (p <= Length(Result)) and (Result[p] in ['0'..'9', '.']) do begin
      if Result[p] in ['0'..'9'] then Inc(n);
      Inc(p);
    end;
    n := n - (Length(Result)-Width);
    if n < 1 then n := 1;
    Result := FloatToStrF(v, ffGeneral, n, 0);
  end;
  if Length(Result) > Width then begin
    { Still not right - try again }
    Dec(n, Length(Result)-Width);
    if n > 1 then
      Result := FloatToStrF(v, ffGeneral, n, 0)
    else
      Result := FormatFloat('0e-', v);
  end;
  p := Pos('E', Result);
  if p > 0 then
    Result[p] := 'e';
  {$ENDIF}
end;

function RoundToIncrement(Value, Increment: Extended): Extended;
{ Given Increment reflecting the approximate precision of Value,
  return Value rounded to the power of 10 less than increment }
var
  PowerOfTen: Integer;
  Scale: Extended;
begin
  Increment := Abs(Increment);
  Result := Value;
  if Increment > 0 then begin
    PowerOfTen := Floor(Log10(Abs(Increment)));
    Scale := IntPower(10, PowerOfTen);
    Result := Value/Scale;
    if Abs(Result) < High(Int64) then
      Result := Round(Result)*Scale { Avoid overflow }
  end;
end;

function RoundedStr(Value, Increment: Double): string;
begin
  Result := FloatToStr(RoundToIncrement(Value, Increment));
end;

procedure Limits(const A: array of Double; var Mi, Mx: Double; First, Last:Integer);
var
  i: Integer;
begin
  if Last <= Low(Integer)+1 then { +1 is bug fix }
    Last := High(A);
  if (First > Last) then Exit;
  if (First < Low(A)) or (Last > High(A)) then
    raise Exception.CreateFmt('Limits: %d %d %d', [High(A), First, Last]);
  Mi := A[First];
  Mx := Mi;
  for i := First+1 to Last do begin
    if A[i] < Mi then Mi := A[i];
    if A[i] > Mx then Mx := A[i];
  end;
end;

procedure Limits(const a: array of Integer; var Mi, Mx: Integer; First:Integer = 0; Last: Integer = Low(Integer));
var
  i: Integer;
begin
  if Last <= Low(Integer)+1 then { +1 is bug fix }
    Last := High(A);
  if (First > Last) then Exit;
  if (First < Low(A)) or (Last > High(A)) then
    raise Exception.CreateFmt('Limits: %d %d %d', [High(A), First, Last]);
  Mi := A[First];
  Mx := Mi;
  for i := First+1 to Last do begin
    if A[i] < Mi then Mi := A[i];
    if A[i] > Mx then Mx := A[i];
  end;
end;

procedure Limits(const a: array of Cardinal; var Mi, Mx: Cardinal; First:Integer = 0; Last: Integer = Low(Integer));
var
  i: Integer;
begin
  if Last <= Low(Integer)+1 then { +1 is bug fix }
    Last := High(A);
  if (First > Last) then Exit;
  if (First < Low(A)) or (Last > High(A)) then
    raise Exception.CreateFmt('Limits: %d %d %d', [High(A), First, Last]);
  Mi := A[First];
  Mx := Mi;
  for i := First+1 to Last do begin
    if A[i] < Mi then Mi := A[i];
    if A[i] > Mx then Mx := A[i];
  end;
end;

function AtLeast(Value, Spaces: Integer):string;
begin
  Str(Value:Spaces, Result);
  if Result[1] <> ' ' then
    Result := ' ' + Result;
end;

function AtLeast(Value: Double; Spaces, Places: Integer):string;
begin
  Str(Value:Spaces:Places, Result);
  if Result[1] <> ' ' then
    Result := ' ' + Result;
end;

procedure InOrder(var Mi, Mx: Double);
var
  v: Double;
begin
  if Mi > Mx then begin
    v := Mi;
    Mi := Mx;
    Mx := v;
  end;
end;

procedure InOrder(var Mi, Mx: Integer);
var
  v: Integer;
begin
  if Mi > Mx then begin
    v := Mi;
    Mi := Mx;
    Mx := v;
  end;
end;

function Within(const v, Mi, Mx: Double): Boolean; overload;
begin
  if Mi < Mx then
    Result := InRange(v,Mi,Mx)
  else
    Result := InRange(v,Mx,Mi)
end;

function Within(const v, Mi, Mx: Integer): Boolean; overload;
begin
  if Mi < Mx then
    Result := InRange(v,Mi,Mx)
  else
    Result := InRange(v,Mx,Mi)
end;

function TryStrToFloatExHelper(S: string; var Value: Extended):Boolean;
var
  i: Integer;
begin
  { In separate routine to avoid string initialization/finalization }
  i := Pos('d', S);
  if i < 0 then
    i := Pos('D', S);
  if i > 0 then
    S[i] := 'e';
  Result := TryStrToFloat(StringReplace(S,'d','e',[rfIgnoreCase]),Value);
  if not Result and (Length(S) in [3..4]) then begin
    if S[1] in ['+','-'] then
      Delete(S,1,1);
    Result := (CompareText(S,'Nan') = 0) or (CompareText(S,'Inf') = 0);
    if Result then
      SetNan(Value);
  end;
end;

function TryStrToFloatEx(const S: string; out Value: Extended):Boolean;
begin
  Result := TryStrToFloat(S,Value);
  if not Result then
    Result := TryStrToFloatExHelper(S, Value)
end;

{$IFDEF FPC_HAS_TYPE_EXTENDED}
function TryStrToFloatEx(const S: string; out Value: Double):Boolean;
var
  ExValue: Extended;
begin
  Result := TryStrToFloat(S,Value);
  if not Result then begin
    Result := TryStrToFloatExHelper(S, ExValue);
    if Result then
      Value := ExValue;
  end;
end;
{$ENDIF}

function StrToFloatEx(const S: string):Extended;
begin
  if not TryStrToFloatEx(S, Result) then
    raise Exception.Create(S+'is not a valid floating point number');
end;

end.

