unit Sysinfo;

{$I DFS.inc}{ Copyright (C) 2005 C M Western; see UCompat.pas for license}

interface

function is_buggy_pentium:Boolean;
function DetermineCpu:string;
function nCPUs:Integer;
function ComputerInfo:string;
function DirectIOPossible: Boolean;

implementation

uses {$IFDEF MSWINDOWS} Windows, Huge, {$ENDIF}
     {$IFDEF LINUX} {$IFDEF CPU64} BaseUnix, {$ELSE} Libc, {$ENDIF} Unix, {$ENDIF}
     {$IFDEF BSD} Unix, {$ENDIF}
     {$IFDEF DARWIN} MacOSAll, {$ENDIF}
     {$IFDEF CLX_} Qt, {$ENDIF}
     SysUtils;

var
  SavednCPUs: Integer = -1;
  DetermineCpuResult: string;
  SavedIOPossible: Integer = -1;

{$IFNDEF DARWIN}
{ Buggy FPU's predate DARWIN }
{$IFDEF CPU86}
{$DEFINE CHECKCPU}
{$ENDIF}
{$ENDIF}

{$IFDEF CHECKCPU}
{$IFDEF FPC}
{$ASMMODE intel}
{$ENDIF}
const
  numerator:Longint = 4195835;
  denominator:Longint = 3145727;

function is_buggy_pentium:Boolean;
var
  t:Smallint;
begin
  asm
    fild dword ptr numerator
    fidiv dword ptr denominator
    fimul dword ptr denominator
    fisub dword ptr numerator
    fistp word ptr t
  end;
  Result := t <> 0;
end;
{$ELSE}
function is_buggy_pentium:Boolean;
begin
  Result := False;
end;
{$ENDIF}

{$IFDEF LINUX}
function GetCpuName: string;
var
  buffer, Field: string;
  F: Text;
  ProcName, Speed: string;
  p, e: Integer;
begin
  Result := '';
  AssignFile(F, '/proc/cpuinfo');
  Reset(F);
  SavednCPUs := 0;
  ProcName := '';
  Speed := '';
  while not EOF(F) do begin
    ReadLn(F, buffer);
    p := Pos(#9, buffer);
    e := Pos(':', buffer);
    if (p <= 0) or (e <= 0) then
      Continue;
    Field := Copy(buffer, 1, p-1);
    if Field = 'processor' then
      Inc(SavednCPUs)
    else if (Field = 'model name') and (ProcName = '') then
      ProcName := Trim(Copy(buffer, e+2, Length(buffer)))
    else if (Field = 'cpu MHz') and (Speed = '') then
      Speed := Trim(Copy(buffer, e+2, Length(buffer))) + ' MHz'
  end;
  CloseFile(F);
  if SavednCPUs > 1 then
    Result := IntToStr(SavednCPUs) + ' cpus:' ;
  Result := Result + ProcName + ' ' + Speed;
end;

function DetermineCpu:string;
var
 buf:TUTSName;
 infobuf: string;
 {$IFDEF CLX_}
 function ModuleFileName(const Name:string):string;
 var
   module: HMODULE;
 begin
   Result := '';
   module := GetModuleHandle(PChar(Name));
   if module <> 0 then begin
     Result := GetModuleName(module);
   end;
   if Result = '' then Result := Name;
 end;
 {$ENDIF}
begin
  if DetermineCpuResult <> '' then begin
    Result := DetermineCpuResult;
    Exit;
  end;
  // TODO: Add cpu info
  infobuf := GetCpuName;
  {$IFDEF CPU64}
    FpUname(buf);
  {$ELSE}
    Uname(buf);
  {$ENDIF}
  Result := Format('%s %s %s %s %s %s '{$IFNDEF CPU64}+'glibc-%s-%s'{$ENDIF},
      [buf.sysname, buf.release, buf.version, buf.machine, buf.nodename, infobuf {$IFNDEF CPU64}, gnu_get_libc_version, gnu_get_libc_release{$ENDIF}]);
  {$IFDEF CLX_}
  Result := Concat(Result, ' ', ModuleFileName(QtIntf));
   if Qt.QtIntf <> Qt.QtLib then
     Result := Concat(Result, ' ', ModuleFileName(QtLib));
  {$ENDIF}
  DetermineCpuResult := Result;
  SavedIOPossible := 0;
end;

function ComputerInfo:string;
begin
  Result := GetHostName;
end;
{$ENDIF}

{$IFDEF BSD}
function DetermineCpu:string;
{$IFDEF DARWIN}
var
  versMaj, versMin, versBugFix: SInt32;
{$ENDIF}
begin
  Result := {$I %FPCTARGETOS%} + ' FPC:' + ' '+{$I %FPCVERSION%} + ' ' + {$I %FPCTARGETCPU%};
  {$IFDEF DARWIN}
  Gestalt(gestaltSystemVersionMajor, versMaj);
  Gestalt(gestaltSystemVersionMinor, versMin);
  Gestalt(gestaltSystemVersionBugFix, versBugFix);
  Result := Result + Format(' MacOSX %d.%d.%d', [versMaj, versMin, versBugFix]);
  {$ENDIF}
  SavedIOPossible := 0;
end;

function ComputerInfo:string;
begin
  Result := GetHostName;
end;
{$ENDIF}

{$IFDEF WIN32}
type
  TIsWOW64Process = function (hProcess: THandle; var Wow64Process: BOOL): BOOL; stdcall;

function DetermineCpu:string;
var
  o:TOSVersionInfo;
  s:TSystemInfo;
  CPUVendorString: string[13];
  CPUid: Longint;
  Processor: Integer;
  HeapStatus: THeapStatus;
  Kernel: HMODULE;
  IsWOW64Process: TIsWOW64Process;
  Wow64Process: BOOL;
begin
  if DetermineCpuResult = '' then begin
    o.dwOSVersionInfoSize := SizeOf(o);
    GetVersionEx(o);
    SavedIOPossible := 1;
    case o.dwPlatformId of
       VER_PLATFORM_WIN32s:        Result := 'Win32s on Windows 3.1';
       VER_PLATFORM_WIN32_WINDOWS: Result := 'Win32c on Windows 95';
       VER_PLATFORM_WIN32_NT:      begin
                                     Result := 'Windows NT';
                                     SavedIOPossible := 0;
                                   end
    else
      Result :='Windows ?';
      SavedIOPossible := 0;
    end;
    {$IFNDEF CPU64}
    Kernel := GetModuleHandle(Windows.kernel32);
    if Kernel <> 0 then begin
      IsWOW64Process := TIsWOW64Process(GetProcAddress(Kernel, 'IsWow64Process'));
      if @IsWOW64Process <> nil then begin
        if not IsWOW64Process(GetCurrentProcess, Wow64Process) then
          RaiseLastOSError;
        if WOW64Process then begin
          Result := Result + ' 32 on 64 bit';
          SavedIOPossible := 0;
        end;
      end;
    end;
    {$ELSE}
    Result := Result + ' 64 bit';
    {$ENDIF}
    Result := Result + Format(' %d.%d.%d %s'#10#13, [o.dwMajorVersion, o.dwMinorVersion,
                              LOWORD(o.dwBuildNumber), o.szCSDVersion]);
    GetSystemInfo(s);
    processor := s.dwProcessorType;
    SavednCPUs := s.dwNumberOfProcessors;
    if is_buggy_pentium then begin
      Result := Result + 'Broken ';
      processor := 586;
    end;
    if processor >= 586 then begin
      CPUVendorString[0] := Chr(12);
      try
        asm
          push ebx

          xor ebx,ebx
          xor eax,eax
          db $0f, $a2  // CPUID
          or ebx,ebx
          je @@done
                    // After B0 stepping
          mov dword ptr CPUVendorString[1], ebx
          mov dword ptr CPUVendorString[5], edx
          mov dword ptr CPUVendorString[9], ecx
          mov eax,1
          db $0f, $a2  // CPUID
        @@done:
          mov CPUid, eax
          pop ebx
        end;
      except
        CPUid := 0;
      end;
    end;
    if SavednCPUs > 1 then
      Result := Result + Format('%d processors:', [SavednCPUs])
    else
      Result := Result + 'Processor:';
    if CPUid <> 0 then begin
      processor := ((CPUid shr 8) and $f)*100 + 86;
      Result := Result + Format(' %d, Model: %d, Step: %d, %s',
                      [processor, (cpuid shr 4) and $f, cpuid and $f, CPUVendorString]);
    end else
      Result := Result + Format(' %d', [processor]);
    DetermineCpuResult := Result;
  end;
  HeapStatus := GetHeapStatus;
  Result := DetermineCpuResult + Format(#10#13'Memory Used: %d M', [(HeapStatus.TotalAllocated + GMemoryCount) div (1024*1024)]);
end;
{$ENDIF}

{$IFDEF DFS_WIN16}
function GetCPUID(var s: string): Longint;
var
  ID: string[13];
begin
  ID[0] := Chr(0);
  asm
    db $66  { 32 bit prefix }
    xor bx, bx
    db $66  { 32 bit prefix }
    xor ax, ax
    db $0f, $a2  { CPUID }
    db $66  { 32 bit prefix }
    or bx, bx
    je @done		{ After B0 stepping }
    db $66  { 32 bit prefix }
    mov word ptr ID[1],bx
    db $66  { 32 bit prefix }
    mov word ptr ID[5],dx
    db $66  { 32 bit prefix }
    mov word ptr ID[9],cx
    mov byte ptr ID[0], 12
    db $66  { 32 bit prefix }
    xor ax, ax
    db $66  { 32 bit prefix }
    xor ax, ax
    inc ax
    db $0f, $a2  { CPUID }
@done:
    db $66  { 32 bit prefix }
    mov word ptr Result, ax
  end;
  s := ID;
end;

function DetermineCpu:string;
var
  dwVersion, dwFlags, CPUid: Longint;
  Processor: Integer;
  OSType: string;
  CPUVendorString: string[13];
begin
  if DetermineCpuResult <> '' then begin
    Result := DetermineCpuResult;
    Exit;
  end;
  dwVersion := GetVersion;
  dwFlags := GetWinFlags;
  if (dwFlags and $4000) <> 0 then
    OSType := 'NT'
  else if (LOBYTE(LOWORD(dwVersion)) > 3) or (HIBYTE(LOWORD(dwVersion)) > 50) then
    OSType := '95'
  else
    OSType := '';
  Result := Format('Windows %s %d.%d ', [OSType, LOBYTE(LOWORD(dwVersion)), HIBYTE(LOWORD(dwVersion))]);
  if (dwFlags and WF_ENHANCED) <> 0 then
    Result := Result + 'enhanced mode '
  else
    Result := Result + 'standard mode ';
  if (dwFlags and WF_CPU286) <> 0 then
    Processor := 286
  else if (dwFlags and WF_CPU386) <> 0 then
    Processor := 386
  else if (dwFlags and WF_CPU486) <> 0 then
    Processor := 486
  else
    Processor := 0;
  if is_buggy_pentium then begin
    Result := Result + 'Broken ';
    processor := 586;
  end;
  Result := Result + #13#10;
  try
    CPUVendorString := '';
    CPUid := GetCPUID(CPUVendorString);
  except
    CPUid := 0;
  end;
  if CPUid <> 0 then begin
    processor := ((CPUid shr 8) and $f)*100 + 86;
    Result := Result + Format('Processor: %d, Model: %d, Step: %d, %s',
                    [processor, (cpuid shr 4) and $f, cpuid and $f, CPUVendorString]);
  end else
    Result := Result + Format('Processor: %d', [processor]);
  if (dwFlags and WF_80x87) <> 0 then
    Result := Result + ' + coprocessor'
  DetermineCpuResult := Result;
end;
{$ENDIF}

function nCPUs:Integer;
begin
  if SavednCPUs < 0 then
    DetermineCpu;
  if SavednCPUs < 0 then
    SavednCPUs := 1; { Default in the absence of other information }
  Result := SavednCPUs;
end;

{$IFDEF MSWINDOWS}
function ComputerInfo:string;
var
  Kernel: HMODULE;
  GetComputerNameEx: function (NameType: Integer; lpBuffer: PChar; var nSize: DWORD): BOOL; stdcall;
  Size: DWORD;
  nTry: Integer;
  OK: Boolean;
begin
  {$IFNDEF CPU64}
  Kernel := GetModuleHandle(Windows.kernel32);
  if Kernel <> 0 then begin
    GetComputerNameEx := GetProcAddress(Kernel, 'GetComputerNameExA');
    if Assigned(GetComputerNameEx) then begin
      Size := 5;
      nTry := 0;
      repeat
        SetLength(Result,Size);
        Inc(nTry);
        OK := GetComputerNameEx(1,PChar(Result),Size);
      until OK or (nTry > 2);
      if OK then begin
        SetLength(Result,Size);
        Exit;
      end;
    end;
  end;
  {$ENDIF}
  Size := 5;
  nTry := 0;
  repeat
    SetLength(Result,Size);
    Inc(nTry);
    OK := GetComputerName(PChar(Result),Size);
  until OK or (nTry > 2);
  if OK then
    SetLength(Result,Size)
  else
    Result := '';
end;
{$ENDIF}

function DirectIOPossible: Boolean;
begin
  if SavedIOPossible < 0 then
    DetermineCpu;
  Result := SavedIOPossible > 0;
end;

end.
