unit UCompat;
{ Various routines to help with compatibility between different operating
  systems and compilers }
{$I DFS.inc}
{
Putils - utility routines for pgopger
Copyright (C) 2005 C M Western
School of Chemistry, University of Bristol, Bristol, UK
For further information please see http://pgopher.chm.bris.ac.uk/
or contact help-pgopher@bristol.ac.uk

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
}
interface

uses
 {$IFDEF MSWINDOWS} Windows, {$ENDIF}
 Classes, IniFiles, SysUtils;

var
  VersionInfo: ShortString;

{$IFDEF UNIX}
const
  LineEnd: string = #10;
  NewLineLength = 1;
{$ENDIF}

{$IFDEF LINUX}
// Not connected at the moment: function CopyFile(Source, Dest:PChar; bFailIfExists: Boolean):Boolean;
function timeGetTime:Cardinal;
{$ENDIF}

{$IFDEF FPC}
function StripHotKey(const s: string):string;
{$IFNDEF MSWINDOWS}
  procedure Yield;
  function MulDiv(A, B, C:Integer):Integer;
{$ENDIF}
function RoundTo(const Value: Double; Places: Integer):Double;
type
  TTextIOFunc = function (Var t:TTextRec):Integer;
procedure SetInOutRes(Value:Word);
{$ELSE}
const
  AllFilesMask: string = '*.*';
{$ENDIF}

{$IFDEF MSWINDOWS}
const
  LineEnd: string = #13#10;
  NewLineLength = 2;

{$IFDEF VER80}
const
  MinusOne:Cardinal = $ffff;
{$ELSE}
const
  MinusOne:LongInt = -1;
{$ENDIF}


{$IFNDEF DFS_COMPILER_6_UP}
{$IFNDEF FPC}
  PathDelim  = '\';
  DriveDelim = ':';
  PathSep    = ';';
procedure RaiseLastOSError;
{$ENDIF}
{$ENDIF}

{$IFNDEF DFS_COMPILER_5_UP}
procedure FreeAndNil(var Obj);
{$ENDIF}
{$ENDIF}

{$ifndef VER80}
procedure hmemcpy(p1, p2: Pointer; L: Longint);
{$endif}

function AddressOfFirstChar(const s:string):PChar;

function ApplicationSettingsDir: string;
function GetIniFile(const Name: string; WithBorland:Boolean{ = False}): TCustomIniFile;
procedure AllowCtrlC; { Allow ^C to kill process }

{$IFNDEF FPC}
type
  TLibHandle = THandle;
  PtrInt = Integer;
{$IFDEF MSWINDOWS}
  TThreadID = THandle;
procedure InitCriticalSection(var lpCriticalSection: TRTLCriticalSection); stdcall;
procedure DoneCriticalSection(var lpCriticalSection: TRTLCriticalSection); stdcall;
{$ENDIF}
procedure DoDirSeparators(var FileName: string);
{$ENDIF}

implementation
{.$DEFINE LIBC_SYSCALLS}
uses
  {$IFDEF MSWINDOWS}
    Registry, ShlObj, 
  {$ELSE}
    {$IFDEF FPC} Unix, BaseUnix, {$ENDIF}
    {$IFDEF LIBC_SYSCALLS} Libc, {$ENDIF}
  {$ENDIF}
  Math;

{$IFNDEF VER80}
{$IFDEF MSWINDOWS}

procedure hmemcpy(p1, p2: Pointer; L: Longint);
begin
  CopyMemory(p1, p2, L);
end;
{$IFNDEF DFS_COMPILER_6_UP}
{$IFNDEF FPC}
procedure RaiseLastOSError;
begin
  RaiseLastWin32Error;
end;
{$ENDIF}
{$ENDIF}
{$ENDIF}

{$IFDEF UNIX}

{$IFDEF FPC}

function MulDiv(A, B, C:Integer):Integer;
begin
  Result := (A * B) div C;
end;

procedure Yield;
begin
  ThreadSwitch;
end;

{$ENDIF}

procedure hmemcpy(p1, p2: Pointer; L: Longint);
begin
  Move(p2^, p1^, L);
end;
{$ENDIF}

{$ELSE}
uses
  SysUtils;
{$ENDIF}

{$IFDEF MSWINDOWS}
{$IFNDEF DFS_COMPILER_5_UP}
procedure FreeAndNil(var Obj);
var
  Temp: TObject;
begin
  Temp := TObject(Obj);
  Pointer(Obj) := nil;
  Temp.Free;
end;
{$ENDIF}
{$ENDIF}

function AddressOfFirstChar(const s:string):PChar;
begin
{$IFNDEF VER80}
  Result := PChar(s);
{$ELSE}
  Result := @s[1];
{$ENDIF}
end;

{$IFDEF LINUX}

{$IFDEF cpVERSION}
var
  CopyDosCommand: TDosCommand;
{$ENDIF}

function timeGetTime:Cardinal;
var
  tz:timeval;
begin
  fpgettimeofday(@tz,nil);
  { Ignore overflow }
  Result := Cardinal(tz.tv_sec*1000) + tz.tv_usec div 1000;
end;

function CopyFile(Source, Dest:PChar; bFailIfExists: Boolean):Boolean;
var
  // ExitCode: LongWord;
  SourceStream, DestStream: TFileStream;
  Count: Int64;
  st, dt: {$IFDEF LIBC_SYSCALLS}TStat{$ELSE}Stat{$ENDIF};
  u: utimbuf;
begin
  Result := False;
  if bFailIfExists and FileExists(Dest) then begin
    {$IFDEF FPC}
    fpseterrno(ESysEEXIST);
    {$ELSE}
    SetLastError(EEXIST);
    {$ENDIF}
    Exit;
  end;
  {$IFDEF cpVERSION}
  if CopyDosCommand = nil then begin
    CopyDosCommand := TDosCommand.Create(Application);
    with CopyDosCommand.Arguments do begin
      Add('cp');
      Add('-p');
      Add(''); { Source }
      Add(''); { Dest }
    end;
  end;
  with CopyDosCommand do begin
    Arguments[2] := Source;
    Arguments[3] := Dest;
    Execute;
    ExitCode := WaitFor;
    if Lines.Count > 0 then
      raise Exception.Create(Lines[0])
    else if ExitCode <> 0 then
      SetLastError(ExitCode) { May not be meaningful }
    else
      Result := True;
  end;
  {$ENDIF}
  {$IFDEF Hmm}
  SourceStream := TFileStream.Create(Source, fmOpenRead);
  try
    DestStream := TFileStream.Create(Dest, fmCreate);
    Count := DestStream.CopyFrom(SourceStream, 0);
    {$IFNDEF LIBC_SYSCALLS}
    if (FPFStat(SourceStream.Handle,st) = -1)
       or (FPFStat(DestStream.Handle, dt) = -1) then
    {$ELSE}
    if (Libc.fpfstat(SourceStream.Handle, st) = -1)
       or (Libc.fpstat(DestStream.Handle, dt) = -1) then
    {$ENDIF}
      RaiseLastOSError;
    if Count <> dt.st_size then
      raise Exception.Create('Wrong number of bytes copied to '+Dest);
    {$IFNDEF LIBC_SYSCALLS}
    if Fpfchmod(DestStream.Handle, st.st_mode) = -1 then
    {$ELSE}
    if Libc.fchmod(DestStream.Handle, st.st_mode) = -1 then
    {$ENDIF}
      RaiseLastOSError;
    { Try and copy owner and group, but ignore errors }
    if st.st_gid <> dt.st_gid then
    {$IFNDEF LIBC_SYSCALLS}
      FpfChown(DestStream.Handle, uid_t(-1), st.st_gid);
    {$ELSE}
      Libc.fchown(DestStream.Handle, uid_t(-1), st.st_gid);
    {$ENDIF}
    {$IFNDEF LIBC_SYSCALLS}
    if (st.st_uid <> dt.st_uid) and (FpGetuid = 0) then
      FpfChown(DestStream.Handle, st.st_uid, gid_t(-1));
    {$ELSE}
    if (st.st_uid <> dt.st_uid) and (Libc.getuid = 0) then
      Libc.fchown(DestStream.Handle, st.st_uid, gid_t(-1));
    {$ENDIF}
    FreeAndNil(DestStream);
    u.actime := st.st_atime;
    u.modtime := st.st_mtime;
    {$IFNDEF LIBC_SYSCALLS}
    if FpUtime(Dest, @u) = -1 then
    {$ELSE}
    if Libc.utime(Dest, u) = -1 then
    {$ENDIF}
      RaiseLastOSError;
    Result := True;
  finally
    DestStream.Free;
  end;
  SourceStream.Free;
  {$ENDIF}
end;
{$ENDIF}

{$IFDEF UNIX}
type
  TMyMemIniFile = class(TMemIniFile)
  public
    destructor Destroy; override;
  end;

destructor TMyMemIniFile.Destroy;
begin
  UpdateFile;
  inherited;
end;
{$ENDIF}

var
  ApplicationSettingsDirCalled: Boolean = False;
  ApplicationSettingsDirResult: string;

function ApplicationSettingsDir: string;
 { per user space for wrtting application settings and other data }
begin
  if ApplicationSettingsDirCalled then begin
    Result := ApplicationSettingsDirResult;
    Exit;
  end;
  {$IFDEF MSWINDOWS}
    SetLength(Result,MAX_PATH+1);
    if SHGetSpecialFolderPath(0, @Result[1], CSIDL_APPDATA, False)
       or SHGetSpecialFolderPath(0, @Result[1], CSIDL_PERSONAL, False) then
      SetLength(Result, Strlen(@Result[1]))
    else
      Result := '';
  {$ENDIF}
  {$IFDEF UNIX}
    Result := GetEnvironmentVariable('HOME');
  {$ENDIF}
  ApplicationSettingsDirCalled := True;
  ApplicationSettingsDirResult := Result;
end;

function GetIniFile(const Name: string; WithBorland:Boolean{ = False}): TCustomIniFile;
var
  KeyName: string;
begin
{$IFDEF MSWINDOWS}
  {$IFDEF VER80} {$DEFINE USEINIFILE} {$ENDIF}
  {$IFDEF FPC} {$DEFINE USEINIFILE} {$ENDIF} { TODO: Implement TRegistryIniFile for FPFC }
  {$IFDEF USEINIFILE}
    KeyName := Name + '.ini';
    Result := TIniFile.Create(KeyName);
  {$ELSE}
    if WithBorland then
      KeyName := 'SOFTWARE\BORLAND\DELPHI\' + Name
    else
      KeyName := 'SOFTWARE\' + Name;
    Result := TRegistryIniFile.Create(KeyName);
  {$ENDIF}
{$ELSE}
  {$IFDEF UNIX}
    KeyName := ApplicationSettingsDir;
    if WithBorland then
      KeyName := KeyName + {$IFDEF KYLIX}'/.borland/'{$ELSE}'/.lazarus/'{$ENDIF} + Name
    else
      KeyName := KeyName + '/.' + Name;
    Result := TMyMemIniFile.Create(KeyName);
  {$ENDIF}
{$ENDIF}
end;

procedure AllowCtrlC; { Allow ^C to kill process }
begin
  {$IFDEF LINUX}
    {$IFNDEF LIBC_SYSCALLS}
    FpSignal(SIGINT, TSignalHandler(SIG_DFL));
    {$ELSE}
    Libc.signal(SIGINT, TSignalHandler(SIG_DFL));
    {$ENDIF}
  {$ENDIF}
end;

{$IFDEF FPC}
function StripHotKey(const s: string):string;
var
  p: Integer;
begin
  Result := s;
  repeat
    p := Pos('&',Result);
    if p > 0 then
      Delete(Result,p,1);
  until p <= 0;
end;

function RoundTo(const Value: Double; Places: Integer):Double;
var
  Scale: Double;
begin
  Scale := IntPower(10, Places);
  Result := Value/Scale;
  if Abs(Result) < High(Int64) then
    Result := Round(Result); { Avoid overflow }
  Result := Result * Scale;
end;

procedure SetInOutRes(Value:Word);
begin
  InOutRes := Value;
end;
{$ENDIF}

{$IFNDEF FPC}
procedure DoDirSeparators(var FileName: string);
var
  i: Integer;
  OtherDelim: Char;
begin
  if PathDelim = '\' then
    OtherDelim := '/'
  else
    OtherDelim := '\';
  for i := 1 to Length(FileName) - 1 do
    if FileName[i] = OtherDelim then
      FileName[i] := PathDelim;
end;

{$IFDEF MSWINDOWS}
procedure InitCriticalSection; external kernel32 name 'InitializeCriticalSection';
procedure DoneCriticalSection; external kernel32 name 'DeleteCriticalSection';
{$ENDIF}
{$ENDIF}

end.


