unit njExpr;
{$I DFS.inc}
interface

uses Expr, SysUtils;

type
  TCGFunction = class (TNadicFunction)
    function CreateExprNode:TExprNode; override;
  end;

  TThreejFunction = class (TNadicFunction)
    function CreateExprNode:TExprNode; override;
  end;

  TSixjFunction = class (TNadicFunction)
    function CreateExprNode:TExprNode; override;
  end;

  TFunctionNodeArg = class (TFunctionNode)
  protected
    function GetArgn(Index: integer): TExprNode;
  end;

  TCGNode = class (TFunctionNodeArg)
  public
    property j1:TExprNode index 0 read GetArgn;
    property j2:TExprNode index 2 read GetArgn;
    property j3:TExprNode index 4 read GetArgn;
    property m1:TExprNode index 1 read GetArgn;
    property m2:TExprNode index 3 read GetArgn;
    property m3:TExprNode index 5 read GetArgn;
    function Simplify(Actions:TSimplifyActions=[]):TExprNode; override;
  end;

  TThreejNode = class (TCGNode)
    function Simplify(Actions:TSimplifyActions=[]):TExprNode; override;
  end;

  TSixjNode = class (TFunctionNodeArg)
  public
    property j1:TExprNode index 0 read GetArgn;
    property j2:TExprNode index 1 read GetArgn;
    property j3:TExprNode index 2 read GetArgn;
    property l1:TExprNode index 3 read GetArgn;
    property l2:TExprNode index 4 read GetArgn;
    property l3:TExprNode index 5 read GetArgn;
    function Simplify(Actions:TSimplifyActions=[]):TExprNode; override;
  end;

implementation

uses nj, Math;

{ TCGFunction }

function TCGFunction.CreateExprNode: TExprNode;
begin
  Result := TCGNode.Create(Self);
end;

function _CG(var a: array of Double): Double;
begin
  Result := cg(Round(a[5]*2), Round(a[4]*2), Round(a[3]*2), Round(a[2]*2), Round(a[1]*2), Round(a[0]*2));
end;

{ TThreejFunction }

function TThreejFunction.CreateExprNode: TExprNode;
begin
  Result := TThreejNode.Create(Self);
end;

function _threej(var a: array of Double): Double;
begin
  Result := threej(Round(a[5]*2), Round(a[4]*2), Round(a[3]*2), Round(a[2]*2), Round(a[1]*2), Round(a[0]*2));
end;

{ TSixjFunction }

function TSixjFunction.CreateExprNode: TExprNode;
begin
  Result :=TSixjNode.Create(Self);
end;

function _sixj(var a: array of Double): Double;
begin
  Result := sixj(Round(a[5]*2), Round(a[4]*2), Round(a[3]*2), Round(a[2]*2), Round(a[1]*2), Round(a[0]*2));
end;

{ TCGNode }

function TCGNode.Simplify(Actions:TSimplifyActions): TExprNode;
var
  i, z, zMax, zMin: Integer;
  Node, Den: TExprNode;
  SumDenom: array [1..5] of TExprNode;
  procedure MakeSD(Index: Integer);
  begin
    case Index of
      1: SumDenom[1] := EB([j1,'+',j2,'-',j3],Actions);
      2: SumDenom[2] := EB([j1,'-',m1],Actions);
      3: SumDenom[3] := EB([j2,'+',m2],Actions);
    end;
  end;
begin
  Result := inherited Simplify(Actions);
  if Result <> Self then
    Exit;
  try
    //for i := 0 to 5 do
    //  WriteLn(i, ':', Children[i].Text(True));
    zMax := High(Integer);
    for i := 1 to 3 do begin
      MakeSD(i);
      Node := SumDenom[i];
      if Node is TIntegerNode then
        zMax := Min(zMax, TIntegerNode(Node).Value);
    end;
    if zMax > 10 then begin
      { Small (known) limit to sum not found - try a cyclic permutation }
      SumDenom[2].Free;
      SumDenom[2] := EB([j3, '-', m3]);
      if SumDenom[2] is TIntegerNode then
        zMax := Min(zMax, TIntegerNode(SumDenom[2]).Value);
      if zMax <= 10 then begin
        { Cyclic permutation will simplify things, so do it }
        Node := Children[0];
        Children[0] := Children[4];
        Children[4] := Children[2];
        Children[2] := Node;
        Node := Children[1];
        Children[1] := Children[5];
        Children[5] := Children[3];
        Children[3] := Node;
        SumDenom[1].Free;
        SumDenom[3].Free;
        MakeSD(1);
        MakeSD(3);
      end else
        Exit;
    end;

    SumDenom[4] := EB([j3,'-',j2,'+',m1],Actions);
    SumDenom[5] := EB([j3,'-',j1,'-',m2],Actions);
    zMin := 0;
    for i := 4 to 5 do begin
      Node := SumDenom[i];
      if Node is TIntegerNode then
        zMin := Max(zMin, -TIntegerNode(Node).Value);
    end;

    if zMax < zMin then begin
      Result := TIntegerNode.Create(0);
      Free;
      Exit;
    end;
    Node := OperatorPlus.CreateExprNode;
    for z := zMin to zMax do begin
      Den := OperatorTimes.CreateExprNode;
      Den.AddNode(EB(['factorial', z],Actions));
      for i := 1 to 3 do
        Den.AddNode(EB([SumDenom[i], '-', z, 'factorial'],Actions));
      Den.AddNode(EB([SumDenom[4], '+', z, 'factorial'],Actions));
      Den.AddNode(EB([SumDenom[5], '+', z, 'factorial'],Actions));
      Den := EN(['/',1,Den],Actions);
      if Odd(z) then
        Den := MonadicMinus.CreateExprNode(Den);
      Node.AddNode(Den);
    end;
    Node := CheckSimplify(Node,Actions);

    Node :=
    EN([ '*', EB([2,'*',j3,'+',1],Actions), EB([j1,'+',j2,'-',j3,'factorial'],Actions), EB([j1,'-',j2,'+',j3,'factorial'],Actions), EB([j2,'-',j1,'+',j3,'factorial'],Actions),
              EB([j1,'+',m1,'factorial'],Actions), EB([j1,'-',m1,'factorial'],Actions), EB([j2,'+',m2,'factorial'],Actions), EB([j2,'-',m2,'factorial'],Actions), EB([j3,'+',m3,'factorial'],Actions), EB([j3,'-',m3,'factorial'],Actions),
         '/', EB([j1,'+',j2,'+',j3,'+',1,'factorial'],Actions),
         'sqrt', '*', Node],Actions);
    Result := CheckSimplify(Node,Actions);
  finally
    for i := 1 to 5 do
      SumDenom[i].Free;
  end;
  Free;
end;

var
  CGFunction, ThreejFunction, SixjFunction: TNadicFunction;

{ TSixjNode }

function TSixjNode.Simplify(Actions:TSimplifyActions): TExprNode;
var
  i, z, zMax, zMin: Integer;
  Node: TExprNode;
  eZ: TExprNode;
  SumDenom: array [1..7] of TExprNode;
  function Delta(a, b, c: TExprNode):TExprNode;
  begin
    Result := EN([ EB([a, '+', b, '-', c, 'factorial'],Actions),
                   '*', EB([a, '-', b, '+', c, 'factorial'],Actions),
                   '*', EB([b, '-', a, '+', c, 'factorial'],Actions),
                   '/', EB([a, '+', b, '+', c, '+', 1, 'factorial'],Actions),
                   'sqrt'],Actions);
    //WriteLn('a:',a.Text(True), ' b:', b.Text(True), ' c:', c.Text(True));
    //WriteLn('Delta: ',Result.Text(True));
  end;
  function Term(z:TExprNode; zn: Integer):TExprNode;
  var
    i: Integer;
  begin
    Result := OperatorTimes.CreateExprNode;
    for i := 1 to 4 do
      Result.AddNode(EB([z, '+', SumDenom[i], 'factorial'],Actions));
    for i := 5 to 7 do
      Result.AddNode(EB([SumDenom[i], '-', z, 'factorial'],Actions));
    Result := EN([z.Duplicate, '+', 1, 'factorial', '/', Result],Actions);
    if Odd(zn) then
      Result := MonadicMinus.CreateExprNode(Result);
  end;
  procedure LimitsZ(First, Second: Integer);
  var
    Diff: Integer;
  begin
    if SimplifiesToInt(Diff,[SumDenom[First], '+', SumDenom[Second]]) then begin
      //WriteLn(First:2, Second:2, Diff:3, ' ', SumDenom[Second].Text(True));
      if (Diff >=0) and (Diff < zMax) then begin
        ez.Free;
        ez := EB([SumDenom[Second], '-', Diff]);
        zMin := 0;
        zMax := Diff;
      end;
    end;
  end;
begin
  Result := inherited Simplify(Actions);
  if Result <> Self then
    Exit;
  try
    SumDenom[1] := EB([0,'-',j1,'-',j2,'-',j3],Actions);
    SumDenom[2] := EB([0,'-',j1,'-',l2,'-',l3],Actions);
    SumDenom[3] := EB([0,'-',l1,'-',j2,'-',l3],Actions);
    SumDenom[4] := EB([0,'-',l1,'-',l2,'-',j3],Actions);
    SumDenom[5] := EB([j1,'+',j2,'+',l1,'+',l2],Actions);
    SumDenom[6] := EB([j2,'+',j3,'+',l2,'+',l3],Actions);
    SumDenom[7] := EB([j1,'+',j3,'+',l1,'+',l3],Actions);
    zMin := 0;
    for i := 1 to 4 do begin
      Node := SumDenom[i];
      //Writeln(i,':',Node.Text(true));
      if Node is TIntegerNode then
        zMin := Max(zMin, -TIntegerNode(Node).Value);
    end;
    zMax := High(Integer);
    for i := 5 to 7 do begin
      Node := SumDenom[i];
      //Writeln(i,':',Node.Text(true));
      if Node is TIntegerNode then
        zMax := Min(zMax, TIntegerNode(Node).Value);
    end;

    if zMax < zMin then begin
      Result := TIntegerNode.Create(0);
      Free;
      Exit;
    end;

    if zMax > 10 then begin
      eZ := nil;
      for i := 1 to 4 do begin
        LimitsZ(i,5);
        LimitsZ(i,6);
        LimitsZ(i,7);
      end;
      if not Assigned(eZ) then
        Exit;
    end else
      eZ := TIntegerNode.Create(zMin);
    Node := OperatorPlus.CreateExprNode;
    for z := zMin to zMax do begin
      Node.AddNode(Term(eZ,z-zMin));
      ez := EN([ez, '+', 1],Actions);
    end;
    Node := CheckSimplify(Node,Actions);
    Result := EN([-1, '^', EN([eZ, '-', zMax-zMin+1],Actions), '*', Node],Actions);
    //WriteLn('Sum: ', Node.Text(True));
    Result := EN([Node, '*', Delta(j1,j2,j3), '*', Delta(j1,l2,l3), '*', Delta(l1,j2,l3), '*', Delta(l1,l2,j3)],Actions);
  finally
    for i := 1 to 7 do
      SumDenom[i].Free;
  end;
  Free;
end;

{ TThreejNode }

function TThreejNode.Simplify(Actions: TSimplifyActions): TExprNode;
var
  i: Integer;
  Node: TExprNode;
begin
  { Convert to Clebsch-Gordan equivalent }
  Result := EB([j1, '-', j2, '-', m3]);
  Result := EN([-1, '^', Result, '/', EB([j3, '*', 2, '+', 1, 'sqrt'])]);
  Node := CGFunction.CreateExprNode;
  for i := 0 to 4 do
    Node.AddNode(MoveChild(i));
  Node.AddNode(MonadicMinus.CreateExprNode(MoveChild(5))); { m3 }
  Result := EN([Result, '*', Node]);
  Free;
end;

{ TFunctionNodeArg }

function TFunctionNodeArg.GetArgn(Index: integer): TExprNode;
begin
  Result := Children[Index];
end;

initialization

CGFunction := TCGFunction.Create({$IFDEF FPC}@{$ENDIF}_CG, 6, 'CG');
GlobalFunctions.AddObject('CG', CGFunction);

ThreejFunction := TThreejFunction.Create({$IFDEF FPC}@{$ENDIF}_threej, 6, 'threej');
GlobalFunctions.AddObject('threej', ThreejFunction);

SixjFunction := TSixjFunction.Create({$IFDEF FPC}@{$ENDIF}_sixj, 6, 'sixj');
GlobalFunctions.AddObject('sixj', SixjFunction);

end.
