unit ExpCan;
{$I DFS.inc}{ Copyright (C) 2005 C M Western; see PcontrolsReg.pas for license}
interface

uses {$IFNDEF CLX_} Graphics, {$ELSE} QGraphics, {$ENDIF}
     {$IFNDEF LCL_} Types, {$ENDIF} Classes;

type

  TExportableCanvas = class
  protected
    procedure SetFont(const AValue: TFont); virtual;
    function GetFont: TFont; virtual; abstract;
    procedure SetPixels(X, Y: Integer; const Value: TColor); virtual; abstract;
  public
    function RealCanvas: TCanvas; virtual;
    function Pen: TPen; virtual; abstract;
    function Brush: TBrush; virtual; abstract;
    function DPI: Integer; { Assumed isotropic } virtual; abstract;
    procedure Ellipse(X1, Y1, X2, Y2: Integer); virtual; abstract;
    procedure FillRect(const Rect: TRect); virtual; abstract;
    procedure Polyline(Points: array of TPoint; const nPoints: Integer); virtual; abstract;
    procedure Polygon(Points: array of TPoint; const nPoints: Integer); virtual; abstract;
    procedure RotatedTextOut(x, y: Integer; const str: string; RealTextAngle: Integer; Justify: TAlignment); virtual; abstract;
    procedure StretchDraw(var Rect: TRect; Graphic: TGraphic); virtual; abstract;
    procedure TextOut(X,Y: Integer; const Text: String); virtual;
    procedure TextRect(ARect: TRect; X, Y: integer; const Text : string); virtual;
    property Pixels[X, Y: Integer]: TColor write SetPixels;
    property Font: TFont read GetFont write SetFont;
  end;

  TCustomStreamCanvas = class(TExportableCanvas)
  public
    constructor Create(AStream: TStream; Xmm, Ymm: Double); virtual;
  end;

  TStreamCanvasClass = class of TCustomStreamCanvas;

implementation

uses {$IFDEF MSWINDOWS} Windows, {$ENDIF} SysUtils;

{ TExportableCanvas }

procedure TExportableCanvas.SetFont(const AValue: TFont);
begin
  GetFont.Assign(AValue);
end;

function TExportableCanvas.RealCanvas: TCanvas;
begin
  Result := nil;
end;

procedure TExportableCanvas.TextOut(X, Y: Integer; const Text: String);
begin
  RotatedTextOut(X, Y, Text, 0, taLeftJustify);
end;

procedure TExportableCanvas.TextRect(ARect: TRect; X, Y: integer; const Text: string);
begin
  TextOut(X, Y, Text);
end;

{ TCustomStreamCanvas }

constructor TCustomStreamCanvas.Create(AStream: TStream; Xmm, Ymm: Double);
begin
  inherited Create;
end;

end.
