unit ExportG;
{$I DFS.inc}{ Copyright (C) 2005 C M Western; see PcontrolsReg.pas for license}
interface

uses {$IFNDEF CLX_} Graphics, {$ELSE} QGraphics, {$ENDIF}
     ExpCan, {$IFDEF LCL_}{$IFNDEF CONSOLE} PostScriptPrinter, {$ENDIF}{$ENDIF} {$IFNDEF LCL_} Types, {$ENDIF} Classes;

type

  { Routines common to all file formats }
  TStreamCanvas = class(TCustomStreamCanvas)
  protected
    FStream: TStream;
    FBrush: TBrush;
    FPen: TPen;
    FFont: TFont;
    function GetFont: TFont; override;
    procedure PF(const Format: string; const Args: array of const); virtual;
  public
    constructor Create(AStream: TStream; Xmm, Ymm: Double); override;
    destructor Destroy; override;
    procedure StretchDraw(var Rect: TRect; Graphic: TGraphic); override;
    function Pen: TPen; override;
    function Brush: TBrush; override;
    property Stream: TStream read FStream;
  end;

  TSVGCanvas = class(TStreamCanvas)
  private
    function ColourName: string;
    procedure PutPoints(Points: array of TPoint; nPoints: Integer);
  public
    constructor Create(AStream: TStream; Xmm, Ymm: Double); override;
    destructor Destroy; override;
    function DPI: Integer; override;
    procedure SetPixels(X, Y: Integer; const Value: TColor); override;
    procedure Ellipse(X1, Y1, X2, Y2: Integer); override;
    procedure FillRect(const Rect: TRect); override;
    procedure Polyline(Points: array of TPoint; const nPoints: Integer); override;
    procedure Polygon(Points: array of TPoint; const nPoints: Integer); override;
    procedure RotatedTextOut(x, y: Integer; const str: string; RealTextAngle: Integer; Justify: TAlignment); override;
  end;

  TXFigCanvas = class(TStreamCanvas)
  private
    ColourTable: array of TColor;
    RealStream: TStream;
    function ColourNumber: Integer;
    procedure PutPoints(Points: array of TPoint; const nPoints: Integer);
  public
    constructor Create(AStream: TStream; Xmm, Ymm: Double); override;
    destructor Destroy; override;
    function DPI: Integer; override;
    procedure SetPixels(X, Y: Integer; const Value: TColor); override;
    procedure Ellipse(X1, Y1, X2, Y2: Integer); override;
    procedure FillRect(const Rect: TRect); override;
    procedure Polyline(Points: array of TPoint; const nPoints: Integer); override;
    procedure Polygon(Points: array of TPoint; const nPoints: Integer); override;
    procedure RotatedTextOut(x, y: Integer; const str: string; RealTextAngle: Integer; Justify: TAlignment); override;
    function nColours: Integer;
  end;

  TTextCanvas = class(TStreamCanvas) { Data in format suitable for spreadsheet or other program }
  public
    function DPI: Integer; override;
    procedure SetPixels(X, Y: Integer; const Value: TColor); override;
    procedure Ellipse(X1, Y1, X2, Y2: Integer); override;
    procedure FillRect(const Rect: TRect); override;
    procedure Polyline(Points: array of TPoint; const nPoints: Integer); override;
    procedure Polygon(Points: array of TPoint; const nPoints: Integer); override;
    procedure RotatedTextOut(x, y: Integer; const str: string; RealTextAngle: Integer; Justify: TAlignment); override;
  end;

  TNullCanvas = class(TStreamCanvas) { Discard everything }
  public
    function DPI: Integer; override;
    procedure SetPixels(X, Y: Integer; const Value: TColor); override;
    procedure Ellipse(X1, Y1, X2, Y2: Integer); override;
    procedure FillRect(const Rect: TRect); override;
    procedure Polyline(Points: array of TPoint; const nPoints: Integer); override;
    procedure Polygon(Points: array of TPoint; const nPoints: Integer); override;
    procedure RotatedTextOut(x, y: Integer; const str: string; RealTextAngle: Integer; Justify: TAlignment); override;
  end;

  {$IFDEF LCL_}{$IFNDEF CONSOLE}
  TExportPostscriptCanvas = class(TStreamCanvas) { Printer postscript }
  private
    FPostScriptCanvas: TPostScriptCanvas;
  protected
    procedure PF(const Format: string; const Args: array of const); override;
  public
    constructor Create(ACanvas: TPostScriptCanvas; Xmm, Ymm: Double);
    procedure SetPixels(X, Y: Integer; const Value: TColor); override;
    procedure Ellipse(X1, Y1, X2, Y2: Integer); override;
    procedure FillRect(const Rect: TRect); override;
    procedure Polyline(Points: array of TPoint; const nPoints: Integer); override;
    procedure Polygon(Points: array of TPoint; const nPoints: Integer); override;
    procedure RotatedTextOut(x, y: Integer; const str: string; RealTextAngle: Integer; Justify: TAlignment); override;
  end;
  {$ENDIF}{$ENDIF}

  function GetStreamCanvasClassFromFilename(Filename: string):TStreamCanvasClass;

implementation

uses {$IFDEF MSWINDOWS} Windows, {$ENDIF}
     {$IFNDEF CONSOLE}
       Forms,
     {$IFDEF LCL_} LCLIntf, {$ENDIF}
     {$ENDIF}
     SysUtils, Math, UCompat;

{ TStreamCanvas }

constructor TStreamCanvas.Create(AStream: TStream; Xmm, Ymm: Double);
begin
  inherited;
  FStream := AStream;
  FBrush := TBrush.Create;
  FPen := TPen.Create;
  FFont := TFont.Create;
  Font.PixelsPerInch := DPI;
  Font.Size := 12;
end;

destructor TStreamCanvas.Destroy;
begin
  FreeAndNil(FFont);
  FreeAndNil(FPen);
  FreeAndNil(FBrush);
  inherited;
end;

function TStreamCanvas.Brush: TBrush;
begin
  Result := FBrush;
end;

function TStreamCanvas.Pen: TPen;
begin
  Result := FPen;
end;

function TStreamCanvas.GetFont: TFont;
begin
  Result := FFont;
end;

procedure TStreamCanvas.PF(const Format: string; const Args: array of const);
var
  Buffer: string;
begin
  Buffer := SysUtils.Format(Format, Args);
  Buffer := Buffer + lineend;
  Stream.WriteBuffer(Buffer[1], Length(Buffer));
end;

procedure TStreamCanvas.StretchDraw(var Rect: TRect; Graphic: TGraphic);
begin
  { Throw away by default }
  { TODO: should be implementable for SVG, PS? }
end;

{ TSVGCanvas }

function TSVGCanvas.ColourName: string;
var
  c: Longint;
begin
  { This format works on most systems.... (can not even skip leading 00) }
  if Pen.Color = clBlack then
    Result := 'black'
  else begin
    c := ColorToRGB(Pen.Color);
    Result := Format('#%.2x%.2x%.2x', [GetRValue(c), GetGValue(c), GetBValue(c)]);
  end;
end;

constructor TSVGCanvas.Create(AStream: TStream; Xmm, Ymm: Double);
begin
  inherited Create(AStream, Xmm, Ymm);
  PF('<?xml version="1.0" encoding="UTF-8" standalone="no"?>', []);
  PF('<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">',[]);
  PF('<svg version="1.1" xmlns="http://www.w3.org/2000/svg" width="%.3f mm" height="%.3f mm">' + lineend,
      [Xmm, Ymm]);
end;

destructor TSVGCanvas.Destroy;
begin
  PF('</svg>', []);
  inherited;
end;

function TSVGCanvas.DPI: Integer;
begin
  Result := 2540; { Use 0.01 mm as resoultion }
end;

procedure TSVGCanvas.Ellipse(X1, Y1, X2, Y2: Integer);
var
  Fill: string;
begin
  if Brush.Style = bsClear then
    Fill := 'none'
  else
    Fill := ColourName;
  PF('<ellipse style="stroke:%s; stroke-linecap:round; stroke-linejoin:round; fill:%s;"', [ColourName, Fill]);
  PF('cx="%.2f" cy="%.2f" rx="%.2f" ry="%.2f" />',
     [(X1+X2)/200, (Y1+Y2)/200, Abs(X2-X1)/200, Abs(Y2-Y1)/200]);
end;

procedure TSVGCanvas.FillRect(const Rect: TRect);
begin
  PF('<rect style="stroke:none; fill:%s;"', [ColourName]);
  PF('x="%.2f" y="%.2f" width="%.2f" height="%.2f" />',
     [Min(Rect.Left, Rect.Right)/100, Min(Rect.Top, Rect.Bottom)/100,
      Abs(Rect.Right-Rect.Left)/100, Abs(Rect.Bottom-Rect.Top)/100]);
end;

procedure TSVGCanvas.Polygon(Points: array of TPoint; const nPoints: Integer);
begin
  PF('<path style="stroke:none; fill:%s;"', [ColourName]);
  PutPoints(Points, nPoints);
end;

procedure TSVGCanvas.Polyline(Points: array of TPoint; const nPoints: Integer);
begin
  PF('<path style="stroke:%s; fill:none; stroke-linecap:round; '
     +'stroke-linejoin:round"', [ColourName]);
  PutPoints(Points, nPoints);
end;

procedure TSVGCanvas.PutPoints(Points: array of TPoint; nPoints: Integer);
var
  i: Integer;
begin
  PF('d="M %.2f %.2f', [Points[0].x/100, Points[0].y/100]);
  for i := 1 to nPoints-1 do
    PF('L %.2f %.2f', [Points[i].x/100, Points[i].y/100]);
  PF('"/>', []);
end;

procedure TSVGCanvas.RotatedTextOut(x, y: Integer; const str: string;
  RealTextAngle: Integer; Justify: TAlignment);
var
  s, JustName: string;
  p: Integer;
begin
  case Justify of
    taLeftJustify: JustName := 'text-anchor:start';
    taCenter: JustName := 'text-anchor:middle';
    taRightJustify: JustName := 'text-anchor:end';
  end;
  if RealTextAngle <> 0 then begin
    PF('<g transform="translate(%.2f,%.2f) rotate(%.1f)">', [x/100,y/100,360-RealTextAngle/10]);
    x := 0;
    y := 0;
  end;
  // Omit: font-family:Helvetica;
  PF('<text x="%.2f" y="%.2f" style="font-size:%.2f; fill:black; '
     + JustName, [x/100,y/100,Abs(Font.Height/100)]);
  { Escape < and & }
  s := str;
  p := 1;
  while p <= Length(s) do
    case s[p] of
      '&': begin Insert('amp;',s,p+1); Inc(p,5); end;
      '<': begin Insert('lt;',s,p+1); s[p] := '&'; Inc(p,4); end;
    else
      Inc(p);
    end;
  PF('">'+s+'</text>', []);
  if RealTextAngle <> 0 then
    PF('</g>',[]);
end;

procedure TSVGCanvas.SetPixels(X, Y: Integer; const Value: TColor);
begin
  WriteLn('How do I set a pixel in SVG?');
end;

{ TXFigCanvas }

const
  fig_colors: array [0..31] of TColor =
  ( clBlack, clBlue, clGreen, clAqua, clRed, clFuchsia, clYellow, clWhite,
    $900000, $b00000, $d00000, $ffce87, $009000, $00b000, $00d000, $909000,
    $b0b000, $d0d000, $000090, $0000b0, $0000d0, $900090, $b000b0, $d000d0,
    $003080, $0040a0, $0060c0, $8080ff, $a0a0ff, $c0c0ff, $e0e0ff, $00d7ff );

function TXFigCanvas.nColours: Integer;
begin
  Result := High(ColourTable);
end;

function TXFigCanvas.ColourNumber: Integer;
var
  i: Integer;
begin
  for i := 0 to nColours do
    if ColourTable[i] = Pen.Color then begin
      Result := i;
      Exit;
    end;
  Result := nColours+1;
  SetLength(ColourTable, Result+1);
  ColourTable[Result] := Pen.Color;
end;

constructor TXFigCanvas.Create(AStream: TStream; Xmm, Ymm: Double);
var
  i: Integer;
begin
  inherited;
  SetLength(ColourTable, High(fig_colors)+1);
  for i := 0 to nColours do
    ColourTable[i] := fig_colors[i];
  PF('#FIG 3.2',[]);
  if Xmm > Ymm then
    PF('Landscape', [])
  else
    PF('Portrait', []);
  PF('Center', []);
  PF('Inches', []);
  PF('A4', []);
  PF('100.00', []);
  PF('Single', []);
  PF('-2', []);
  PF('# Generated by %s', [UCompat.VersionInfo]);
  PF('%d 2', [DPI]);
  { As custom colours must be defined before anything else, divert
    rest of stream to memory }
  RealStream := AStream;
  FStream := TMemorySTream.Create;
end;

destructor TXFigCanvas.Destroy;
var
  i: Integer;
  c: Longint;
  t: TStream;
begin
  if RealStream <> nil then begin
    { Swap streams back }
    t := RealStream;
    RealStream := Stream;
    FStream := t;
    for i := High(fig_colors)+1 to High(ColourTable) do begin
      c := ColourTable[i];
      PF('0 %d #%.2x%.2x%.2x', [i, GetRValue(c), GetGValue(c), GetBValue(c)]);
    end;
    Stream.CopyFrom(RealStream, 0);
    FreeAndNil(RealStream);
  end;
  inherited;
end;

function TXFigCanvas.DPI: Integer;
begin
  Result := 1200;
end;

procedure TXFigCanvas.Ellipse(X1, Y1, X2, Y2: Integer);
var
  area_fill: Integer;
begin
  if Brush.Style = bsClear then
    area_fill := -1
  else
    area_fill := 20;
  { ellipse defined by diameters }
  PF('1 2 0 %d %d %d 10 0 %d 10 1 0 %d %d %d %d %d %d %d %d',
     [Pen.Width, ColourNumber, ColourNumber, area_fill,
      Round((X1+X2)/2), Round((Y1+Y2)/2), Round(Abs(X2-X1)/2), Round(Abs(Y1-Y2)/2),
      X1, Y1, X2, Y2]);
end;

procedure TXFigCanvas.FillRect(const Rect: TRect);
begin
  PF('2 2 0 0 0 %d 10 0 20 10 1 1 10 0 0 5', [ColourNumber]);
  PF('%d %d', [Rect.Left, Rect.Top]);
  PF('%d %d', [Rect.Right, Rect.Top]);
  PF('%d %d', [Rect.Right, Rect.Bottom]);
  PF('%d %d', [Rect.Left, Rect.Bottom]);
  PF('%d %d', [Rect.Left, Rect.Top]);
end;

procedure TXFigCanvas.Polygon(Points: array of TPoint; const nPoints: Integer);
begin
  PF('2 3 0  0  0 %d 10 0 20 10 1 1 10 0 0 %d', [ColourNumber, nPoints]);
  PutPoints(Points, nPoints);
end;

procedure TXFigCanvas.Polyline(Points: array of TPoint;  const nPoints: Integer);
begin
  { 2=polyline 1=polyline 0=line style=solid %d %d
    0=fill colour 10=depth 0=pen style -1=area_fill=none 10=style_val
    1=join_style=Round 1=cap_style=Round 10=radius 0=forward_arrow=off 0=backward_arrow=off }
  PF('2 1 0 %d %d 0 10 0 -1 10 1 1 10 0 0 %d', [Pen.Width, ColourNumber, nPoints]);
  PutPoints(Points, nPoints);
end;

procedure TXFigCanvas.PutPoints(Points: array of TPoint; const nPoints: Integer);
var
  i: Integer;
begin
  for i := 0 to nPoints-1 do
    PF('%d %d ', [Points[i].x, Points[i].y]);
end;

procedure TXFigCanvas.RotatedTextOut(x, y: Integer; const str: string;
  RealTextAngle: Integer; Justify: TAlignment);
var
  j: Integer;
begin
  case Justify of
    taLeftJustify:  j := 0;
    taCenter:       j := 1;
    taRightJustify: j := 2;
  else
    j := 0;
  end;
  { 4=text %d 0=colour 10=depth 0=pen_style 4=font=sans serif %d %f
    0=font flags 0=height 0=length }
  PF('4 %d 0 10 0 4 %d %.4f 0 0 0 %d %d %s\001',
     [j, Abs(Font.Size), RealTextAngle*0.1*Pi/180, x, y, str]);
end;

procedure TXFigCanvas.SetPixels(X, Y: Integer; const Value: TColor);
begin
  WriteLn('How do I set a fig pixel?');
end;

{$IFDEF LCL_}{$IFNDEF CONSOLE}
{ TExportPostscriptCanvas }

procedure TExportPostscriptCanvas.PF(const Format: string;
  const Args: array of const);
begin
  FPostScriptCanvas.PostScript.Add(SysUtils.Format(Format, Args));
end;

constructor TExportPostscriptCanvas.Create(ACanvas: TPostScriptCanvas; Xmm,  Ymm: Double);
begin
  inherited Create(nil, Xmm, Ymm); { Could map stringlist to stream }
  FPostScriptCanvas := ACanvas;
end;

procedure TExportPostscriptCanvas.SetPixels(X, Y: Integer; const Value: TColor);
begin
  WriteLn('How do I set a postscript point?');
end;

procedure TExportPostscriptCanvas.Ellipse(X1, Y1, X2, Y2: Integer);
begin
  FPostScriptCanvas.Ellipse(X1, Y1, X2, Y2);
end;

procedure TExportPostscriptCanvas.FillRect(const Rect: TRect);
begin
  FPostScriptCanvas.Rectangle(Rect); { TODO : Use FillRect? }
end;

procedure TExportPostscriptCanvas.Polyline(Points: array of TPoint; const nPoints: Integer);
begin
  FPostScriptCanvas.Polyline(@Points[0], High(Points));
end;

procedure TExportPostscriptCanvas.Polygon(Points: array of TPoint; const nPoints: Integer);
begin
  WriteLn('FPostScriptCanvas.Polygon(Points)');
end;

procedure TExportPostscriptCanvas.RotatedTextOut(x, y: Integer;
  const str: string; RealTextAngle: Integer; Justify: TAlignment);
begin
  WriteLn('FPostScriptCanvas.RotatedTextOut(x, y, str, RealTextAngle, Justify)');
end;
{$ENDIF}{$ENDIF}

{ TTextCanvas }

function TTextCanvas.DPI: Integer;
begin
  Result := 1000; { Not necessarily used }
end;

procedure TTextCanvas.Ellipse(X1, Y1, X2, Y2: Integer);
begin
  PF('%d'#9'%d'#9'%d'#9'%d', [X1, Y1, X2, Y2]);
end;

procedure TTextCanvas.FillRect(const Rect: TRect);
begin
  PF('%d'#9'%d'#9'%d'#9'%d', [Rect.Left, Rect.Top, Rect.Right, Rect.Bottom]);
end;

procedure TTextCanvas.Polygon(Points: array of TPoint; const nPoints: Integer);
var
  i: Integer;
begin
  for i := 0 to nPoints-1 do
    PF('%d'#9'%d', [Points[i].x, Points[i].y]);
  PF('',[]);
end;

procedure TTextCanvas.Polyline(Points: array of TPoint; const nPoints: Integer);
var
  i: Integer;
begin
  for i := 0 to nPoints-1 do
    PF('%d'#9'%d', [Points[i].x, Points[i].y]);
  PF('',[]);
end;

procedure TTextCanvas.RotatedTextOut(x, y: Integer; const str: string; RealTextAngle: Integer; Justify: TAlignment);
begin
  PF('%d'#9'%d'#9'%s'#9'%d'#9'%d', [x, y, str, RealTextAngle, Ord(Justify)]);
end;

function GetStreamCanvasClassFromFilename(Filename: string):TStreamCanvasClass;
begin
  Filename := ExtractFileExt(Filename);
  if Filename = '.xfig' then
    Result := TXFigCanvas
  else if Filename = '.svg' then
    Result := TSVGCanvas
  else { Assume text file such as .txt, .dat }
    Result := TTextCanvas
end;

procedure TTextCanvas.SetPixels(X, Y: Integer; const Value: TColor);
begin
  PF('%d'#9'%d'#9'%d', [X, Y, Value]);
end;

{ TNullCanvas }

function TNullCanvas.DPI: Integer;
begin
  Result := 1000; { Not necessarily used }
end;

procedure TNullCanvas.Ellipse(X1, Y1, X2, Y2: Integer);
begin
end;

procedure TNullCanvas.FillRect(const Rect: TRect);
begin
end;

procedure TNullCanvas.Polygon(Points: array of TPoint; const nPoints: Integer);
begin
end;

procedure TNullCanvas.Polyline(Points: array of TPoint; const nPoints: Integer);
begin
end;

procedure TNullCanvas.RotatedTextOut(x, y: Integer; const str: string; RealTextAngle: Integer; Justify: TAlignment);
begin
end;

procedure TNullCanvas.SetPixels(X, Y: Integer; const Value: TColor);
begin
end;

end.
