unit Gmarks;
{$I DFS.inc}{ Copyright (C) 2005 C M Western; see PcontrolsReg.pas for license}
interface

uses Classes, Graphics,
     {$IFDEF MSWINDOWS} Windows, {$ENDIF}
     ExpCan;

type

  TPreparePointsEvent = procedure (var p: array of TPoint; np:Integer) of object;


  TMarkParameters = record
    v, h: Integer;
    BlobV, BlobH: Integer;
    IsRealDevice,
    ForceSymmetricalMarks, DotAtCentre:Boolean;
    rgbColour: TColor;
    PreparePoints: TPreparePointsEvent;
  end;

  TArrowType = (arSimple, arHead, arOpen, arSolid);

  TMarkType = (mtDiamond, mtPlus, mtBox, mtCross, mtTriangle, mtStar, mtITriangle, mtCircle,
               mtFDiamond, mtFBox, mtFTriangle, mtFITriangle, mtFCircle, mtWhiteCircle, mtPlusCircle, mtMinusCircle,
               mtVbar, mtHbar, mtDot, mtXMajor, mtXMinor, mtYMajor, mtYMinor);

const  { Range of "normal" marks }
  FirstMark: TMarkType =  mtDiamond;
  LastMark: TMarkType = mtMinusCircle;
  nMarkTypes = Ord(High(TMarkType))+1;


function QuantOff(d:Integer; plus1:Boolean):Integer;
procedure ClosedFigure(Canvas: TExportableCanvas; x:Integer; y:Integer; const input: array of TPoint; np:Integer;
                       fill :Boolean; var mp: TMarkParameters);
procedure OpenFigure(Canvas: TExportableCanvas; x:Integer; y:Integer; const input: array of TPoint; np:Integer;
                     Circle, DoNotDouble: Boolean; var mp: TMarkParameters);
procedure Mark(Canvas: TExportableCanvas; x:Integer; y:Integer; MarkType:TMarkType; var mp:TMarkParameters);
procedure Dot(Canvas: TExportableCanvas; x, y:Integer; var mp: TMarkParameters);    { Plot a dot }
procedure Arrow(Canvas: TExportableCanvas; x0, y0, x1, y1: Integer; ArrowType:TArrowType; var mp:TMarkParameters);

implementation

uses SysUtils, Math, UCompat;

const
  NullHandle = {$IFDEF MSWINDOWS} 0; {$ENDIF} {$IFDEF UNIX} nil; {$ENDIF}

{$IFNDEF VER80}
{$T+} // What is this?
{$ENDIF}

function QuantOff(d:Integer; plus1:Boolean):Integer;
begin
  if plus1 and (d <> 0) then
    if d > 0 then
      Result := d + 1
    else
      Result := d - 1
  else
    Result := d;
end;

const MAXnp = 20;

procedure ClosedFigure(Canvas: TExportableCanvas; x:Integer; y:Integer; const input: array of TPoint; np:Integer;
                       fill :Boolean; var mp: TMarkParameters);
var
  out:array[0..MAXnp] of TPoint;
  i:Integer;
begin
  for i := 0 to np-1 do begin
    out[i].x := x+input[i].x*mp.h;
    out[i].y := y+input[i].y*mp.v;
  end;
  if Assigned(mp.PreparePoints) then
    mp.PreparePoints(out, np);
  if fill then begin
    {$IFDEF VER80}
    Polygon(Canvas.Handle, out[0], np);
    {$ELSE}
    Canvas.Polygon(out, np);
    {$ENDIF}
  end else begin
    out[np] := out[0];
    {$IFDEF VER80}
    Polyline(Canvas.Handle, out[0], np+1);
    {$ELSE}
    Canvas.Polyline(out, np+1);
    {$ENDIF}
  end;
end;

const
  BlobBound:array [0..1] of TPoint = ((X:-1; Y:+1), (X:+1; Y:-1));

procedure Dot(Canvas: TExportableCanvas; x, y:Integer; var mp: TMarkParameters);    (* Plot a dot *)
var
  BlobP: TMarkParameters;
  r: TRect;
  OldStyle: TPenStyle;
  OldWidth: Integer;
begin
  OldStyle := Canvas.Pen.Style;
  OldWidth := Canvas.Pen.Width;
  if mp.BlobV = 0 then begin
    r.Left := x;
    r.Top := y;
    if Assigned(mp.PreparePoints) then
      mp.PreparePoints(r.TopLeft, 1);
    {$IFDEF KYLIX1}
    Canvas.Pen.Style := psSolid;
    Canvas.DrawPoint(r.TopLeft);
    {$ELSE}
    Canvas.Pixels[r.Left, r.Top] := Canvas.Pen.Color;
    {$ENDIF}
  end else begin
    BlobP := mp;
    BlobP.v := mp.BlobV;
    BlobP.h := mp.BlobH;
    { QPainter_setPen(DC, QColor(mp.rgbColour)); }
    Canvas.Pen.Style := psSolid;
    { ? Needed ? Canvas.Pen.Mode := pmCopy; }
    Canvas.Pen.Width := 0;
    OpenFigure(Canvas, x, y, BlobBound, 2, True, True, BlobP);
  end;
  if OldStyle <> Canvas.Pen.Style then
    Canvas.Pen.Style := OldStyle;
  if OldWidth <> Canvas.Pen.Width then
    Canvas.Pen.Width := OldWidth;
end;

const
  box:array [0..3] of TPoint = ((X:-2; Y:-2), (X:+2; Y:-2), (X:+2; Y:+2), (X:-2; Y:+2));
  diamond:array [0..3] of TPoint = ((X: 0; Y:+2), (X:-2; Y: 0), (X: 0; Y:-2), (X:+2; Y: 0));
  triangle:array [0..2] of TPoint =((X: 0; Y:-2), (X:-2; Y: 1), (X: 2; Y: 1));
  itriangle:array [0..2] of TPoint =((X: 0; Y: 2), (X:-2; Y:-1), (X: 2; Y:-1));
  pluscross:array [0..7] of TPoint =((X:-2; Y: 0), (X:+2; Y: 0), (X: 0; Y:-2), (X: 0; Y:+2),
                                     (X:-2; Y:-2), (X:+2; Y:+2), (X:-2; Y:+2), (X:+2; Y:-2));
  smallpluscross:array [0..7] of TPoint =((X:-1; Y: 0), (X:+1; Y: 0), (X: 0; Y:-1), (X: 0; Y:+1),
                                          (X:-1; Y:-1), (X:+1; Y:+1), (X:-1; Y:+1), (X:+1; Y:-1));
  xmajor: array [0..1] of TPoint = ((X:0; Y:0), (X:0; Y:2));
  xminor: array [0..1] of TPoint = ((X:0; Y:0), (X:0; Y:1));
  ymajor: array [0..1] of TPoint = ((X:0; Y:0), (X:2; Y:0));
  yminor: array [0..1] of TPoint = ((X:0; Y:0), (X:1; Y:0));
  smallHbar: array [0..1] of TPoint = ((X:-1; Y:0), (X:1; Y:0));

procedure Mark(Canvas: TExportableCanvas; x:Integer; y:Integer; MarkType:TMarkType; var mp: TMarkParameters);
var
  DoDot:Boolean;
  circle:array [0..1] of TPoint;
  OldBrushStyle: TBrushStyle;
  OldBrushColor: TColor;
  procedure MakeCircle;
  begin
    if mp.h > 0 then
      circle[0].x := -2
    else
      circle[0].x := 2;
    if mp.v > 0 then
      circle[0].y := -2
    else
      circle[0].y := 2;
    circle[1].x := -circle[0].x;
    circle[1].y := -circle[0].y;
  end;
begin
  DoDot := false;
  case MarkType of
      mtDot: (* do dot *)
          DoDot := true;
      mtHbar:    (* Horizontal bar (for error bars) *)
          OpenFigure(Canvas, x, y, pluscross, 2, False, False, mp);
      mtVbar:    (* Vertical bar (for error bars) *)
          OpenFigure(Canvas, x, y, pluscross[2], 2, False, False, mp);
      mtDiamond: (* do diamond *)
        begin
          ClosedFigure(Canvas, x, y, diamond, 4, false, mp);
          DoDot := mp.DotAtCentre;
        end;
      mtPlus: (* do plus *)
          OpenFigure(Canvas, x, y, pluscross, 4, False, False, mp);
      mtBox: (* do box *)
        begin
          ClosedFigure(Canvas, x, y, box, 4, false, mp);
          DoDot := mp.DotAtCentre;
        end;
      mtCross: (* do X *)
          OpenFigure(Canvas, x, y, pluscross[4], 4, False, False, mp);
      mtTriangle: (* do triangle *)
        begin
          ClosedFigure(Canvas, x, y, triangle, 3, false, mp);
          DoDot := mp.DotAtCentre;
        end;
      mtStar: (* do star *)
          OpenFigure(Canvas, x, y, pluscross, 8, False, False, mp);
      mtITriangle: (* do inverted triangle *)
        begin
          ClosedFigure(Canvas, x, y, itriangle, 3, false, mp);
          DoDot := mp.DotAtCentre;
        end;
      mtCircle: (* Open Circle *)
        begin
          OldBrushColor := Canvas.Brush.Color;
          OldBrushStyle := Canvas.Brush.Style;
          Canvas.Brush.Style := bsClear;
          MakeCircle;
          OpenFigure(Canvas, x, y, Circle, 2, True, True, mp);
          Canvas.Brush.Style := OldBrushStyle;
          Canvas.Brush.Color := OldBrushColor; { Changing brush Style can also change color }
          DoDot := mp.DotAtCentre;
        end;
      mtFDiamond: (* Filled diamond *)
          ClosedFigure(Canvas, x, y, diamond, 4, True, mp);
      mtFBox: (* Filled box *)
          ClosedFigure(Canvas, x, y, box, 4, True, mp);
      mtFTriangle: (* Filled triangle *)
          ClosedFigure(Canvas, x, y, triangle, 3, True, mp);
      mtFITriangle: (* Filled inverted triangle *)
          ClosedFigure(Canvas, x, y, itriangle, 3, True, mp);
      mtFCircle: (* Filled Circle *)
        begin
          MakeCircle;
          OpenFigure(Canvas, x, y, Circle, 2, True, True, mp);
        end;
      mtWhiteCircle: { Circle filled with white, rather than pen colour }
        begin
          OldBrushColor := Canvas.Brush.Color;
          OldBrushStyle := Canvas.Brush.Style;
          Canvas.Brush.Color := clWhite;
          Canvas.Brush.Style := bsSolid;
          MakeCircle;
          OpenFigure(Canvas, x, y, Circle, 2, True, True, mp);
          Canvas.Brush.Style := OldBrushStyle;
          Canvas.Brush.Color := OldBrushColor;
        end;
      mtPlusCircle: { Filled circle with + sign }
        begin
          Mark(Canvas, x, y, mtWhiteCircle, mp);
          OpenFigure(Canvas, x, y, smallpluscross, 4, False, False, mp);
        end;
      mtMinusCircle: { Filled circle with - sign }
        begin
          Mark(Canvas, x, y, mtWhiteCircle, mp);
          OpenFigure(Canvas, x, y, smallHbar, 2, False, True, mp);
        end;
      { Axis marks }
      mtXMinor: OpenFigure(Canvas, x, y, xminor, 2, False, True, mp);
      mtXMajor: OpenFigure(Canvas, x, y, xmajor, 2, False, True, mp);
      mtYMinor: OpenFigure(Canvas, x, y, yminor, 2, False, True, mp);
      mtYMajor: OpenFigure(Canvas, x, y, ymajor, 2, False, True, mp);
  end;
  if DoDot then
    Dot(Canvas, x, y, mp);
end;

{$R-}
procedure OpenFigure(Canvas: TExportableCanvas; x:Integer; y:Integer; const input: array of TPoint; np:Integer;
                     Circle, DoNotDouble: Boolean; var mp: TMarkParameters);
var
  out:array[0..MAXnp] of TPoint;
  i:Integer;
  p:TPoint;
  ExtraPixel: Boolean;
begin
  if 2*np > MAXnp then raise Exception.CreateFmt('OpenFigure (%d)', [np]);
  if mp.ForceSymmetricalMarks and not (Circle or DoNotDouble) then begin
    p.x := x; p.y := y;
    for i := 0 to np-1 do begin
      out[2*i].x := x + input[i].x*mp.h;
      out[2*i].y := y + input[i].y*mp.v;
      out[2*i+1] := p;
    end;
    np := np * 2;
  end else
    for i := 0 to np-1 do begin
      { As last pixel not set, need to extend lines that are not continued }
      ExtraPixel := (i and 1) <> 0;
      out[i].x := x + QuantOff(input[i].x*mp.h, ExtraPixel);
      { Circles require similar change, except that scale switch means first y rather than second y needs shifting }
      out[i].y := y + QuantOff(input[i].y*mp.v, ExtraPixel xor (Circle and Assigned(mp.PreparePoints)));
    end;
  if Assigned(mp.PreparePoints) then
    mp.PreparePoints(out, np);
  if Circle then begin
    {$IFDEF MSWINDOWS}
    Canvas.Ellipse(out[0].x, out[0].y, out[1].x, out[1].y);
    {$ENDIF}
    {$IFDEF UNIX}
    if out[0].y > out[1].y then begin
      i := out[0].y;
      out[0].y := out[1].y;
      out[1].y := i;
    end;
    Canvas.Ellipse(out[0].x, out[0].y+1, out[1].x, out[1].y+1); { +1 corrects for Windows/Kylix difference }
    {$ENDIF}
  end else
    for i := 0 to np div 2 -1 do begin
      {$IFDEF VER80}
      Polyline(Canvas.Handle, out[2*i], 2);
      {$ELSE}
      { Copy required as can't do fortran style slice on dynamic arrays }
      out[0] := out[2*i];
      out[1] := out[2*i+1];
      Canvas.Polyline(out, 2);
      {$ENDIF}
    end;
end;

procedure Arrow(Canvas: TExportableCanvas; x0, y0, x1, y1: Integer; ArrowType:TArrowType; var mp:TMarkParameters);
var
  P: array [0..7] of TPoint;
  Base: TPoint;
  dx, dy, t: Integer;
  d, h: Double;
begin
  P[0].X := x0;
  P[0].Y := y0;
  P[1].X := x1;
  P[1].Y := y1;
  d := Sqrt(Sqr((x1-x0)*mp.v/mp.h)+Sqr(y1-y0));
  if d > 0 then begin
    h := Min(3*mp.v,d); { Make sure head is not bigger than line }
    { Move to base of head }
    dx := Round((x1-x0)*h/mp.v*mp.h/d);
    dy := Round((y1-y0)*h/d);
    Base.X := P[1].X - dx;
    Base.Y := P[1].Y - dy;
    { Rotate by 90 and scale for arrow corner }
    t := dy;
    dy := MulDiv(dx,mp.v,mp.h*3);
    dx := -MulDiv(t,mp.h,mp.v*3);
    if ArrowType <= arHead then begin
      P[2].X := Base.X + dx;
      P[2].Y := Base.Y + dy;
      { Reverse direction for other corner }
      P[3].X := Base.X - dx;
      P[3].Y := Base.Y - dy;
      if Assigned(mp.PreparePoints) then
        mp.PreparePoints(P,4);
      Canvas.Polyline(P,2);
      if ArrowType = arSimple then begin
        Canvas.Polyline([P[1],p[2]],2);
        Canvas.Polyline([P[1],p[3]],2);
      end else
        Canvas.Polygon([P[1],P[2],P[3]],3);
    end else begin
      P[0] := P[1];
      P[1].X := Base.X + 2*dx;
      P[1].Y := Base.Y + 2*dy;
      P[2].X := Base.X + dx;
      P[2].Y := Base.Y + dy;
      P[3].X := x0 + dx;
      P[3].Y := y0 + dy;
      P[4].X := x0 - dx;
      P[4].Y := y0 - dy;
      P[5].X := Base.X - dx;
      P[5].Y := Base.Y - dy;
      P[6].X := Base.X - 2*dx;
      P[6].Y := Base.Y - 2*dy;
      P[7] := P[0];
      if Assigned(mp.PreparePoints) then
        mp.PreparePoints(P,8);
      if ArrowType = arOpen then
        Canvas.Polyline(P,8)
      else
        Canvas.Polygon(P,7);
    end;
  end else begin
    if Assigned(mp.PreparePoints) then
      mp.PreparePoints(P,2);
    Canvas.Polyline(P,2);
  end;
end;

end.
