unit Graphs;
{$I DFS.inc}{ Copyright (C) 2005,2009 C M Western; see PcontrolsReg.pas for license}
interface
{ TODO: Default context menu for graphs }
uses
  {$IFDEF MSWINDOWS} Windows, Messages, {$ENDIF}
  Graphics, Controls, Forms, Dialogs, StdCtrls, ExtCtrls, Menus, ClipBrd, Printers,
  {$IFDEF LCL_} Types, LCLType, {$ENDIF}
  SysUtils, Classes, ExpCan, Huge, Prevwdlg, PrintPre, GStyle, Gmarks,
  GraphI, GraphM, MathSup;

{$IFNDEF NONWINLCL}
 {$DEFINE METAFILES}
{$ENDIF}

const
  DefPixels = 25000;

type
  TGraphB = class;

  TReplotEvent = procedure (Sender:TObject; G:TGraphB) of object;

  TGraphPrintPreview = class;

  TDCType = (dcMF, dcClip, dcExport, dcScreen, dcScreenBuffer, dcNone, dcPreview, dcPrinter);

  { TGraphB }

  TGraphB = class(TGraphM)
  private
    OriginalWeight: Longint;
    FOnReplot:TReplotEvent;
    FOnInvalidate:TNotifyEvent;
    FXmm, FYmm: Double;           { Size of plot in mm on Printer }
    FAvailableHeight: Integer;    { Height argument to ShowOnCanvas - used for font scaling }
    FPrintPreview:TGraphPrintPreview; { Private Print Preview Component, for simple printing }
    FPrOnInvalidate:TNotifyEvent; { For Use by print preview }
    FLastMouse: TPoint;
    FPlotFailed: Boolean;         { If last call of FOnReplot raised an exception }
    FAreaSelected:Boolean;
    FSavedAngle, FSavedLinestyle, FSavedTextSize:Integer;
    FSavedJustify: TAlignment;
    FStartPlottingCount: Integer; { Allow nesting of StartPlotting / StopPlotting calls }
    FOnPaint: TNotifyEvent;
    FScreenBuffer: TBitmap;   { For double buffering; note that this buffer is only for
                              make up of plot between Start/Finish pairs, and is drawn and discarded
                              by Finish. (Could keep, and duplicate TGraphW logic) }
    {$IFDEF UNIX}
    FDoubleBuffered: Boolean;
    {$ENDIF}
    procedure SetSelectedArea(r:TRect);
    procedure SetAreaSelected(b: Boolean);
    function GetLogicalSelectedArea: TRect;
    procedure SetLogicalSelectedArea(ARect: TRect);
    procedure MakePrintPreview;
    function IsNonDefaultStyle(i:Integer):Boolean;
    function IsNonDefaultMarkStyle(i:Integer):Boolean;
    function GetTickSize:TPoint;
    procedure SetMetafileScaling;
    {$IFDEF METAFILES}
    function GetPicture:{$IFDEF MSWINDOWS} {$IFDEF WINLCL} HMETAFILE {$ELSE} TMetafile {$ENDIF} {$ENDIF} {$IFDEF UNIX} TDrawing {$ENDIF};
    {$ENDIF}
  protected
    {$IFDEF METAFILES}
    { WIN32 uses old style metafiles, as these seem to be compatible with more applications }
    hMF: {$IFDEF MSWINDOWS} {$IFDEF VER80} TMetafile; {$ELSE} HMETAFILE; {$ENDIF}{$ENDIF}
         {$IFDEF UNIX} TDrawing; {$ENDIF}
    {$IFDEF MSWINDOWS}
    MetafileCanvas: TCanvas;
    MetaFileWidth, MetaFileHeight: Integer;
    {$ENDIF}
    {$ENDIF}
    DCType: TDCType;
    InReplot:Boolean;             (* If OnReplot event call *)
    XSCALE:Double;                (* User selected plot scale *)
    YSCALE:Double;
    oldXSCALE, oldYSCALE: Double;      { Previous User selected plot scale }
    oldHPos, oldVPos: Double;          { Previous Scroll Bar Position (before last zoom) }
    Selecting:Boolean;
    FSelectedArea:TRect;

    GraphicsLogFont: TLogFont;          { Current font description }
    UnrotatedGraphicsLogFont: TLogFont; { Current font description with rotation set to zero }
    {$IFDEF VCL_}
    GraphicsFont: TFont;
    UnrotatedGraphicsFont: TFont;
    text_flags:Word;         { Current text justification }
    {$ENDIF}

    s_TextAngle:Double;      (* sin and cos of TextAngle *)
    c_TextAngle:Double;

    procedure InitGraphics; override;
    procedure cancel_graphics_DC;
    procedure setup_graphics_DC;
    procedure InitFont;
    procedure DoChangeFont; override;
    procedure FontHasChanged(Sender:TObject);
    procedure SetScaling(ACanvas: TCanvas; X, Y, AWidth, AHeight:Longint);
    procedure Paint; override;
    procedure MakeScreenImage(ScreenCanvas: TCanvas);
    procedure SetOnReplot(e:TReplotEvent);
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure UpdateSelection( X, Y: Integer; NewPoint:Boolean);
    procedure DrawSelectedArea(InPaint: Boolean = False);
    procedure ChangeZoom(newXSCALE, newYSCALE, newHscroll, newVscroll: Double); virtual;
    procedure ChangeScrollPos(Hpos, Vpos: double); virtual;
    procedure Resize; override;
    procedure UpdateScrollBars; override;
    function HScrollFraction: Double; { Scroll Bar Positions as fraction of plot space }
    function VScrollFraction: Double;
    procedure ShowOnCanvas(ACanvas: TCanvas; X, Y, AWidth, AHeight:Longint); virtual;
    procedure SetLastMouse(X, Y: Integer); virtual;
    procedure Loaded; override;
    procedure SetRealCanvas(ACanvas: TCanvas);
  protected
    procedure DoCopyToClipboard(Sender: TObject);
    procedure DoExport(Sender: TObject);
    procedure DoPrint(Sender: TObject);
    procedure DoToggleVariableCharSize(Sender: TObject);
    procedure DoToggleLimitSegments(Sender: TObject);
    procedure DoOnPopup(Sender: TObject);
    procedure CreateHandle; override;
    function  UnrotatedTextExtent(const str:string):TSize;
    procedure SetJustify(mode:TAlignment); override;
  public
    property Canvas; { Re-export }
    property MarkStyles[i:Integer]:TMarkStyle read GetMarkStyle write SetMarkStyle;
    property LastMouse: TPoint read FLastMouse;
    property AreaSelected: Boolean read FAreaSelected write SetAreaSelected;
    function SignificantAreaSelected:Boolean;
    property TickSize:TPoint read GetTickSize;
    procedure StuffPopupMenu(APopupMenu: TPopupMenu);
    { May want to make the following private or protected }
    function ClientY:Integer;
    function ClientX:Integer;
    procedure SetScreenScaling;
    procedure ChooseFont(Sender: TObject);

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Start; override;
    procedure Finish; override;
    procedure Suspend; {\ To allow display of partly completed metafile }
    procedure Resume;  {/ Ignored unless OnReplot not set }
    {$IFDEF METAFILES}
    procedure StartMetafile;
    procedure StopMetafile;
    {$ENDIF}
    procedure StartPlotting; override; { Call to ensure DC present }
    procedure StopPlotting; override;  { Call when done with DC }
    function CellHeight:Double; override;
    procedure TextI(x:Integer; y:Integer; str:string); override;
    procedure ClearBox(const Rect: TRect);
    procedure DrawGraphic(Rect: TRect; Graphic: TGraphic); overload;
    procedure XorTextI(x:Integer; y:Integer; str:string);
    function  TextExtentF(const str:string; Fraction: Single):TSize;
    function  TextExtent(const str:string):TSize; override;
    procedure Clear;
    property  SelectedArea:TRect read FSelectedArea write SetSelectedArea;
    property  LogicalSelectedArea:TRect read GetLogicalSelectedArea write SetLogicalSelectedArea;
    procedure CancelSelection;
    procedure Invalidate; override;
    procedure ZoomSelected(Anisotropic:Boolean);
    procedure ResetZoom;
    procedure PreviousZoom; virtual;
    procedure ZoomOut;
    procedure ShowPrintPreview(Sender:TObject; ACanvas:TExportableCanvas; X, Y, AWidth, AHeight:Longint;
                               APage: Integer; OnPrinter:Boolean);
    procedure ExportTo(AStream: TStream; CanvasClass: TStreamCanvasClass); virtual;
    procedure ExecuteExport;
    function IsRealDevice:Boolean; override;
    function IsClipboard:Boolean; override;
    function IsPrinter:Boolean; override;
    function IsScreen:Boolean;
    function PenDialog(Pen:Integer):Boolean;
    function MarkDialog(AMark:Integer):Boolean;
    procedure PrinterSetup;
    procedure OptionChanged; override; { Options changed, but basic plot has not }
    procedure PlotChanged; virtual;   { Plot has changed; forces call of OnReplot }
    {$IFDEF METAFILES}
    property Picture: {$IFDEF MSWINDOWS} {$IFDEF WINLCL} HMETAFILE {$ELSE} TMetafile {$ENDIF} {$ENDIF} {$IFDEF UNIX} TDrawing {$ENDIF} read GetPicture;
    {$ENDIF}
    property PlotFailed:Boolean read FPlotFailed;

  published

    procedure PrintPreview;
    procedure Print;
    function CanPrint: Boolean; virtual;
    procedure CopyToClipboard; virtual;
    function CanCopyToClipboard: Boolean;

    property Align;
    property Anchors;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderWidth;
    property BorderStyle;
    property Color;
    {$IFDEF MSWINDOWS}
    property Ctl3D;
    property ParentCtl3D;
    property DragCursor;
    {$ENDIF}
    property DragMode;
    property DoubleBuffered {$IFDEF UNIX} :Boolean read FDoubleBuffered write FDoubleBuffered {$ENDIF} ;
    property Enabled;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
{$ifdef WIN32}
    property OnStartDrag;
{$endif}

    property OnPaint: TNotifyEvent read FOnPaint write FOnPaint;

   (* Max plotting range - in mm*100 on Printer *)
    property Xmm;
    property Ymm;
    property OnReplot:TReplotEvent read FOnReplot write SetOnReplot;
    property OnInvalidate:TNotifyEvent read FOnInvalidate write FOnInvalidate;
    property GOptions;

    property LineStyle0:TLineStyle index 0 read GetLineStyles write SetLineStyles stored IsNonDefaultStyle;
    property LineStyle1:TLineStyle index 1 read GetLineStyles write SetLineStyles stored IsNonDefaultStyle;
    property LineStyle2:TLineStyle index 2 read GetLineStyles write SetLineStyles stored IsNonDefaultStyle;
    property LineStyle3:TLineStyle index 3 read GetLineStyles write SetLineStyles stored IsNonDefaultStyle;
    property LineStyle4:TLineStyle index 4 read GetLineStyles write SetLineStyles stored IsNonDefaultStyle;
    property LineStyle5:TLineStyle index 5 read GetLineStyles write SetLineStyles stored IsNonDefaultStyle;
    property LineStyle6:TLineStyle index 6 read GetLineStyles write SetLineStyles stored IsNonDefaultStyle;
    property LineStyle7:TLineStyle index 7 read GetLineStyles write SetLineStyles stored IsNonDefaultStyle;
    property LineStyle8:TLineStyle index 8 read GetLineStyles write SetLineStyles stored IsNonDefaultStyle;

    property MarkStyle0:TMarkStyle index 0 read GetMarkStyle write SetMarkStyle stored IsNonDefaultMarkStyle;
    property MarkStyle1:TMarkStyle index 1 read GetMarkStyle write SetMarkStyle stored IsNonDefaultMarkStyle;
    property MarkStyle2:TMarkStyle index 2 read GetMarkStyle write SetMarkStyle stored IsNonDefaultMarkStyle;
    property MarkStyle3:TMarkStyle index 3 read GetMarkStyle write SetMarkStyle stored IsNonDefaultMarkStyle;
    property MarkStyle4:TMarkStyle index 4 read GetMarkStyle write SetMarkStyle stored IsNonDefaultMarkStyle;
    property MarkStyle5:TMarkStyle index 5 read GetMarkStyle write SetMarkStyle stored IsNonDefaultMarkStyle;
    property MarkStyle6:TMarkStyle index 6 read GetMarkStyle write SetMarkStyle stored IsNonDefaultMarkStyle;
    property MarkStyle7:TMarkStyle index 7 read GetMarkStyle write SetMarkStyle stored IsNonDefaultMarkStyle;
    property MarkStyle8:TMarkStyle index 8 read GetMarkStyle write SetMarkStyle stored IsNonDefaultMarkStyle;

  end;

  TGraphPrintPreview = class(TPrintPreview)
  private
    FGraph: TGraphB;
  protected
    procedure SetGraph(G:TGraphB);
    procedure DoShowPreview(Sender:TObject; Canvas:TExportableCanvas; X, Y, AWidth, AHeight:Longint;
                            APage: Integer; OnPrinter:Boolean); override;
  public
    procedure SetMargins(AXSize, AYSize, AXMargin, AYMargin:Double); override;
  published
    property Graph:TGraphB read FGraph write SetGraph;
  end;

  TGraphD = class;

  TReplotDEvent = procedure (Sender:TObject; G:TGraphM) of object;

  TGraphD = class(TGraphB)
  private
    FRecordCalls:Boolean;  { Set to save calls for better replots }
    FOnDReplot:TReplotDEvent;
    FLastMouseX, FLastMouseY: Double;
    XofSave, XsSave, XminSave, YofSave, YsSave, YminSave: Double;
    FZoomArea: TDRect;
    FPreviousZoomArea: TDRect;
    procedure SetRecordCalls(ARecording:Boolean);
    procedure DoGraphReplot(Sender:TObject; G:TGraphB);
    function GetSelectedMathArea: TDrect;
    procedure SetSelectedMathArea(ARect: TDRect);
    procedure SetZoomArea(const Value: TDRect);
    function IsNotAxisDefault: Boolean;
  protected
    procedure SetOnReplot(e:TReplotDEvent);
    procedure SetLastMouse(X, Y: Integer); override;
    procedure ShowOnCanvas(ACanvas: TCanvas; X, Y, AWidth, AHeight:Longint); override;
    procedure ChangeZoom(newXSCALE, newYSCALE, newHscroll, newVscroll: Double); override;
    procedure ChangeScrollPos(Hpos, Vpos: double); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ExportTo(AStream: TStream; CanvasClass: TStreamCanvasClass); override;
    function PixelSize(x: Boolean): Double; override;
    function CloseToX(x1, x2: Double):Boolean;
    function CloseToY(y1, y2: Double):Boolean;
    function InPlotArea(x, y: Double):Boolean;
    function RoundToDisplay(value: Double; xvalue: Boolean): Double;
    procedure Start; override;
    procedure Finish; override;
    procedure DrawGraphic(x1, x2, y1, y2:Double; Graphic: TGraphic); override;
    { Character Sizes }
    function  PlotLength:Double;
    procedure PlotChanged; override;
    procedure LPtoXY(lp: TPoint; var x, y:Double);
    procedure XYtoLP(out lp: TPoint; x, y:Double);
    procedure PreviousZoom; override;
    property LastMouseX: Double read FLastMouseX;
    property LastMouseY: Double read FLastMouseY;
    property SelectedMathArea: TDRect read GetSelectedMathArea write SetSelectedMathArea;
    property ZoomArea: TDRect read FZoomArea write SetZoomArea;
    property PreviousZoomArea: TDRect read FPreviousZoomArea write FPreviousZoomArea;
  published
    function CanPrint: Boolean; override;
    procedure CopyToClipboard; override;
    property RecordCalls:Boolean read FRecordCalls write SetRecordCalls;
    property OnReplot:TReplotDEvent read FOnDReplot write SetOnReplot;
    property AxisStyle stored IsNotAxisDefault;
end;

procedure DrawFocusRectHelper(Canvas: TCanvas; r: TRect);

implementation

uses GraphPen, GPenDlg, GMarkDlg, {$IFNDEF VER80} Math, {$ENDIF}
     {$IFDEF LCL_}
       LCLIntf,
       {$IFDEF LCLanygtk} {$IFNDEF LCLgtk2} gdk, GTKDef, {$ELSE} gdk2, Gtk2Def, {$ENDIF} {$ENDIF}
     {$ENDIF}
     CanCan, ExportG, TextAssign, HugeClip, ScrlCtrl, UCompat;


{$IFNDEF VER80}
{$IFNDEF FPC} { TODO: Remove when FPC bug fixed ? }
{$T+}
{$ENDIF}
{$ENDIF}

{$IFDEF MSWINDOWS}
{$IFDEF VER80}
const
  NullMF = nil;
procedure DiscardMF(var hMF: TMetafile);
begin
  FreeAndNil(hMF);
end;
{$ELSE}
const
  NullMF = 0;
procedure DiscardMF(var hMF: HMETAFILE);
begin
  if hMF <> 0 then
    DeleteMetafile(hMF);
  hMF := 0;
end;
{$ENDIF}
{$ENDIF}

function Min(a, b:Integer):Integer;
begin
  if a < b then
    Result := a
  else
    Result := b;
end;

constructor TGraphB.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  VertScrollBar.ScrollJump := 10;
  HorzScrollBar.ScrollJump := 10;
  Width := 100;
  Height := 100;
  {$IFDEF MSWINDOWS}
  Font.Name := 'Arial';
  {$ENDIF}
  {$IFDEF UNIX}
  Font.Name := 'Helvetica';
  {$ENDIF}
  {$IFDEF LCL_}
  Font.Height := 12;
  {$ENDIF}
  ParentFont := false;
  Font.OnChange := {$IFDEF FPC}@{$ENDIF}FontHasChanged;
  Brush.OnChange := {$IFDEF FPC}@{$ENDIF}FontHasChanged;

  FCurrentCanvas := nil;
  DCType := dcNone;
  FStartPlottingCount := 1;
  LoadMP;
  {$IFDEF VCL_}
  GraphicsFont := TFont.Create;
  UnrotatedGraphicsFont := TFont.Create;
  {$ENDIF}
  {$IFDEF METAFILES}
  hMF := NullMF;
  {$ENDIF}
  XSCALE := 1;
  YSCALE := 1;   { User selected plot scale }
  ResetZoom;
  SetScaling(Canvas, 0, 0, Width, Height); { Sensible defaults for scaling }
end;

procedure TGraphB.SetRealCanvas(ACanvas: TCanvas);
begin
  SetCurrentCanvas(TCanvasCanvas.Create(ACanvas));
end;

function TGraphB.PenDialog(Pen:Integer):Boolean;
var
 Dialog:TGraphPenDialog;
begin
  Dialog := TGraphPenDialog.Create(Application);
  try
    Dialog.Graph := Self;
    Dialog.Pen := Pen;
    Result := Dialog.Execute;
    if Result then
      Dialog.Apply;
  finally
    Dialog.Free;
  end;
end;

function TGraphB.MarkDialog(AMark:Integer):Boolean;
var
 Dialog:TGraphMarkDialog;
begin
  Dialog := TGraphMarkDialog.Create(Application);
  try
    Dialog.Graph := Self;
    Dialog.Mark := AMark;
    Result := Dialog.Execute;
    if Result then
      Dialog.Apply;
  finally
    Dialog.Free;
  end;
end;

procedure TGraphB.Loaded;
var
  i: Integer;
begin
  inherited Loaded;
  for i := 0 to MaxPens do begin
    FDefaultLineStyles[i].Assign(LineStyles[i]);
    if i < MaxPens then
      FDefaultMarkStyles[i].Assign(MarkStyles[i]);
  end;
  if not (csDesigning in ComponentState) and not Assigned(PopupMenu) then begin
    PopupMenu := TPopupMenu.Create(Self);
    StuffPopupMenu(PopupMenu);
  end;
end;

function TGraphB.IsNonDefaultStyle(i:Integer):Boolean;
begin
  Result := (LineStyles[i].Width <> 0)
            or (LineStyles[i].Colour <> DefaultPenColour[i])
            or (Ord(LineStyles[i].Pattern) <> (i mod MAXSTYLE));
end;

function TGraphB.IsNonDefaultMarkStyle(i:Integer):Boolean;
begin
  Result := (MarkStyles[i].Size <> 2)
            or (MarkStyles[i].Colour <> DefaultPenColour[i])
            or (Ord(MarkStyles[i].MarkType) <> (i mod nMarkTypes))
            or not MarkStyles[i].DotAtCentre;
end;

function TGraphB.IsRealDevice:Boolean;
begin
  Result := DCType >= dcScreen;
end;

function TGraphB.IsClipboard:Boolean;
begin
  Result := DCType = dcClip;
end;

function TGraphB.IsPrinter:Boolean;
begin
  Result := DCType >= dcPreview;
end;

function TGraphB.IsScreen:Boolean;
begin
  Result := (DCType = dcScreen) or (DCType = dcScreenBuffer) or (DCType = dcNone);
end;

procedure TGraphB.StartPlotting;        (* Call to ensure DC present *)
begin
  if DCType = dcNone then begin
    FStartPlottingCount := 0;
    DCType := dcScreen;
    SetRealCanvas(Canvas);
    SetScreenScaling;
    DoChangeFont;
  end;
  Inc(FStartPlottingCount);
end;

procedure TGraphB.StopPlotting;         (* Call when done with DC *)
begin
  Dec(FStartPlottingCount);
  if FStartPlottingCount = 0 then begin
    if DCType <> dcScreen then
      raise Exception.Create('StopPlotting mismatch');
    DCType := dcNone;
    SetCurrentCanvas(nil);
    FStartPlottingCount := 1; { Other DCTypes will not use this mechanism }
  end;
end;

function TGraphB.ClientY:Integer;
begin
  Result := PaintHeight
end;

function TGraphB.ClientX:Integer;
begin
  Result := PaintWidth;
end;

procedure TGraphB.InitFont;
var
  ph: Integer;
begin
  CurFontHeight := 0;
  GetObject(Font.Handle, sizeof(GraphicsLogFont), @GraphicsLogFont);
  OriginalWeight := GraphicsLogFont.lfWeight;
  {$IFDEF UNIX}
  if GraphicsLogFont.lfFaceName = '' then
    GraphicsLogFont.lfFaceName := 'helvetica';
  {$ENDIF}
  GraphicsLogFont.lfOutPrecision := OUT_TT_ONLY_PRECIS;
  GraphicsLogFont.lfClipPrecision := CLIP_LH_ANGLES;
  if (grFixedCharSize in GOptions) and (IsPrinter or IsClipboard or (DCType = dcExport)) then
    { Font is fixed at Font.Size points }
    FontSize := Font.Size / 72 * 25.4 / Ymm * YPixels
  else begin
    ph := PaintHeight;
    if ph = 0 then
      ph := Height; { In case paint rect hasn't been calculated yet }
    if ph = 0 then
      ph := FAvailableHeight;
    if ph = 0 then
      ph := 1;
    FontSize := Font.Height / ph * FAvailableHeight;
    if IsScreen then
      FontSize := FontSize / YSCALE;
  end;
  if Abs(FontSize) < 32000 then
    GraphicsLogFont.lfHeight := Round(FontSize);
  {$IFDEF VCL_}
  text_flags := TA_LEFT;
  {$ENDIF}
  FTextDeciAngle := 0;
  FJustify := taLeftJustify;
  FTextSize := 100;
end;

procedure TGraphB.InitGraphics;
begin
  InitFont;
  setup_graphics_DC;
end;

procedure TGraphB.Resize;
begin
  inherited Resize;
  Invalidate;
end;

{$IFDEF METAFILES}
procedure TGraphB.StartMetafile;
begin
  DiscardMF(hMF);
  SetMetafileScaling;
  {$IFDEF MSWINDOWS}
  {$IFDEF VER80}
  hMF := TMetafile.Create;
  {$ENDIF}
  MetafileCanvas := TCanvas.Create;
  MetafileCanvas.Handle := CreateMetafile(nil);
  SetWindowExtEx(MetafileCanvas.Handle, XPixels, YPixels, nil);
  SetRealCanvas(MetafileCanvas);
  FXPixels := MetaFileWidth;
  FYPixels := MetaFileHeight;
  {$ENDIF}
  {$IFDEF UNIX}
  hMF := TDrawing.Create;
  hMF.Height := YPixels;
  hMF.Width := XPixels;
  hMF.Canvas.Start(True);
  SetRealCanvas(hMF.Canvas);
  {$ENDIF}
end;

procedure TGraphB.StopMetafile;
begin
  { Finish Metafile }
  {$IFDEF MSWINDOWS}
  {$IFDEF VER80}
  hMF.Handle := CloseMetaFile(MetafileCanvas.Handle);
  hMF.Width := XPixels;
  hMF.Height := YPixels;
  {$ELSE}
  hMF := CloseMetaFile(MetafileCanvas.Handle);
  {$ENDIF}
  {$ENDIF}
  {$IFDEF UNIX}
  hMF.Canvas.Stop;
  {$ENDIF}
  SetCurrentCanvas(nil);
end;
{$ENDIF}

procedure TGraphB.Start;
var
  ACanvas: TCanvas;
begin
  inherited;
  if not InReplot then begin
    if CurrentCanvas = nil then
      if Assigned(FOnReplot) then begin
        { Assume direct screen access required }
        FreeAndNil(FScreenBuffer);
        if DoubleBuffered then begin
          FScreenBuffer := TBitmap.Create;
          FScreenBuffer.Height := PaintHeight;
          FScreenBuffer.Width := PaintWidth;
          DCType := dcScreenBuffer;
          ACanvas := FScreenBuffer.Canvas;
          SetRealCanvas(ACanvas);
          SetScreenScaling;
          { Bitmap only holds visible part of screen }
          FXOffset := HorzScrollBar.Position;
          FYOffset := VertScrollBar.Position;
        end else begin
          DCType := DCScreen;
          ACanvas := Canvas;
          SetRealCanvas(ACanvas);
          SetScreenScaling;
        end;
        ACanvas.Brush.Color := Color;
        ACanvas.FillRect(Canvas.ClipRect);
      {$IFDEF METAFILES}
      end else begin
        StartMetafile;
        DCType := dcMF;
      {$ENDIF}
      end;
    InitGraphics;
  end;
end;

procedure TGraphB.cancel_graphics_DC;
begin
  DCType := dcNone;
  { Leave scaling and other things in place for mouse }
  StartPlotting;
  SetScreenScaling;
  InitGraphics;
  StopPlotting;
end;

procedure TGraphB.Finish;
var
  NeedInvalidate: Boolean;
begin
  inherited;
  if not InReplot then begin
    NeedInvalidate := True;
    if CurrentCanvas <> nil then begin
      EndDraw;
      if CurrentCanvas.RealCanvas <> Canvas then
       {$IFDEF METAFILES}
        if DCType = dcMF then begin
          StopMetafile;
        end else
        {$ENDIF}
        if DCType = dcScreenBuffer then begin
          Canvas.Draw(PaintRect.Left - HorzScrollBar.Position,
                      PaintRect.Top - VertScrollBar.Position,
                      FScreenBuffer);
          FreeAndNil(FScreenBuffer);
          NeedInvalidate := False;
        end else
          raise Exception.Create('Help!');
      SetCurrentCanvas(nil);
      cancel_graphics_DC;
    end;
    if NeedInvalidate then
      Invalidate;
    FTextDeciAngle := 0;
  end
end;

procedure TGraphB.DoCopyToClipboard(Sender: TObject);
begin
  CopyToClipboard;
end;

procedure TGraphB.DoExport(Sender: TObject);
begin
  ExecuteExport;
end;

procedure TGraphB.DoOnPopup(Sender: TObject);
var
  i: Integer;
begin
  with (Sender as TPopupMenu).Items do
    for i := 0 to Count - 1 do
      if StripHotkey(Items[i].Caption) = 'Variable Char Size' then
        Items[i].Checked := not (grFixedCharSize in GOptions)
      else if StripHotkey(Items[i].Caption) = 'Limit Line Segments' then
        Items[i].Checked := not (grUnlimitedSegmentLength in GOptions)
end;

procedure TGraphB.DoPrint(Sender: TObject);
begin
  PrintPreview;
end;

procedure TGraphB.DoToggleLimitSegments(Sender: TObject);
begin
  if grUnlimitedSegmentLength in GOptions then
    GOptions := GOptions - [grUnlimitedSegmentLength]
  else
    GOptions := GOptions + [grUnlimitedSegmentLength];
end;

procedure TGraphB.DoToggleVariableCharSize(Sender: TObject);
begin
  if grFixedCharSize in GOptions then
    GOptions := GOptions - [grFixedCharSize]
  else
    GOptions := GOptions + [grFixedCharSize];
end;

procedure TGraphB.StuffPopupMenu(APopupMenu: TPopupMenu);
var
  MenuItem: TMenuItem;
begin
  MenuItem := TMenuItem.Create(APopupMenu);
  APopupMenu.Items.Add(MenuItem);
  with MenuItem do begin
    Caption := 'Copy to clipboard';
    Hint := 'Use Paste Special to paste picture or underlying data (if available) in the target application';
    OnClick := {$IFDEF FPC}@{$ENDIF}DoCopyToClipboard;
  end;
  MenuItem := TMenuItem.Create(APopupMenu);
  APopupMenu.Items.Add(MenuItem);
  with MenuItem do begin
    Caption := 'Print...';
    Hint := 'Show print peview and allow print of plot';
    OnClick := {$IFDEF FPC}@{$ENDIF}DoPrint;
  end;
  MenuItem := TMenuItem.Create(APopupMenu);
  APopupMenu.Items.Add(MenuItem);
  with MenuItem do begin
    Caption := 'Export to file...';
    Hint := 'Export plot to external file';
    OnClick := {$IFDEF FPC}@{$ENDIF}DoExport;
  end;
  MenuItem := TMenuItem.Create(PopupMenu);
  APopupMenu.Items.Add(MenuItem);
  with MenuItem do begin
    Caption := 'Font...';
    Hint := 'Choose font for graph text';
    OnClick := {$IFDEF FPC}@{$ENDIF}ChooseFont;
  end;
  MenuItem := TMenuItem.Create(PopupMenu);
  APopupMenu.Items.Add(MenuItem);
  with MenuItem do begin
    Caption := 'Variable Char Size';
    Hint := 'Character size on printer and clipboard depends on window size';
    OnClick := {$IFDEF FPC}@{$ENDIF}DoToggleVariableCharSize;
  end;
  MenuItem := TMenuItem.Create(PopupMenu);
  APopupMenu.Items.Add(MenuItem);
  with MenuItem do begin
    Caption := 'Limit Line Segments';
    Hint := 'Limit number of segments on a plotted curve';
    OnClick := {$IFDEF FPC}@{$ENDIF}DoToggleLimitSegments;
  end;
  APopupMenu.OnPopup := {$IFDEF FPC}@{$ENDIF}DoOnPopup;
end;

procedure TGraphB.Suspend;
begin
  if not InReplot and not Assigned(FOnReplot) then begin
    FSavedAngle := TextDeciAngle;
    FSavedTextSize := TextSize;
    FSavedLinestyle := FLinetype;
    FSavedJustify := FJustify;
    Finish;
  end;
end;

procedure TGraphB.Resume;
{$IFDEF MSWINDOWS}
var
  LastMF: {$IFDEF VER80} TMetafile; {$ELSE} HMETAFILE; {$ENDIF}
{$ENDIF}
begin
  if not InReplot and not Assigned(FOnReplot) then begin
    {$IFDEF METAFILES}
    if hMF <> NullMF then begin
      {$IFDEF MSWINDOWS}
        LastMF := hMF;
        hMF := NullMF;
        StartMetafile;
        {$IFDEF VER80}
          CurrentCanvas.Draw(0, 0, LastMF);
        {$ELSE}
          PlayMetafile(CurrentCanvas.RealCanvas.Handle, LastMF);
        {$ENDIF}
        DiscardMF(LastMF);
      {$ENDIF}
      {$IFDEF UNIX}
        SetMetafileScaling;
        hMF.Canvas.Start(True);
        SetRealCanvas(hMF.Canvas);
      {$ENDIF}
    end;
    {$ENDIF}
    Start;
    TextSize := FSavedTextSize;
    TextDeciAngle := FSavedAngle;
    Justify := FSavedJustify;
    if FSavedLinestyle >= 0 then
      Linetype := FSavedLinestyle
    else
      FLinetype := -1;
  end;
end;

procedure TGraphB.setup_graphics_DC;
begin
  PenInvalid;
  DoChangeFont;
end;

procedure TGraphB.ClearBox(const Rect: TRect);
begin
  CurrentCanvas.Brush.Color := Color;
  CurrentCanvas.FillRect(Rect);
  { Restore Brush }
  CurrentCanvas.Brush.Color := mp.rgbColour;
end;

procedure TGraphB.DrawGraphic(Rect: TRect; Graphic: TGraphic);
var
  t: Integer;
  Buffer: TBitmap;
  VisibleRect, LogRect: TRect;
begin
  StartPlotting;
  LogToDev(Rect.Left, Rect.Top);
  LogToDev(Rect.Right, Rect.Bottom);
  t := Rect.Top;
  Rect.Top := Rect.Bottom;
  Rect.Bottom := t;
  {$IFDEF OFFLINUX}
  if Graphic.Transparent and (Graphic is TBitmap) and Assigned(CurrentCanvas.RealCanvas) then
    StretchBlt(CurrentCanvas.RealCanvas.Handle, Rect.Left, Rect.Top, Rect.Right-Rect.Left, Rect.Bottom-Rect.Top,
               TBitmap(Graphic).Canvas.Handle, 0, 0, Graphic.Width, Graphic.Height, SRCINVERT)
  else
  {$ENDIF}
    {$IFNDEF MSWINDOWS}
    CurrentCanvas.StretchDraw(Rect, Graphic);
    {$ELSE} // It should be possible to get this to work?
//    if Graphic.Transparent and (Graphic is TBitmap) and (CurrentCanvas.RealCanvas <> nil) then begin
//      StretchBlt(CurrentCanvas.RealCanvas.Handle, Rect.Left, Rect.Top, Rect.Right-Rect.Left, Rect.Bottom-Rect.Top,
//                 TBitmap(Graphic).Canvas.Handle, 0, 0, Graphic.Width, Graphic.Height, SRCAND)
//    end else
      if (Rect.Left < -100) or (Rect.Right > XPixels+100) then begin
        { Problem with large enlargements, typically on clipboard; crop picture first }
        Buffer := TBitmap.Create;
        try
          VisibleRect := Rect;
          t := 0;
          if Rect.Left < 0 then begin
            t := VisibleRect.Left;
            VisibleRect.Left := 0;
          end;
          if Rect.Right >= XPixels then
            VisibleRect.Right := XPixels-1;
          if VisibleRect.Right > VisibleRect.Left then begin
            { create buffer for visible hosizontal fraction }
            Buffer.SetSize(MulDiv(VisibleRect.Right-VisibleRect.Left+1,Graphic.Width,Rect.Right-Rect.Left+1),Graphic.Height);
            t := MulDiv(t,Graphic.Width,Rect.Right-Rect.Left+1);
            if Buffer.Width > 2 then begin
              Buffer.Canvas.Draw(t,0,Graphic);
              CurrentCanvas.StretchDraw(VisibleRect,Buffer);
            end;
          end;
        finally
          Buffer.Free;
        end;
      end else
        CurrentCanvas.StretchDraw(Rect, Graphic);
    {$ENDIF}
  StopPlotting;
end;

procedure TGraphB.FontHasChanged(Sender:TObject);
begin
  CurFontHeight := 0;
  if CurrentCanvas <> nil then
    DoChangeFont;
  OptionChanged;
end;

function TGraphB.CellHeight:Double;
var
  sz: TSize;
  Cnvs: TCanvas;
  tm: TTextMetric;
  {$IFDEF VCL_}
  DC: HDC;
  {$ENDIF}
begin
  if DCType = dcNone then
    InitFont
  else if not IsRealDevice and (grFixedCharSize in GOptions) then begin
    Result := inherited CellHeight;
    Exit;
  end;
  {$IFDEF VCL_}
  if CurFontHeight = 0 then begin
    if IsPrinter then begin
      Cnvs := CurrentCanvas.RealCanvas;
      if TextDeciAngle <> 0 then
        Cnvs.Font.Assign(UnrotatedGraphicsFont);
    end else begin
      Cnvs := TCanvas.Create;
      DC := GetDC(0);
      Cnvs.Handle := DC;
      Cnvs.Font.Assign(Font);
    end;
    {$IFDEF VER80}
    sz.cx := Cnvs.TextWidth('0123456789');
    sz.cy := Cnvs.TextHeight('0123456789');
    {$ELSE}
    sz := Cnvs.TextExtent('0123456789');
    {$ENDIF}
    GetTextMetrics(Cnvs.Handle, tm);
    CurFontHeight := tm.tmHeight;
    CurFontLeading := tm.tmExternalLeading;
    CurFontInternalLeading := tm.tmInternalLeading;
    CurFontAscent := tm.tmAscent;
    if CurFontHeight = 0 then { Driver bug, or just very small font - get a number from somewhere }
      CurFontHeight := sz.cy;
    if CurFontHeight = 0 then
      CurFontHeight := 1;
    CurFont10Width := sz.cx;
    if not IsPrinter then begin
      Cnvs.Free;
      ReleaseDC(0,DC);
    end;
  end;
  {$ENDIF}
  {$IFDEF LCL_}
  if CurFontHeight = 0 then begin
    if (DCType = dcNone) or (CurrentCanvas.RealCanvas = nil) then
      Cnvs := Canvas
      { TODO: May need to assign font }
    else
      Cnvs := CurrentCanvas.RealCanvas;
    sz := Cnvs.TextExtent('0123456789');
    CurFont10Width := sz.cx;
    if IsPrinter then begin
      { Printer does not have a proper handle }
      Result := sz.cy;
      { TODO: Refine the values below }
      CurFontHeight := sz.cy;
      CurFontLeading := 0;
      CurFontInternalLeading := 0;
      CurFontAscent := sz.cy;
    end else begin
      GetTextMetrics(Cnvs.Handle, tm);
      CurFontHeight := tm.tmHeight;
      CurFontLeading := tm.tmExternalLeading;
      CurFontInternalLeading := tm.tmInternalLeading;
      CurFontAscent := tm.tmAscent;
      if CurFontHeight = 0 then { Driver bug, or just very small font - get a number from somewhere }
        CurFontHeight := sz.cy;
      if CurFontHeight = 0 then
        CurFontHeight := 1;
     end;
   end;
  {$ENDIF}
  if IsPrinter then
    Result := CurFontHeight + CurFontLeading
  else if FontSize < 0 then
    Result := -FontSize * (CurFontHeight + CurFontLeading) / (CurFontHeight - CurFontInternalLeading)
  else
    Result := FontSize * (CurFontHeight + CurFontLeading) / CurFontHeight;
{$IFDEF FIX_CHARACTER_BOX_SIZE_ON_ZOOM}
  if IsScreen then
    Result := Result * YSCALE;
{$ENDIF}
end;

procedure TGraphB.DoChangeFont; { Update font from GraphicsLogFont;
                                  Angle of rotation from TextDeciAngle }
var
 Fnt: TFont;
 {$IFDEF VCL_}
 procedure STA(Mode: Uint);
 begin
   if not Assigned(CurrentCanvas) then
     SetTextAlign(Canvas.Handle, Mode)
   else
     SetTextAlign(CurrentCanvas.RealCanvas.Handle, Mode);
 end;
 {$ENDIF}
 {$IFDEF LCL_}
 FontHandle: HFONT;
 {$ENDIF}
begin
  if not Assigned(CurrentCanvas) then
    Fnt := Canvas.Font
  else
    Fnt := CurrentCanvas.Font;
  GraphicsLogFont.lfHeight := Round((FontSize*TextSize)/100);
  if IsRealDevice and (TextDeciAngle <> 0) then begin
    UnrotatedGraphicsLogFont := GraphicsLogFont;
    UnrotatedGraphicsLogFont.lfEscapement := 0;
    UnrotatedGraphicsLogFont.lfOrientation := 0;
    {$IFDEF VCL_}
    UnrotatedGraphicsFont.PixelsPerInch := Fnt.PixelsPerInch;
    UnrotatedGraphicsFont.Handle := CreateFontIndirect(UnrotatedGraphicsLogFont);
    {$ENDIF}
  end;
  GraphicsLogFont.lfOrientation := TextDeciAngle;
  GraphicsLogFont.lfEscapement := TextDeciAngle;
  if Bold then
    GraphicsLogFont.lfWeight := FW_BOLD
  else
    GraphicsLogFont.lfWeight := OriginalWeight;
  {$IFDEF VCL_}
  if DCType <> dcExport then
    if IsRealDevice and (TextDeciAngle <> 0) then
      STA(TA_LEFT or TA_TOP or TA_NOUPDATECP)
    else
      STA(text_flags or TA_BASELINE or TA_NOUPDATECP);
  GraphicsFont.PixelsPerInch := Fnt.PixelsPerInch;
  GraphicsFont.Handle := CreateFontIndirect(GraphicsLogFont);
  Fnt.Assign(GraphicsFont);
  {$ENDIF}
  {$IFDEF LCL_}
  FontHandle := CreateFontIndirect(GraphicsLogFont);
  Fnt.Handle := FontHandle;
  {$IFDEF LCLcarbon}
    DeleteObject(FontHandle); { TODO: Should not be necessary }
  {$ENDIF}
  {$ENDIF}
  Fnt.Color := Font.Color;
  if TextDeciAngle = 0 then begin
    s_TextAngle := 0;
    c_TextAngle := 1;
  end else begin
    s_TextAngle := sin(TextDeciAngle*0.1*Pi/180);
    c_TextAngle := cos(TextDeciAngle*0.1*Pi/180);
  end;
  inherited;
end;

function TGraphB.UnrotatedTextExtent(const str:string):TSize;
var
  Cnvs: TCanvas;
  tm : TTextMetric;
  descent: Integer;
  FontHandle: HFONT;
begin
  Cnvs := CurrentCanvas.RealCanvas;
  if Cnvs = nil then
    Cnvs := Canvas;
  {$IFDEF VCL_}
  if IsRealDevice and (TextDeciAngle <> 0) then
    Cnvs.Font.Assign(UnrotatedGraphicsFont);
  { Determine size using rotation = 0, as extent at any other rotation is suspect }
  Result.cx := Cnvs.TextWidth(str);
  GetTextMetrics(Cnvs.Handle, tm);
  if IsRealDevice and (TextDeciAngle <> 0) then
    Cnvs.Font.Assign(GraphicsFont);
  Result.cy := tm.tmAscent;   { Assume baseline - add tm.tmDescent for bottom }
  {$ENDIF}
  {$IFDEF LCL_}
  if IsRealDevice and (TextDeciAngle <> 0) then begin
    FontHandle := CreateFontIndirect(UnrotatedGraphicsLogFont);
    Cnvs.Font.Handle := FontHandle;
    {$IFDEF LCLcarbon}
      DeleteObject(FontHandle); { TODO: Should not be necessary }
    {$ENDIF}
  end;
  Result := Cnvs.TextExtent(str);
  if DCType = dcPrinter then
    tm.tmAscent:= Result.cy; { The next line does nothing if Cnvs is a TPostsciptCanvas, so set up a useful default }
  GetTextMetrics(Cnvs.Handle, tm);
  if tm.tmAscent > 0 then
    Result.cy := tm.tmAscent
  {$IFDEF LCLgtk2}
  else begin
    descent := tm.tmDescent;
    if descent = 0 then { ?not yet implemented }
      descent := TGtkDeviceContext(Cnvs.Handle).DCTextMetric.TextMetric.tmDescent;
    if descent = 0 then { Should not be zero, but is on current gtk2 }
      descent := Result.cy div 4; { guess }
    Dec(Result.cy,descent);
  end
  {$ENDIF}
  ;
  if IsRealDevice and (TextDeciAngle <> 0) then begin
    FontHandle := CreateFontIndirect(GraphicsLogFont);
    Cnvs.Font.Handle := FontHandle;
    {$IFDEF LCLcarbon}
      DeleteObject(FontHandle); { TODO: Should not be necessary }
    {$ENDIF}
  end;
  {$ENDIF}
end;

function TGraphB.TextExtentF(const str:string; Fraction: Single):TSize;
var
  v,h: Single;
  xf, yf: Double;
begin
  Result := UnrotatedTextExtent(str);
  h := Result.cx*Fraction;
  v := Result.cy;
  xf := v/YPerX*s_TextAngle + h*c_TextAngle;
  yf := h*YPerX*s_TextAngle - v*c_TextAngle;
  if Abs(xf) >= MaxPlot then
    Result.cx := MaxPlot
  else
    Result.cx := Round(xf);
  if Abs(yf) >= MaxPlot then
    Result.cy := MaxPlot
  else
    Result.cy := Round(yf);
end;

function TGraphB.TextExtent(const str:string):TSize;
begin
  Result := TextExtentF(str, 1);
end;

procedure TGraphB.XorTextI(x, y: Integer; str: string);
var
  Extent: TSize;
  Bmp: TBitmap;
  CopyMode: TCopyMode;
begin
  Bmp := nil;
  CopyMode := cmSrcCopy;
  if not IsRealDevice or (str = '') then
    TextI(x, y, str)
  else
    try
      StartPlotting;
      CopyMode := Canvas.CopyMode;
      LogToDev(x, y);
      Extent := TextExtent(str);
      if (Extent.cx <> 0) and (Extent.cy <> 0) then begin
        Bmp := TBitmap.Create;
        with Bmp do begin
          {$IFNDEF UNIX}
          Monochrome := True; { Seem to be problems creating monochrome bitmaps? }
          {$ENDIF}
          Height := Abs(Extent.cy);
          Width := Abs(Extent.cx);
          with Canvas do Begin
            Brush.Color := clBlack;
            FillRect(Rect(0, 0, Height, Width));
            Font := CurrentCanvas.RealCanvas.Font;
            Font.Color := clWhite;
            TextOut(0, 0, str);
          end;
        end;
        Canvas.CopyMode := cmSrcInvert;
        Canvas.Draw(x, y-Bmp.Height, Bmp);
      end;
    finally
      Canvas.CopyMode := CopyMode;
      Bmp.Free;
      StopPlotting;
    end;
end;

procedure TGraphB.TextI(x:Integer; y:Integer; str:string);
var
{$IFDEF VCL_}
  CanPlot:Boolean;
  h: Single;
  xo, yo: Integer;
  Extent: TSize;
{$ELSE}
{$IFDEF LCL_}
  h: Single;
  Extent: TSize;
  sz: TSize;
  i: Integer;
  lbearing, rbearing, descent: LongInt;
  buf: string[2];
  //ws: WideString;
  //ps: PChar;
  Cinc: Integer;
  {$IFDEF LCLgtk}
  Lengths: array of Integer;
  RotatedGraphicsFont: TLogFont;
  {$ENDIF}
  {$IFDEF LCLcarbon}
    OldFont, FontHandle: HFONT;
  {$ENDIF}
{$ENDIF}
{$ENDIF}
begin
  if {$IFDEF LCL_} not IsPrinter and {$ENDIF} Assigned(CurrentCanvas) and (CurrentCanvas.RealCanvas <> nil) then begin
    StartPlotting;
    CurrentCanvas.Brush.Style := bsClear;
    {$IFDEF VCL_}
    CanPlot := True;
    LogToDev(x, y);
    if IsRealDevice and (TextDeciAngle <> 0) then begin
      case text_flags and (TA_LEFT or TA_RIGHT or TA_CENTER) of
        TA_RIGHT:  h := 1;
        TA_CENTER: h := 0.5;
        TA_LEFT:   h := 0;
      else
        h := 0;
      end;
      Extent := TextExtentF(str, h);
      CanPlot := (Extent.cx <> MaxPlot) and (Extent.cy <> MaxPlot);
      if CanPlot then begin
        xo := x;
        Dec(x, Extent.cx);
        yo := y;
        Inc(y, Extent.cy);
        {$IFNDEF VER80}
        { Keep graphics calls to 16 bits, as Win95/8 can give overflow
          problems otherwise }
        CanPlot := (Abs(x) < MaxPlot) and (Abs(x) < MaxPlot)
                   and (Abs(xo) < MaxPlot) and (Abs(yo) < MaxPlot);
        {$ELSE}
        { This odd sequence checks for 16 bit overflow }
        if (x xor xo) < 0 then CanPlot := False;
        if (y xor yo) < 0 then CanPlot := False;
        {$ENDIF}
      end;
    end;
    if CanPlot then
      CurrentCanvas.RealCanvas.TextOut(x, y, str);
    {$ELSE}
    {$IFDEF LCL_}
    LogToDev(x, y);
    case FJustify of
        taLeftJustify:  h := 0;
        taRightJustify: h := 1;
        taCenter:       h := 0.5;
    end;
    Extent := UnrotatedTextExtent(str);
    if TextDeciAngle = 0 then begin
      if FJustify <> taLeftJustify then
        x := x - Round(h*Extent.cx);
      ExtUTF8Out(CurrentCanvas.RealCanvas.Handle, x, y-Extent.cy, 0, nil, PChar(Str), Length(Str), nil)
    end else begin
      x := x - Round(h*Extent.cx*c_TextAngle+Extent.cy*s_TextAngle);
      y := y + Round(h*Extent.cx*s_TextAngle-Extent.cy*c_TextAngle);
      {$IFDEF LCLgtk}
      SetLength(Lengths,Length(str));
      for i := 0 to Length(Str)-1 do begin
        GetTextExtentPoint(CurrentCanvas.RealCanvas.Handle, @Str[i+1], 1, sz);
        Lengths[i] := sz.cx;
      end;
      { Rotated text has to be done one character at a time under GTK1 at least }
      { TODO: Angles other than 0 and 90 need work (180 is particularly odd) }
      { TODO: Fragile implementation - does not work on all systems }
      RotatedGraphicsFont := GraphicsLogFont;
      if (RotatedGraphicsFont.lfFaceName = '') or (RotatedGraphicsFont.lfFaceName = 'Arial') or (RotatedGraphicsFont.lfFaceName = 'helvetica')
         or (RotatedGraphicsFont.lfFaceName = 'default') then
        RotatedGraphicsFont.lfFaceName := 'luxi sans'; { This is a fudge, as only some fonts will rotate }
      CurrentCanvas.RealCanvas.Font.Handle := CreateFontIndirect(RotatedGraphicsFont);
      buf := #0#0;
      if TGtkDeviceContext(CurrentCanvas.RealCanvas.Handle).DCTextMetric.IsDoubleByteChar then
        Cinc := 2
      else
        Cinc := 1;
      for i := 1 to Length(str) do begin
        buf[Cinc] := Str[i];
        with TGtkDeviceContext(CurrentCanvas.RealCanvas.Handle) do
          gdk_draw_text(Drawable, PGdiObject(CurrentCanvas.RealCanvas.Font.Handle)^.GDIFontObject, GC, x { - Round(sz.cy*s_TextAngle)}, y {- Round(sz.cx*s_TextAngle)}, @buf[1], Cinc)
        x := x + Round(Lengths[i-1]*c_TextAngle);
        y := y - Round(Lengths[i-1]*s_TextAngle);
      end;
      {$ELSE}
      {$IFDEF LCLcarbon}
        FontHandle := CreateFontIndirect(GraphicsLogFont);
        OldFont := SelectObject(CurrentCanvas.RealCanvas.Handle,FontHandle);
        ExtUTF8Out(CurrentCanvas.RealCanvas.Handle, x, y, 0, nil, PChar(Str), Length(Str), nil);
        SelectObject(CurrentCanvas.RealCanvas.Handle,OldFont);
        DeleteObject(FontHandle);
      {$ELSE}
        { works without special handling using Pango or other default mechanism }
        ExtUTF8Out(CurrentCanvas.RealCanvas.Handle, x, y, 0, nil, PChar(Str), Length(Str), nil)
      {$ENDIF}
      {$ENDIF}
    end;
    {$ENDIF}
    {$ENDIF}
    PenInvalid;
    StopPlotting;
  end else
    inherited;
end;

function TGraphB.GetTickSize:TPoint;
begin
  if DCType = dcNone then InitFont;
  HintDisplacement(Round(Abs(FontSize) / 3), Result.y, Result.x, False);
end;

procedure TGraphB.SetJustify(mode:TAlignment);
{$IFDEF VCL_}
var
  textm:Word;
{$ENDIF}
begin
  inherited;
  {$IFDEF VCL_}
  case mode of
      taLeftJustify:  textm := TA_LEFT;
      taRightJustify: textm := TA_RIGHT;
      taCenter:       textm := TA_CENTER;
  else
    raise Exception.CreateFmt('TGraphB.Justify (%d)', [Ord(mode)]);
  end;
  if (textm <> text_flags) and (CurrentCanvas <> nil) and not (IsRealDevice and (TextDeciAngle <> 0))
     and (CurrentCanvas.RealCanvas <> nil) then
    SetTextAlign(CurrentCanvas.RealCanvas.Handle, textm or TA_BASELINE or TA_NOUPDATECP);
  text_flags := textm;
  {$ENDIF}
end;

procedure  TGraphB.Clear;
begin
  if CurrentCanvas = nil then begin
    {$IFDEF METAFILES}
    DiscardMF(hMF);
    {$ENDIF}
    Invalidate;
  end;
end;

procedure TGraphB.SetMetafileScaling;
var
  w, h: Integer;
begin
  w := Round(Xmm*100);
  h := Round(Ymm*100);
  if not (grIsotropic in GOptions) then
    { Adjust pixel ratio to follow Window size }
    if w/PaintWidth > h/PaintHeight then
      w := Round(h/PaintHeight*PaintWidth)
    else
      h := Round(w/PaintWidth*PaintHeight);
  SetScaling(nil, 0, 0, w, h);
  {$IFDEF MSWINDOWS}
  MetaFileWidth := XPixels;
  MetaFileHeight := YPixels;
  {$ENDIF}
end;

procedure TGraphB.SetScaling(ACanvas: TCanvas; X, Y, AWidth, AHeight:Longint);
begin
  FAvailableHeight := AHeight;
  if (ACanvas <> nil) and (IsRealDevice or (ACanvas = Canvas)) then begin
    if (ACanvas <> Canvas) or HandleAllocated then begin
      DPIX := GetDeviceCaps(ACanvas.Handle, LOGPIXELSX);
      DPIY := GetDeviceCaps(ACanvas.Handle, LOGPIXELSY);
      { Interestingly, when directly querying a monitor, the two
        values returned as above, and by xdpyinfo can be slightly different.
        This leads to horrible looking marks, so assume the two values are different
        only if the difference is significant }
      if (DPIX - DPIY) / (DPIX + DPIY) < 0.3 then begin
        DPIY := (DPIX+DPIY) div 2;
        DPIX := DPIY;
      end;
    end else begin
      { Assume on screen }
      DPIX := Screen.PixelsPerInch;
      DPIY := Screen.PixelsPerInch;
    end;
    YPerX := DPIY / DPIX;
  end else begin
    YPerX := 1;
    { Use standard for enhanced metafiles - 0.01 mm }
    DPIX := 2540;
    DPIY := 2540;
  end;
  FXPixels := AWidth;
  FYPixels := AHeight;
  if grIsotropic in GOptions then begin
    { Shrink one axis to maintain Xmm/Ymm ratio }
    if XPixels * YPerX / YPixels > Xmm/Ymm then
      FXPixels := Round(YPixels * Xmm/Ymm / YPerX)
    else
      FYPixels := Round(XPixels * Ymm/Xmm * YPerX);
  end;
  FXOffset := X;
  FYOffset := Y;
end;

procedure TGraphB.ShowOnCanvas(ACanvas: TCanvas; X, Y, AWidth, AHeight:Longint);
  {$IFDEF MSWINDOWS}
var
  SavedDC: Integer;
  ViewOrg: TPoint;
  {$ENDIF}
  procedure ResetDC;
  begin
    InReplot := False;
    if Assigned(ACanvas) then
      {$IFDEF MSWINDOWS}
      RestoreDC(ACanvas.Handle, SavedDC);
      {$ENDIF}
    if CurrentCanvas <> nil then
      cancel_graphics_DC;
    DCType := dcNone;
    LoadMP;
  end;
begin
  if Assigned(ACanvas) then begin
    {$IFDEF MSWINDOWS}
    { Clear all handles, otherwise VCL looses track, though it only seems to be a problem under Win9x }
    ACanvas.Pen.Handle := 0;
    ACanvas.Brush.Handle := 0;
    ACanvas.Font.Handle := 0;
    SavedDC := SaveDC(ACanvas.Handle);
    {$ENDIF}
  end;
  if Assigned(FOnReplot) then begin
    if Assigned(ACanvas) then
      SetRealCanvas(ACanvas);
    try
      if Assigned(ACanvas) then begin
        ACanvas.Refresh;
        ACanvas.Pen.Width := 1; { Refresh sets pen width to zero, which can not normally
                                  be chosen under CLX, giving consistency problems in plots }
        ACanvas.Brush.Style := bsSolid;
      end;
      SetScaling(ACanvas, X, Y, AWidth, AHeight);
      InitGraphics;
      InReplot := True;
      if not PlotFailed then FOnReplot(Self, Self);
      EndDraw;
      InReplot := False;
      ResetDC;
    except
      ResetDC;
      FPlotFailed := True;
      raise;
    end;
  end
  {$IFDEF METAFILES}
  else begin
    SetScaling(ACanvas, X, Y, AWidth, AHeight);
    {$IFDEF MSWINDOWS}
      {$IFDEF VER80}
      if hMF <> nil then
        ACanvas.StretchDraw(Bounds(X, Y, AWidth, AHeight), hMF);
      {$ELSE}
      if hMF <> 0 then begin
        { Stretch draw logic from Delphi 1 }
        SetMapMode(ACanvas.Handle, MM_ANISOTROPIC);
        SetWindowExtEx(ACanvas.Handle, MetaFileWidth, MetaFileHeight, nil);
        SetViewportExtEx(ACanvas.Handle, AWidth, AHeight, nil);
        GetViewPortOrgEx(ACanvas.Handle, ViewOrg);
        SetViewportOrgEx(ACanvas.Handle, ViewOrg.X + X, ViewOrg.Y + Y, nil);
        PlayMetafile(ACanvas.Handle, hMF);
      end;
      {$ENDIF}
      ResetDC;
    {$ELSE}
    if hMF <> nil then
      ACanvas.StretchDraw(Bounds(X, Y, AWidth, AHeight), hMF)
    {$ENDIF}
  end
  {$ENDIF}
  ;
  ResetDC;
end;

procedure TGraphB.OptionChanged;
begin
  if CurrentCanvas = nil then begin
    LoadMP;
    if not Assigned(FOnReplot) then
      Clear
    else
      Invalidate;
  end;
end;

procedure TGraphB.PlotChanged;
begin
  if not Assigned(FOnReplot) then
    Clear
  else
    Invalidate;
  FPlotFailed := False;
end;

procedure TGraphB.MakeScreenImage(ScreenCanvas: TCanvas);
begin
  if (PaintWidth > 4) and (PaintHeight > 4) then begin
    ShowOnCanvas(ScreenCanvas,
                 PaintRect.Left - HorzScrollBar.Position,
                 PaintRect.Top - VertScrollBar.Position,
                 Round(PaintWidth*XSCALE),
                 Round(PaintHeight*YSCALE));
  end;
end;

procedure TGraphB.SetScreenScaling;
begin
  SetScaling(Canvas, PaintRect.Left - HorzScrollBar.Position,
             PaintRect.Top - VertScrollBar.Position,
             Round(PaintWidth*XSCALE), Round(PaintHeight*YSCALE));
end;

procedure TGraphB.SetLastMouse(X, Y: Integer);
begin
  FLastMouse.x := X;
  FLastMouse.y := Y;
  DevToLog(FLastMouse.x, FLastMouse.y);
end;

procedure TGraphB.Paint;
{$IFDEF UNIX}
var
  Buffered: Boolean;
  SaveHandle: {$IFDEF LCL_} HDC {$ENDIF} ;
{$ENDIF}
begin
  DCType := dcScreen;
  {$IFDEF UNIX}
  Buffered := False;
  { Force double buffer under gtk2, as invalid rect seems to be always the entire screen }
  if {$IFNDEF LCLgtk2} DoubleBuffered and {$ENDIF} not (csDesigning in ComponentState) then begin
    if FScreenBuffer <> nil then Exit; { Construction in progress - ignore paint }
    FScreenBuffer := TBitmap.Create;
    FScreenBuffer.Height := Height;
    FScreenBuffer.Width := Width;
    DCType := dcScreenBuffer;
    SaveHandle := Canvas.Handle;
    Canvas.Handle := FScreenBuffer.Canvas.Handle;
    Canvas.Refresh;
    Buffered := True;
  end;
  try
  {$ENDIF}
  Canvas.Brush.Color := Color;
  inherited Paint;
  Canvas.FillRect(Canvas.ClipRect);
  {$IFDEF MSWINDOWS}
  IntersectClipRect(Canvas.Handle, PaintRect.Left, PaintRect.Top, PaintRect.Right, PaintRect.Bottom);
  {$ENDIF}
  MakeScreenImage(Canvas);
  DrawSelectedArea(True);
  if Assigned(FOnPaint) then
   FOnPaint(Self);
  {$IFDEF UNIX}
  finally
    if Buffered then begin
      Canvas.Handle := SaveHandle;
      Canvas.Draw(0, 0, FScreenBuffer);
      FreeAndNil(FScreenBuffer);
    end;
  end;
  {$ENDIF}
end;

procedure TGraphB.ShowPrintPreview(Sender:TObject; ACanvas:TExportableCanvas; X, Y, AWidth, AHeight:Longint;
                                   APage: Integer; OnPrinter:Boolean);
begin
  if OnPrinter then
    DCType := dcPrinter
  else
    DCType := dcPreview;
  FCurrentCanvas := nil;
  ShowOnCanvas(ACanvas.RealCanvas(), X, Y, AWidth, AHeight)
end;

function TGraphB.SignificantAreaSelected: Boolean;
begin
  Result := AreaSelected
            and ((Abs(SelectedArea.Left-SelectedArea.Right) > SmallX)
                 or (Abs(SelectedArea.Top-SelectedArea.Bottom) > SmallY));
end;

procedure TGraphB.MakePrintPreview;
begin
  if not Assigned(FPrintPreview) then begin
    FPrintPreview := TGraphPrintPreview.Create(Application);
    FPrintPreview.Graph := Self;
    FPrintPreview.Orientation := Printer.Orientation;
    FPrintPreview.SetMargins(Xmm, Ymm, FPrintPreview.XMargin, FPrintPreview.YMargin);
  end;
end;

procedure TGraphB.PrintPreview;
begin
  MakePrintPreview;
  FPrintPreview.Show;
end;

procedure TGraphB.Print;
begin
  MakePrintPreview;
  FPrintPreview.Print;
end;

procedure TGraphB.PrinterSetup;
begin
  MakePrintPreview;
  FPrintPreview.PrinterSetup;
end;

destructor TGraphB.Destroy;
begin
  SetCurrentCanvas(nil);
  {$IFDEF METAFILES}
  DiscardMF(hMF);
  {$ENDIF}
  FreeAndNil(FScreenBuffer);
  {$IFDEF VCL_}
  FreeAndNil(UnrotatedGraphicsFont);
  FreeAndNil(GraphicsFont);
  {$ENDIF}
  inherited Destroy;
end;

procedure TGraphB.SetOnReplot(e:TReplotEvent);
begin
  if @FOnReplot <> @e then begin
    FOnReplot := e;
    PlotChanged;
  end;
end;

function TGraphB.CanCopyToClipboard: Boolean;
begin
  if Assigned(FOnReplot) then
    Result := not PlotFailed
  {$IFDEF METAFILES}
  else
    Result := hMF <> NullMF
  {$ENDIF}
    ;
end;

function TGraphB.CanPrint: Boolean;
begin
  Result := CanCopyToClipboard;
end;

procedure TGraphB.SetAreaSelected(b: Boolean);
begin
  if b <> AreaSelected then
    if b then
      SetSelectedArea(FSelectedArea)
    else begin
      DrawSelectedArea;
      FAreaSelected := False;
    end;
end;

procedure TGraphB.SetSelectedArea(r:TRect);
begin
  DrawSelectedArea;
  FSelectedArea := r;
  FAreaSelected := ( abs(FSelectedArea.Right - FSelectedArea.Left) > 0 )
                  or (abs(FSelectedArea.Bottom - FSelectedArea.Top) > 0 );
  DrawSelectedArea;
end;

function TGraphB.GetLogicalSelectedArea: TRect;
begin
  Result := SelectedArea;
  DevToLog(Result.Left, Result.Top);
  DevToLog(Result.Right, Result.Bottom);
end;

procedure TGraphB.SetLogicalSelectedArea(ARect: TRect);
begin
  LogToDev(ARect.Left, ARect.Top);
  LogToDev(ARect.Right, ARect.Bottom);
  SelectedArea := ARect;
end;

procedure DrawFocusRectHelper(Canvas: TCanvas; r: TRect);
var
  t: Longint;
begin
  if r.Left > r.Right then begin
    t := r.Left;
    r.Left := r.Right;
    r.Right := t;
  end;
  if r.Top > r.Bottom then begin
    t := r.Top;
    r.Top := r.Bottom;
    r.Bottom := t;
  end;
  Canvas.Brush.Style := bsSolid; { These two are required under MSWINDOWS, else no focus rect appears }
  Canvas.Brush.Color := clWhite;
  {$IFNDEF LCL_}
  Canvas.DrawFocusRect(r);
  {$ELSE}
    {$IFNDEF LCLanygtk}
      DrawFocusRect(Canvas.Handle, r);
    {$ELSE}
      Canvas.Pen.Style := Graphics.psDot;
      gdk_gc_set_function(TGtkDeviceContext(Canvas.Handle).GC, GDK_INVERT);
      Canvas.Frame(r);
      gdk_gc_set_function(TGtkDeviceContext(Canvas.Handle).GC, GDK_COPY);
      Canvas.Pen.Style := Graphics.psSolid;
    {$ENDIF}
  {$ENDIF}
end;

procedure TGraphB.DrawSelectedArea(InPaint: Boolean);
var
  r:TRect;
begin
  if not (grNoAreaSelection in GOptions) and AreaSelected then begin
    r := FSelectedArea;
    OffsetRect(r, PaintRect.Left, PaintRect.Top);
    {$IFDEF LCLcarbon}
    if not InPaint then begin
      { Carbon can only draw during Paint events }
      InflateRect(r,1,1);
      InvalidateRect(r,False);
    end else
    {$ENDIF}
      DrawFocusRectHelper(Canvas, r);
  end else
    FAreaSelected := False;
end;

procedure TGraphB.CancelSelection;
begin
  DrawSelectedArea;
  Selecting := False;
  FAreaSelected := False;
end;

procedure TGraphB.Invalidate;
begin
  FAreaSelected := False;
  if HandleAllocated then
    inherited Invalidate;
  if Assigned(FPrOnInvalidate) then FPrOnInvalidate(Self);
  if Assigned(FOnInvalidate) then FOnInvalidate(Self);
end;

procedure TGraphB.UpdateSelection( X, Y: Integer; NewPoint:Boolean);
begin
  DrawSelectedArea;
  if NewPoint and Selecting then begin
    FSelectedArea.Right := X - PaintRect.Left;
    FSelectedArea.Bottom := Y - PaintRect.Top;
    { Make sure rectangle shown includes current cursor point }
    if FSelectedArea.Right > FSelectedArea.Left then
      Inc(FSelectedArea.Right);
    if FSelectedArea.Bottom > FSelectedArea.Top then
      Inc(FSelectedArea.Bottom);
    FAreaSelected := ( abs(FSelectedArea.Right - FSelectedArea.Left) > SmallX )
                    or (abs(FSelectedArea.Bottom - FSelectedArea.Top) > SmallY );
    DrawSelectedArea;
  end;
end;

procedure TGraphB.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  p: TMouseEvent;
begin
  if (Button = mbLeft) or (Button = mbRight) then begin
    CancelSelection;
    Selecting := True;
    FSelectedArea.Left := X - PaintRect.Left;
    FSelectedArea.Top := Y - PaintRect.Top;
  end;
  p := OnMouseDown;
  if Assigned(p) then begin
    SetLastMouse(X, Y);
    OnMouseDown(Self, Button, Shift, LastMouse.x, LastMouse.y);
  end;
end;

procedure TGraphB.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  p: TMouseMoveEvent;
begin
  if Selecting then
    UpdateSelection(X, Y, True);
  p := OnMouseMove;
  if Assigned(p) then begin
    SetLastMouse(X, Y);
    OnMouseMove(Self, Shift, LastMouse.x, LastMouse.y);
  end;
end;

procedure TGraphB.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  p: TMouseEvent;
begin
  if ((Button = mbLeft) or (Button = mbRight)) and Selecting then begin
    UpdateSelection(X, Y, True);
    Selecting := False;
  end;
  p := OnMouseUp;
  if Assigned(p) then begin
    SetLastMouse(X, Y);
    OnMouseUp(Self, Button, Shift, LastMouse.x, LastMouse.y);
  end;
end;

procedure TGraphB.UpdateScrollBars;
begin
  if not( Assigned(VertScrollBar) and Assigned(HorzScrollBar) ) then
    Exit;
  HorzScrollBar.Range := Round(XSCALE*PaintWidth) - PaintWidth;
  HorzScrollBar.PageSize := PaintWidth div 2;
  VertScrollBar.Range := Round(YSCALE*PaintHeight) - PaintHeight;
  VertScrollBar.PageSize := PaintHeight div 2;
  inherited UpdateScrollBars;
end;

{ Scroll Bar Positions as fraction of plot space }
function TGraphB.HScrollFraction: Double;
begin
  if HorzScrollBar.Range > 0 then
    Result := HorzScrollBar.Position/(XSCALE*PaintWidth)
  else
    Result := 0;
end;

function TGraphB.VScrollFraction: Double;
begin
  if VertScrollBar.Range > 0 then
    Result := VertScrollBar.Position/(YSCALE*PaintHeight)
  else
    Result := 0;
end;

procedure TGraphB.ChangeScrollPos(Hpos, Vpos: double);
begin
  HorzScrollBar.Position := Round(Hpos*XSCALE*PaintWidth);
  VertScrollBar.Position := Round(Vpos*YSCALE*PaintHeight);
end;

procedure TGraphB.ChangeZoom(newXSCALE, newYSCALE, newHscroll, newVscroll: Double);
begin
  if (newXSCALE*PaintWidth > 32700) or (newYSCALE*PaintHeight > 32700) then
    ShowMessage('Magnification too high')
  else begin
    oldXSCALE := XSCALE;
    oldYSCALE := YSCALE;
    oldHPos := HScrollFraction;
    oldVPos := VScrollFraction;

    XSCALE := newXSCALE;
    YSCALE := newYSCALE;
    UpdateScrollBars;
    ChangeScrollPos(newHscroll, newVscroll);
  end;
  {$IFDEF METAFILES}
  if hMF = NullMF then
    PlotChanged
  else
  {$ENDIF}
    Invalidate;
end;

procedure TGraphB.ResetZoom;
begin
  ChangeZoom(1, 1, 0, 0);
end;

procedure TGraphB.PreviousZoom;
begin
  ChangeZoom(oldXSCALE, oldYSCALE, oldHPos, oldVPos);
end;

procedure TGraphB.ZoomOut;
var
  magx,magy, h, v: Double;
begin
  if AreaSelected then begin
    magx := Abs(SelectedArea.Left-SelectedArea.Right)/PaintWidth;
    magy := Abs(SelectedArea.Top-SelectedArea.Bottom)/PaintHeight;
    if grIsotropic in GOptions then
      if magx < magy then
        magx := magy
      else
        magy := magx;
    h := HScrollFraction - Min(SelectedArea.left,SelectedArea.right)/(XSCALE*PaintWidth*magx);
    v := VScrollFraction - Min(SelectedArea.top,SelectedArea.bottom)/(YSCALE*PaintHeight*magy);
    ChangeZoom(magx*XSCALE, magy*YSCALE, h, v);
  end else
    ChangeZoom(XSCALE/2, YSCALE/2, HScrollFraction - 1/(2*XSCALE), VScrollFraction - 1/(2*YSCALE));
end;

procedure TGraphB.ZoomSelected(Anisotropic:Boolean);
var
  Magx, Magy, AMagx, AMagy, h, v:Double;
  function Adjust(Mag: Double): Double;
  begin
    { Adjust mag to correct for problems }
    if Mag > 2 then
      Result := 0.9
    else if Mag > 1.05 then
      Result := 0.95
    else
      Result := 1;
    Result := Mag * Result;
  end;
begin
  if AreaSelected then begin
    magx := PaintWidth/Abs(SelectedArea.Left-SelectedArea.Right);
    AMagx := Adjust(magx);
    magy := PaintHeight/Abs(SelectedArea.Top-SelectedArea.Bottom);
    AMagy := Adjust(magy);
    if not Anisotropic then
      if AMagx > AMagy then
        AMagx := AMagy
      else
        AMagy := AMagx;
    h := HScrollFraction - (1-AMagx/Magx)/2/AMagx/XSCALE
         + Min(SelectedArea.left,SelectedArea.right)/(XSCALE*PaintWidth);
    v := VScrollFraction - (1-AMagy/Magy)/2/AMagy/YSCALE
         + Min(SelectedArea.top,SelectedArea.bottom)/(YSCALE*PaintHeight);
    ChangeZoom(AMagx*XSCALE, AMagy*YSCALE, h, v);
  end else
    ChangeZoom(2*XSCALE, 2*YSCALE, HScrollFraction + 1/(4*XSCALE), VScrollFraction + 1/(4*YSCALE));
end;

var
  ExportPlotDialog: TSaveDialog;

procedure TGraphB.ExecuteExport;
var
  StreamClass: TStreamCanvasClass;
  Stream: TStream;
begin
  if not Assigned(ExportPlotDialog) then begin
    ExportPlotDialog := TSaveDialog.Create(Application);
    with ExportPlotDialog do begin
      Filter := 'Text File (*.txt,*.dat)|*.txt;*.dat|SVG File (*.svg)|*.svg|XFig File (*.xfig)|*.xfig';
      Options := Options + [ofOverwritePrompt,ofPathMustExist,ofNoReadOnlyReturn];
      Title := 'Export Plot To';
    end;
  end;
  if ExportPlotDialog.Execute then begin
    StreamClass := GetStreamCanvasClassFromFilename(ExportPlotDialog.FileName);
    Stream := TFileStream.Create(ExportPlotDialog.FileName,fmCreate);
    try
      ExportTo(Stream, StreamClass);
    finally
      Stream.Free;
    end;
  end;
end;

procedure TGraphB.ExportTo(AStream: TStream; CanvasClass: TStreamCanvasClass);
var
  CustomStreamCanvas: TCustomStreamCanvas;
begin
  if not Assigned(FOnReplot) or InReplot then
    raise Exception.Create('OnReplot must be set for ExportTo');
  CustomStreamCanvas := CanvasClass.Create(AStream, Xmm, Ymm);
  try
    SetCurrentCanvas(CustomStreamCanvas);
    FAvailableHeight := YPixels;
    { TODO : this stuff appears in several places }
    DCType := dcExport;
    InitGraphics;
    InReplot := True;
    FOnReplot(Self, Self);
    EndDraw;
    cancel_graphics_DC;
  finally
    InReplot := False;
    SetCurrentCanvas(nil); { Will free CustomStreamCanvas }
  end;
end;

{$IFDEF METAFILES}
function TGraphB.GetPicture: {$IFDEF MSWINDOWS} {$IFDEF WINLCL} HMETAFILE {$ELSE} TMetafile {$ENDIF} {$ENDIF} {$IFDEF UNIX} TDrawing {$ENDIF};
{$IFDEF WIN32}
var
  nBytes: Cardinal;
  buffer: string;
{$ENDIF}
begin
  if Assigned(FOnReplot) then begin
    try
      StartMetafile;
      DCType := dcClip;
      InitGraphics;
      InReplot := True;
      FOnReplot(Self, Self);
      EndDraw;
      InReplot := False;
      StopMetafile;
      cancel_graphics_DC;
    except
      InReplot := False;
      {$IFDEF MSWINDOWS}
      if (MetafileCanvas <> nil) and (MetafileCanvas.Handle <> 0) then
        CloseMetaFile(MetafileCanvas.Handle);
      FreeAndNil(MetafileCanvas);
      {$ENDIF}
      DiscardMF(hMF);
      cancel_graphics_DC;
      raise;
    end;
    {$IFDEF MSWINDOWS}
      {$IFDEF LCL}
        Result := hMF;
        hMF := 0; { Hand over ownership }
      {$ELSE}
        Result := TMetafile.Create;
        {$IFDEF VER80}
          Result.Handle := hMF;
        {$ELSE}
          nBytes := GetMetaFileBitsEx(hMF, 0, nil);
          SetLength(buffer, nBytes);
          GetMetaFileBitsEx(hMF, nBytes, PChar(buffer));
          Result.Handle := SetWinMetaFileBits(nBytes, PChar(buffer), HDC(nil), PMetafilePict(nil)^);
          DiscardMF(hMF);
        {$ENDIF}
        hMF := 0; { Hand over ownership }
      {$ENDIF}
    {$ENDIF}
    {$IFDEF UNIX}
      Result := hMF;
      hMF := nil; { Hand over ownership }
    {$ENDIF}
  end else begin
    {$IFDEF MSWINDOWS}
      {$IFDEF LCL}
        Result := CopyEnhMetafile(hMF, nil);
      {$ELSE}
        Result := TMetafile.Create;
        Result.Handle := CopyEnhMetafile(hMF, nil);
      {$ENDIF}
    {$ENDIF}
    {$IFDEF UNIX}
    Result := TDrawing.Create;
    Result.Assign(hMF);
    {$ENDIF}
  end;
end;
{$ENDIF}

procedure TGraphB.CopyToClipboard;
{$IFDEF MSWINDOWS}
var
  hGMem:THandle;
  AFormat: Word;
  lpMFP:^TMetaFilePict;
{$ENDIF}
begin
  {$IFDEF MSWINDOWS}
  if Assigned(FOnReplot) then
    try
      StartMetafile;
      DCType := dcClip;
      InitGraphics;
      InReplot := True;
      FOnReplot(Self, Self);
      EndDraw;
      InReplot := False;
      {$IFDEF VER80}
      hMF.Width := MetafileWidth;
      hMF.Height := MetafileHeight;
      { Do not set hMF.Inch := DPIX; as this affects width and height }
      {$ELSE}
      hMF := CloseMetaFile(MetafileCanvas.Handle);
      {$ENDIF}
      FreeAndNil(MetafileCanvas);
      cancel_graphics_DC;
    except
      InReplot := False;
      if (MetafileCanvas <> nil) and (MetafileCanvas.Handle <> 0) then
        CloseMetaFile(MetafileCanvas.Handle);
      FreeAndNil(MetafileCanvas);
      DiscardMF(hMF);
      cancel_graphics_DC;
      raise;
    end;
  if hMF <> NullMF then begin
    hGMem := GlobalAlloc(GHND, SizeOf(TMetaFilePict));
    if hGMem = 0 then raise Exception.Create('No Handle');
    try
      lpMFP := GlobalLock(hGMem);
      try
        lpMFP^.mm := MM_ANISOTROPIC;
        { grIsotropic just determines whether the aspect ratio of the picture
          reflects the window size or the Xmm/Ymm settings }
        lpMFP^.xExt := MetafileWidth;
        lpMFP^.yExt := MetafileHeight;
        lpMFP^.hMF := CopyMetafile(hMF{$IFDEF VER80}.Handle{$ENDIF}, nil);
      finally
        GlobalUnlock(hGMem);
      end;
    except
      GlobalFree(hGMem);
      raise;
    end;
    AFormat := CF_METAFILEPICT;
    {$IFDEF LCL}
      { TODO: Clipboard.SetAsHandle }
    {$ELSE}
      Clipboard.SetAsHandle(AFormat, hGMem);
    {$ENDIF}
  end;
  {$ENDIF}
  {$IFNDEF METAFILES}
  // TODO:
  {$ENDIF}
  {$IFDEF UNIX}
  { TODO: Copy a picture to the clipboard - SVG? }
  ExportTo(nil, TNullCanvas); { For the time being this will force the data to be copied in readable format for TGraphD }
  {$ENDIF}
end;

procedure TGraphB.ChooseFont(Sender: TObject);
var
  FontDialog: TFontDialog;
begin
  FontDialog := TFontDialog.Create(Self);
  try
    FontDialog.Font := Font;
    if FontDialog.Execute then
      Font := FontDialog.Font;
    OptionChanged;
  finally
    FontDialog.Free;
  end;
end;

procedure TGraphB.CreateHandle;
begin
  inherited CreateHandle;
end;

constructor TGraphD.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  SetRecordCalls(True);
end;

destructor TGraphD.Destroy;
begin
  inherited Destroy;
end;

procedure  TGraphD.DoGraphReplot(Sender:TObject; G:TGraphB);
begin
  Recording := false;
  DefaultScale;
  if Assigned(FOnDReplot) then
      FOnDReplot(Self, Self)
  else
      PlayBack(Rstream);
  { Need to save scaling factors for mouse event scaling to work }
  if IsScreen then begin
    XofSave := Xof;
    XsSave := Xs;
    XminSave := Xmin;
    YofSave := Yof;
    YsSave := Ys;
    YminSave := Ymin;
  end else begin
    Xof := XofSave;
    Xs := XsSave;
    FXmin := XminSave;
    Yof := YofSave;
    Ys := YsSave;
    FYmin := YminSave;
  end
end;

procedure TGraphD.ShowOnCanvas(ACanvas: TCanvas; X, Y, AWidth, AHeight:Longint);
var
  RecPos: Longint;
begin
  if Recording then begin
    RecPos := RStream.Position;
    try
      Recording := False;
      PutType(tyEnd);
      inherited ShowOnCanvas(ACanvas, X, Y, AWidth, AHeight);
    finally
      Recording := True;
      RStream.Position := RecPos;
    end
  end else
    inherited ShowOnCanvas(ACanvas, X, Y, AWidth, AHeight);
end;

procedure TGraphD.PlotChanged;
begin
  if RStream <> nil then begin
    RStream.Position := 0;
    PutType(tyEnd);
  end;
  inherited PlotChanged;
end;

procedure TGraphD.PreviousZoom;
begin
  ZoomArea := PreviousZoomArea;
end;

procedure TGraphD.ChangeScrollPos(Hpos, Vpos: double);
begin
  with FZoomArea do begin
    Left := (Hpos-Xp1)/(Xp2-Xp1);
    Right := (Hpos+1/XSCALE-Xp1)/(Xp2-Xp1);
    Top := (Vpos-Yp1)/(Yp2-Yp1);
    Bottom := (Vpos+1/YSCALE-Yp1)/(Yp2-Yp1);
  end;
  inherited ChangeScrollPos(Hpos, Vpos);
end;

procedure TGraphD.ChangeZoom(newXSCALE, newYSCALE, newHscroll, newVscroll: Double);
var
  RStreamSave: TStream;
  Xp1p, Xp2p, Yp1p, Yp2p: Double; { New pspace }
begin
  { Do not clear recorded calls on change of zoom }
  RStreamSave := RStream;
  RStream := nil;
  try
    PreviousZoomArea := ZoomArea;
    { Assume pspace stuff stays constant pixel size, so scales inversely with XSCALE/YSCALE }
    Xp1p := Xp1 * XSCALE/newXSCALE;
    Xp2p := 1 - (1-Xp2) * XSCALE/newXSCALE;
    Yp1p := Yp1 * YSCALE/newYSCALE;
    Yp2p := 1 - (1-Yp2) * YSCALE/newYSCALE;
    newHscroll := (newHscroll-Xp1)*(Xp2p-Xp1p)/(Xp2-Xp1) + Xp1p;
    newVscroll := (newVscroll-Yp1)*(Yp2p-Yp1p)/(Yp2-Yp1) + Yp1p;
    inherited ChangeZoom(newXSCALE, newYSCALE, newHscroll, newVscroll);
    { Assume pspace stuff stays constant pixel size, so scales inversely with XSCALE/YSCALE }
    Xp1 := Xp1p;
    Xp2 := Xp2p;
    Yp1 := Yp1p;
    Yp2 := Yp2p;
  finally
    RStream := RStreamSave;
  end;
end;

procedure TGraphD.SetZoomArea(const Value: TDRect);
var
  Previous: TDRect;
begin
  Previous := ZoomArea;
  with Value do
    if (Right > Left) and (Bottom > Top) then
      ChangeZoom(1/(Right-Left)/(Xp2-Xp1), 1/(Bottom-Top)/(Yp2-Yp1),
                 Left*(Xp2-Xp1) + Xp1, Top*(Yp2-Yp1) + Yp1)
    else
      ResetZoom;
  PreviousZoomArea := Previous;
end;

procedure  TGraphD.SetOnReplot(e:TReplotDEvent);
begin
  FOnDReplot := e;
  SetRecordCalls(FRecordCalls);
  PlotChanged;
end;

procedure  TGraphD.SetRecordCalls(ARecording:Boolean);
begin
  FRecordCalls := ARecording;
  if ARecording or Assigned(FOnDReplot) then
      inherited OnReplot := {$IFDEF FPC}@{$ENDIF}DoGraphReplot
  else
      inherited OnReplot := Nil;
  if not ARecording then begin
    RStream.Free;
    RStream := nil;
  end;
end;

function TGraphD.CanPrint: Boolean;
begin
  if RecordCalls then
    Result := Assigned(FOnDReplot) or ( (RStream <> nil) and (RStream.Size > 1) )
  else
    Result := inherited CanPrint;
end;

procedure TGraphD.CopyToClipboard;
var
  Stream: THugeMemoryStream;
begin
  if RecordCalls and (Assigned(FOnDReplot) or ( (RStream <> nil) and (RStream.Size > 1)) ) then begin
    Stream := THugeMemoryStream.Create;
    Clipboard.Open;
    try
      TextStream := Stream;
      inherited;
    finally
      TextStream := nil;
      Stream.CopyToClipboard(CF_TEXT);
      Stream.Free;
      Clipboard.Close;
    end;
  end else
    inherited;
end;

procedure TGraphD.ExportTo(AStream: TStream; CanvasClass: TStreamCanvasClass);
begin
  if CanvasClass = TTextCanvas then begin
    { Special case - treat text file as wanting real number output }
    try
      TextStream := AStream;
      inherited ExportTo(nil, TNullCanvas)
    finally
      TextStream := nil;
    end;
  end else
    inherited;
end;

procedure TGraphD.LPtoXY(lp: TPoint; var x, y:Double);
begin
  if Xs <> 0 then
    x := (lp.x-Xof)/Xs + Xmin;
  if Ys <> 0 then
    y := (lp.y-Yof)/Ys + Ymin;
end;

procedure TGraphD.XYtoLP(out lp: TPoint; x, y:Double);
begin
  lp.x := ProcX(x);
  lp.y := ProcY(y);
end;

function TGraphD.PlotLength:Double; { Plot length in mm on printer }
begin
  Result := (Xp2-Xp1) * Xmm;
end;

procedure TGraphD.DrawGraphic(x1, x2, y1, y2:Double; Graphic: TGraphic);
var
  Rect: TRect;
begin
  if not Recording then begin
    Rect.Left := Round((x1-Xmin)*Xs + Xof); { Limiting to 16 bit is done elsewhere, if at all }
    Rect.Right := Round((x2-Xmin)*Xs + Xof);
    Rect.Top := ProcY(y1);
    Rect.Bottom := ProcY(y2);
    inherited DrawGraphic(Rect, Graphic);
  end else
    inherited DrawGraphic(x1,x2,y1,y2,Graphic);
end;

procedure TGraphD.Start;
begin
  if not InReplot then begin
    SetRecordCalls(FRecordCalls);   (* Ensure Event handler set correctly *)
    if (FRecordCalls) then begin
      DefaultScale; { Make some half way sensible character size values }
      Recording := true;
    end else begin
      inherited Start;
      DefaultScale;
    end;
  end;
end;

procedure TGraphD.Finish;
begin
  if not InReplot then begin
    if Recording then begin
      PutType(tyEnd);
      Recording := false;
      Invalidate;
    end else
      inherited Finish;
  end;
end;

function TGraphD.IsNotAxisDefault: Boolean;
begin
  Result := (AxisStyle.Width <> 0)
            or (AxisStyle.Colour <> clBlack)
            or (AxisStyle.Pattern <> lpSolid);
end;

function TGraphD.PixelSize(x: Boolean): Double;
{ The size of one screen pixel }
begin
  if x then begin
    Result := XPixels / PaintWidth / XSCALE;
    if Xs <> 0 then Result := Result / Xs;
  end else begin
    Result := YPixels / PaintHeight / YSCALE;
    if Ys <> 0 then Result := Result / Ys;
  end;
  Result := Abs(Result);
end;

function TGraphD.InPlotArea(x, y: Double): Boolean;
var
  dx, dy: Double;
begin
  dx := SmallX*PixelSize(true);
  dy := SmallY*PixelSize(false);
  if Xmin < Xmax then
    Result := (x >= Xmin - dx) and (x <= Xmax + dx)
  else
    Result := (x <= Xmin + dx) and (x >= Xmax - dx);
  if Result then
    if Ymin < Ymax then
      Result := (y >= Ymin - dy) and (y <= Ymax + dy)
    else
      Result := (y <= Ymin + dy) and (y >= Ymax - dy);
end;

function TGraphD.CloseToX(x1, x2: Double):Boolean;
begin
  if Xs <> 0 then begin
    Result := Abs(x1-x2) < PixelSize(True)*SmallX;
  end else
    Result := False;
end;

function TGraphD.CloseToY(y1, y2: Double):Boolean;
begin
  if Ys <> 0 then begin
    Result := Abs(y1-y2) < PixelSize(False)*SmallY;
  end else
    Result := False;
end;

procedure TGraphPrintPreview.SetMargins(AXSize, AYSize, AXMargin, AYMargin:Double);
begin
  inherited SetMargins(AXSize, AYSize, AXMargin, AYMargin);
  if Assigned(FGraph) then begin
    if AXSize > 0 then Graph.Xmm := AXSize;
    if AYSize > 0 then Graph.Ymm := AYSize;
  end;
end;

procedure TGraphPrintPreview.SetGraph(G:TGraphB);
begin
  if Assigned(FGraph) then
    FGraph.FPrOnInvalidate := nil;
  FGraph := G;
  if Assigned(FGraph) then begin
    FGraph.FPrOnInvalidate := {$IFDEF FPC}@{$ENDIF}PInvalidate;
    if XSize > 0 then FGraph.Xmm := XSize;
    if YSize > 0 then FGraph.Ymm := YSize;
  end;
end;

procedure TGraphPrintPreview.DoShowPreview(Sender:TObject; Canvas:TExportableCanvas;
  X, Y, AWidth, AHeight:Longint; APage: Integer; OnPrinter:Boolean);
begin
  if Assigned(FGraph) then
    FGraph.ShowPrintPreview(Sender, Canvas, X, Y, AWidth, AHeight, APage, OnPrinter);
  inherited DoShowPreview(Sender, Canvas, X, Y, AWidth, AHeight, APage, OnPrinter);
end;

function TGraphD.GetSelectedMathArea: TDrect;
var
  r: Trect;
begin
  r := LogicalSelectedArea;
  LPtoXY(r.TopLeft, Result.Left, Result.Top);
  LPtoXY(r.BottomRight, Result.Right, Result.Bottom);
end;

procedure TGraphD.SetSelectedMathArea(ARect: TDrect);
var
  r: Trect;
begin
  XYtoLP(r.TopLeft, ARect.Left, ARect.Top);
  XYtoLP(r.BottomRight, ARect.Right, ARect.Bottom);
  LogicalSelectedArea := r;
end;

function TGraphD.RoundToDisplay(value: Double; xvalue: Boolean): Double;
var
  dRes: Double;
begin
  { Discard significant figures (in decimal representation!)
    to match display resolution }
  dRes := PixelSize(xvalue)/2;
  { Truncate to next lowest power of 10 }
  dRes := Power(10, Floor(Log10(dRes)));
  Result := Round(value/dRes)*dREs;
  if Abs(value-Result) > dRes then
    raise Exception.Create('RoundToDisplay');
end;

procedure TGraphD.SetLastMouse(X, Y: Integer);
begin
  inherited SetLastMouse(X, Y);
  LPtoXY(LastMouse, FLastMouseX, FLastMouseY);
end;

end.

