unit LogFiDlg;

interface
{$I DFS.inc}{ Copyright (C) 2005 C M Western; see PcontrolsReg.pas for license}

uses
  {$IFDEF MSWINDOWS} Windows, {$ENDIF}
  {$IFNDEF CLX_} Graphics, Forms, Controls, Buttons,
                 StdCtrls, ExtCtrls, Dialogs, Menus,
  {$ELSE} QGraphics, QForms, QControls, QButtons, QStdCtrls, QExtCtrls,
          QDialogs, QMenus, QTypes, {$ENDIF}
  {$IFDEF LCL_} SrchStrs, {$ENDIF}
  Classes;

type

  { TLogFindDlg }

  TLogFindDlg = class(TForm)
    Label1: TLabel;
    FindNext: TButton;
    FindText: TMemo;
    Cancel: TButton;
    WholeWord: TCheckBox;
    MatchCase: TCheckBox;
    Direction: TRadioGroup;
    NewLine: TButton;
    Tab: TButton;
    Label2: TLabel;
    ReplaceText: TMemo;
    Replace: TButton;
    ReplaceAll: TButton;
    LocalPopupMenu: TPopupMenu;
    PasteItem: TMenuItem;
    CopyItem: TMenuItem;
    CutItem: TMenuItem;
    N3: TMenuItem;
    UndoItem: TMenuItem;
    DeleteItem: TMenuItem;
    N1: TMenuItem;
    SelectAllItem: TMenuItem;
    N2: TMenuItem;
    NewlineItem: TMenuItem;
    TabItem: TMenuItem;
    PopupItem: TMenuItem;
    procedure NewLineClick(Sender: TObject);
    procedure TabClick(Sender: TObject);
    procedure CancelClick(Sender: TObject);
    procedure FindNextClick(Sender: TObject);
    procedure ReplaceClick(Sender: TObject);
    procedure ReplaceAllClick(Sender: TObject);
    procedure FindTextEnter(Sender: TObject);
    procedure CopyItemClick(Sender: TObject);
    procedure CutItemClick(Sender: TObject);
    procedure PasteItemClick(Sender: TObject);
    procedure UndoItemClick(Sender: TObject);
    procedure SelectAllItemClick(Sender: TObject);
    procedure PopupItemClick(Sender: TObject);
    procedure DeleteItemClick(Sender: TObject);
    procedure PopupMenuPopup(Sender: TObject);
  private
    FIsReplace: Boolean;
    procedure SetIsReplace(const Value: Boolean);
  public
    OnFindNext: TNotifyEvent;
    OnReplace: TNotifyEvent;
    LastOption: TFindOption;
    LastText: TMemo;
    property IsReplace: Boolean read FIsReplace write SetIsReplace;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

var
  LogFindDlg: TLogFindDlg;

implementation

uses {$IFDEF MSWINDOWS} Messages, {$ENDIF} WCompat, UCompat;

{$IFNDEF LCL_}
{$R *.DFM}
{$ELSE}
{$R *.lfm}
{$ENDIF}

constructor TLogFindDlg.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  LastText := FindText;
  FIsReplace := True;
  Left := Screen.Width-Width;
  Top := Screen.Height - Height * 12 div 10;
end;

procedure TLogFindDlg.NewLineClick(Sender: TObject);
begin
  LastText.SelText := LineEnd;
  LastText.SetFocus;
end;

procedure TLogFindDlg.TabClick(Sender: TObject);
begin
  LastText.SelText := #9;
  LastText.SetFocus;
end;

procedure TLogFindDlg.CancelClick(Sender: TObject);
begin
  Close;
end;

procedure TLogFindDlg.FindNextClick(Sender: TObject);
begin
  if FindText.GetTextLen = 0 then Exit;
  if Assigned(OnFindNext) then OnFindNext(Self);
end;

procedure TLogFindDlg.ReplaceClick(Sender: TObject);
begin
  LastOption := frReplace;
  if Assigned(OnReplace) then OnReplace(Self);
  FindNextCLick(Self);
end;

procedure TLogFindDlg.ReplaceAllClick(Sender: TObject);
begin
  LastOption := frReplaceAll;
  if Assigned(OnReplace) then OnReplace(Self);
end;

procedure TLogFindDlg.FindTextEnter(Sender: TObject);
begin
  LastText := Sender as TMemo;
end;

procedure TLogFindDlg.CopyItemClick(Sender: TObject);
begin
  LastText.CopyToClipboard;
end;

procedure TLogFindDlg.CutItemClick(Sender: TObject);
begin
  LastText.CutToClipboard;
end;

procedure TLogFindDlg.PasteItemClick(Sender: TObject);
begin
  LastText.PasteFromClipboard;
end;

procedure TLogFindDlg.UndoItemClick(Sender: TObject);
begin
  {$IFDEF MSWINDOWS}
  SendMessage(LastText.Handle, EM_UNDO, 0, 0);
  {$ENDIF}
  {$IFDEF UNIX}
  // TODO: send undo char to control
  {$ENDIF}
end;

procedure TLogFindDlg.SelectAllItemClick(Sender: TObject);
begin
  LastText.SelectAll;
end;

procedure TLogFindDlg.PopupItemClick(Sender: TObject);
begin
  LastText.PopupMenu.PopUp(Left+FindText.Left+10, Top+FindText.Top+10);
end;

procedure TLogFindDlg.DeleteItemClick(Sender: TObject);
begin
  with LastText do begin
    if SelLength = 0 then
      SelLength := 1;
    ClearSelection;
  end;
end;

procedure TLogFindDlg.PopupMenuPopup(Sender: TObject);
begin
  {$IFDEF MSWINDOWS}
  UndoItem.Enabled := SendMessage(LastText.Handle, EM_CANUNDO, 0, 0) <> 0;
  {$ENDIF}
  {$IFDEF UNIX}
  // TODO: send undo char to control
  UndoItem.Enabled := True;
  {$ENDIF}
  CutItem.Enabled := LastText.SelLength > 0;
  CopyItem.Enabled := CutItem.Enabled;
  PasteItem.Enabled := ClipboardHasText;
  SelectAllItem.Enabled := LastText.GetTextLen > 0;
end;

procedure TLogFindDlg.SetIsReplace(const Value: Boolean);
var
  i, sign, NewTop: Integer;
begin
  if Value <> FIsReplace then begin
    FIsReplace := Value;
    if IsReplace then begin
      Caption := 'Replace';
      sign := 1;
    end else begin
      Caption := 'Find';
      sign := -1;
    end;
    {$IFDEF LCL_}
    MatchCase.Tag := 0;
    WholeWord.Tag := 0;
    Cancel.Tag := 0;
    if IsReplace then
      Cancel.AnchorSideTop.Control := ReplaceAll
    else
      Cancel.AnchorSideTop.Control := FindNext;
    {$ENDIF}
    for i := 0 to ControlCount-1 do
      with Controls[i] do
        case Tag of
          0: Continue;
         -1: Visible := IsReplace;
        {$IFNDEF LCL_}
        else
          Top := Top + sign*Tag
        {$ENDIF}
        end;
    Height := Height + sign*59;
    NewTop := Screen.Height - Height * 12 div 10;
    if NewTop < Top then Top := NewTop;
  end;
end;

destructor TLogFindDlg.Destroy;
begin
  if Self = LogFindDlg then
    LogFindDlg := nil;
  inherited Destroy;
end;

end.
