unit MDIHelp;
{$I DFS.inc}{ Copyright (C) 2005 C M Western; see PcontrolsReg.pas for license}
interface

uses
  {$IFNDEF CLX_} Graphics, Controls, Forms, Menus, Dialogs,
  {$ELSE} QGraphics, QControls, QForms, QMenus, QDialogs, Types, {$ENDIF}
  SysUtils, Classes;

type
  TMDIHelper = class(TComponent)
  private
    FWindowMenu: TMenuItem;
    FUpdateNeeded: Boolean;
    FSeparatorPosition: Integer; { Where to put menu items }
    procedure OnMenuShow(Sender: TObject);
    procedure SwitchToWindow(Sender: TObject);
    procedure ClearExtraEntries(RemoveSeparator: Boolean);
    procedure SetWindowMenu(const Value: TMenuItem);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property WindowMenu: TMenuItem read FWindowMenu write SetWindowMenu;
  end;

implementation

uses MDIChildForm;

type TOpenCustomForm = class(TCustomForm);

{ TMDIHelper }

procedure TMDIHelper.ClearExtraEntries(RemoveSeparator: Boolean);
var
  i, e: Integer;
begin
  if MDIAppStyle then
    Exit;
  e := FSeparatorPosition;
  if not RemoveSeparator then
    Inc(e);
  if not (csDesigning in ComponentState) and (WindowMenu <> nil)
     and (FSeparatorPosition >= 0) then
    for i := WindowMenu.Count -1 downto e do
      WindowMenu.Items[i].Free;
end;

constructor TMDIHelper.Create(AOwner: TComponent);
begin
  inherited;
  FSeparatorPosition := -1;
  FUpdateNeeded := False;
end;

procedure TMDIHelper.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  case Operation of
    {$IFDEF CLX_}
    opInsert:
      if AComponent is TCustomForm then
        FUpdateNeeded := True;
    {$ENDIF}
    opRemove:
      if AComponent = WindowMenu then
        WindowMenu := nil;
  end;
end;

procedure TMDIHelper.OnMenuShow(Sender: TObject);
var
  NewItem: TMenuItem;
  i: Integer;
begin
  if FUpdateNeeded and (WindowMenu <> nil) then begin
    { Add separator if required }
    if FSeparatorPosition < 0 then begin
      if (WindowMenu.Count = 0)
         or (WindowMenu.Items[WindowMenu.Count-1].Caption <> '-') then begin
           NewItem := TMenuItem.Create(WindowMenu.Owner);
           NewItem.Caption := '-';
           WindowMenu.Add(NewItem);
      end;
      FSeparatorPosition := WindowMenu.Count - 1;
    end else
      ClearExtraEntries(False);
    if WindowMenu.Owner is TMDIChildForm then
      with TMDIChildForm(WindowMenu.Owner) do
        for i := 0 to MDIChildCount - 1 do begin
          NewItem  := TMenuItem.Create(WindowMenu.Owner);
          NewItem.Caption := '&' + IntToStr(i) + ' ' +MDIChildren[i].Caption;
          NewItem.Tag := i;
          NewItem.OnClick := {$IFDEF FPC}@{$ENDIF}SwitchToWindow;
          WindowMenu.Add(NewItem);
        end;
    FUpdateNeeded := False;
  end;
end;

procedure TMDIHelper.SwitchToWindow(Sender: TObject);
begin
  if (Sender is TMenuItem) and (WindowMenu <> nil)
    and (WindowMenu.Owner is TMDIChildForm) then
      TMDIChildForm(WindowMenu.Owner).MDIChildren[TMenuItem(Sender).Tag].SetFocus;
end;

procedure TMDIHelper.SetWindowMenu(const Value: TMenuItem);
begin
  if (Value <> FWindowMenu) and (WindowMenu <> nil) then
    ClearExtraEntries(True);
  FWindowMenu := Value;
  {$IFDEF VCL_}
  { TODO: LCL_ equivalent }
  if not (csDesigning in ComponentState) and (Owner is TCustomForm) then
    TOpenCustomForm(Owner).WindowMenu := Value;
  {$ENDIF}
  if {$IFNDEF CLX_} not MDIAppStyle and {$ENDIF} (WindowMenu <> nil) and not (csDesigning in ComponentState) then begin
    WindowMenu.FreeNotification(Self);
    {$IFDEF CLX_}
    WindowMenu.OnHighlighted
    {$ELSE}
      WindowMenu.OnClick
    {$ENDIF}
       := {$IFDEF FPC}@{$ENDIF}OnMenuShow;
    FSeparatorPosition := -1;
    FUpdateNeeded := True;
  end;
end;

end.
