unit MakeHelp;
{ Drop this button on a form and press it at run time to create a skeleton
  .htm documentation file }
{$I DFS.inc}{ Copyright (C) 2005 C M Western; see PcontrolsReg.pas for license}
interface

uses
  Forms, SysUtils, Buttons, Classes, Controls, StdCtrls;

type
  TMakeHelpButton = class(TButton)
  public
    constructor Create(AOwner: TComponent); override;
    procedure Click; override;
  published
    property Width default 5;
    property Height default 5;
  end;

{$IFDEF SINGLE}
procedure Register;
{$ENDIF}

procedure MakeSkeletonHelp(Form: TCustomForm);

implementation

uses {$IFDEF MSWINDOWS} Windows, ComCtrls, {$ENDIF}
     {$IFDEF VCL_} GIFImage, { PNGImage, } {$ENDIF}
     {$IFDEF LCL_} LCLType, {$ENDIF}
     UCompat, Menus, Graphics, TextAssign, Dialogs, GraphI, ScrlCtrl;

{$IFDEF SINGLE}
procedure Register;
begin
  RegisterComponents('CMW', [TMakeHelpButton]);
end;
{$ENDIF}
{$DEFINE TABLE}

procedure FileHeader(var F: Text; Title: string);
begin
  Rewrite(F);
  WriteLn(F,'<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">');
  WriteLn(F,'<HTML>');
  WriteLn(F,'<HEAD>');
  WriteLn(F,'<Title>',Title,'</Title>');
  WriteLn(F,'</HEAD>');
  WriteLn(F,'<BODY>');
  WriteLn(F,'<H1>', Title, '</H1>');
end;

procedure FileFooter(var F: Text);
begin
  WriteLn(F,'</BODY>');
  WriteLn(F,'</HTML>');
  CloseFile(F);
end;

function StartFile(var F: Text; var Name: string): Boolean;
var
  SaveDialog: TSaveDialog;
begin
  Result := False;
  SaveDialog := TSaveDialog.Create(Application);
  try
    SaveDialog.DefaultExt := 'htm';
    SaveDialog.Filter := 'HTML files (*.htm; *.html)|*.htm;*.html|All files (*.*)|'+AllFilesMask;
    SaveDialog.FileName := LowerCase(Name + '.htm');
    if SaveDialog.Execute then begin
      Result := True;
      AssignFile(F, SaveDialog.FileName);
      Name := SaveDialog.FileName;
    end;
  finally
    SaveDialog.Free;
  end;
end;

function BitmapOfControl(Control: TControl):TBitmap;
var
  WinControl: TWinControl;
  Org, POrg: TPoint;
  DC: HDC;
begin
  Result := nil;
  try
    Result := TBitmap.Create;
    Result.PixelFormat := pfDevice;
    Result.Width := Control.Width;
    Result.Height := Control.Height;
    Application.ProcessMessages;
    if Control is TGraphI then
      with TGraphI(Control) do begin
        Org.X := PaintRect.Left;
        Org.Y := PaintRect.Top;
        Result.Width := PaintRect.Right - PaintRect.Left;
        Result.Height := PaintRect.Bottom - PaintRect.Top;
        WinControl := Control as TWinControl;
      end
    else if Control is TWinControl then begin
      WinControl := Control as TWinControl;
      Org.x := 0;
      Org.y := 0;
    end else begin
      WinControl := Control.Parent;
      Org := Control.ClientOrigin;
      if WinControl is TCustomForm then begin
        POrg.x := WinControl.Left;
        POrg.y := WinControl.Top;
      end else begin
        POrg := WinControl.ClientOrigin;
      end;
      Dec(Org.X,POrg.X);
      Dec(Org.Y,POrg.Y);
    end;
    {$IFDEF MSWINDOWS}
    if (WinControl is TToolBar) and (Control <> WinControl) then
      BitBlt(Result.Canvas.Handle, 0, 0, Result.Width, Result.Height,
             TToolBar(WinControl).Canvas.Handle, Control.Left, Control.Top, SRCCOPY)
    else begin
      DC := GetWindowDC(WinControl.Handle);
      BitBlt(Result.Canvas.Handle, 0, 0, Result.Width, Result.Height, DC, Org.X, Org.Y, SRCCOPY);
      ReleaseDC(WinControl.Handle, DC);
    end;
    {$ENDIF}
  except
    Result.Free;
  end;
end;

procedure DumpAsGif(Bitmap:TBitmap; FileName: string);
{$IFDEF VCL_}
var
  GIFImage: TGIFImage;
{$ENDIF}
begin
  {$IFDEF VCL_}
  GIFImage := nil;
  try
    GIFImage := TGIFImage.Create;
    GIFImage.ColorReduction := rmQuantizeWindows;
    GIFImage.Assign(Bitmap);
    GIFImage.SaveToFile(ChangeFileExt(FileName,'.gif'));
  finally
    Bitmap.Free;
    GIFImage.Free;
  end;
  {$ENDIF}
end;

procedure DumpControl(Control: TControl; FileName: string);
begin
  DumpAsGif(BitmapOfControl(Control),FileName);
end;

function DumpAssociatedBitmap(Bitmap:TBitmap; FileName, Name: string):string;
var
  s: string;
begin
  s := ChangeFileExt(FileName, '');
  if not DirectoryExists(s) then
    MkDir(s);
  s := LowerCase(s+PathDelim+Name+'.gif');
  DumpAsGif(Bitmap, s);
  Result := ChangeFileExt(ExtractFileName(FileName),'') + '/' + ExtractFileName(s);
end;

{$IFDEF LCL_}
function StripHotKey(const s: string): string;
begin
  Result := STringReplace(s, '&', '', []);
end;

{$ENDIF}
procedure DumpMenu(Menu: TMenuItem);
var
  F: Text;
  s, FileName: string;
  Bitmap: TBitmap;
  procedure DumpHelper(M: TMenuItem);
  var
    i: Integer;
  begin
    for i := 0 to M.Count-1 do
      if M[i].Caption <> '-' then begin
        Write(F,'<a name="',M[i].Name,'"><h2>',StripHotkey(M[i].Caption),'</h2>');
        if Assigned(M[i].Bitmap) and not M[i].Bitmap.Empty then begin
          Bitmap := TBitmap.Create;
          Bitmap.Assign(M[i].Bitmap);
          Write(F,'<IMG SRC="',DumpAssociatedBitmap(Bitmap, FileName, M[i].Name),'">');
        end;
        WriteLn(F,'</a>');
        if M[i].Hint <> '' then
          WriteLn(F,M[i].Hint);
        WriteLn(F);
        DumpHelper(M[i]);
      end;
  end;
begin
  FileName := Menu.Name;
  if FileName = '' then FileName := Menu.Owner.Name;
  if not StartFile(F, FileName) then
    Exit;
  s := StripHotkey(Menu.Caption);
  if s = '' then
    s := Menu.Owner.Name;
  FileHeader(F, s + ' Menu');
  DumpHelper(Menu);
  FileFooter(F);
end;

type
  TOpenControl = class(TControl)
  end;

procedure MakeSkeletonHelp(Form: TCustomForm);
var
  Org: TPoint;
  F: Text;
  i, j: Integer;
  FileName, BaseName: string;
  procedure DumpSubComponents(Control: TWinControl; IsMap:Boolean);
  var
    i: Integer;
    s, h, cap: string;
    c: TControl;
    p: TPoint;
  begin
    for i := 0 to Control.ControlCount -1 do begin
      c := Control.Controls[i];
      if c.Visible and not (c is TMakeHelpButton) then begin
        s := c.Name;
        if s = '' then Continue;
        h := c.Hint;
        if IsMap then begin
          if c is TWinControl then
            DumpSubComponents(TWinControl(c),IsMap);  { Dump sub components first so that overlapping areas work }
          if (c is TLabel) and Assigned(TLabel(c).FocusControl) then begin
            s := TLabel(c).FocusControl.Name;
            h := TLabel(c).FocusControl.Hint;
          end;
          p.x := 0;
          p.y := 0;
          p := c.ClientToScreen(p);
          Write(F,'<AREA SHAPE=RECT COORDS="', p.X-Org.X, ',', p.Y-Org.Y, ',',
                                             p.X-Org.X+c.Width, ',', P.Y-Org.Y+c.Height,
                                             '" HREF = "#', s, '" ');
          if h <> '' then
            WriteLn(F,' TITLE = "', h, '">')
          else
            WriteLn(F,'>');
        end else begin
          if not (c is TLabel) or Assigned(TLabel(c).FocusControl) then begin
            cap := TOpenControl(c).Caption;
            if cap = '' then
              cap := s;
            {$IFDEF TABLE}
              Write(F, '<tr><td>');
            {$ENDIF}
            Write(F,'<a name="',s,'">');
            {$IFNDEF TABLE}
              Write(F, '<h2>',s,'</h2>');
            {$ENDIF}
            WriteLn(F,'<IMG SRC="',DumpAssociatedBitmap(BitmapOfControl(c), FileName, c.Name),'" ALT="', s, '"></a>');
            {$IFDEF TABLE}
              Write(F, '</td><td>');
            {$ENDIF}
            if h <> '' then
              Write(F,h);
            {$IFDEF TABLE}
              Write(F, '</td></tr>');
            {$ENDIF}
            Writeln(F);
          end;
          if c is TWinControl then
            DumpSubComponents(TWinControl(c),IsMap);
        end;
      end
    end;
  end;
begin
  Org.X := Form.Left;
  Org.Y := Form.Top;
  FileName := Form.Name;
  if StartFile(F, FileName) then begin
    BaseName := ChangeFileExt(ExtractFileName(FileName),'');
    FileHeader(F, Form.Caption);
    WriteLn(F,'<IMG SRC="',BaseName+'.gif','" USEMAP="#',LowerCase(Form.Name+'map'),'">');
    WriteLn(F,'<MAP NAME="',BaseName+'map','">');
    DumpSubComponents(Form, True);
    WriteLn(F,'</MAP><br>');
    {$IFDEF TABLE}
      WriteLn(F, '<table>');
    {$ENDIF}
    DumpSubComponents(Form, False);
    {$IFDEF TABLE}
      WriteLn(F, '</table>');
    {$ENDIF}
    FileFooter(F);
    DumpControl(Form, BaseName);
  end;
  for i := 0 to Form.ComponentCount-1 do
    if Form.Components[i] is TMainMenu then
      with TMainMenu(Form.Components[i]) do
        for j := 0 to Items.Count - 1 do
          DumpMenu(Items[j])
    else if Form.Components[i] is TPopupMenu then
      DumpMenu(TPopupMenu(Form.Components[i]).Items)
end;

{ TMakeHelpButton }

procedure TMakeHelpButton.Click;
begin
  Visible := False;
  MakeSkeletonHelp(GetParentForm(Self));
  Visible := True;
end;

constructor TMakeHelpButton.Create(AOwner: TComponent);
begin
  inherited;
  Width := 5;
  Height := 5;
end;

end.
