unit MuScroll;

{$I DFS.inc}{ Copyright (C) 2005 C M Western; see PcontrolsReg.pas for license}

interface

uses
  {$IFDEF VCL_} Windows, Messages, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Menus, Buttons, ExtCtrls, {$ENDIF}
  {$IFDEF CLX_} Types, QGraphics, QControls, QForms, QDialogs, QStdCtrls, QMenus, QButtons, QExtCtrls, {$ENDIF}
  SysUtils, Classes;

const
  InitRepeatPause = 400;  { pause before repeat timer (ms) }
  RepeatPause     = 100;  { pause before hint window displays (ms)}

type

  TTimerSpeedButton = class;

{ TMicroScroll }

  TMicroScroll = class (TWinControl)
  private
    FUpButton: TTimerSpeedButton;
    FDownButton: TTimerSpeedButton;
    FFocusedButton: TTimerSpeedButton;
    FFocusControl: TWinControl;
    FOnUpClick: TNotifyEvent;
    FOnDownClick: TNotifyEvent;
    function CreateButton: TTimerSpeedButton;
    function GetUpGlyph: TBitmap;
    function GetDownGlyph: TBitmap;
    procedure SetUpGlyph(Value: TBitmap);
    procedure SetDownGlyph(Value: TBitmap);
    procedure BtnClick(Sender: TObject);
    procedure BtnMouseDown (Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SetFocusBtn (Btn: TTimerSpeedButton);
    procedure AdjustButtonSizes (var W: Integer; var H: Integer);
    {$IFDEF VCL_}
    procedure WMSize(var Message: TWMSize);  message WM_SIZE;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    {$ENDIF}
  protected
    procedure Loaded; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    {$IFDEF VCL_}
    procedure CMEnabledchanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    {$ENDIF}
    {$IFDEF CLX_}
    procedure EnabledChanged; override;
    {$ENDIF}
  public
    constructor Create(AOwner: TComponent); override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
  published
    property Align;
    property DownGlyph: TBitmap read GetDownGlyph write SetDownGlyph;
    {$IFDEF VCL_}
    property DragCursor;
    {$ENDIF}
    property DragMode;
    property Enabled;
    property FocusControl: TWinControl read FFocusControl write FFocusControl;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
{    property TabOrder;
    property TabStop; }
    property OnEnter;
    property OnExit;
    property UpGlyph: TBitmap read GetUpGlyph write SetUpGlyph;
    property Visible;
    property OnDownClick: TNotifyEvent read FOnDownClick write FOnDownClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnUpClick: TNotifyEvent read FOnUpClick write FOnUpClick;
  end;

{ TTimerSpeedButton }

  TTimeBtnState = set of (tbFocusRect, tbAllowTimer);

  TTimerSpeedButton = class(TSpeedButton)
  private
    FRepeatTimer: TTimer;
    FTimeBtnState: TTimeBtnState;
    FOnPressed: TNotifyEvent;
    procedure TimerExpired(Sender: TObject);
  protected
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;  X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    destructor Destroy; override;
    procedure Press;
    property TimeBtnState: TTimeBtnState read FTimeBtnState write FTimeBtnState;
    property OnPressed: TNotifyEvent read FOnPressed write FOnPressed;
  end;

{$IFDEF SINGLE}
procedure Register;
{$ENDIF}

implementation

{$IFDEF SINGLE}
{$IFNDEF VER80}
{$R MuScroll.d32}
{$ELSE}
{$R Muscroll.d16}
{$ENDIF}
{$ENDIF}

uses MathSup;

{ TMicroScroll }

constructor TMicroScroll.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  TabStop := False;
  ControlStyle := ControlStyle - [csAcceptsControls, csSetCaption] +
    [csFramed, csOpaque];
  { ControlStyle := ControlStyle + [csNoFocus]; }
  {$IFDEF VCL_}
  Ctl3D := False;
  ParentCtl3D := False;
  {$ENDIF}

  FUpButton := CreateButton;
  FDownButton := CreateButton;
  UpGlyph := nil;
  DownGlyph := nil;

  Width := 20;
  Height := 25;
  FFocusedButton := FUpButton;
end;

function TMicroScroll.CreateButton: TTimerSpeedButton;
begin
  Result := TTimerSpeedButton.Create (Self);
  { Result.ControlStyle := Result.ControlStyle + [csNoFocus]; }
  Result.OnMouseDown := BtnMouseDown;
  Result.OnPressed := BtnClick;
  {$IFDEF VCL_}
  Result.OnClick := BtnClick;
  {$ENDIF}
  Result.Visible := True;
  Result.Enabled := True;
  Result.TimeBtnState := [tbAllowTimer];
  Result.NumGlyphs := 1;
  Result.Parent := Self;
  {$IFNDEF VER80}
  Result.Transparent := False;
  {$ENDIF}
end;

procedure TMicroScroll.AdjustButtonSizes (var W: Integer; var H: Integer);
begin
  if (FUpButton = nil) or (csLoading in ComponentState) then Exit;
  if W < 13 then W := 13;
  FUpButton.SetBounds (0, 0, W, (H+1) div 2);   { CMW: even up heights }
  FDownButton.SetBounds (0, FUpButton.Height - (H and 1), W, (H+1) div 2);
end;

procedure TMicroScroll.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  W, H: Integer;
begin
  W := AWidth;
  H := AHeight;
  AdjustButtonSizes (W, H);
  inherited SetBounds (ALeft, ATop, W, H);
end;

{$IFDEF VCL_}
procedure TMicroScroll.WMSize(var Message: TWMSize);
var
  W, H: Integer;
begin
  inherited;

  { check for minimum size }
  W := Width;
  H := Height;
  AdjustButtonSizes (W, H);
  if (W <> Width) or (H <> Height) then
    inherited SetBounds(Left, Top, W, H);
  Message.Result := 0;
end;

procedure TMicroScroll.WMSetFocus(var Message: TWMSetFocus);
begin
  FFocusedButton.TimeBtnState := FFocusedButton.TimeBtnState + [tbFocusRect];
  FFocusedButton.Invalidate;
end;

procedure TMicroScroll.WMKillFocus(var Message: TWMKillFocus);
begin
  FFocusedButton.TimeBtnState := FFocusedButton.TimeBtnState - [tbFocusRect];
  FFocusedButton.Invalidate;
end;

procedure TMicroScroll.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  Message.Result := DLGC_WANTARROWS;
end;
{$ENDIF}

procedure TMicroScroll.KeyDown(var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_UP:
      begin
        SetFocusBtn (FUpButton);
        FUpButton.Press;
      end;
    VK_DOWN:
      begin
        SetFocusBtn (FDownButton);
        FDownButton.Press;
      end;
    VK_SPACE:
      FFocusedButton.Press;
  end;
end;

procedure TMicroScroll.BtnMouseDown (Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then begin
    SetFocusBtn (TTimerSpeedButton (Sender));
    if (FFocusControl <> nil) and FFocusControl.TabStop and
        FFocusControl.CanFocus and
{$IFDEF VCL_}
        (GetFocus <> FFocusControl.Handle)
{$ENDIF}
{$IFDEF CLX_}
      (Screen.ActiveControl <> FFocusControl) // TODO - MSWIN ok also?
{$ENDIF}
      then
      FFocusControl.SetFocus
    else if TabStop and
{$IFDEF VCL_}
     (GetFocus <> Handle)
{$ENDIF}
{$IFDEF CLX_}
      (Screen.ActiveControl <> self) // TODO - MSWIN ok also?
{$ENDIF}
     and CanFocus then
      SetFocus;
  end;
end;

procedure TMicroScroll.BtnClick(Sender: TObject);
begin
  if Sender = FUpButton then
  begin
    if Assigned(FOnUpClick) then FOnUpClick(Self);
  end
  else
    if Assigned(FOnDownClick) then FOnDownClick(Self);
end;

procedure TMicroScroll.SetFocusBtn (Btn: TTimerSpeedButton);
begin
  if TabStop and CanFocus and  (Btn <> FFocusedButton) then
  begin
    FFocusedButton.TimeBtnState := FFocusedButton.TimeBtnState - [tbFocusRect];
    FFocusedButton := Btn;
    if
{$IFDEF VCL_}
     (GetFocus = Handle)
{$ENDIF}
{$IFDEF CLX_}
     (Screen.ActiveControl = self) // TODO - MSWIN ok also?
{$ENDIF}
    then begin
       FFocusedButton.TimeBtnState := FFocusedButton.TimeBtnState + [tbFocusRect];
       Invalidate;
    end;
  end;
end;

procedure TMicroScroll.Loaded;
var
  W, H: Integer;
begin
  inherited Loaded;
  W := Width;
  H := Height;
  AdjustButtonSizes (W, H);
  if (W <> Width) or (H <> Height) then
    inherited SetBounds (Left, Top, W, H);
end;

function TMicroScroll.GetUpGlyph: TBitmap;
begin
  Result := FUpButton.Glyph;
end;

procedure SetDefaultGlyph(Button:TTimerSpeedButton; Up:Boolean);
var
  p:array [0..2] of TPoint;
  base: Integer;
begin
  with Button.Glyph do begin
    if Up then
      Base := 3
    else
      Base := 0;
    p[0].x := 0; { Was 1 }
    p[0].y := Base;
    p[1].x := 3; { Was 4 }
    p[1].y := 3 - Base;
    p[2].y := Base;
    p[2].x := 6; { Was 7 }
    {$IFDEF VCL_}
    Canvas.Brush.Color := clBlack;
    Width := 7; { Was 8 }
    {$ENDIF}
    {$IFDEF CLX_}
    Canvas.Brush.Color := clWhite;
    Canvas.Pen.Color := clWhite;
    Transparent := False;
    Width := 8; { Was 9 }
    {$ENDIF}
    Height := 5;
    Monochrome := True;
    Canvas.Polygon(p);
  end;
  Button.NumGlyphs := 1;
  Button.Invalidate;
end;

procedure TMicroScroll.SetUpGlyph(Value: TBitmap);
begin
  if Value <> nil then
    FUpButton.Glyph := Value
  else
    SetDefaultGlyph(FUpButton, True); { CMW }
end;

function TMicroScroll.GetDownGlyph: TBitmap;
begin
  Result := FDownButton.Glyph;
end;

procedure TMicroScroll.SetDownGlyph(Value: TBitmap);
begin
  if Value <> nil then
    FDownButton.Glyph := Value
  else
    SetDefaultGlyph(FDownButton, False); { CMW }
end;

{$IFDEF VCL_}
procedure TMicroScroll.CMEnabledchanged(var Message: TMessage);
{$ENDIF}
{$IFDEF CLX_}
procedure TMicroScroll.EnabledChanged;
{$ENDIF}
begin
  inherited;
  FUpButton.Enabled := Enabled;
  FDownButton.Enabled := Enabled;
end;

{TTimerSpeedButton}

destructor TTimerSpeedButton.Destroy;
begin
  if FRepeatTimer <> nil then
    FRepeatTimer.Free;
  inherited Destroy;
end;

procedure TTimerSpeedButton.Press;
begin
  if Assigned(FOnPressed) then
    FOnPressed(Self);
end;

procedure TTimerSpeedButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseDown (Button, Shift, X, Y);
  {$IFDEF CLX_}
  { Click is called at a bad time, so do it this way under linux }
  if Button = mbLeft then
    Press;
  {$ENDIF}
  if tbAllowTimer in FTimeBtnState then begin
    if FRepeatTimer = nil then
      FRepeatTimer := TTimer.Create(Self);

    FRepeatTimer.OnTimer := TimerExpired;
    FRepeatTimer.Interval := InitRepeatPause;
    FRepeatTimer.Enabled  := True;
  end;
end;

procedure TTimerSpeedButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
                                  X, Y: Integer);
begin
  inherited MouseUp (Button, Shift, X, Y);
  if FRepeatTimer <> nil then
    FRepeatTimer.Enabled  := False;
end;

procedure TTimerSpeedButton.TimerExpired(Sender: TObject);
begin
  FRepeatTimer.Interval := RepeatPause;
  if (FState = bsDown) and MouseCapture then
  begin
    try
      Press;
    except
      FRepeatTimer.Enabled := False;
      raise;
    end;
  end;
end;

procedure TTimerSpeedButton.Paint;
var
  R: TRect;
begin
  inherited Paint;
  if tbFocusRect in FTimeBtnState then
  begin
    R := Bounds(0, 0, Width, Height);
    InflateRect(R, -3, -3);
    if FState = bsDown then
      OffsetRect(R, 1, 1);
{$IFDEF VCL_}
    { TODO: Linux? }
    DrawFocusRect(Canvas.Handle, R);
{$ENDIF}
  end;
end;

{$IFDEF SINGLE}
procedure Register;
begin
  RegisterComponents('CMW', [TMicroScroll, TSpinBox, TFixedSpin]);
end;
{$ENDIF}

end.
