unit PenChse;
{$I DFS.inc}{ Copyright (C) 2005 C M Western; see PcontrolsReg.pas for license}
interface

uses {$IFDEF MSWINDOWS} Windows, {$ENDIF}
     {$IFNDEF CLX_} Messages, Graphics, Controls,
                    Forms, Dialogs, StdCtrls, Menus, ExtCtrls, Buttons,
     {$ELSE} Types, QGraphics, QControls, QForms, QDialogs, QStdCtrls, QMenus, QExtCtrls, QButtons, {$ENDIF}
     {$IFDEF LCL_} LCLType, LMessages, {$ENDIF}
     GStyle, FrameGrp, SysUtils, Classes, NewSpin, AlEdit, EditExtr;

type
  TPenChooseItem = (pcColour, pcWidth, pcStyle);
  TPenChooseItems = set of TPenChooseItem;

  TPenChoose = class(TFrameGroupBox)
    Colour: TPanel;
    WidthLbl: TLabel;
    StyleLbl: TLabel;
    Style: TComboBox;
    PenColorDialog: TColorDialog;
    PenWidth: TFixedSpin;
    SetColour: TBitBtn;
    procedure SetColourClick(Sender: TObject);
    procedure SetLineStyle(AStyle: TLineStyle);
    procedure WidthChange(Sender: TObject);
    procedure StyleChange(Sender: TObject);
    procedure UpdateEnabled;
  protected
    procedure Loaded; override;
    {$IFNDEF CLX_}
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CreateHandle; override;
    {$ELSE}
    procedure FontChanged; override;
    procedure EnabledChanged; override;
    {$ENDIF}
    procedure StyleDrawItem({$IFDEF CLX_}Sender: TObject;{$ELSE} Sender: TWinControl;{$ENDIF} Index: Integer;
      Rect: TRect; State: TOwnerDrawState {$IFDEF CLX_}; var Handled:Boolean{$ENDIF});
  private
    FVisibleItems: TPenChooseItems;
    FLineStyle: TLineStyle;
    FVertSpacing: Integer;
    FEnabledItems: TPenChooseItems;
    procedure LineStyleChanged(Sender: TObject);
    procedure SetVisibleItems(AVisibleItems: TPenChooseItems);
    procedure SetEnabledItems(const Value: TPenChooseItems);
  public
    ChangedItems: TPenChooseItems;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
  published
    property LineStyle:TLineStyle read FLineStyle write SetLineStyle;
    property VisibleItems: TPenChooseItems read FVisibleItems write SetVisibleItems;
    property EnabledItems: TPenChooseItems read FEnabledItems write SetEnabledItems;

    property Caption;
    property Color;
    {$IFDEF MSWINDOWS}
    property Ctl3D;
    property ParentCtl3D;
    property DragCursor;
    {$ENDIF}
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;

  end;

var
  PenChoose: TPenChoose;

implementation

{$IFDEF LCL_} uses LResources; {$ENDIF}

{$IFDEF MSWINDOWS}
{$IFNDEF LCL_}
{$R *.dfm}
{$ENDIF}
{$ENDIF}
{$IFDEF CLX_}
{$R *.xfm}
{$ENDIF}

procedure TPenChoose.SetColourClick(Sender: TObject);
begin
  PenColorDialog.Color := LineStyle.Colour;
  if PenColorDialog.Execute then begin
    LineStyle.Colour := PenColorDialog.Color;
    ChangedItems := ChangedItems + [pcColour];
  end;
end;

procedure TPenChoose.LineStyleChanged(Sender: TObject);
var
  i: Integer;
begin
  if Colour <> nil then begin
    Colour.Font := Font;
    if SetColour.Enabled then begin
      Colour.Color := LineStyle.Colour;
      Colour.Font.Color := LineStyle.Colour xor clWhite;
    end else begin
      Colour.Color := clBtnFace;
      Colour.Font.Color := LineStyle.Colour;
    end;
    PenWidth.Value := LineStyle.Width;
    if Style.HandleAllocated then begin
      if Style.Items.Count < 2 then
        for i := 0 to MAXSTYLE do
          Style.Items.Add('');
      Style.ItemIndex := Ord(LineStyle.Pattern);
    end;
    Invalidate;
  end;
end;

procedure TPenChoose.SetLineStyle(AStyle:TLineStyle);
begin
  FLineStyle.Assign(AStyle);
end;

procedure TPenChoose.Loaded;
begin
  inherited Loaded;
  Style.OnDrawItem := {$IFDEF FPC}@{$ENDIF}StyleDrawItem;
  {$IFDEF UNIX}
  LineStyleChanged(Self);
  UpdateEnabled;
  {$ENDIF}
end;

procedure TPenChoose.StyleDrawItem({$IFDEF CLX_}Sender: TObject;{$ELSE} Sender: TWinControl;{$ENDIF} Index: Integer;
      Rect: TRect; State: TOwnerDrawState {$IFDEF CLX_}; var Handled:Boolean{$ENDIF});
var
  pat: TPat;
  x, xe, y, i: Integer;
  c: TColor;
begin
  {$IFDEF CLX_}
  Handled := Sender is TComboBox;
  if not Handled then Exit;
  {$ENDIF}
  pat := pats[Index];
  y := (Rect.Top + Rect.Bottom) div 2;
  x := Rect.Left + 2;
  xe := Rect.Right - 2;
  if (odSelected in State) and (odFocused in State) then
    c := clHighlightText
  else
    c := clWindowText;
  with (Sender as TComboBox).Canvas do begin { draw on the control canvas, not on the form }
    FillRect(rect);
    Pen.Color := c;
    if pat.length = 0 then begin
      {$IFNDEF CLX_}
      Pixels[x,y] := c;
      Pixels[xe,y] := c;
      {$ELSE}
      DrawPoint(x,y);
      DrawPoint(xe,y);
      {$ENDIF}
    end else begin
      i := 0;
      repeat
        if pat.Segments[i] < 0 then
          {$IFNDEF CLX_}
          Pixels[x,y] := c
          {$ELSE}
          DrawPoint(x,y)
          {$ENDIF}
        else begin
          MoveTo(x,y);
          x := x + pat.Segments[i];
          if x > xe then x := xe;
          LineTo(x,y);
        end;
        x := x + pat.Segments[i+1];
        i := i + 2;
        if i >= pat.length then i := 0;
      until x >= xe;
    end;
  end;
end;

{$IFDEF FPC}
var
  ResourcesLoaded: Boolean;
{$ENDIF}
constructor TPenChoose.Create(AOwner: TComponent);
begin
  {$IFDEF FPC}
  if not ResourcesLoaded then
    {$I PenChse.lrs}
  ResourcesLoaded := True;
  {$ENDIF}
  FVisibleItems := [pcColour, pcWidth, pcStyle];
  FEnabledItems := [pcColour, pcWidth, pcStyle];
  FLineStyle := TLineStyle.Create;
  FLineStyle.OnChange := {$IFDEF FPC}@{$ENDIF}LineStyleChanged;
  inherited Create(AOwner);
  ParentFont := True;
  FVertSpacing := PenWidth.Top - (Colour.Top + Colour.Height);
  SetBounds(Left, Top, 0, 1);
  LineStyleChanged(Self);
  ChangedItems := [];
end;

{$IFNDEF CLX_}
procedure TPenChoose.CMFontChanged(var Message: TMessage);
begin
  inherited;
  LineStyleChanged(Self);
end;

procedure TPenChoose.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  UpdateEnabled;
end;

procedure TPenChoose.CreateHandle;
begin
  inherited CreateHandle;
  Style.HandleNeeded;
  LineStyleChanged(Self);
  UpdateEnabled;
end;

{$ELSE}

procedure TPenChoose.FontChanged;
begin
  inherited;
  LineStyleChanged(Self);
end;

procedure TPenChoose.EnabledChanged;
begin
  inherited;
  UpdateEnabled;
end;
{$ENDIF}

destructor TPenChoose.Destroy;
begin
  FLineStyle.Free;
  inherited Destroy;
end;

procedure TPenChoose.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  cp, mp: Integer;
  procedure Ck(var A: Integer; l: Integer);
  begin
    if A < l then A := l;
  end;
begin
  {$IFNDEF LCL_}
  if Colour <> nil then
    if AHeight > AWidth div 2  then begin
      { Portrait }
      Ck(AWidth, {$IFDEF LCL_} Width - ClientWidth {$ENDIF} + 2*Colour.Left + Colour.Width);
      cp := Colour.Top;
      if Colour.Visible then
        Inc(cp, Colour.Height + FVertSpacing);
      if PenWidth.Visible then begin
        PenWidth.Left := Colour.Left + Colour.Width - PenWidth.Width - PenWidth.ExtraWidth;
        PenWidth.Top := cp;
        WidthLbl.Left := Colour.Left;
        WidthLbl.Top := PenWidth.Top + PenWidth.Height div 2 - WidthLbl.Height div 2;
        cp := PenWidth.Top + PenWidth.Height + FVertSpacing;
      end;
      if Style.Visible then begin
        Style.Left := Colour.Left + Colour.Width - Style.Width;
        Style.Top := cp;
        StyleLbl.Left := Colour.Left;
        StyleLbl.Top := Style.Top + Style.Height div 2 - StyleLbl.Height div 2;
        cp := Style.Top + Style.Height + FVertSpacing;
      end;
      if not (csLoading in ComponentState) then
        Ck(AHeight, cp + FVertSpacing div 2 {$IFDEF LCL_} + Height - ClientHeight {$ENDIF} );
    end else begin
      if Colour.Visible then begin
        Ck(AHeight, Colour.Top + Colour.Height + FVertSpacing {$IFDEF LCL_} + Height - ClientHeight {$ENDIF});
        mp := Colour.Top + Colour.Height div 2;
        cp := Colour.Left + Colour.Width + Colour.Left;
      end else begin
        Ck(AHeight, Colour.Top + PenWidth.Height + FVertSpacing {$IFDEF LCL_} + Height - ClientHeight {$ENDIF});
        mp := Colour.Top + PenWidth.Height div 2;
        cp := Colour.Left;
      end;
      if PenWidth.Visible then begin
        WidthLbl.Left := cp;
        WidthLbl.Top := mp - WidthLbl.Height div 2;
        PenWidth.Left := WidthLbl.Left + WidthLbl.Width + 3;
        PenWidth.Top := mp - PenWidth.Height div 2;
        cp := PenWidth.Left + PenWidth.Width + PenWidth.ExtraWidth + Colour.Left;
      end;
      if Style.Visible then begin
        StyleLbl.Left := cp;
        StyleLbl.Top := mp - StyleLbl.Height div 2;
        Style.Left := StyleLbl.Left + StyleLbl.Width + 5;
        Style.Top := mp - Style.Height div 2;
        cp := Style.Left + Style.Width + Colour.Left;
      end;
      if not (csLoading in ComponentState) then Ck(AWidth, cp {$IFDEF LCL_} + Width - ClientWidth {$ENDIF});
    end;
  {$ENDIF}
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
  Invalidate;
end;


procedure TPenChoose.WidthChange(Sender: TObject);
begin
  Linestyle.Width := PenWidth.Value;
  ChangedItems := ChangedItems + [pcWidth];
end;

procedure TPenChoose.StyleChange(Sender: TObject);
begin
  LineStyle.Pattern := TLinePattern(Style.ItemIndex);
  ChangedItems := ChangedItems + [pcStyle];
end;

procedure TPenChoose.SetVisibleItems(AVisibleItems: TPenChooseItems);
begin
  if AVisibleItems <> FVisibleItems then begin
    FVisibleItems := AVisibleItems;
    Colour.Visible := pcColour in VisibleItems;
    WidthLbl.Visible := pcWidth in VisibleItems;
    PenWidth.Visible := pcWidth in VisibleItems;
    StyleLbl.Visible := pcStyle in VisibleItems;
    Style.Visible := pcStyle in VisibleItems;
    SetBounds(Left, Top, inherited Width, Height);
  end;
end;

procedure TPenChoose.SetEnabledItems(const Value: TPenChooseItems);
begin
  if Value <> FEnabledItems then begin
    FEnabledItems := Value;
    SetColour.Enabled := pcColour in FEnabledItems;
    PenWidth.Enabled := pcWidth in FEnabledItems;
    WidthLbl.Enabled := pcWidth in FEnabledItems;
    Style.Enabled := pcStyle in FEnabledItems;
    StyleLbl.Enabled := pcStyle in FEnabledItems;
    LineStyleChanged(Self);
  end;
end;

procedure TPenChoose.UpdateEnabled;
begin
  if csLoading in ComponentState then Exit;
  SetColour.Enabled := Enabled;
  if Enabled then
    ParentFont := True
  else
    Font.Color := clGrayText;
end;

{$IFDEF LCL_}
initialization
  {$I PenChse.lrs}
{$ENDIF}
end.

