unit TextActn;

{$I DFS.inc}{ Copyright (C) 2005 C M Western; see PcontrolsReg.pas for license}

interface

uses
  {$IFNDEF CLX_} ActnList,
  {$ELSE} QActnList, {$ENDIF}
  Classes, TextWin, ExActn;

type

  TTextChildAction = class(TActionBase)
  public
    function HandlesTarget(Target: TObject): Boolean; override;
    procedure UpdateTarget(Target: TObject); override;
    procedure ExecuteTarget(Target: TObject); override;
    procedure ExecuteTextTarget(Target: TTextChild); virtual; abstract;
  end;

  TTextNew = class(TTextChildAction)
  public
    constructor Create(AOwner: TComponent); override;
    function Execute: Boolean; override;
    procedure ExecuteTextTarget(Target: TTextChild); override;
  end;

  TTextOpen = class(TTextChildAction)
  public
    constructor Create(AOwner: TComponent); override;
    function Execute: Boolean; override;
    procedure ExecuteTextTarget(Target: TTextChild); override;
  end;

  { TTextSave }

  TTextSave = class(TTextChildAction)
  public
    constructor Create(AOwner: TComponent); override;
    procedure ExecuteTextTarget(Target: TTextChild); override;
    procedure UpdateTarget(Target: TObject); override;
  end;

  TTextSaveAll = class(TTextChildAction)
  public
    constructor Create(AOwner: TComponent); override;
    function Execute: Boolean; override;
    procedure ExecuteTextTarget(Target: TTextChild); override;
    procedure UpdateTarget(Target: TObject); override;
  end;

{$IFDEF SINGLE}
procedure Register;
{$ENDIF}

implementation

uses
  {$IFNDEF CLX} Forms, Menus, { Windows, } Dialogs,
  {$ELSE} QForms, QMenus, QDialogs, {$ENDIF}
  LogFind, SysUtils;

{$IFDEF SINGLE}
procedure Register;
begin
    RegisterActions('Text', [TTextNew, TTextOpen, TTextSave, TTextSaveAll], nil);
end;
{$ENDIF}

{ TTextChildAction }

procedure TTextChildAction.ExecuteTarget(Target: TObject);
begin
  ExecuteTextTarget(Target as TTextChild);
end;

function TTextChildAction.HandlesTarget(Target: TObject): Boolean;
begin
  Result := ((Control <> nil) and (Target = Control))
            or ((Control = nil) and (Target is TTextChild) {$IFNDEF lCL_} and TTextChild(Target).Memo.Focused{$ENDIF});
end;

procedure TTextChildAction.UpdateTarget(Target: TObject);
begin
  Enabled := True;
end;

{ TTextNew }

constructor TTextNew.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SetShortCut(Word('N'), [ssCtrl]);
  Hint := 'New|Create New text document';
  ImageIndex := 6;
  Caption := '&New';
  DisableIfNoHandler := False;
end;

function TTextNew.Execute: Boolean;
var
  TextChild: TTextChild;
begin
  Result := inherited Execute;
  if not Result then begin
    TextChild := AnyTextChild; { Have a stab at copying defaults }
    if TextChild <> nil then
      TextChild.NewClick(nil)
    else
      TTextChild.Create(Application);
  end;
  Result := True;
end;

procedure TTextNew.ExecuteTextTarget(Target: TTextChild);
begin
  Target.NewClick(Self);
end;

{ TTextOpen }

constructor TTextOpen.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SetShortCut(Word('O'), [ssCtrl]);
  Hint := 'Open|Open a file';
  ImageIndex := 7;
  Caption := '&Open';
  DisableIfNoHandler := False;
end;

function TTextOpen.Execute: Boolean;
begin
  Result := inherited Execute;
  if not Result then
    AskAndOpenFileInTextChild;
  Result := True;
end;

procedure TTextOpen.ExecuteTextTarget(Target: TTextChild);
begin
  Target.FileOpenClick(Self);
end;

{ TTextSave }

constructor TTextSave.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SetShortCut(Word('S'), [ssCtrl]);
  Hint := 'Save|Save current file';
  ImageIndex := 8;
  Caption := '&Save';
end;

procedure TTextSave.ExecuteTextTarget(Target: TTextChild);
begin
  Target.SaveClick(Self);
end;

procedure TTextSave.UpdateTarget(Target: TObject);
begin
  Enabled := (Target as TTextChild).Memo.Modified;
end;

{ TTextSaveAll }

constructor TTextSaveAll.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Hint := 'Save All|Save all changed files';
  ImageIndex := 8;
  Caption := 'Sa&ve All';
  DisableIfNoHandler := False;
end;

function TTextSaveAll.Execute: Boolean;
begin
  SaveAllTextChildren;
  Result := True;
end;

procedure TTextSaveAll.ExecuteTextTarget(Target: TTextChild);
begin
  SaveAllTextChildren;
end;

procedure TTextSaveAll.UpdateTarget(Target: TObject);
begin
  Enabled := AnyTextChildModified;
end;

end.
