unit ToolBarEx;
{ Replacement for TToolBar, to allow toolbar items
  to work for Forms/dialogs separate from form containing
  ToolBar (such as Copy/Paste Buttons to operate in a text window
  in a dialog box) }

interface
{$I DFS.inc}{ Copyright (C) 2005 C M Western; see PcontrolsReg.pas for license}
uses
  {$IFNDEF CLX_} { Windows, } Messages, Graphics, Controls, Forms, Dialogs, ToolWin, ComCtrls, Buttons, ExtCtrls,
  {$ELSE} QGraphics, QControls, QForms, QDialogs, QComCtrls, QExtCtrls, {$ENDIF}
  SysUtils, Classes;

{$IFDEF OffLCL_}
{$DEFINE USE_PANEL}
{$ENDIF}

type

  { TToolBarEx }

  TToolBarEx = class({$IFDEF USE_PANEL}TPanel{$ELSE}TToolBar{$ENDIF})
  private
    FActiveForm: TCustomForm;
    {$IFDEF USE_PANEL}
    FIndent: Integer;
    FImages: TImageList;
    FFlat: Boolean;
    FEdgeBorders: TEdgeBorders;
    FEdgeInner: TEdgeStyle;
    FEdgeOuter: TEdgeStyle;
    {$ENDIF}
  {$IFDEF MSWINDOWS}
  protected
    procedure WMMouseActivate(var Message: TMessage); message WM_MOUSEACTIVATE;
    procedure WndProc(var Message: TMessage); override;
  {$ENDIF}
  public
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    {$IFDEF USE_PANEL}
    constructor Create(AOwner: TComponent); override;
    {$ENDIF}
  published
    {$IFDEF USE_PANEL}
    property Flat: Boolean read FFlat write FFlat default False;
    property Indent: Integer read FIndent write FIndent default 0;
    property Images: TImageList read FImages write FImages;
    property EdgeBorders: TEdgeBorders read FEdgeBorders write FEdgeBorders default [ebTop];
    property EdgeInner: TEdgeStyle read FEdgeInner write FEdgeInner default esRaised;
    property EdgeOuter: TEdgeStyle read FEdgeOuter write FEdgeOuter default esLowered;
    {$ENDIF}
  end;

  { TToolButtonEx }

  TToolButtonEx = class({$IFDEF USE_PANEL}TSpeedButton{$ELSE}TToolButton{$ENDIF})
  private
    {$IFDEF USE_PANEL}
    FImageIndex: Integer;
    FStyle: TToolButtonStyle;
    procedure SetImageIndex(const AValue: Integer);
    procedure SetStyle(const AValue: TToolButtonStyle);
    {$ENDIF}
  protected
    {$IFDEF USE_PANEL}
    {$IFDEF LCL_}
    procedure RealSetText(const Value: TCaption); override;
    {$ELSE}
    procedure SetText(const Value: TCaption); override;
    {$ENDIF}
    {$ENDIF}
  public
    {$IFDEF USE_PANEL}
    constructor Create(AOwner: TComponent); override;
    {$ENDIF}
  published
    {$IFDEF USE_PANEL}
    property ImageIndex: Integer read FImageIndex write SetImageIndex;
    property Style: TToolButtonStyle read FStyle write SetStyle default tbsButton;
    property Flat default True;
    {$ENDIF}
  end;

{$IFDEF SINGLE}
procedure Register;
{$ENDIF}

implementation

{$IFDEF SINGLE}
procedure Register;
begin
  RegisterComponents('CMW', [TToolBarEx, TToolButtonEx]);
end;
{$ENDIF}

{ TToolBarEx }

procedure TToolBarEx.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  if (Operation = opRemove) and (AComponent = FActiveForm) then
    FActiveForm := nil;
  inherited;
end;

{$IFDEF MSWINDOWS}
// TODO: Linux
procedure TToolBarEx.WMMouseActivate(var Message: TMessage);
var
  ParentForm: TCustomForm;
begin
  FActiveForm := nil;
  if Screen.ActiveCustomForm <> nil then begin
    ParentForm := GetParentForm(Self);
    if (ParentForm <> Screen.ActiveCustomForm)
       and ( not(ParentForm is TForm)
             {$IFNDEF LCL}or ((ParentForm as TForm).ActiveMDIChild <> Screen.ActiveCustomForm){$ENDIF} ) then
      FActiveForm := Screen.ActiveCustomForm;
    if FActiveForm <> nil then
      FActiveForm.FreeNotification(Self);
  end;
  inherited;
end;

procedure TToolBarEx.WndProc(var Message: TMessage);
var
  Control: TControl;
begin
  if FActiveForm <> nil then
    case Message.Msg of
      WM_LBUTTONDOWN, WM_RBUTTONDOWN, WM_MBUTTONDOWN:
        begin
          inherited WndProc(Message);
          Control := GetCaptureControl;
          if (Control = nil) or (Control.Parent <> Self) then
            FActiveForm := nil;
          Exit;
        end;
      WM_LBUTTONUP, WM_RBUTTONUP, WM_MBUTTONUP:
        begin
          with FActiveForm do begin
            if Visible and Enabled then
            SetFocus;
          end;
          FActiveForm := nil;
        end;
    end;
  inherited WndProc(Message);
end;
{$ENDIF}


{$IFDEF USE_PANEL}
constructor TToolBarEx.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Align := alTop;
  EdgeBorders := [ebTop];
  EdgeInner := esRaised;
  EdgeOuter := esLowered;
end;
{$ENDIF}

{ TToolButtonEx }

{$IFDEF USE_PANEL}
procedure TToolButtonEx.SetImageIndex(const AValue: Integer);
var
  ImageList: TImageList;
  Bmp, Mask: TBitmap;
  ImageRect: TRect;
begin
  if FImageIndex=AValue then exit;
  FImageIndex:=AValue;
  if (FImageIndex >= 0) and (Parent is TToolBarEx) then begin
    ImageList := TToolBarEx(Parent).Images;
    if Assigned(ImageList) and (ImageList.Count > ImageIndex) then begin
      ImageList.GetInternalImage(ImageIndex, Bmp, Mask, ImageRect);
      Glyph.Assign(Bmp);
    end;
  end;
end;

procedure TToolButtonEx.SetStyle(const AValue: TToolButtonStyle);
begin
  if FStyle=AValue then exit;
  FStyle:=AValue;
  if Style in [tbsSeparator, tbsDivider] then
    Enabled := False;
end;

{$IFDEF LCL_}
procedure TToolButtonEx.RealSetText(const Value: TCaption);
begin
  // inherited RealSetText(Value);
end;
{$ELSE}
procedure TToolButtonEx.SetText(const Value: TCaption);
begin
  // inherited RealSetText(Value);
end;
{$ENDIF}

constructor TToolButtonEx.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Flat := True;
  FImageIndex := -1;
  FStyle := tbsButton;
  SetBounds(1,1,23,22);
end;

{$ENDIF}

end.
